########################################################
# SDP_CHECK_OPENGL - tests for an installed OpenGL library
AC_DEFUN(SDP_CHECK_OPENGL, [
AC_SUBST(OPENGL_CFLAGS)
AC_SUBST(OPENGL_LIBS)
AC_MSG_CHECKING([for an OpenGL installation])

sdp_check_opengl_lib_dir=""
sdp_check_opengl_directories="/usr/lib /usr/local/lib /usr/X11R6/lib"
for sdp_check_opengl_directory in $sdp_check_opengl_directories; do
	if test -f $sdp_check_opengl_directory/libGL.so || test -f $sdp_check_opengl_directory/libGL.a; then
		sdp_check_opengl_lib_dir=$sdp_check_opengl_directory
	fi
done

sdp_check_opengl_header_dir=""
sdp_check_opengl_directories="/usr/include /usr/local/include /usr/X11R6/include"
for sdp_check_opengl_directory in $sdp_check_opengl_directories; do
	if test -f $sdp_check_opengl_directory/GL/gl.h; then
		sdp_check_opengl_header_dir=$sdp_check_opengl_directory
	fi
done

if test -n $sdp_check_opengl_lib_dir && test -n $sdp_check_opengl_header_dir; then
	OPENGL_CFLAGS="-I$sdp_check_opengl_header_dir"
	OPENGL_LIBS="-L$sdp_check_opengl_lib_dir -lGL"
	AC_MSG_RESULT([found in $sdp_check_opengl_lib_dir and $sdp_check_opengl_header_dir])
else
	AC_MSG_ERROR([OpenGL required but not found])
fi

])

########################################################
# SDP_CHECK_OPENGLU - tests for an installed OpenGLU library
AC_DEFUN(SDP_CHECK_OPENGLU, [
AC_SUBST(OPENGLU_CFLAGS)
AC_SUBST(OPENGLU_LIBS)
AC_MSG_CHECKING([for an OpenGLU installation])

sdp_check_openglu_lib_dir=""
sdp_check_openglu_directories="/usr/lib /usr/local/lib /usr/X11R6/lib"
for sdp_check_openglu_directory in $sdp_check_openglu_directories; do
	if test -f $sdp_check_openglu_directory/libGLU.so || test -f $sdp_check_openglu_directory/libGLU.a; then
		sdp_check_openglu_lib_dir=$sdp_check_openglu_directory
	fi
done

sdp_check_openglu_header_dir=""
sdp_check_openglu_directories="/usr/include /usr/local/include /usr/X11R6/include"
for sdp_check_openglu_directory in $sdp_check_openglu_directories; do
	if test -f $sdp_check_openglu_directory/GL/gl.h; then
		sdp_check_openglu_header_dir=$sdp_check_openglu_directory
	fi
done

if test -n $sdp_check_openglu_lib_dir && test -n $sdp_check_openglu_header_dir; then
	OPENGLU_CFLAGS="-I$sdp_check_openglu_header_dir"
	OPENGLU_LIBS="-L$sdp_check_openglu_lib_dir -lGLU"
	AC_MSG_RESULT([found in $sdp_check_openglu_lib_dir and $sdp_check_openglu_header_dir])
else
	AC_MSG_ERROR([OpenGL required but not found])
fi

])

########################################################
# SDP_CHECK_GLIB - tests for an installed glib library
AC_DEFUN(SDP_CHECK_GLIB, [
AC_MSG_CHECKING([for glib])
if glib-config --version >/dev/null 2>/dev/null; then
        AC_MSG_RESULT([found version `glib-config --version`])
else
        AC_MSG_ERROR([glib required but not found])
fi
])

########################################################
# SDP_WITH_READLINE - prompts/tests for the GNU readline library
AC_DEFUN(SDP_WITH_READLINE, [
AC_SUBST(READLINE_CFLAGS)
AC_SUBST(READLINE_LIBS)
AC_ARG_WITH(readline, [  --with-readline         use GNU readline for command-line input [[yes]]],
[ if test x"$withval" != xno; then
AC_DEFINE(HAVE_READLINE)
READLINE_CFLAGS=""
READLINE_LIBS="-lreadline -ltermcap"
fi],
[
AC_DEFINE(HAVE_READLINE)
READLINE_CFLAGS=""
READLINE_LIBS="-lreadline -ltermcap"
])
])

########################################################
# K3D_WITH_EXPERIMENTAL
AC_DEFUN(K3D_WITH_EXPERIMENTAL, [
AC_SUBST(EXPERIMENTAL_SUBDIR)
AC_ARG_WITH(experimental, [  --with-experimental     build experimental tools & objects [[no]]],
[
if test x"$withval" != xno; then
EXPERIMENTAL_SUBDIR="experimentallib"
AC_DEFINE(K3D_HAVE_EXPERIMENTAL)
fi
])
])

########################################################
# K3D_WITH_STATIC_PLUGINS
AC_DEFUN(K3D_WITH_STATIC_PLUGINS, [
AC_SUBST(K3D_STATIC_PLUGIN_LIBS)
AC_ARG_WITH(static-plugins, [  --with-static-plugins   statically-link plugins into executable [[no]]],
[
if test x"$withval" != xno; then
AC_DEFINE(K3D_HAVE_STATIC_PLUGINS)
fi
AM_CONDITIONAL(K3D_STATIC_PLUGINS, test x$withval != xno)
])
])

########################################################
# K3D_WITH_PLIB
AC_DEFUN(K3D_WITH_PLIB, [
AC_SUBST(PLIB_SUBDIR)
AC_SUBST(PLIB_LIBS)
AC_ARG_WITH(plib, [  --with-plib             build plib-based import/export library [[yes]]])
if test x$withval != xno; then
AC_CHECK_HEADER([plib/ssg.h], [],
[
AC_MSG_ERROR([couldn't find plib/ssg.h ... if you don't require plib support, re-run configure with the --without-plib option])
])
AC_CHECK_LIB(plibul, ulInit,
[
PLIB_SUBDIR="plib"
PLIB_LIBS="-lplibssg -lplibssgaux -lplibsg -lplibul"
],
[
AC_MSG_ERROR([couldn't link with plib ... if you don't require plib support, re-run configure with the --without-plib option])
])
fi
])

########################################################
# SDP_CHECK_GTK - tests for an installed gtk+ library
AC_DEFUN(SDP_CHECK_GTK, [
AC_MSG_CHECKING([for gtk])
if gtk-config --version >/dev/null 2>/dev/null; then
        AC_MSG_RESULT([found version `gtk-config --version`])
else
        AC_MSG_ERROR([gtk required but not found])
fi
])

########################################################
# SDP_CHECK_SDPTYPES - tests for an installed sdptypes library
AC_DEFUN(SDP_CHECK_SDPTYPES, [
AC_SUBST(SDPTYPES_CFLAGS)
AC_SUBST(SDPTYPES_LIBS)
AC_MSG_CHECKING([for sdptypes])
if sdptypes-config --version >/dev/null 2>/dev/null; then
	SDPTYPES_CFLAGS=`sdptypes-config --cflags`
	SDPTYPES_LIBS=`sdptypes-config --libs`
        AC_MSG_RESULT([found version `sdptypes-config --version`])
else
	AC_MSG_ERROR([sdptypes required but not found])
fi
])

########################################################
# SDP_CHECK_SDPXML - tests for an installed sdpxml library
AC_DEFUN(SDP_CHECK_SDPXML, [
AC_SUBST(SDPXML_CFLAGS)
AC_SUBST(SDPXML_LIBS)
AC_MSG_CHECKING([for sdpxml])
if sdpxml-config --version >/dev/null 2>/dev/null; then
	SDPXML_CFLAGS=`sdpxml-config --cflags`
	SDPXML_LIBS=`sdpxml-config --libs`
	AC_MSG_RESULT([found version `sdpxml-config --version`])
else
	AC_MSG_ERROR([sdpxml required but not found])
fi
])

########################################################
# SDP_CHECK_SDPGTK - tests for an installed sdpgtk library
AC_DEFUN(SDP_CHECK_SDPGTK, [
AC_SUBST(SDPGTK_CFLAGS)
AC_SUBST(SDPGTK_LIBS)
AC_MSG_CHECKING([for sdpgtk])
if sdpgtk-config --version >/dev/null 2>/dev/null; then
	SDPGTK_CFLAGS=`sdpgtk-config --cflags`
	SDPGTK_LIBS=`sdpgtk-config --libs`
	AC_MSG_RESULT([found version `sdpgtk-config --version`])
else
	AC_MSG_ERROR([sdpgtk required but not found])
fi
])

########################################################
# SDP_CHECK_SDPGTK2 - tests for an installed sdpgtk2 library
AC_DEFUN(SDP_CHECK_SDPGTK2, [
AC_SUBST(SDPGTK2_CFLAGS)
AC_SUBST(SDPGTK2_LIBS)
AC_MSG_CHECKING([for sdpgtk2])
if sdpgtk2-config --version >/dev/null 2>/dev/null; then
	SDPGTK2_CFLAGS=`sdpgtk2-config --cflags`
	SDPGTK2_LIBS=`sdpgtk2-config --libs`
	AC_MSG_RESULT([found version `sdpgtk2-config --version`])
else
	AC_MSG_ERROR([sdpgtk2 required but not found])
fi
])

########################################################
# SDP_CHECK_SDPGL - tests for an installed sdpgl library
AC_DEFUN(SDP_CHECK_SDPGL, [
AC_SUBST(SDPGL_CFLAGS)
AC_SUBST(SDPGL_LIBS)
AC_MSG_CHECKING([for sdpgl])
if sdpgl-config --version >/dev/null 2>/dev/null; then
	SDPGL_CFLAGS=`sdpgl-config --cflags`
	SDPGL_LIBS=`sdpgl-config --libs`
	AC_MSG_RESULT([found version `sdpgl-config --version`])
else
	AC_MSG_ERROR([sdpgl required but not found])
fi
])

########################################################
# SDP_CHECK_SDPSL - tests for an installed sdpsl library
AC_DEFUN(SDP_CHECK_SDPSL, [
AC_SUBST(SDPSL_CFLAGS)
AC_SUBST(SDPSL_LIBS)
AC_MSG_CHECKING([for sdpsl])
if sdpsl-config --version >/dev/null 2>/dev/null; then
	SDPSL_CFLAGS=`sdpsl-config --cflags`
	SDPSL_LIBS=`sdpsl-config --libs`
	AC_MSG_RESULT([found version `sdpsl-config --version`])
else
	AC_MSG_ERROR([sdpsl required but not found])
fi
])

