#ifndef ANIMATIONPARAMETERS_H
#define ANIMATIONPARAMETERS_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the CAnimationParametersDialog, for altering a document's animation parameters
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3ddialog.h>
#include <k3dtimecontroller.h>

// Forward class references ...
class CTimeStorage;
class k3dIDocument;

///////////////////////////////////////////////////////////////////
// CAnimationParametersDialog

typedef k3dDialog CAnimationParametersDialogBase;

/// Provides a user interface for altering a document's start/end times, framerate, etc.
class CAnimationParametersDialog : public CAnimationParametersDialogBase
{
public:
	CAnimationParametersDialog(sdpIUnknown* const ParentCommandNode, CTimeStorage* const TimeStorage, k3dIDocument* const Document);
	virtual ~CAnimationParametersDialog();

	// k3dIEventObserver implementation
	sdpResult ObserveEvent(k3dIEventSource* const Source, SDPCSTRING Event, SDPCSTRING EventData, SDPCSTRING ObserverData);

protected:
	/// Stores a reference to the owning document
	k3dIDocument* const m_Document;
	/// Stores the time storage
	CTimeStorage* const m_TimeStorage;

	static const sdpString dialogname;
	static const sdpString control_duration;
	static const sdpString control_framerate;
	static const sdpString control_totalframes;
	static const sdpString control_firstframe;
	static const sdpString control_viewframe;
	static const sdpString control_lastframe;
};

#endif // ANIMATIONPARAMETERS_H


