#ifndef BLACKBOXRECORDER_H
#define BLACKBOXRECORDER_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the CBlackBoxRecorder class, which provides a UI for real-time recording of user actions for troubleshooting purposes
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3ddialog.h>
#include <k3dicommandrecorder.h>
#include <fstream>

// Forward class references ...
class CApplication;
class k3dIScriptEngine;

////////////////////////////////////////////////////////////////////////////
// CBlackBoxRecorder

typedef k3dDialog CBlackBoxRecorderBase;

/// Implements the application user interface
class CBlackBoxRecorder : public CBlackBoxRecorderBase,
										public k3dICommandRecorder
{
public:
	CBlackBoxRecorder(CApplication* Application, SDPCSTRING OutputFilePath);
	~CBlackBoxRecorder();

	// k3dICommandRecorder overrides
	sdpResult IsRecording();
	sdpResult RecordCommand(k3dICommandNode* CommandNode, k3dICommandNode::CommandType Type, SDPCSTRING Name, SDPCSTRING Arguments);

	// k3dIEventObserver implementation
	sdpResult ObserveEvent(k3dIEventSource* const Source, SDPCSTRING Event, SDPCSTRING EventData, SDPCSTRING ObserverData);

	static const sdpString dialogname;
	static const sdpString control_cancel;

protected:
	/// Overrides sdpGtkObjectContainer to handle custom events
	void OnEvent(sdpGtkEvent* Event);
	void OnDelete(sdpGtkEvent* Event);
	void OnCancel();

	/// The owning application
	CApplication* m_Application;
	/// Script engine for the script being recorded
	k3dIScriptEngine* m_ScriptEngine;
	/// Path to the output file stream during recording
	sdpString m_OutputFilepath;
	/// Output file stream during recording
	std::ofstream m_OutputFile;
};

#endif // BLACKBOXRECORDER_H

