// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Implements the CCommandNodeInspector class
		\author Tim Shead (tshead@k-3d.com)
*/

#include "commandnodeinspector.h"
#include "application.h"

#include <k3dmisc.h>

#include <k3dicommandnode.h>

namespace
{

void InsertNode(sdpGtkCTree& Tree, GtkCTreeNode* const ParentTreeNode, k3dICommandNode* const CommandNode)
{
	// Sanity checks ...
	g_return_if_fail(CommandNode);
	
	// Insert this node ...
	SDPCSTRING text[] = 
	{
		CommandNode->CommandNodeName()
	};
	GtkCTreeNode* const node = Tree.InsertNode(ParentTreeNode, 0, text, 0, 0, 0, 0, 0);
	
	// Recursively insert children ...
	for(K3DCOMMANDNODEHANDLE handle = CommandNode->FirstCommandNode(); handle; handle = CommandNode->NextCommandNode(handle))
		InsertNode(Tree, node, CommandNode->CommandNode(handle));
}

} // namespace

//////////////////////////////////////////////////////////////////////
// CCommandNodeInspector

CCommandNodeInspector::CCommandNodeInspector(CApplication* const Application) :
	CCommandNodeInspectorBase(Application, "commandnodeinspector", new k3dOptionsWindowGeometryStore(k3dApplication(), 0)),
	m_Selection(0),
	m_ApplicationCloseEvent(dynamic_cast<k3dIEventSource*>(Application), this, k3dIApplication::Event_Close, g_pretty_function(), 0)
{
	// Sanity checks ...
	g_assert(Application);

	// Load dialog template ...
	g_return_if_fail(LoadGTKMLTemplate("commandnodeinspector.gtkml"));
	
	// Populate our controls ...
	UpdateTree();
	
	Show();
}

sdpResult CCommandNodeInspector::ObserveEvent(k3dIEventSource* const Source, SDPCSTRING Event, SDPCSTRING EventData, SDPCSTRING ObserverData)
{
	// Sanity checks ...
	g_return_val_if_string_fail(Event, SDP_E_FAIL);

	const sdpString event(Event);
	if(event == k3dIApplication::Event_Close)
		{
			delete this;
			return SDP_S_OK;
		}
		
	return CCommandNodeInspectorBase::ObserveEvent(Source, Event, EventData, ObserverData);
}

void CCommandNodeInspector::UpdateTree()
{
	// Get the tree ...
	sdpGtkCTree tree = CTree("tree");

	// Start out fresh ...
	tree.Clear();

	// Freeze updates ...
	tree.Freeze();

	// Create the tree structure ...
	InsertNode(tree, 0, dynamic_cast<k3dICommandNode*>(k3dApplication()));

	// Setup our columns ...
	tree.ShowColumnTitles();
	tree.SetColumnTitle(0, "Nodes");
	for(int column = 0; column < 1; column++)
		{
			tree.SetColumnVisibility(column, true);
			tree.SetColumnWidth(column, tree.GetOptimalColumnWidth(column));
		}

	// Restore normal tree updates ...
	tree.Thaw();
}


