#ifndef DIALOGS_H
#define DIALOGS_H

// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the CMessageBox informational dialog
		\author Tim Shead (tshead@k-3d.com)
		\author Dan Erikson <derikson@montana.com>
*/

#include <k3ddialog.h>

////////////////////////////////////////////////////////////////////////////
// CMessageBox

typedef k3dDialog CMessageBoxBase;

/// Creates a modal message box that can be displayed by clients
class CMessageBox :
	public CMessageBoxBase
{
public:
	/**
		 \brief Create a message box with a caption and any number of buttons.
		 \param DefaultButton One-based index of the button selected by default.  If DefaultButton is 0, then no button is selected by default.
	*/
	CMessageBox(k3dIApplication* const Application, SDPCSTRING Message, SDPCSTRING Title, const guint DefaultButton, const sdpStringArray Buttons, SDPCSTRING DialogTemplate);

	/**
		 \brief Make the message box modal and return the button clicked.
		 \return the number of the button pressed.  0 is returned if the dialog box is closed by the user.
	*/
	guint Run();

	/**
		 \brief Add a button
		 \param Label the text of the button to be added.
		 \return the number of the button added.
	*/
	guint AddButton(SDPCSTRING Label);

	/**
		 \brief Add any number of buttons.
		 \param Buttons a collection of buttons to be added.
	 */
   void AddButtons(const sdpStringArray Buttons);

protected:
	void OnEvent(sdpGtkEvent* Event);
	void OnDelete(sdpGtkEvent* Event);
	
	/**
		 \brief the number of the button pressed.  This value is returned by Run().
	*/
	guint m_Result;
	/**
		 \brief the number of the last button to be added to the dialog box.
	*/
	guint m_LastButton;

private:
	static const sdpString dialogname;
	static const sdpString control_buttonbox;
};

#endif // DIALOGS_H
