#ifndef OBJECTCOLLECTION_H
#define OBJECTCOLLECTION_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares CObjectCollection, an implementation of k3dIObjectCollection
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3deditable.h>
#include <k3deventsource.h>

#include <k3diobject.h>
#include <k3diobjectcollection.h>

// Forward class references
class CHierarchy;
class k3dIDependencies;
class k3dIDocument;
class k3dIObjectInfo;

///////////////////////////////////////////////////////////
// CObjectCollection

/// Implements k3dIObjectCollection
class CObjectCollection :
	public k3dIObjectCollection,
	public k3dEditable,
	public k3dEventSource
{
public:
	CObjectCollection(k3dIDocument* const Document, CHierarchy* const Hierarchy);
	~CObjectCollection();

	// k3dIObjectCollection implementation
	k3dIObject* CreateObject(const sdp::ClassID& ClassID);
	k3dIObject* CreateObject(sdpxml::Document& Document, sdpxml::Element& Element);
	bool DeleteObject(k3dIObject* const Object);
	void VisitObjects(k3dIObjectVisitor* const Visitor);
	K3DOBJECTHANDLE FirstObject();
	K3DOBJECTHANDLE NextObject(K3DOBJECTHANDLE Object);
	k3dIObject* Object(K3DOBJECTHANDLE Object);

	// Partial k3dIObject implementation
	bool InitComplete();

	// Partial k3dIPersist implementation
	bool Load(sdpxml::Document& Document, sdpxml::Element& Element);
	bool LoadComplete();
	bool Save(sdpxml::Element& Element, k3dIDependencies* const Dependencies);
	bool SaveComplete();

	// k3dIEditable implementation
	void PushEditing();
	void PopEditing();

private:
	CObjectCollection(const CObjectCollection& RHS);
	CObjectCollection& operator=(CObjectCollection& RHS);

	/// Non-destructively deletes an object
	bool RawDeleteObject(k3dIObject* const Object);
	/// Permanently destroys an object
	bool RawDestroyObject(k3dIObject* const Object);
	
	/// Returns a guaranteed-unique object ID
	const k3dIObject::ID NextUniqueObjectID();
	k3dIObject* RawCreateObject(k3dIObjectInfo* const ObjectInfo, const k3dIObject::ID ObjectID);
	
	/// Stores a reference to the owning document
	k3dIDocument* const m_Document;
	/// Stores a reference to an associated hierarchical view of this collection's objects
	CHierarchy* const m_Hierarchy;
	
	/// Stores a collection of objects
	typedef std::vector<k3dIObject*> Objects;
	typedef Objects::iterator ObjectIterator;
	Objects m_Objects;
	
	/// Set to true iff the collection has been modified during the current edit session
	bool m_CollectionChanged;
};

#endif // OBJECTCOLLECTION_H

