#ifndef OPTIONS_H
#define OPTIONS_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the COptionsDialog, which provides a UI for changing global user options
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3ddialog.h>

// Forward class declarations
class CApplication;

///////////////////////////////////////////////////////////////////
// COptionsDialog

typedef k3dDialog COptionsDialogBase;

/// Provides a user interface for altering global user options.
class COptionsDialog :
	public COptionsDialogBase
{
public:
	COptionsDialog(CApplication* const Application);

protected:
	/// Called to handle our custom events
	void OnEvent(sdpGtkEvent* Event);

	void OnDelete(sdpGtkEvent* Event);
	void OnShowTutorialMenu();
	void OnRestoreWindows();
	void OnTemplateDocument();
	void OnPreferredLanguage();
	void OnChooseTemplateDocument();
	void OnHTMLViewer();
	void OnBitmapViewer();
	void OnMPEGViewer();
	void OnSLPreprocessor();
	void OnSLParser();
	void OnDefaultEngine();
	void OnRenderFarmPath();

	/// Called to put formatted data in all dialog controls
	void UpdateControls();

	/// Stores a reference to the parent application
	CApplication* const m_Application;

	static const sdpString dialogname;
	static const sdpString control_showtutorialmenu;
	static const sdpString control_restorewindows;
	static const sdpString control_templatedocument;
	static const sdpString control_choosetemplatedocument;
	static const sdpString control_preferredlanguage;
	static const sdpString control_defaultengine;
	static const sdpString control_htmlviewer;
	static const sdpString control_bitmapviewer;
	static const sdpString control_mpegviewer;
	static const sdpString control_slpreprocessor;
	static const sdpString control_slparser;
	static const sdpString control_renderfarmpath;
};

#endif // OPTIONS_H
