#ifndef RENAMEOBJECTDIALOG_H
#define RENAMEOBJECTDIALOG_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the CRenameObjectDialog class, which provides a UI for renaming objects
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3ddialog.h>

// Forward class declarations
class k3dIObject;

///////////////////////////////////////////////////////////////////
// CRenameObjectDialog

typedef k3dDialog CRenameObjectDialogBase;

/// Provides a user interface for changing an object's name
class CRenameObjectDialog : public CRenameObjectDialogBase
{
public:
	CRenameObjectDialog(k3dIObject* const Object);
	~CRenameObjectDialog();

	// k3dIEventObserver implementation
	sdpResult ObserveEvent(k3dIEventSource* const Source, SDPCSTRING Event, SDPCSTRING EventData, SDPCSTRING ObserverData);

protected:
	/// Called to handle our custom events
	void OnEvent(sdpGtkEvent* Event);

	void OnDelete(sdpGtkEvent* Event);
	void OnNewName();
	void OnOK();
	void OnCancel();

	/// Called to put formatted data in all dialog controls
	void UpdateControls();

	/// Stores the object to be renamed
	k3dIObject* const m_Object;
	
	static const sdpString dialogname;
	static const sdpString control_oldname;
	static const sdpString control_newname;
	static const sdpString control_ok;
	static const sdpString control_cancel;
};

#endif // !RENAMEOBJECTDIALOG_H

