// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Implements the CShaderEditor class, which provides a UI for editing / compiling RenderMan Shading Language source
		\author Tim Shead (tshead@k-3d.com)
*/

#include "shadereditor.h"
#include "application.h"

#include <k3dmisc.h>
#include <k3doptions.h>

//////////////////////////////////////////////////////////////////////
// CShaderEditor

CShaderEditor::CShaderEditor(CApplication* const Application) :
	CShaderEditorBase(Application, "shadereditor")
{
	// Sanity checks ...
	g_assert(Application);

	// Load dialog template ...
	g_return_if_fail(LoadGTKMLTemplate("shadereditor.gtkml"));
	
	k3dIEventSource* const eventserver = dynamic_cast<k3dIEventSource*>(k3dApplication());
	if(eventserver)
		eventserver->AddEventObserver(this, k3dIApplication::Event_Close, 0);
		
	Show();
}

CShaderEditor::~CShaderEditor()
{
	k3dIEventSource* const eventserver = dynamic_cast<k3dIEventSource*>(k3dApplication());
	if(eventserver)
		eventserver->RemoveEventObserver(this);
}

sdpResult CShaderEditor::ObserveEvent(k3dIEventSource* const Source, SDPCSTRING Event, SDPCSTRING EventData, SDPCSTRING ObserverData)
{
	// Sanity checks ...
	g_return_val_if_string_fail(Event, SDP_E_FAIL);

	const sdpString event(Event);
	if(event == k3dIApplication::Event_Close)
		{
			OnFileClose();
			return SDP_S_OK;
		}
		
	return CShaderEditorBase::ObserveEvent(Source, Event, EventData, ObserverData);
}

