#ifndef TIMESTORAGE_H
#define TIMESTORAGE_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the CTimeStorage class, which provides an implementation of k3dITime
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3deventobserver.h>
#include <k3deventsource.h>
#include <k3dvariable.h>
#include <k3dvariablecontainer.h>

#include <k3ditime.h>

// Forward class declarations
class k3dIStateRecorder;

/////////////////////////////////////////////////////////////////////
// CTimeStorage

/// Provides an implementation of k3dITime
class CTimeStorage : public k3dITime,
									public k3dEventSource,
									public k3dEventObserver,
									public k3dVariableContainer
{
public:
	CTimeStorage(k3dIStateRecorder* const StateRecorder);
	virtual ~CTimeStorage();

	// k3dITime overrides ...
	gdouble StartTime();
	gdouble ViewTime();
	gdouble EndTime();
	gdouble Duration();
	gdouble FrameRate();
	gdouble FrameDelta();
	gdouble RenderTime();
	gdouble RenderDeltaTime();

	sdpResult SetStartTime(const gdouble Time);
	sdpResult SetViewTime(const gdouble Time);
	sdpResult SetEndTime(const gdouble Time);
	sdpResult SetDuration(const gdouble Duration);
	sdpResult SetFrameRate(const gdouble Rate);
	sdpResult SetRenderTime(const gdouble Time);
	sdpResult SetRenderDeltaTime(const gdouble Time);

	sdpResult Load(sdpxml::Document& Document, sdpxml::Element& Element);
	sdpResult Save(sdpxml::Element& Element, k3dIDependencies* Dependencies);

	/// Stores the current animation start time, as a percentage
	k3dVariable<gdouble> m_StartTime;
	/// Stores the current animation view time, as a percentage
	k3dVariable<gdouble> m_ViewTime;
	/// Stores the current animation end time, as a percentage
	k3dVariable<gdouble> m_EndTime;
	/// Stores the current animation duration, in seconds
	k3dVariable<gdouble> m_Duration;
	/// Stores the current animation frame rate, in frames-per-second
	k3dVariable<gdouble> m_FrameRate;
	/// Stores the current render time
	gdouble m_RenderTime;
	/// Stores the current render delta time
	gdouble m_RenderDeltaTime;
};

#endif // TIMESTORAGE_H


