#ifndef TUTORIALMENU_H
#define TUTORIALMENU_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the CTutorialMenu class, which provides a UI for playing tutorials
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3ddialog.h>

// Forward class references ...
class CApplication;

////////////////////////////////////////////////////////////////////////////
// CTutorialMenu

typedef k3dDialog CTutorialMenuBase;

/// Implements the application user interface
class CTutorialMenu : public CTutorialMenuBase
{
public:
	CTutorialMenu(CApplication* Application);
	~CTutorialMenu();
	
	// k3dIEventObserver implementation
	sdpResult ObserveEvent(k3dIEventSource* const Source, SDPCSTRING Event, SDPCSTRING EventData, SDPCSTRING ObserverData);

	static const sdpString dialogname;
	static const sdpString control_tutorials;
	static const sdpString control_play;
	static const sdpString control_cancel;

protected:
	/// Overrides sdpGtkObjectContainer to handle custom events
	void OnEvent(sdpGtkEvent* Event);

	void OnDelete(sdpGtkEvent* Event);
	void OnPlay();
	void OnCancel();
	void OnTutorials(sdpGtkEvent* Event);

	/// The owning frame for this editor
	CApplication* m_Application;
	
	/// Maps user-visible tutorial titles to filepaths
	typedef std::map<sdpString, sdpString> PathMap;
	typedef PathMap::iterator PathIterator;
	PathMap m_PathMap;
};

#endif // TUTORIALMENU_H
