#ifndef TUTORIALMESSAGE_H
#define TUTORIALMESSAGE_H

// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the CTutorialMessage informational dialog
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3ddialog.h>
#include <k3diapplication.h>

////////////////////////////////////////////////////////////////////////////
// CTutorialMessage

typedef k3dDialog CTutorialMessageBase;

/// Creates a semi-modal message box that can be displayed by interactive tutorial scripts
class CTutorialMessage :
	public CTutorialMessageBase
{
public:
	CTutorialMessage(CTutorialMessage*& OwnerReference);
	~CTutorialMessage();
	
	void Update(SDPCSTRING Message);
	void Update(SDPCSTRING Message, k3dIApplication::TutorialMouseAction Action, GdkModifierType Modifiers);

	bool Cancelled();

	// k3dICommandNode overrides
	sdpResult Command(SDPCSTRING Name, SDPCSTRING Arguments, sdpIUnknown** Result);

private:
	/// Common UI creation code
	bool Create();

	/// Overrides sdpGtkObjectContainer to handle custom events
	void OnEvent(sdpGtkEvent* Event);
	
	void OnDelete(sdpGtkEvent* const Event);
	void OnKeyPress(sdpGtkEvent* const Event);
	void OnMouseEvent(sdpGtkEvent* const Event);
	void OnSpeed();
	void OnCruiseControl();
	void OnContinue();
	void OnQuit();

	void SetMessage(SDPCSTRING Message);
	void SetMouseDiagram(SDPCSTRING MouseDiagram);
	void SetModifiersDiagram(SDPCSTRING ModifiersDiagram);

	CTutorialMessage*& m_OwnerReference;

	sdpString m_CurrentMessage;
	sdpString m_CurrentMouseDiagram;
	sdpString m_CurrentModifiersDiagram;

	gdouble m_CurrentSpeed;

	bool m_Cancelled;

	static const sdpString dialogname;
	static const sdpString control_close;
	static const sdpString control_keypress;
	static const sdpString control_cruisecontrol;
	static const sdpString control_speed;
	static const sdpString control_currentspeed;
};

#endif // TUTORIALMESSAGE_H

