#ifndef K3DIBEZIERCHANNEL_H
#define K3DIBEZIERCHANNEL_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares 3dIBezierChannel, an abstract interface for a two-dimensional Bezier control spline
		\author Tim Shead (tshead@k-3d.com)
*/

#ifndef SDPTYPES_H
#include <sdptypes.h>
#endif // !SDPTYPES_H

// Forward class declarations
class k3dIBezierChannelNode;
class k3dIEditable;

/// Abstract interface for objects that wish to enumerate over a collection of k3dIBezierChannelNode objects
class k3dIBezierChannelNodeVisitor
{
public:
	virtual sdpResult VisitBezierChannelNode(k3dIBezierChannelNode* const Node) = 0;

protected:
	virtual ~k3dIBezierChannelNodeVisitor() {}
};

/// Abstract interface for a two-dimensional Bezier control spline
class k3dIBezierChannel :
	public virtual sdpIUnknown
{
public:
	/// Returns a k3dIEditable object for this channel
	virtual k3dIEditable* Editable() = 0;

	/// Sets the range of allowable values for one-dimensional data
	virtual sdpResult SetRange1(const gdouble Min, const gdouble Max) = 0;
	/// Sets the range of allowable values for two-dimensional data
	virtual sdpResult SetRange2(const gdouble Min[2], const gdouble Max[2]) = 0;
	/// Sets the range of allowable values for three-dimensional data
	virtual sdpResult SetRange3(const gdouble Min[3], const gdouble Max[3]) = 0;
	/// Returns the range of allowable values for one-dimensional data
	virtual sdpResult Range1(gdouble* Min, gdouble* Max) = 0;
	/// Returns the range of allowable values for two-dimensional data
	virtual sdpResult Range2(gdouble Min[2], gdouble Max[2]) = 0;
	/// Returns the range of allowable values for three-dimensional data
	virtual sdpResult Range3(gdouble Min[3], gdouble Max[3]) = 0;
	/// Sets the total number of segments in the channel ... total number of nodes will be (3 * SegmentCount) + 1
	virtual sdpResult SetSegmentCount(const gulong SegmentCount) = 0;
	/// Enumerates nodes using the Visitor Design Pattern
	virtual sdpResult VisitNodes(k3dIBezierChannelNodeVisitor* const Visitor) = 0;

protected:
	virtual ~k3dIBezierChannel() {}
};

#endif // K3DIBEZIERCHANNEL_H
