#ifndef K3DIBEZIERCHANNELVALUENODE_H
#define K3DIBEZIERCHANNELVALUENODE_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares k3dIBezierChannelValueNode, an abstract interface for a two-dimensional Bezier control point with extra value data
		\author Tim Shead (tshead@k-3d.com)
*/

#ifndef SDPTYPES_H
#include <sdptypes.h>
#endif // !SDPTYPES_H

/// Abstract interface for a two-dimensional Bezier control point with extra value data
class k3dIBezierChannelValueNode :
	public virtual sdpIUnknown
{
public:
	/// Returns the one-dimensional node value
	virtual sdpResult Value1(gdouble& Value) = 0;
	/// Returns the two-dimensional node value
	virtual sdpResult Value2(sdpVector2& Value) = 0;
	/// Returns the three-dimensional node value
	virtual sdpResult Value3(sdpVector3& Value) = 0;
	/// Sets the value of one-dimensional data
	virtual sdpResult SetValue1(const gdouble Value) = 0;
	/// Sets the value of two-dimensional data
	virtual sdpResult SetValue2(const sdpVector2& Value) = 0;
	/// Sets the value of three-dimensional data
	virtual sdpResult SetValue3(const sdpVector3& Value) = 0;
	
	static SDPCSTRING Event_Changed;

protected:
	virtual ~k3dIBezierChannelValueNode() {}
};

#endif // K3DIBEZIERCHANNELVALUENODE_H
