#ifndef K3DIBITMAP_H
#define K3DIBITMAP_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares k3dIBitmap, an abstract interface for a 32-bit RGBA bitmap image
		\author Tim Shead (tshead@k-3d.com)
*/

#ifndef SDPTYPES_H
#include <sdptypes.h>
#endif // !SDPTYPES_H

// Forward class references
class k3dIBitmapReadFormat;
class k3dIBitmapWriteFormat;

/// Encapsulates a single pixel within a K-3D bitmap image
class k3dPixel
{
public:
	guchar Red;
	guchar Green;
	guchar Blue;
	guchar Alpha;
};

//////////////////////////////////////////////////////////////////////////////////
// k3dIBitmap

/// Abstract interface for a 32-bit RGBA bitmap image
class k3dIBitmap :
	public virtual sdpIUnknown
{
public:
	/// Creates a new, empty (black) image with the given width and height
	virtual sdpResult NewImage(const gulong Width, const gulong Height) = 0;
	/// Loads an image from the filesystem
	virtual sdpResult LoadImage(SDPCSTRING FilePath, k3dIBitmapReadFormat* const FormatFilter) = 0;
	/// Saves an image to the filesystem
	virtual sdpResult SaveImage(SDPCSTRING FilePath, k3dIBitmapWriteFormat* const FormatFilter) = 0;

	/// Return a pointer to the image data (a 2D array of k3dPixels)
	virtual k3dPixel* Data() = 0;
	/// Returns the current image width
	virtual gulong Width() = 0;
	/// Returns the current image height
	virtual gulong Height() = 0;

	/// Mirrors the current image around the X axis
	virtual sdpResult FlipVertical() = 0;
	/// Mirrors the current image around the Y axis
	virtual sdpResult FlipHorizontal() = 0;
	/// Scales the image in-place
	virtual sdpResult Scale(const gulong NewWidth, const gulong NewHeight) = 0;

	static SDPCSTRING Event_Changed;

protected:
	virtual ~k3dIBitmap() {}
};

#endif // K3DIBITMAP_H
