#ifndef K3DICHANNEL_H
#define K3DICHANNEL_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares k3dIChannel, abstract interface for an object encapsulating a data source that changes over time
		\author Tim Shead (tshead@k-3d.com)
*/

#ifndef SDPTYPES_H
#include <sdptypes.h>
#endif // !SDPTYPES_H

/// Abstract interface for an object encapsulating data that can change over time
class k3dIChannel :
	public virtual sdpIUnknown
{
public:
	typedef enum
	{
		DATA_ANGLE,
		DATA_AREA,
		DATA_COLOR,
		DATA_DISTANCE,
		DATA_FORCE,
		DATA_MASS,
		DATA_PRESSURE,
		DATA_ROTATIONXYZ,
		DATA_SCALAR,
		DATA_SCALEXYZ,
		DATA_TIME,
		DATA_VOLUME,
	} DataType;

	/// Sets the type of data stored by this channel
	virtual sdpResult SetDatatype(const DataType Data) = 0;
	/// Returns the type of data stored by this channel
	virtual DataType Datatype() = 0;
	/// Returns the range of curve values stored by this channel
	virtual sdpResult CurveRange(gdouble* Min, gdouble* Max) = 0;
	/// Returns the one-dimensional value of the channel at the given time
	virtual gdouble Value1(const gdouble Time, const gdouble MaxError) = 0;
	/// Returns the two-dimensional value of the channel at the given time
	virtual sdpResult Value2(const gdouble Time, const gdouble MaxError, gdouble Value[2]) = 0;
	/// Returns the three-dimensional value of the channel at the given time
	virtual sdpResult Value3(const gdouble Time, const gdouble MaxError, gdouble Value[3]) = 0;

	static SDPCSTRING Event_Changed;

protected:
	virtual ~k3dIChannel() {}
};

#endif // K3DICHANNEL_H
