#ifndef K3DICOMMANDNODE_H
#define K3DICOMMANDNODE_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares k3dICommandNode, an abstract interface for objects that can accept commands from external entities
		\author Tim Shead (tshead@k-3d.com)
*/

#ifndef SDPTYPES_H
#include <sdptypes.h>
#endif // !SDPTYPES_H

SDP_DECLARE_HANDLE(K3DCOMMANDNODEHANDLE);

/// Abstract interface for objects that can accept commands from external entities
class k3dICommandNode :
	public virtual sdpIUnknown
{
public:
	/// Enumerates the types of commands that can be executed by a node
	typedef enum
	{
		USERINTERFACE,
		SCRIPTING,
	} CommandType;

	/// Executes a command, with optional return object
	virtual sdpResult Command(SDPCSTRING Name, SDPCSTRING Arguments, sdpIUnknown** Result) = 0;

	/// Returns the name of this command node	
	virtual SDPCSTRING CommandNodeName() = 0;
	/// Returns the parent of this command node, or NULL if it's the root node
	virtual k3dICommandNode* CommandNodeParent() = 0;
	/// Returns a command-node by path (could be relative or absolute)
	virtual k3dICommandNode* CommandNodeByPath(SDPCSTRING Path) = 0;
	
	// Enumeration of child nodes
	virtual K3DCOMMANDNODEHANDLE FirstCommandNode() = 0;
	virtual K3DCOMMANDNODEHANDLE NextCommandNode(K3DCOMMANDNODEHANDLE CommandNode) = 0;
	virtual k3dICommandNode* CommandNode(K3DCOMMANDNODEHANDLE CommandNode) = 0;
	virtual sdpResult AddCommandNode(k3dICommandNode* CommandNode) = 0;
	virtual sdpResult RemoveCommandNode(k3dICommandNode* CommandNode) = 0;

protected:
	virtual ~k3dICommandNode() {}
};

#endif // K3DICOMMANDNODE_H
