#ifndef K3DICURVE_H
#define K3DICURVE_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the k3dICurve abstract interface for objects that represent a NURBS curve of arbitrary degree and configuration
		\author Tim Shead (tshead@k-3d.com)
*/

#ifndef SDPTYPES_H
#include <sdptypes.h>
#endif // !SDPTYPES_H

// Forward class declarations
class k3dIMesh;
class k3dIState;
class k3dIPathPointCollection;
class k3dILocationCollection;
class k3dIKnotCollection;

/// Abstract interface for objects that represent a NURBS curve of arbitrary degree and configuration
class k3dICurve :
	public virtual sdpIUnknown
{
public:
	virtual k3dIMesh* Mesh() = 0;
	virtual k3dIPathPointCollection* Points() = 0;
	virtual k3dILocationCollection* Locations() = 0;
	virtual k3dIKnotCollection* Knots() = 0;
	virtual k3dIState* State() = 0;
	virtual gulong Order() = 0;
	virtual sdpResult SetOrder(gulong Order) = 0;
	
	/// Defines a unique identifier that can be serialized
	typedef gulong ID;
	/// Returns a unique identifier for this object that can be serialized
	virtual const ID CurveID() = 0;

protected:
	virtual ~k3dICurve() {}
};

#endif // K3DICURVE_H

