#ifndef K3DICURVECOLLECTION_H
#define K3DICURVECOLLECTION_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the k3dICurveCollection abstract interface for collections of curve objects
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3dicurve.h>

// Forward class declarations
class k3dICurve;

SDP_DECLARE_HANDLE(K3DCURVEHANDLE);

/// Abstract interface for collections of k3dICurve objects
class k3dICurveCollection :
	public virtual sdpIUnknown
{
public:
	virtual gulong CurveCount() = 0;
	virtual K3DCURVEHANDLE FirstCurve() = 0;
	virtual K3DCURVEHANDLE NextCurve(K3DCURVEHANDLE Curve) = 0;
	virtual k3dICurve* Curve(K3DCURVEHANDLE Curve) = 0;
	virtual sdpResult AddCurve(k3dICurve* Curve) = 0;
	virtual sdpResult RemoveCurve(k3dICurve* Curve) = 0;

protected:
	virtual ~k3dICurveCollection() {}
};

#endif // K3DICURVECOLLECTION_H

