#ifndef K3DIDOCUMENT_H
#define K3DIDOCUMENT_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares k3dIDocument, an abstract interface for an open K-3D document
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3diobject.h>

// Forward class references
class k3dIGeometryReadFormat;
class k3dIGeometryWriteFormat;
class k3dIHierarchy;
class k3dIMouseEventObserver;
class k3dIObjectCollection;
class k3dIStateRecorder;
class k3dITime;

/// Abstract interface for an open K-3D document
class k3dIDocument :
	public virtual sdpIUnknown
{
public:
	/// Returns the collection of K-3D objects within this document
	virtual k3dIObjectCollection* Objects() = 0;
	/// Returns a hierarchical view of the K-3D objects within this document
	virtual k3dIHierarchy* Hierarchy() = 0;
	/// Returns the k3dIStateRecorder interface for this document
	virtual k3dIStateRecorder* StateRecorder() = 0;
	/// Returns the k3dITime interface for this document
	virtual k3dITime* Time() = 0;

	/// Returns the full filepath to the document, or NULL
	virtual SDPCSTRING Path() = 0;
	/// Imports a geometry file into the document, using the given file format
	virtual sdpResult Import(SDPCSTRING FilePath, k3dIGeometryReadFormat* const FormatFilter) = 0;
	/// Exports the document to a geometry file, using the given file format
	virtual sdpResult Export(SDPCSTRING FilePath, k3dIGeometryWriteFormat* const FormatFilter) = 0;
	/// Saves the document to the given file
	virtual sdpResult Save(SDPCSTRING FilePath) = 0;

	/// Sets the current mouse input focus
	virtual void SetMouseFocus(k3dIMouseEventObserver* const Observer) = 0;
	/// Returns the current mouse input focus
	virtual k3dIMouseEventObserver* MouseFocus() = 0;

	// Command-node commands ...
	static SDPCSTRING Command_CreateObject;
	static SDPCSTRING Command_DeleteObject;
	static SDPCSTRING Command_Save;
	static SDPCSTRING Command_Close;

	// K-3D events ...	
	static SDPCSTRING Event_Close;
	static SDPCSTRING Event_TitleChanged;
	static SDPCSTRING Event_MouseFocusChanged;
	static SDPCSTRING Event_PlaybackModeChanged;

protected:
	virtual ~k3dIDocument() {}
};

#endif // K3DIDOCUMENT_H
