#ifndef K3DIEDITORRENDER_H
#define K3DIEDITORRENDER_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares k3dIEditorRender, an abstract interface implemented by objects that can draw themselves during editor-mode rendering
		\author Tim Shead (tshead@k-3d.com)
*/

#ifndef SDPTYPES_H
#include <sdptypes.h>
#endif // !SDPTYPES_H

// Forward class references
class k3dICamera;
class k3dIEditorEngine;

/// Abstract interface implemented by objects that can draw themselves during editor-mode rendering
class k3dIEditorRender :
	public virtual sdpIUnknown
{
public:
	virtual sdpResult EditorPreRender(k3dIEditorEngine* const Engine) = 0;
	virtual sdpResult EditorPushRender(k3dIEditorEngine* const Engine) = 0;
	virtual sdpResult EditorRender(k3dIEditorEngine* const Engine) = 0;
	virtual sdpResult EditorPushChildRender(k3dIEditorEngine* const Engine) = 0;
	virtual sdpResult EditorPopChildRender(k3dIEditorEngine* const Engine) = 0;
	virtual sdpResult EditorPopRender(k3dIEditorEngine* const Engine) = 0;
	virtual sdpResult EditorPostRender(k3dIEditorEngine* const Engine) = 0;

protected:
	virtual ~k3dIEditorRender() {}
};

#endif // K3DIEDITORRENDER_H
