#ifndef K3DIEVENTSOURCE_H
#define K3DIEVENTSOURCE_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the k3dIEventSource abstract interface for objects that can emit events
		\author Tim Shead (tshead@k-3d.com)
*/

#ifndef SDPTYPES_H
#include <sdptypes.h>
#endif // !SDPTYPES_H

// Forward class references
class k3dIEventObserver;

/// Abstract interface for objects that can emit events
class k3dIEventSource :
	public virtual sdpIUnknown
{
public:
	/// Called to request that the given client be notified when the specified event occurs
	virtual sdpResult AddEventObserver(k3dIEventObserver* const Observer, SDPCSTRING Event, SDPCSTRING ObserverData) = 0;
	/// Called to remove a client from future event notification
	virtual sdpResult RemoveEventObserver(k3dIEventObserver* const Observer) = 0;
	/// Called to remove a client from future event notification
	virtual sdpResult RemoveEventObserverEvent(k3dIEventObserver* const Observer, SDPCSTRING Event, SDPCSTRING ObserverData) = 0;

protected:
	virtual ~k3dIEventSource() {}
};

#endif // K3DIEVENTSOURCE_H
