#ifndef K3DIHIERARCHY_H
#define K3DIHIERARCHY_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares k3dIHierarchy, an abstract interface for a hierarchical collection of K-3D objects 
		\author Tim Shead (tshead@k-3d.com)
*/

#ifndef SDPTYPES_H
#include <sdptypes.h>
#endif // !SDPTYPES_H

// Forward class declarations
class k3dIObject;

SDP_DECLARE_HANDLE(K3DHIERARCHYNODEHANDLE);

// We have a clash with <windows.h> ... go figure!
#ifdef SDPWIN32
#ifdef ERROR
#undef ERROR
#endif // ERROR
#endif // SDPWIN32

/// Abstract interface for a hierarchical collection of K-3D objects
class k3dIHierarchy :
	public virtual sdpIUnknown
{
public:
	/// Enumerates the types of nodes available within the hierarchy
	typedef enum
	{
		COMPLETE,
		PUSH,
		NODE,
		POP,
		ERROR
	} NodeType;

	/// Returns a handle to the PUSH node of the first K-3D object in the hierarchy, or COMPLETE if empty
	virtual NodeType FirstHierarchyNode(K3DHIERARCHYNODEHANDLE* Handle) = 0;
	/// Returns a handle to the next node in the hierarchy
	virtual NodeType NextHierarchyNode(K3DHIERARCHYNODEHANDLE* Handle) = 0;
	/// Returns a handle to the PUSH node of the given object in the hierarchy, or ERROR if there is none
	virtual NodeType HierarchyNode(k3dIObject* Object, K3DHIERARCHYNODEHANDLE* Handle) = 0;
	/// Converts a node handle into a k3dIObject interface on the underlying object
	virtual k3dIObject* HierarchyObject(K3DHIERARCHYNODEHANDLE Handle) = 0;

	/// Returns an object's parent
	virtual k3dIObject* ObjectParent(k3dIObject* Object) = 0;

	/// Sets an object's parent object
	virtual sdpResult SetObjectParent(k3dIObject* Object, k3dIObject* NewParent) = 0;
	/// Sets an object's sibling object
	virtual sdpResult SetObjectSibling(k3dIObject* Object, k3dIObject* NewSibling) = 0;
	
   	/// Makes k3dILocation WorldXYZ cooordinates current
   	virtual void SnapshotWorldCoordinates() = 0;
   	/// Updates hierarchy objects to reflect the current time
   	virtual void Animate(const gdouble Time, const gdouble FrameDelta) = 0;
	
	/// Event that will be fired when the hierarchy is modified
	static SDPCSTRING Event_Changed;

protected:
	virtual ~k3dIHierarchy() {}
};

#endif // K3DIHIERARCHY_H
