#ifndef K3DISOFTOBJECT_H
#define K3DISOFTOBJECT_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares k3dIImplicitObject, an abstract interface for implicit objects
		\author Romain Behar (romainbehar@yahoo.com)
*/

#ifndef SDPTYPES_H
#include <sdptypes.h>
#endif // !SDPTYPES_H

// Forward class declarations
class k3dIObject;
class k3dIDocument;

/// Defines a point list
typedef std::vector<sdpVector3> PointList;

/// Abstract interface for softobjects returning a float value
class k3dIImplicitObject :
	public virtual sdpIUnknown
{
public:
	/// Returns the parent document
	virtual k3dIDocument* Document() = 0;

	/// Returns a list of points inside object
	// (every topologically distinct surface -must- provide at
	//  least one point inside it, for polygonizer dectection)
	virtual PointList InsidePoints() = 0;

	/// Returns implicit surface value for any point in space
	virtual gdouble ImplicitValue(const sdpVector3& position) = 0;

protected:
	virtual ~k3dIImplicitObject() {}
};

#endif // K3DISOFTOBJECT_H
