#ifndef K3DIIMPLICITOBJECTCOLLECTION_H
#define K3DIIMPLICITOBJECTCOLLECTION_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the k3dIImplicitObjectCollection abstract interface for collections of k3dIImplicitObject objects
		\author Romain Behar (romainbehar@yahoo.com)
*/

// Forward class declarations
class k3dIImplicitObject;

/// Abstract interface for objects that want to enumerate through a collection of k3dIImplicitObject objects
class k3dIImplicitObjectVisitor
{
public:
	virtual void VisitImplicitObject(k3dIImplicitObject* const ImplicitObject) = 0;

protected:
	virtual ~k3dIImplicitObjectVisitor() {}
};

/// Abstract interface for collections of k3dIImplicitObject objects
class k3dIImplicitObjectCollection :
	public virtual sdpIUnknown
{
public:
	/// Enumerates through the collection (Visitor Design Pattern)
	virtual void VisitImplicitObjects(k3dIImplicitObjectVisitor* const Visitor) = 0;

	/// Adds a k3dIImplicitObject object to the collection
	virtual sdpResult AddImplicitObject(k3dIImplicitObject* const ImplicitObject) = 0;
	/// Removes a k3dIImplicitObject object from the collection
	virtual sdpResult RemoveImplicitObject(k3dIImplicitObject* const ImplicitObject) = 0;

protected:
	virtual ~k3dIImplicitObjectCollection() {}
};

#endif // K3DIIMPLICITOBJECTCOLLECTION_H
