#ifndef K3DILOCATION_H
#define K3DILOCATION_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares k3dILocation, an abstract interface implemented by objects that can act as a geometric point in the 3D workspace
		\author Tim Shead (tshead@k-3d.com)
*/

#ifndef SDPTYPES_H
#include <sdptypes.h>
#endif // !SDPTYPES_H

#include <list>
#include <set>
#include <vector>

// Forward class declarations
class k3dIState;

/// Abstract interface implemented by objects that can act as a geometric point in the 3D workspace
class k3dILocation :
	public virtual sdpIUnknown
{
public:
	virtual k3dIState* State() = 0;
	virtual const sdpVector3& LocalXYZ() = 0;
	virtual const sdpVector3& AnimXYZ() = 0;
	virtual const sdpVector3& WorldXYZ() = 0;
	virtual const sdpVector3& CameraXYZ() = 0;
	virtual const sdpVector3& ScreenXYZ() = 0;
	virtual const sdpVector3& NormalXYZ() = 0;
	virtual const sdpVector3& TextureXYZ() = 0;
	virtual const sdpVector3& ImplicitTextureXYZ() = 0;
	virtual void SetLocalXYZ(const sdpVector3& Coords) = 0;
	virtual void SetAnimXYZ(const sdpVector3& Coords) = 0;
	virtual void SetWorldXYZ(const sdpVector3& Coords) = 0;
	virtual void SetCameraXYZ(const sdpVector3& Coords) = 0;
	virtual void SetScreenXYZ(const sdpVector3& Coords) = 0;
	virtual void SetNormalXYZ(const sdpVector3& Coords) = 0;
	virtual void SetTextureXYZ(const sdpVector3& Coords) = 0;
	virtual void SetImplicitTextureXYZ(const sdpVector3& Coords) = 0;

protected:
	virtual ~k3dILocation() {}
};

#endif // K3DILOCATION_H

