#ifndef K3DILOCATIONCOLLECTION_H
#define K3DILOCATIONCOLLECTION_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the k3dILocationCollection abstract interface for collections of k3dILocation objects
		\author Tim Shead (tshead@k-3d.com)
*/

#ifndef SDPTYPES_H
#include <sdptypes.h>
#endif // !SDPTYPES_H

// Forward class declarations
class k3dILocation;

SDP_DECLARE_HANDLE(K3DLOCATIONHANDLE);

/// Abstract interface for objects that wish to enumerate through a collection of k3dILocation objects (Visitor Design Pattern)
class k3dILocationVisitor
{
public:
	virtual void VisitLocation(k3dILocation* const Location) = 0;
	
protected:
	virtual ~k3dILocationVisitor() {}
};

/// Abstract interface for collections of k3dILocation objects
class k3dILocationCollection :
	public virtual sdpIUnknown
{
public:
	/// Enumerates through the collection (Visitor Design Pattern)
	virtual void VisitLocations(k3dILocationVisitor* const Visitor) = 0;

	// Deprecated interface for enumerating through the collection
	/** \todo get rid of deprecated enumeration interface **/
	virtual gulong LocationCount() = 0;
	virtual K3DLOCATIONHANDLE FirstLocation() = 0;
	virtual K3DLOCATIONHANDLE NextLocation(K3DLOCATIONHANDLE Location) = 0;
	virtual k3dILocation* Location(K3DLOCATIONHANDLE Location) = 0;

protected:
	virtual ~k3dILocationCollection() {}
};

#endif // K3DILOCATIONCOLLECTION_H
