#ifndef K3DIMATERIAL_H
#define K3DIMATERIAL_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares k3dIMaterial, an abstract interface implemented by objects that can act as geometric surface materials
		\author Tim Shead (tshead@k-3d.com)
*/

#ifndef SDPTYPES_H
#include <sdptypes.h>
#endif // !SDPTYPES_H

/// Abstract interface implemented by objects that can act as geometric surface materials
class k3dIMaterial :
	public virtual sdpIUnknown
{
public:
	virtual sdpResult SetAmbient(double[3]) = 0;
	virtual sdpResult SetDiffuse(double[3]) = 0;
	virtual sdpResult SetSpecular(double[3]) = 0;
	virtual sdpResult SetEmissive(double[3]) = 0;
	virtual sdpResult SetSpecularSize(double) = 0;
	virtual sdpResult SetTransparency(double) = 0;

protected:
	virtual ~k3dIMaterial() {}
};

#endif // K3DIMATERIAL_H
