#ifndef K3DIMOUSEEVENTOBSERVER_H
#define K3DIMOUSEEVENTOBSERVER_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares k3dIMouseEventObserver, an abstract interface implemented by objects that want to receive notification of mouse events
		\author Tim Shead (tshead@k-3d.com)
*/

#ifndef SDPTYPES_H
#include <sdptypes.h>
#endif // !SDPTYPES_H

#include <gdk/gdktypes.h>

// Forward class declarations
class k3dICamera;

/// Abstract interface for objects that want to receive interactive user events
class k3dIMouseEventObserver :
	public virtual sdpIUnknown
{
public:
	/// Called to return a human-readable description of how the observer plans to use the input events
	virtual SDPCSTRING Usage() = 0;

	/// Called when the user moves the mouse
	virtual sdpResult OnMouseMove(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY) = 0;
	/// Called when the user clicks the left mouse button
	virtual sdpResult OnLButtonClick(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY) = 0;
	/// Called when the user double-clicks the left mouse button
	virtual sdpResult OnLButtonDoubleClick(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY) = 0;
	/// Called when the user begins dragging the mouse with the left mouse button down
	virtual sdpResult OnLButtonStartDrag(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY) = 0;
	/// Called while the user drags the mouse with the left mouse button down
	virtual sdpResult OnLButtonDrag(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY, gdouble LastX, gdouble LastY, gdouble StartX, gdouble StartY) = 0;
	/// Called when the user finishes dragging the mouse with the left mouse button down
	virtual sdpResult OnLButtonEndDrag(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY, gdouble LastX, gdouble LastY, gdouble StartX, gdouble StartY) = 0;
	/// Called when the user clicks the middle mouse button
	virtual sdpResult OnMButtonClick(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY) = 0;
	/// Called when the user double-clicks the middle mouse button
	virtual sdpResult OnMButtonDoubleClick(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY) = 0;
	/// Called when the user begins dragging the mouse with the middle mouse button down
	virtual sdpResult OnMButtonStartDrag(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY) = 0;
	/// Called while the user drags the mouse with the middle mouse button down
	virtual sdpResult OnMButtonDrag(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY, gdouble LastX, gdouble LastY, gdouble StartX, gdouble StartY) = 0;
	/// Called when the user finishes dragging the mouse with the middle mouse button down
	virtual sdpResult OnMButtonEndDrag(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY, gdouble LastX, gdouble LastY, gdouble StartX, gdouble StartY) = 0;
	/// Called when the user clicks the right mouse button
	virtual sdpResult OnRButtonClick(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY) = 0;
	/// Called when the user double-clicks the right mouse button
	virtual sdpResult OnRButtonDoubleClick(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY) = 0;
	/// Called when the user begins dragging the mouse with the right mouse button down
	virtual sdpResult OnRButtonStartDrag(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY) = 0;
	/// Called while the user drags the mouse with the right mouse button down
	virtual sdpResult OnRButtonDrag(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY, gdouble LastX, gdouble LastY, gdouble StartX, gdouble StartY) = 0;
	/// Called when the user finishes dragging the mouse with the right mouse button down
	virtual sdpResult OnRButtonEndDrag(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY, gdouble LastX, gdouble LastY, gdouble StartX, gdouble StartY) = 0;
	/// Called when the user begins dragging the mouse with the left and right mouse buttons down
	virtual sdpResult OnLRButtonStartDrag(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY) = 0;
	/// Called while the user drags the mouse with the left and right mouse buttons down
	virtual sdpResult OnLRButtonDrag(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY, gdouble LastX, gdouble LastY, gdouble StartX, gdouble StartY) = 0;
	/// Called when the user finishes dragging the mouse with the left and right mouse buttons down
	virtual sdpResult OnLRButtonEndDrag(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY, gdouble LastX, gdouble LastY, gdouble StartX, gdouble StartY) = 0;
	/// Called when the user moves the mouse wheel up
	virtual sdpResult OnMouseWheelUp(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY) = 0;
	/// Called when the user moves the mouse wheel down
	virtual sdpResult OnMouseWheelDown(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY) = 0;

protected:
	virtual ~k3dIMouseEventObserver() {}
};

#endif // K3DIMOUSEEVENTOBSERVER_H
