#ifndef K3DINURBSPOINTCOLLECTION_H
#define K3DINURBSPOINTCOLLECTION_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the k3dIPointCollection abstract interface for collections of k3dIPoint objects
		\author Romain Behar with NURBS students (romainbehar@yahoo.com)
*/

// Forward class declarations
class k3dINurbsControlPoint;

SDP_DECLARE_HANDLE(K3DNURBSCONTROLPOINTHANDLE);

/// Abstract interface for objects that want to enumerate through a collection of k3dINurbsControlPoint objects
class k3dINurbsControlPointVisitor
{
public:
	virtual void VisitPoint(k3dINurbsControlPoint* const Point) = 0;

protected:
	virtual ~k3dINurbsControlPointVisitor() {}
};

/// Abstract interface for collections of k3dINurbsControlPoint objects
class k3dINurbsControlPointCollection :
	public virtual sdpIUnknown
{
public:
	/// Enumerates through the collection (Visitor Design Pattern)
	virtual void VisitPoints(k3dINurbsControlPointVisitor* const Visitor) = 0;

	/// Adds a k3dINurbsControlPoint object to the collection
	virtual sdpResult AddPoint(k3dINurbsControlPoint* const Point) = 0;
	/// Removes a k3dINurbsControlPoint object from the collection
	virtual sdpResult RemovePoint(k3dINurbsControlPoint* const Point) = 0;

	// Deprected interface for enumerating through the collection
	virtual gulong PointCount() = 0;
	virtual K3DNURBSCONTROLPOINTHANDLE FirstPoint() = 0;
	virtual K3DNURBSCONTROLPOINTHANDLE NextPoint(K3DNURBSCONTROLPOINTHANDLE Point) = 0;
	virtual k3dINurbsControlPoint* Point(K3DNURBSCONTROLPOINTHANDLE Point) = 0;

protected:
	virtual ~k3dINurbsControlPointCollection() {}
};

#endif // K3DINURBSPOINTCOLLECTION_H
