#ifndef K3DINURBSPATH_H
#define K3DINURBSPATH_H

// K-3D
// Copyright  1995 - 2002, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the k3dINurbsPath interface, which encapsulates a geometric nurbs path (with no restriction on number of nurbs points, convexity, or simplicity)
		\author Romain Behar with NURBS students (romainbehar@yahoo.com)
*/

// Forward class declarations
class k3dINurbsMesh;
class k3dIState;
class k3dINurbsPathPointCollection;
class k3dILocationCollection;
class k3dIMaterialCollection;

/// Abstract interface for a geometric nurbs path (with no restriction on number of nurbs points, convexity, or simplicity)
/** \todo Ultimately, k3dINurbsPath should become a more generic geometric object, with k3dIPathLine, k3dINurbsPath, k3dICurve, and k3dIPatch deriving from it */
class k3dINurbsPath :
	public virtual sdpIUnknown
{
public:
	virtual k3dINurbsMesh* Mesh() = 0;
	virtual k3dINurbsPathPointCollection* Points() = 0;
	virtual k3dILocationCollection* Locations() = 0;
	//virtual k3dIMaterialCollection* Materials() = 0;
	virtual k3dIState* State() = 0;
	//virtual sdpResult PathNormal(double Normal[3]) = 0;
	//virtual sdpResult SetPathNormal(double Normal[3]) = 0;
	//virtual sdpResult FlipOrientation() = 0;
	//virtual sdpResult AdvanceFirstPoint() = 0;

	//virtual const gdouble* Nodal() = 0;

	/// Defines a unique identifier that can be serialized
	typedef gulong ID;
	/// Returns a unique identifier for this object that can be serialized
	virtual const ID PathID() = 0;

protected:
	virtual ~k3dINurbsPath() {}
};

#endif // K3DINURBSPATH_H
