#ifndef K3DINURBSPATHCOLLECTION_H
#define K3DINURBSPATHCOLLECTION_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the k3dINurbsPathCollection abstract interface for collections of k3dINurbsPath objects
		\author Romain Behar with NURBS students (romainbehar@yahoo.com)
*/

// Forward class declarations
class k3dINurbsPath;

SDP_DECLARE_HANDLE(K3DNURBSPATHHANDLE);

/// Abstract interface for objects that want to enumerate over a collection of k3dINurbsPath objects
class k3dIPathVisitor
{
public:
	virtual void VisitPath(k3dINurbsPath* const Path) = 0;

protected:
	virtual ~k3dIPathVisitor() {}
};

/// Abstract interface for collections of k3dINurbsPath objects
class k3dINurbsPathCollection :
	public virtual sdpIUnknown
{
public:
	/// Enumerates through the collection (Visitor Design Pattern)
	virtual void VisitPaths(k3dIPathVisitor* const Visitor) = 0;

	/// Adds a k3dINurbsPath object to the collection
	virtual sdpResult AddPath(k3dINurbsPath* const Path) = 0;
	/// Removes a k3dINurbsPath object from the collection
	virtual sdpResult RemovePath(k3dINurbsPath* const Path) = 0;

	// Deprecated interface for enumerating the contents of the nurbs collection
	virtual gulong PathCount() = 0;
	virtual K3DNURBSPATHHANDLE FirstPath() = 0;
	virtual K3DNURBSPATHHANDLE NextPath(K3DNURBSPATHHANDLE Path) = 0;
	virtual k3dINurbsPath* Path(K3DNURBSPATHHANDLE Path) = 0;

protected:
	virtual ~k3dINurbsPathCollection() {}
};

#endif // K3DINURBSPATHCOLLECTION_H
