#ifndef K3DINURBSPATHPOINTCOLLECTION
#define K3DINURBSPATHPOINTCOLLECTION

// K-3D
// Copyright  1995 - 2002, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the k3dIPathPointCollection abstract interface for collections of k3dINurbsPathPoint objects
		\author Romain Behar with NURBS students (romainbehar@yahoo.com)
*/

// Forward class declarations
class k3dINurbsPathPoint;
class k3dINurbsControlPoint;

SDP_DECLARE_HANDLE(K3DNURBSPATHPOINTHANDLE);

/// Abstract interface for objects that want to enumerate over a collection of k3dINurbsPathPoint objects
class k3dIPathPointVisitor
{
public:
	virtual void VisitPathPoint(k3dINurbsPathPoint* const PathPoint) = 0;

protected:
	virtual ~k3dIPathPointVisitor() {}
};

/// Abstract interface for collections of k3dIPathPoint objects
class k3dINurbsPathPointCollection :
	public virtual sdpIUnknown
{
public:
	/// Enumerates over the collection (Visitor Design Pattern)
	virtual void VisitPathPoints(k3dIPathPointVisitor* const Visitor) = 0;

	/// Creates a k3dINurbsPathPoint attached to the given k3dINurbsControlPoint
	virtual k3dINurbsPathPoint* AddPoint(k3dINurbsControlPoint* const Point) = 0;
	/// Removes any k3dINurbsPathPoint associated with the given k3dINurbsControlPoint
	virtual sdpResult RemovePoint(k3dINurbsControlPoint* const Point) = 0;
	/// Removes a k3dINurbsPathPoint
	virtual sdpResult RemovePathPoint(k3dINurbsPathPoint* const PathPoint) = 0;

	// Deprecated interface for enumerating over the collection
	/** \todo Remove deprected enumeration interface **/
	virtual gulong PointCount() = 0;
	virtual K3DNURBSPATHPOINTHANDLE FirstPathPoint() = 0;
	virtual K3DNURBSPATHPOINTHANDLE NextPathPoint(K3DNURBSPATHPOINTHANDLE Point) = 0;
	virtual k3dINurbsPathPoint* PathPoint(K3DNURBSPATHPOINTHANDLE Point) = 0;

protected:
	virtual ~k3dINurbsPathPointCollection() {}
};

#endif // K3DINURBSPATHPOINTCOLLECTION
