#ifndef K3DIOBJECT_H
#define K3DIOBJECT_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares k3dIObject, the abstract interface which MUST be implemented by ALL K-3D objects
		\author Tim Shead (tshead@k-3d.com)
*/

#ifndef SDPTYPES_H
#include <sdptypes.h>
#endif // !SDPTYPES_H

// Forward class references
class k3dIDocument;
class k3dIObjectInfo;
class k3dIState;

//////////////////////////////////////////////////////////////////////////////////////
// k3dIObject

/// Abstract interface which MUST be implemented by ALL K-3D objects
class k3dIObject :
	public virtual sdpIUnknown
{
public:
	/// Called after the object has been fully initialized and integrated into the document (note - for objects that support k3dIPersist, this method will be called *after* InitNew() or LoadComplete().
	virtual sdpResult InitComplete() = 0;

	/// Defines a unique identifier that can be serialized
	typedef gulong ID;
	/// Returns a unique identifier for this object that can be serialized
	virtual const ID ObjectID() = 0;
	
	/// Sets the object ID; note  ... this will only be called when importing / pasting objects
	virtual void SetObjectID(const ID ObjectID) = 0;
	/// Returns the object name
	virtual SDPCSTRING Name() = 0;
	/// Sets the object name (could fail or be overridden)
	virtual sdpResult SetName(SDPCSTRING Name) = 0;

	/// Returns a k3dIState interface describing the current state of the object
	virtual k3dIState* State() = 0;
	/// Returns a k3dIDocument interface to the document that owns this object
	virtual k3dIDocument* Document() = 0;
	/// Returns a k3dIObjectInfo interface describing the owned-object's sdpCom class
	virtual k3dIObjectInfo* ObjectInfo() = 0;

	/// Called to request that the object open its user interface
	virtual sdpResult EditObject() = 0;

	/// Called to notify the object that it has been deleted
	virtual sdpResult ObjectDeleted() = 0;
	/// Called to notify the object that it has been un-deleted
	virtual sdpResult ObjectUnDeleted() = 0;

	static SDPCSTRING Event_IDChanged;
	static SDPCSTRING Event_Renamed;
	static SDPCSTRING Event_Deleted;
	static SDPCSTRING Event_UnDeleted;
	static SDPCSTRING Event_Destroyed;
	static SDPCSTRING Event_UpdateUI;

protected:
	virtual ~k3dIObject() {}
};

#endif // K3DIOBJECT_H

