#ifndef K3DIOBJECTCOLLECTION_H
#define K3DIOBJECTCOLLECTION_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares k3dIObjectCollection, a container of k3dIObject objects
		\author Tim Shead (tshead@k-3d.com)
*/

#ifndef SDPTYPES_H
#include <sdptypes.h>
#endif // !SDPTYPES_H

// Forward class references
class k3dIObject;

namespace sdpxml
{
class Document;
class Element;
} // namespace sdpxml

/// Abstract interface for enumerating over a collection of k3dIObject objects
class k3dIObjectVisitor
{
public:
	/// Called once for each k3dIObject within the collection
	virtual void VisitObject(k3dIObject* const Object) = 0;

protected:
	virtual ~k3dIObjectVisitor() {}
};

/** \todo Deprecated */
SDP_DECLARE_HANDLE(K3DOBJECTHANDLE);

/// Abstract interface for a collection of K-3D objects
class k3dIObjectCollection :
	public virtual sdpIUnknown
{
public:
	/// Creates a new K-3D object
	virtual k3dIObject* CreateObject(const sdp::ClassID& ClassID) = 0;
	/// "Thaws" an object in XML form
	virtual k3dIObject* CreateObject(sdpxml::Document& Document, sdpxml::Element& Element) = 0;
	/// Deletes (can be undone) a K-3D object
	virtual bool DeleteObject(k3dIObject* const Object) = 0;

	/// Enumerates through all of the K-3D objects in the collection (Visitor Design Pattern)
	virtual void VisitObjects(k3dIObjectVisitor* const Visitor) = 0;

	/** \todo Deprecated */
	virtual K3DOBJECTHANDLE FirstObject() = 0;
	/** \todo Deprecated */
	virtual K3DOBJECTHANDLE NextObject(K3DOBJECTHANDLE Object) = 0;
	/** \todo Deprecated */
	virtual k3dIObject* Object(K3DOBJECTHANDLE Object) = 0;

	/// Event that will be fired whenever the collection changes
	static SDPCSTRING Event_Changed;

protected:
	virtual ~k3dIObjectCollection() {}
};

#endif // K3DIOBJECTCOLLECTION_H
