#ifndef K3DIOBJECTINFO_H
#define K3DIOBJECTINFO_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares k3dIObjectInfo, an abstract interface which encapsulates information about a K-3D object type
		\author Tim Shead (tshead@k-3d.com)
*/

#ifndef SDPTYPES_H
#include <sdptypes.h>
#endif // !SDPTYPES_H

// Forward class declarations ...
class k3dIObjectModule;

/// Abstract visitor interface for enumerating through object bookmark collections
class k3dIBookmarkVisitor
{
public:
	virtual void VisitBookmark(SDPCSTRING Bookmark) = 0;

protected:
	virtual ~k3dIBookmarkVisitor() {}
};

/// Abstract interfac which encapsulates information about a K-3D object type
class k3dIObjectInfo :
	public virtual sdpIUnknown
{
public:

	virtual k3dIObjectModule* Module() = 0;
	virtual const sdp::ClassID& ClassID() = 0;
	virtual SDPCSTRING Name() = 0;
	virtual SDPCSTRING ShortDescription() = 0;
	virtual sdpResult QueryObjectInterface(const sdp::ITypeInfo& InterfaceType) = 0;
	virtual sdpResult QueryIncomplete() = 0;
	virtual sdpResult QueryExperimental() = 0;
	virtual void VisitBookmarks(k3dIBookmarkVisitor* const BookmarkVisitor) = 0;

protected:
	virtual ~k3dIObjectInfo() {}
};

#endif // K3DIOBJECTINFO_H

