#ifndef K3DIOBJECTINITIALIZER_H
#define K3DIOBJECTINITIALIZER_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares k3dIObjectInitializer, an abstract interface which is passed to K-3D objects at creation time to supply initialization data
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3diobject.h>

// Forward class declarations
class k3dIDocument;
class k3dIObjectInfo;

/// Abstract interface which is passed to K-3D objects at creation time to supply initialization data 
class k3dIObjectInitializer :
	public virtual sdpIUnknown
{
public:
	/// Returns the new object ID
	virtual k3dIObject::ID ObjectID() = 0;
	/// Returns the initializing document
	virtual k3dIDocument* Document() = 0;
	/// Returns the initializing object info
	virtual k3dIObjectInfo* ObjectInfo() = 0;

protected:
	virtual ~k3dIObjectInitializer() {}
};

#endif // K3DIOBJECTINITIALIZER_H
