#ifndef K3DIPATH_H
#define K3DIPATH_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the k3dIPath interface, which encapsulates a geometric path (with no restriction on number of points, convexity, or simplicity)
		\author Tim Shead (tshead@k-3d.com)
*/

// Forward class declarations
class k3dIMesh;
class k3dIState;
class k3dIPathPointCollection;
class k3dILocationCollection;
class k3dIMaterialCollection;

/// Abstract interface for a geometric path (with no restriction on number of points, convexity, or simplicity)
/** \todo Ultimately, k3dIPath should become a more generic geometric object, with k3dIPathLine, k3dIPath, k3dICurve, and k3dIPatch deriving from it */
class k3dIPath :
	public virtual sdpIUnknown
{
public:
	virtual k3dIMesh* Mesh() = 0;
	virtual k3dIPathPointCollection* Points() = 0;
	virtual k3dILocationCollection* Locations() = 0;
	virtual k3dIMaterialCollection* Materials() = 0;
	virtual k3dIState* State() = 0;
	virtual sdpResult PathNormal(double Normal[3]) = 0;
	virtual sdpResult SetPathNormal(double Normal[3]) = 0;
	virtual sdpResult FlipOrientation() = 0;
	virtual sdpResult AdvanceFirstPoint() = 0;
	
	/// Defines a unique identifier that can be serialized
	typedef gulong ID;
	/// Returns a unique identifier for this object that can be serialized
	virtual const ID PathID() = 0;

protected:
	virtual ~k3dIPath() {}
};

#endif // K3DIPATH_H
