#ifndef K3DIPATHCOLLECTION_H
#define K3DIPATHCOLLECTION_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the k3dIPathCollection abstract interface for collections of k3dIPath objects
		\author Tim Shead (tshead@k-3d.com)
*/

// Forward class declarations
class k3dIPath;

SDP_DECLARE_HANDLE(K3DPATHHANDLE);

/// Abstract interface for objects that want to enumerate over a collection of k3dIPath objects
class k3dIPathVisitor
{
public:
	virtual void VisitPath(k3dIPath* const Path) = 0;

protected:
	virtual ~k3dIPathVisitor() {}
};

/// Abstract interface for collections of k3dIPath objects
class k3dIPathCollection :
	public virtual sdpIUnknown
{
public:
	/// Enumerates through the collection (Visitor Design Pattern)
	virtual void VisitPaths(k3dIPathVisitor* const Visitor) = 0;

	/// Adds a k3dIPath object to the collection
	virtual sdpResult AddPath(k3dIPath* const Path) = 0;
	/// Removes a k3dIPath object from the collection
	virtual sdpResult RemovePath(k3dIPath* const Path) = 0;

	// Deprecated interface for enumerating the contents of the collection
	virtual gulong PathCount() = 0;
	virtual K3DPATHHANDLE FirstPath() = 0;
	virtual K3DPATHHANDLE NextPath(K3DPATHHANDLE Path) = 0;
	virtual k3dIPath* Path(K3DPATHHANDLE Path) = 0;

protected:
	virtual ~k3dIPathCollection() {}
};

#endif // K3DIPATHCOLLECTION_H
