#ifndef K3DIPATHPOINTCOLLECTION
#define K3DIPATHPOINTCOLLECTION

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the k3dIPathPointCollection abstract interface for collections of k3dIPathPoint objects
		\author Tim Shead (tshead@k-3d.com)
*/

// Forward class declarations
class k3dIPathPoint;
class k3dIPoint;

SDP_DECLARE_HANDLE(K3DPATHPOINTHANDLE);

/// Abstract interface for objects that want to enumerate over a collection of k3dIPathPoint objects
class k3dIPathPointVisitor
{
public:
	virtual void VisitPathPoint(k3dIPathPoint* const PathPoint) = 0;
	
protected:
	virtual ~k3dIPathPointVisitor() {}
};

/// Abstract interface for collections of k3dIPathPoint objects
class k3dIPathPointCollection :
	public virtual sdpIUnknown
{
public:
	/// Enumerates over the collection (Visitor Design Pattern)
	virtual void VisitPathPoints(k3dIPathPointVisitor* const Visitor) = 0;

	/// Creates a k3dIPathPoint attached to the given k3dIPoint
	virtual k3dIPathPoint* AddPoint(k3dIPoint* const Point) = 0;
	/// Removes any k3dIPathPoint associated with the given k3dIPoint
	virtual sdpResult RemovePoint(k3dIPoint* const Point) = 0;
	/// Removes a k3dIPathPoint
	virtual sdpResult RemovePathPoint(k3dIPathPoint* const PathPoint) = 0;

	// Deprecated interface for enumerating over the collection
	/** \todo Remove deprected enumeration interface **/
	virtual gulong PointCount() = 0;
	virtual K3DPATHPOINTHANDLE FirstPathPoint() = 0;
	virtual K3DPATHPOINTHANDLE NextPathPoint(K3DPATHPOINTHANDLE Point) = 0;
	virtual k3dIPathPoint* PathPoint(K3DPATHPOINTHANDLE Point) = 0;

protected:
	virtual ~k3dIPathPointCollection() {}
};

#endif // K3DIPATHPOINTCOLLECTION
