#ifndef K3DIPERSIST_H
#define K3DIPERSIST_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares k3dIPersist, an abstract interface implemented by all objects that can be serialized to/from a K-3D document
		\author Tim Shead (tshead@k-3d.com)
*/

#ifndef SDPTYPES_H
#include <sdptypes.h>
#endif // !SDPTYPES_H

// Forward class declarations
class k3dIDependencies;

namespace sdpxml
{
class Document;
class Element;
} // namespace sdpxml

/**
  
The methods in k3dIPersist will be called in a specific order at various stages of an object's lifetime:

First time the object is ever created:

1. InitNew()

When the object is saved as part of a document:

1. Save()
2. SaveComplete()

When the object is reloaded from a document:

1. Load()
2. LoadComplete()

Note that k3dIObject::InitComplete() will always be called *after* InitNew() or LoadComplete().

*/

/// Abstract interface implemented by objects that can be serialized to/from a K-3D document
class k3dIPersist :
	public virtual sdpIUnknown
{
public:
	/// Called once when a new object is created for the first time 
	virtual sdpResult InitNew() = 0;
	/// Called once during document loading
	virtual sdpResult Load(sdpxml::Document& Document, sdpxml::Element& Element) = 0;
	/// Called once after document loading is complete
	virtual sdpResult LoadComplete() = 0;
	/// Called once during document save
	virtual sdpResult Save(sdpxml::Element& Element, k3dIDependencies* Dependencies) = 0;
	/// Called once after document save is complete
	virtual sdpResult SaveComplete() = 0;

protected:
	virtual ~k3dIPersist() {}
};

#endif // K3DIPERSIST_H

