#ifndef K3DIRENDERFARM_H
#define K3DIRENDERFARM_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares k3dIRenderFarm, an abstract interface which encapsulates a (possibly virtual) "render farm" capable of rendering "jobs"
		\author Tim Shead (tshead@k-3d.com)
*/

#ifndef SDPTYPES_H
#include <sdptypes.h>
#endif // !SDPTYPES_H

// Forward class declarations
class k3dIRenderJob;

/// Abstract interface that encapsulates a virtual "render farm" capable of rendering "jobs"
class k3dIRenderFarm :
	public virtual sdpIUnknown
{
public:
	/// Defines some standard render priorities
	typedef enum
	{
		IMMEDIATE_PRIORITY = 0,
		HIGHEST_PRIORITY = 1,
		HIGH_PRIORITY = 64,
		MODERATE_PRIORITY = 128,
		LOW_PRIORITY = 190,
		LOWEST_PRIORITY = 255
	};

	/// Creates a new job, in a "waiting" state
	virtual k3dIRenderJob* CreateJob(SDPCSTRING JobName, guchar Priority) = 0;
	/// Tells the render farm to begin executing the given job
	virtual sdpResult StartJob(k3dIRenderJob* Job) = 0;

protected:
	virtual ~k3dIRenderFarm() {}
};

#endif // K3DIRENDERFARM_H
