#ifndef K3DIRENDERFRAME_H
#define K3DIRENDERFRAME_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares k3dIRenderFrame abstract interface, which encapsulates a single animation frame to be rendered as part of a k3dIRenderJob
		\author Tim Shead (tshead@k-3d.com)
*/

#ifndef SDPTYPES_H
#include <sdptypes.h>
#endif // !SDPTYPES_H

// Forward class declarations ...
class k3dIRenderJob;

SDP_DECLARE_HANDLE(K3DRENDERFRAMEHANDLE);

/// Abstract interface encapsulating a single animation frame to be rendered as part of a k3dIRenderJob
class k3dIRenderFrame :
	public virtual sdpIUnknown
{
public:
	/// Returns the job that owns this frame
	virtual k3dIRenderJob* Job() = 0;
	/// Returns a unique filepath that can be used as an input file for this frame
	virtual sdpResult AddInputFile(SDPCSTRING Name, SDPSTRING Buffer, gulong* BufferLength) = 0;
	/// Returns a unique filepath that can be used as an output file for this frame
	virtual sdpResult AddOutputFile(SDPCSTRING Name, SDPSTRING Buffer, gulong* BufferLength) = 0;
	/// Sets-up a render operation for transforming input files to outputfiles (the render engine will be matched against choices in the user options file)
	virtual sdpResult AddRenderOperation(SDPCSTRING EngineType, SDPCSTRING Engine, SDPCSTRING SourceFilePath, bool VisibleRender) = 0;
	/// Sets-up a copy operation once rendering is complete
	virtual sdpResult AddCopyOperation(SDPCSTRING SourceFilePath, SDPCSTRING TargetFilePath) = 0;
	/// Sets-up a file view operation once copying is complete
	virtual sdpResult AddViewOperation(SDPCSTRING FilePath) = 0;
	
	/// Defines the name of the control file for this frame
	static SDPCSTRING const ControlFile;
	/// Defines the name of the render-engine log file
	static SDPCSTRING const EngineLogFile;
	/// Defines the name of the render-engine error log file
	static SDPCSTRING const EngineErrorFile;
	/// Defines the name of the render-server log file
	static SDPCSTRING const SystemLogFile;
	/// Defines the name of the render-server error log file
	static SDPCSTRING const SystemErrorFile;
	/// Defines the name of the status file when a frame is ready for rendering
	static SDPCSTRING const StatusReadyFile;
	/// Defines the name of the status file while a frame is being rendered
	static SDPCSTRING const StatusRunningFile;
	/// Defines the name of the status file when a frame was aborted due to errors
	static SDPCSTRING const StatusErrorFile;
	/// Defines the name of the status file when a frame was rendered successfully
	static SDPCSTRING const StatusCompleteFile;
	/// Defines the name of the file the rendered image will be stored in
	static SDPCSTRING const OutputImageFile;

protected:
	virtual ~k3dIRenderFrame() {}
};

#endif // K3DIRENDERFRAME_H
