#ifndef K3DIRIENGINE_H
#define K3DIRIENGINE_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/////////////////////////////////////////////////////////////////////////////
//
// kiriengine.h - an sdpCom language binding to the RenderMan (R) interface
// 
// The RenderMan (R) Interface Procedures and RIB Protocol are:
// Copyright 1988, 1989, Pixar.  All rights reserved.
// RenderMan (R) is a registered trademark of Pixar.
//
// Note: this interface conforms to the RenderMan (R) RIB Protocol, with
// the exception of varargs functions, which are specifically prohibited
// by sdpCom.  Use the Ri...V() versions of the functions, instead.
//
/////////////////////////////////////////////////////////////////////////////

/** \file
		\brief Declares k3dIRiEngine, an abstract version of the Pixar RenderMan interface
		\author Tim Shead (tshead@k-3d.com)
*/

#ifndef SDPTYPES_H
#include <sdptypes.h>
#endif // !SDPTYPES_H

// Forward class declarations ...
class k3dICamera;

/// Abstract version of the Pixar RenderMan interface
class k3dIRiEngine :
	public virtual sdpIUnknown
{
public:
	virtual k3dICamera* Camera() = 0;
	virtual gulong RiAreaLightSourceV(SDPCSTRING Name, const gulong ParameterCount, SDPCSTRING Tokens[], gpointer Values[], gulong ValueCounts[]) = 0;
	virtual sdpResult RiAtmosphereV(SDPCSTRING Name, const gulong ParameterCount, SDPCSTRING Tokens[], gpointer Values[], gulong ValueCounts[]) = 0;
	virtual sdpResult RiAttributeBegin() = 0;
	virtual sdpResult RiAttributeEnd() = 0;
	virtual sdpResult RiAttributeV(SDPCSTRING Name, const gulong ParameterCount, SDPCSTRING Tokens[], gpointer Values[], gulong ValueCounts[]) = 0;
	virtual sdpResult RiBasis(gdouble UBasis[16], const gulong UStep, gdouble VBasis[16], const gulong VStep) = 0;
	virtual sdpResult RiBound(gdouble Bound[6]) = 0;
	virtual sdpResult RiClipping(const gdouble Hither, const gdouble Yon) = 0;
	virtual sdpResult RiColor(gdouble Color[3]) = 0;
	virtual sdpResult RiColorSamples(const gulong ParameterCount, gdouble nRGB[], gdouble RGBn[]) = 0;
	virtual sdpResult RiComment(SDPCSTRING Comment) = 0;
	virtual sdpResult RiConcatTransform(gdouble Transform[16]) = 0;
	virtual sdpResult RiConeV(const gdouble Height, const gdouble Radius, const gdouble ThetaMax, const gulong ParameterCount, SDPCSTRING Tokens[], gpointer Values[], gulong ValueCounts[]) = 0;
	virtual sdpResult RiCoordSysTransform(SDPCSTRING Space) = 0;
	virtual sdpResult RiCoordinateSystem(SDPCSTRING Space) = 0;
	virtual sdpResult RiCropWindow(const gdouble XMin, const gdouble XMax, const gdouble YMin, const gdouble YMax) = 0;
	virtual sdpResult RiCurvesV(SDPCSTRING Degree, const gulong ParameterCountCurves, const gulong ParameterCountVerts[], SDPCSTRING Wrap, const gulong ParameterCount, SDPCSTRING Tokens[], gpointer Values[], gulong ValueCounts[]) = 0;
	virtual sdpResult RiCylinderV(const gdouble Radius, const gdouble ZMin, const gdouble ZMax, const gdouble ThetaMax, const gulong ParameterCount, SDPCSTRING Tokens[], gpointer Values[], gulong ValueCounts[]) = 0;
	virtual sdpResult RiDeclare(SDPCSTRING Name, SDPCSTRING Type) = 0;
	virtual sdpResult RiDeformationV(SDPCSTRING Name, const gulong ParameterCount, SDPCSTRING Tokens[], gpointer Values[], gulong ValueCounts[]) = 0;
	virtual sdpResult RiDepthOfField(const gdouble FStop, const gdouble FocalLength, const gdouble FocalDistance) = 0;
	virtual sdpResult RiDetail(gdouble Bound[6]) = 0;
	virtual sdpResult RiDetailRange(const gdouble MinVis, const gdouble LowTran, const gdouble UpTran, const gdouble MaxVis) = 0;
	virtual sdpResult RiDiskV(gdouble Height, gdouble Radius, gdouble ThetaMax, 	const gulong ParameterCount, SDPCSTRING Tokens[], gpointer Values[], gulong ValueCounts[]) = 0;
	virtual sdpResult RiDisplacementV(SDPCSTRING Name, const gulong ParameterCount, SDPCSTRING Tokens[], gpointer Values[], gulong ValueCounts[]) = 0;
	virtual sdpResult RiDisplayV(SDPCSTRING Name, SDPCSTRING Type, SDPCSTRING Mode, const gulong ParameterCount, SDPCSTRING Tokens[], gpointer Values[], gulong ValueCounts[]) = 0;
	virtual sdpResult RiErrorAbort(const gint Code, const gint Severity, SDPCSTRING Message) = 0;
	virtual sdpResult RiErrorIgnore(const gint Code, const gint Severity, SDPCSTRING Message) = 0;
	virtual sdpResult RiErrorPrint(const gint Code, const gint Severity, SDPCSTRING Message) = 0;
	virtual sdpResult RiExposure(const gdouble Gain, const gdouble Gamma) = 0;
	virtual sdpResult RiExteriorV(SDPCSTRING Name, const gulong ParameterCount, SDPCSTRING Tokens[], gpointer Values[], gulong ValueCounts[]) = 0;
	virtual sdpResult RiFormat(const gulong XResolution, const gulong YResolution, const gdouble AspectRatio) = 0;
	virtual sdpResult RiFrameAspectRatio(const gdouble AspectRatio) = 0;
	virtual sdpResult RiFrameBegin(const gulong FrameNumber) = 0;
	virtual sdpResult RiFrameEnd() = 0;
	virtual sdpResult RiGeneralPathV(const gulong LoopCount, const gulong VertexCounts[], const gulong ParameterCount, SDPCSTRING Tokens[], gpointer Values[], gulong ValueCounts[]) = 0;
	virtual sdpResult RiGeometricApproximation(SDPCSTRING Type, const gdouble Value) = 0;
	virtual sdpResult RiGeometricRepresentation(SDPCSTRING Type) = 0;
	virtual sdpResult RiGeometryV(SDPCSTRING Type, const gulong ParameterCount, SDPCSTRING Tokens[], gpointer Values[], gulong ValueCounts[]) = 0;
	virtual sdpResult RiHiderV(SDPCSTRING Type, const gulong ParameterCount, SDPCSTRING Tokens[], gpointer Values[], gulong ValueCounts[]) = 0;
	virtual sdpResult RiHyperboloidV(gdouble Point1[3], gdouble Point2[3], const gdouble ThetaMax, const gulong ParameterCount, SDPCSTRING Tokens[], gpointer Values[], gulong ValueCounts[]) = 0;
	virtual sdpResult RiIdentity() = 0;
	virtual sdpResult RiIlluminate(const gulong LightHandle, const bool OnOff) = 0;
	virtual sdpResult RiImagerV(SDPCSTRING Name, const gulong ParameterCount, SDPCSTRING Tokens[], gpointer Values[], gulong ValueCounts[]) = 0;
	virtual sdpResult RiInteriorV(SDPCSTRING Name, const gulong ParameterCount, SDPCSTRING Tokens[], gpointer Values[], gulong ValueCounts[]) = 0;
	virtual gulong RiLightSourceV(SDPCSTRING Name, const gulong ParameterCount, SDPCSTRING Tokens[], gpointer Values[], gulong ValueCounts[]) = 0;
	virtual sdpResult RiMatte(const bool OnOff) = 0;
	virtual sdpResult RiMotionBeginV(const gulong ParameterCount, gdouble Times[]) = 0;
	virtual sdpResult RiMotionEnd() = 0;
	virtual sdpResult RiNewline() = 0;
	virtual sdpResult RiNuPatchV(const gulong UCount, const gulong UOrder, gdouble UKnot[], const gdouble UMin, const gdouble UMax, const gulong VCount, const gulong VOrder, gdouble VKnot[], const gdouble VMin, const gdouble VMax, const gulong ParameterCount, SDPCSTRING Tokens[], gpointer Values[], gulong ValueCounts[]) = 0;
	virtual gulong RiObjectBegin() = 0;
	virtual sdpResult RiObjectEnd() = 0;
	virtual sdpResult RiObjectInstance(const gulong Object) = 0;
	virtual sdpResult RiOpacity(gdouble Color[3]) = 0;
	virtual sdpResult RiOptionV(SDPCSTRING Name, const gulong ParameterCount, SDPCSTRING Tokens[], gpointer Values[], gulong ValueCounts[]) = 0;
	virtual sdpResult RiOrientation(SDPCSTRING Orientation) = 0;
	virtual sdpResult RiParaboloidV(const gdouble RMax, const gdouble ZMin, const gdouble ZMax, const gdouble ThetaMax, const gulong ParameterCount, SDPCSTRING Tokens[], gpointer Values[], gulong ValueCounts[]) = 0;
	virtual sdpResult RiPatchMeshV(SDPCSTRING Type, const gulong UCount, SDPCSTRING UWrap, const gulong VCount, SDPCSTRING VWrap, const gulong ParameterCount, SDPCSTRING Tokens[], gpointer Values[], gulong ValueCounts[]) = 0;
	virtual sdpResult RiPatchV(SDPCSTRING Type, const gulong ParameterCount, SDPCSTRING Tokens[], gpointer Values[], gulong ValueCounts[]) = 0;
	virtual sdpResult RiPerspective(const gdouble FieldOfView) = 0;
	virtual sdpResult RiPixelFilter(SDPCSTRING FilterName, const gdouble XWidth, const gdouble YWidth) = 0;
	virtual sdpResult RiPixelSamples(const gdouble XSamples, const gdouble YSamples) = 0;
	virtual sdpResult RiPixelVariance(const gdouble Variation) = 0;
	virtual sdpResult RiPointsGeneralPathsV(const gulong PathCount, gulong LoopCounts[], gulong VertexCounts[], gulong Vertices[], const gulong ParameterCount, SDPCSTRING Tokens[], gpointer Values[], gulong ValueCounts[]) = 0;
	virtual sdpResult RiPointsPathsV(const gulong PathCount, gulong VertexCounts[], gulong Vertices[], const gulong ParameterCount, SDPCSTRING Tokens[], gpointer Values[], gulong ValueCounts[]) = 0;
	virtual sdpResult RiPathV(const gulong VertexCount, const gulong ParameterCount, SDPCSTRING Tokens[], gpointer Values[], gulong ValueCounts[]) = 0;
	virtual sdpResult RiProjectionV(SDPCSTRING Name, const gulong ParameterCount, SDPCSTRING Tokens[], gpointer Values[], gulong ValueCounts[]) = 0;
	virtual sdpResult RiQuantize(SDPCSTRING Type, const gint One, const gint QMin, const gint QMax, const gdouble Amplitude) = 0;
	virtual sdpResult RiReadArchive(SDPCSTRING Archive) = 0;
	virtual sdpResult RiRelativeDetail(const gdouble RelativeDetail) = 0;
	virtual sdpResult RiReverseOrientation() = 0;
	virtual sdpResult RiRotate(const gdouble angle, const gdouble DX, const gdouble DY, const gdouble DZ) = 0;
	virtual sdpResult RiScale(const gdouble DX, const gdouble DY, const gdouble DZ) = 0;
	virtual sdpResult RiScreenWindow(const gdouble Left, const gdouble Right, const gdouble Bottom, const gdouble Top) = 0;
	virtual sdpResult RiShadingInterpolation(SDPCSTRING Type) = 0;
	virtual sdpResult RiShadingRate(const gdouble Size) = 0;
	virtual sdpResult RiShutter(const gdouble SMin, const gdouble SMax) = 0;
	virtual sdpResult RiSides(const gulong Sides) = 0;
	virtual sdpResult RiSkew(const gdouble Angle, const gdouble DX1, const gdouble DY1, const gdouble DZ1, const gdouble DX2, const gdouble DY2, const gdouble DZ2) = 0;
	virtual sdpResult RiSolidBegin(SDPCSTRING Type) = 0;
	virtual sdpResult RiSolidEnd() = 0;
	virtual sdpResult RiSphereV(const gdouble Radius, const gdouble ZMin, const gdouble ZMax, const gdouble ThetaMax, const gulong ParameterCount, SDPCSTRING Tokens[], gpointer Values[], gulong ValueCounts[]) = 0;
	virtual sdpResult RiStructure(SDPCSTRING Structure) = 0;
	virtual sdpResult RiSubdivisionMeshV(SDPCSTRING Scheme, const gulong PathCount, gulong VertexCounts[], gulong Vertices[], const gulong TagCount, SDPSTRING Tags[], gulong ArgCounts[], gulong IntegerArgs[], gdouble FloatArgs[], const gulong ParameterCount, SDPCSTRING Tokens[], gpointer Values[], gulong ValueCounts[]) = 0;
	virtual sdpResult RiSurfaceV(SDPCSTRING Name, const gulong ParameterCount, SDPCSTRING Tokens[], gpointer Values[], gulong ValueCounts[]) = 0;
	virtual sdpResult RiTextureCoordinates(const gdouble S1, const gdouble T1, const gdouble S2, const gdouble T2, const gdouble S3, const gdouble T3, const gdouble S4, const gdouble T4) = 0;
	virtual sdpResult RiTorusV(const gdouble MajorRadius, const gdouble MinorRadius, const gdouble PhiMin,	const gdouble PhiMax, const gdouble ThetaMax, const gulong ParameterCount, SDPCSTRING Tokens[], gpointer Values[], gulong ValueCounts[]) = 0;
	virtual sdpResult RiTransform(gdouble Transform[16]) = 0;
	virtual sdpResult RiTransformBegin() = 0;
	virtual sdpResult RiTransformEnd() = 0;
	virtual sdpResult RiTranslate(const gdouble DX, const gdouble DY, const gdouble DZ) = 0;
	virtual sdpResult RiTrimCurve(const gulong LoopCount, gulong CuveCounts[], gulong Order[], gdouble Knot[], gdouble AMin[], gdouble AMax[], const gulong ParameterCount, gdouble U[], gdouble V[], gdouble W[]) = 0;
	virtual sdpResult RiWorldBegin() = 0;
	virtual sdpResult RiWorldEnd() = 0;
	
	static SDPCSTRING const RI_A;
	static SDPCSTRING const RI_ABORT; 
	static SDPCSTRING const RI_AMBIENTLIGHT;
	static SDPCSTRING const RI_AMPLITUDE;
	static SDPCSTRING const RI_AZ;
	static SDPCSTRING const RI_BACKGROUND;
	static SDPCSTRING const RI_BEAMDISTRIBUTION;
	static SDPCSTRING const RI_BICUBIC;
	static SDPCSTRING const RI_BILINEAR; 
	static SDPCSTRING const RI_BLACK;
	static SDPCSTRING const RI_BOX;
	static SDPCSTRING const RI_BUMPY;
	static SDPCSTRING const RI_CAMERA;
	static SDPCSTRING const RI_CATMULL_ROM; 
	static SDPCSTRING const RI_CLAMP; 
	static SDPCSTRING const RI_COMMENT;
	static SDPCSTRING const RI_CONEANGLE;
	static SDPCSTRING const RI_CONEDELTAANGLE;
	static SDPCSTRING const RI_CONSTANT;
	static SDPCSTRING const RI_CONSTANTWIDTH;	
	static SDPCSTRING const RI_CS;
	static SDPCSTRING const RI_CUBIC; 
	static SDPCSTRING const RI_DEPTHCUE;
	static SDPCSTRING const RI_DIFFERENCE;
	static SDPCSTRING const RI_DISTANCE; 
	static SDPCSTRING const RI_DISTANTLIGHT;
	static SDPCSTRING const RI_FILE;
	static SDPCSTRING const RI_FLATNESS; 
	static SDPCSTRING const RI_FOG;
	static SDPCSTRING const RI_FOV;
	static SDPCSTRING const RI_FRAMEBUFFER;
	static SDPCSTRING const RI_FROM;
	static SDPCSTRING const RI_GAUSSIAN;
	static SDPCSTRING const RI_HANDLER;
	static SDPCSTRING const RI_HIDDEN;
	static SDPCSTRING const RI_IDENTIFIER;
	static SDPCSTRING const RI_IGNORE; 
	static SDPCSTRING const RI_INSIDE;
	static SDPCSTRING const RI_INTENSITY;
	static SDPCSTRING const RI_INTERSECTION;
	static SDPCSTRING const RI_KA; 
	static SDPCSTRING const RI_KD; 
	static SDPCSTRING const RI_KR;
	static SDPCSTRING const RI_KS; 
	static SDPCSTRING const RI_LH;
	static SDPCSTRING const RI_LIGHTCOLOR;
	static SDPCSTRING const RI_LINEAR;
	static SDPCSTRING const RI_MATTE;
	static SDPCSTRING const RI_MAXDISTANCE;
	static SDPCSTRING const RI_METAL; 
	static SDPCSTRING const RI_MINDISTANCE; 
	static SDPCSTRING const RI_N; 
	static SDPCSTRING const RI_NAME;
	static SDPCSTRING const RI_NONPERIODIC; 
	static SDPCSTRING const RI_NP; 
	static SDPCSTRING const RI_OBJECT;
	static SDPCSTRING const RI_ORIGIN;
	static SDPCSTRING const RI_ORTHOGRAPHIC;
	static SDPCSTRING const RI_OS;
	static SDPCSTRING const RI_OUTSIDE;
	static SDPCSTRING const RI_P;
	static SDPCSTRING const RI_PAINT;
	static SDPCSTRING const RI_PAINTEDPLASTIC;
	static SDPCSTRING const RI_PERIODIC; 
	static SDPCSTRING const RI_PERSPECTIVE;
	static SDPCSTRING const RI_PLASTIC;
	static SDPCSTRING const RI_POINTLIGHT; 
	static SDPCSTRING const RI_PRIMITIVE; 
	static SDPCSTRING const RI_PRINT; 
	static SDPCSTRING const RI_PW;
	static SDPCSTRING const RI_PZ; 
	static SDPCSTRING const RI_RASTER;
	static SDPCSTRING const RI_RGB;
	static SDPCSTRING const RI_RGBA;
	static SDPCSTRING const RI_RGBAZ;
	static SDPCSTRING const RI_RGBZ;
	static SDPCSTRING const RI_RH;
	static SDPCSTRING const RI_ROUGHNESS; 
	static SDPCSTRING const RI_S; 
	static SDPCSTRING const RI_SCREEN; 
	static SDPCSTRING const RI_SHINYMETAL;
	static SDPCSTRING const RI_SINC;
	static SDPCSTRING const RI_SMOOTH;
	static SDPCSTRING const RI_SPECULARCOLOR;
	static SDPCSTRING const RI_SPOTLIGHT;
	static SDPCSTRING const RI_ST;
	static SDPCSTRING const RI_STRUCTURE;
	static SDPCSTRING const RI_T; 
	static SDPCSTRING const RI_TEXTURENAME;
	static SDPCSTRING const RI_TRIANGLE;
	static SDPCSTRING const RI_TO;
	static SDPCSTRING const RI_UNION;
	static SDPCSTRING const RI_WIDTH; 
	static SDPCSTRING const RI_WORLD;
	static SDPCSTRING const RI_Z;

protected:
	virtual ~k3dIRiEngine() {}
};

#endif // K3DIRIENGINE_H
