#ifndef K3DIRILIGHT_H
#define K3DIRILIGHT_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares k3dIRiLight, an abstract interface implemented by objects that can act as light sources while drawing RenderMan graphics
		\author Tim Shead (tshead@k-3d.com)
*/

#ifndef SDPTYPES_H
#include <sdptypes.h>
#endif // !SDPTYPES_H

// Forward class declarations
class k3dIRenderJob;
class k3dIRenderFrame;
class k3dICamera;
class k3dIRiEngine;

/// Abstract interface for objects that can act as light sources while drawing RenderMan graphics
class k3dIRiLight :
	public virtual sdpIUnknown
{
public:
	virtual sdpResult RiLightStartJob(k3dIRenderJob* const Job, k3dIRiEngine* const Engine) = 0;
	virtual sdpResult RiLightPreRender(k3dIRenderFrame* const Frame, k3dIRiEngine* const Engine) = 0;
	virtual sdpResult RiLightSetup(k3dIRenderFrame* const Frame, k3dIRiEngine* const Engine) = 0;
	virtual sdpResult RiLightPostRender(k3dIRenderFrame* const Frame, k3dIRiEngine* const Engine) = 0;
	virtual sdpResult RiLightFinishJob(k3dIRenderJob* const Job, k3dIRiEngine* const Engine) = 0;

protected:
	virtual ~k3dIRiLight() {}
};

#endif // K3DIRILIGHT_H
