#ifndef K3DIRIRENDER_H
#define K3DIRIRENDER_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares k3dIRiRender, an abstract interface implemented by objects that can render themselves as RenderMan graphics
		\author Tim Shead (tshead@k-3d.com)
*/

#ifndef SDPTYPES_H
#include <sdptypes.h>
#endif // !SDPTYPES_H

// Foward class references
class k3dIRenderJob;
class k3dIRenderFrame;
class k3dICamera;
class k3dIRiEngine;

/// Abstract interface for objects that can render themselves as RenderMan graphics
class k3dIRiRender :
	public virtual sdpIUnknown
{
public:
	virtual sdpResult RiStartJob(k3dIRenderJob* Job, k3dIRiEngine* const Engine) = 0;
	virtual sdpResult RiSetSolid(bool Solid) = 0;
	virtual sdpResult RiPreRender(k3dIRenderFrame* const Frame, k3dIRiEngine* const Engine) = 0;
	virtual sdpResult RiPushRender(k3dIRenderFrame* const Frame, k3dIRiEngine* const Engine) = 0;
	virtual sdpResult RiRender(k3dIRenderFrame* const Frame, k3dIRiEngine* const Engine) = 0;
	virtual sdpResult RiPushChildRender(k3dIRenderFrame* const Frame, k3dIRiEngine* const Engine) = 0;
	virtual sdpResult RiPopChildRender(k3dIRenderFrame* const Frame, k3dIRiEngine* const Engine) = 0;
	virtual sdpResult RiPopRender(k3dIRenderFrame* const Frame, k3dIRiEngine* const Engine) = 0;
	virtual sdpResult RiPostRender(k3dIRenderFrame* const Frame, k3dIRiEngine* const Engine) = 0;
	virtual sdpResult RiFinishJob(k3dIRenderJob* Job, k3dIRiEngine* const Engine) = 0;

protected:
	virtual ~k3dIRiRender() {}
};

#endif // K3DIRIRENDER_H
