#ifndef K3DISCRIPTENGINE_H
#define K3DISCRIPTENGINE_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares k3dIScriptEngine, an abstract interface implemented by objects that can execute scripts written in a specific scripting language
		\author Tim Shead (tshead@k-3d.com)
*/

#ifndef SDPTYPES_H
#include <sdptypes.h>
#endif // !SDPTYPES_H

// Forward class declarations ...
class k3dICommandNode;

/// Abstract interface implemented by objects that can execute scripts written in a specific scripting language
class k3dIScriptEngine :
	public virtual sdpIUnknown
{
public:
	/// Returns the language name associated with this engine
	virtual SDPCSTRING Language() = 0;
	/// Executes a script
	virtual sdpResult Execute(SDPCSTRING Script, SDPCSTRING ScriptName) = 0;
	/// Converts a command-node command into code appropriate to this language
	virtual sdpResult ConvertCommand(k3dICommandNode* CommandNode, SDPCSTRING Name, SDPCSTRING Arguments, SDPSTRING Buffer, gulong& BufferLength) = 0;

protected:
	virtual ~k3dIScriptEngine() {}
};

#endif // K3DISCRIPTENGINE_H
