#ifndef K3DTIME_H
#define K3DTIME_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the k3dITime interface for managing a document's time-related information
		\author Tim Shead (tshead@k-3d.com)
*/

#ifndef SDPTYPES_H
#include <sdptypes.h>
#endif // !SDPTYPES_H

/// Manages a document's time-related information
class k3dITime :
	public virtual sdpIUnknown
{
public:
	/// Returns the animation start time as a percentage
	virtual gdouble StartTime() = 0;
	/// Returns the current view time as a percentage
	virtual gdouble ViewTime() = 0;
	/// Returns the animation end time as a percentage
	virtual gdouble EndTime() = 0;
	/// Returns the length of the animation in seconds
	virtual gdouble Duration() = 0;
	/// Returns the animation frame rate in frames-per-second
	virtual gdouble FrameRate() = 0;
	/// Returns the length of an animation frame as a percentage
	virtual gdouble FrameDelta() = 0;
	/// Returns the current render time as a percentage
	virtual gdouble RenderTime() = 0;
	/// Returns the length of the current render as a percentage
	virtual gdouble RenderDeltaTime() = 0;

	/// Sets the animation start time as a percentage
	virtual sdpResult SetStartTime(const gdouble Time) = 0;
	/// Sets the current view time as a percentage
	virtual sdpResult SetViewTime(const gdouble Time) = 0;
	/// Sets the animation end time as a percentage
	virtual sdpResult SetEndTime(const gdouble Time) = 0;
	/// Sets the length of the animation in seconds
	virtual sdpResult SetDuration(const gdouble Duration) = 0;
	/// Sets the animation frame rate in frames-per-second
	virtual sdpResult SetFrameRate(const gdouble Rate) = 0;
	/// Sets the current render time as a percentage
	virtual sdpResult SetRenderTime(const gdouble Time) = 0;
	/// Sets the length of the current render as a percentage
	virtual sdpResult SetRenderDeltaTime(const gdouble Delta) = 0;
	
	static SDPCSTRING Event_StartTimeChanged;
	static SDPCSTRING Event_ViewTimeChanged;
	static SDPCSTRING Event_EndTimeChanged;
	static SDPCSTRING Event_DurationChanged;
	static SDPCSTRING Event_FrameRateChanged;

protected:
	virtual ~k3dITime() {} 
};

#endif // K3DTIME_H
