#ifndef K3DITRANSFORM_H
#define K3DITRANSFORM_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares k3dITransform, an abstract interface which defines operations for objects that maintain a homogeneous transformation frame
		\author Timothy M. Shead (tshead@k-3d.com)
*/

#ifndef SDPTYPES_H
#include <sdptypes.h>
#endif // !SDPTYPES_H

/// Abstract interface which defines operations for objects that contain an internal coordinate reference frame
class k3dITransform :
	public virtual sdpIUnknown
{
public:
	/// Enumerates supported coordinate references frames
	typedef enum
	{
		CURRENT_FRAME,
		ANIMATED_FRAME,
		CURRENT_CHILD_FRAME,
		ANIMATED_CHILD_FRAME
	} ReferenceFrame;

	virtual const sdpVector3& Position(const ReferenceFrame Frame) = 0;
	virtual const sdpAngleAxis& Orientation(const ReferenceFrame Frame) = 0;
	virtual const sdpVector3& Scale(const ReferenceFrame Frame) = 0;

	virtual void SetPosition(const ReferenceFrame Frame, const sdpVector3& Position) = 0;
	virtual void SetOrientation(const ReferenceFrame Frame, const sdpAngleAxis& Orientation) = 0;
	virtual void SetScale(const ReferenceFrame Frame, const sdpVector3& Scale) = 0;
	
	/// Event transmitted if the object's position changes
	static const SDPCSTRING Event_PositionChanged;
	static const SDPCSTRING Event_ChildPositionChanged;
	/// Event transmitted if the object's orientation changes
	static const SDPCSTRING Event_OrientationChanged;
	static const SDPCSTRING Event_ChildOrientationChanged;
	/// Event transmitted if the object's scale changes
	static const SDPCSTRING Event_ScaleChanged;
	static const SDPCSTRING Event_ChildScaleChanged;
	/// Event transmitted if the object's target changes
	static const SDPCSTRING Event_TargetChanged;

protected:
	virtual ~k3dITransform() {}
};

#endif // K3DITRANSFORM_H
