#ifndef K3DIVARIABLE_H
#define K3DIVARIABLE_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares k3dIVariable, which provides an abstract interface for objects that can store and manipulate serializable data in a standardized way
		\author Tim Shead (tshead@k-3d.com)
*/

#ifndef SDPTYPES_H
#include <sdptypes.h>
#endif // !SDPTYPES_H

// Forward class declarations
class k3dIDependencies;
namespace sdpxml { class Element; }

/// Provides an abstract interface for objects that can store and manipulate serializable data in a standardized way
class k3dIVariable : public sdpIUnknown
{
public:
	/// Returns the variable's name
	virtual SDPCSTRING Name() const = 0;

	/// Saves the variable to a K-3D XML document output stream
	virtual sdpResult Save(sdpxml::Element& Element, k3dIDependencies* Dependencies) const = 0;
	/// Loads the variable from a K-3D XML document
	virtual sdpResult Load(sdpxml::Element& Element) = 0;
	/// Called after loading is complete
	virtual sdpResult LoadComplete() = 0;

	/// Event transmitted when the variable's value changes
	static const SDPCSTRING Event_Changed;
	/// Event transmitted when the variable is deleted
	static const SDPCSTRING Event_Deleted;

protected:
	virtual ~k3dIVariable() {}
};

#endif // K3DIVARIABLE_H

