#ifndef K3DIVIEWVOLUME_H
#define K3DIVIEWVOLUME_H

// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares k3dIViewVolume, an abstract interface which encapsulates 3D-to-2D transformation volumes
		\author Tim Shead (tshead@k-3d.com)
*/

#ifndef SDPTYPES_H
#include <sdptypes.h>
#endif // !SDPTYPES_H

// Forward class declarations
class k3dIViewport;

///////////////////////////////////////////////////////////
// k3dIViewVolume

/// Abstract interface encapsulating 3D-to-2D camera viewing volumes
class k3dIViewVolume :
	public virtual sdpIUnknown
{
public:
	/// Returns the k3dIViewport associated with this volume
	virtual k3dIViewport* Viewport() = 0;

	/// Returns true iff this is an orthographic volume
	virtual const bool Orthographic() = 0;
	
	/// Returns the nearplane distance from the camera
	virtual const gdouble NearPlane() = 0;
	/// Returns the farplane distance from the camera
	virtual const gdouble FarPlane() = 0;
	/// Returns viewing volume distance to the left of the origin, measured a distance of 1.0 from the camera
	virtual const gdouble Left() = 0;
	/// Returns viewing volume distance to the right of the origin, measured a distance of 1.0 from the camera
	virtual const gdouble Right() = 0;
	/// Returns viewing volume distance above the origin, measured a distance of 1.0 from the camera
	virtual const gdouble Top() = 0;
	/// Returns viewing volume distance below the origin, measured a distance of 1.0 from the camera
	virtual const gdouble Bottom() = 0;
	
	/// Converts camera coordinates to GUI window coordinates, given the width and height of the GUI window
	virtual void CameraToScreen(gdouble Coords[3], const gdouble WindowPixelWidth, const gdouble WindowPixelHeight) = 0;

protected:
	virtual ~k3dIViewVolume() {}
};

#endif // K3DIVIEWVOLUME_H


