// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Implements the k3dAction implementation of k3dIAction
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3daction.h>
#include <k3dmisc.h>

#include <k3dicommandnode.h>

///////////////////////////////////////////////////////////////////////////
// k3dAction

k3dAction::k3dAction(k3dIAction* const NextAction) :
	m_NextAction(NextAction)
{
}

k3dAction::~k3dAction()
{
	delete m_NextAction;
}

void k3dAction::DoNextAction()
{
	if(m_NextAction)
		m_NextAction->DoAction();
}

///////////////////////////////////////////////////////////////////////////
// k3dActionRedrawAll

k3dActionRedrawAll::k3dActionRedrawAll(k3dIDocument* const Document, const bool Synchronous, k3dIAction* const NextAction) :
	k3dAction(NextAction),
	m_Document(Document),
	m_Synchronous(Synchronous)
{
	// Sanity checks ...
	g_assert(m_Document);
}

void k3dActionRedrawAll::DoAction()
{
	k3dRedrawAll(m_Document, m_Synchronous);
	DoNextAction();
}

////////////////////////////////////////////////////////////////////////////
// k3dActionSendCommand

k3dActionSendCommand::k3dActionSendCommand(k3dICommandNode* const Target, SDPCSTRING Command, SDPCSTRING Arguments, k3dIAction* const NextAction) :
	k3dAction(NextAction),
	m_Target(Target),
	m_Command(Command),
	m_Arguments(Arguments)
{
	// Sanity checks ...
	g_assert(Target);
	g_assert_string(Command);
}

void k3dActionSendCommand::DoAction()
{
	m_Target->Command(m_Command, m_Arguments, 0);
	DoNextAction();
}

///////////////////////////////////////////////////////////////////////////
// k3dActionDebugMessage

k3dActionDebugMessage::k3dActionDebugMessage(SDPCSTRING Message, k3dIAction* const NextAction) :
	k3dAction(NextAction),
	m_Message(Message)
{
}

void k3dActionDebugMessage::DoAction()
{
	std::cerr << m_Message << std::endl;
	DoNextAction();
}

