#ifndef K3DCHANNELBUTTON_H
#define K3DCHANNELBUTTON_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the k3dChannelButton class, which provides a UI for representing objects that support k3dIChannel
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3ddialog.h>
#include <k3deventobserver.h>
#include <k3duserinterfaceelement.h>

#include <k3dichannel.h>

#include <sdpgtkobjectcontainer.h>
#include <sdpgtkopengldrawingarea.h>

//////////////////////////////////////////////////////////////////////////////////////////////////
// k3dChannelButtonSetColor

typedef k3dDialog k3dChannelButtonSetColorBase;

/// Provides a UI for setting a channel to a single color
class k3dChannelButtonSetColor :
	public k3dChannelButtonSetColorBase
{
public:
	k3dChannelButtonSetColor(sdpIUnknown* const ParentCommandNode, k3dIChannel* const Channel);
	~k3dChannelButtonSetColor();

	// k3dICommandNode implementation
	sdpResult Command(SDPCSTRING Name, SDPCSTRING Arguments, sdpIUnknown** Result);

	// k3dIEventObserver implementation
	sdpResult ObserveEvent(k3dIEventSource* const Source, SDPCSTRING Event, SDPCSTRING EventData, SDPCSTRING ObserverData);

protected:	
	void OnEvent(sdpGtkEvent* Event);
	
	void OnClose();
	void RawOnColorChanged();
	void OnColorChanged(const sdpVector3 NewColor);

private:
	k3dIChannel* const m_Channel;
	sdpVector3 m_LastColor;
	
	static const sdpString control_close;
	static const sdpString control_colorchanged;
	static const sdpString control_colorselection;
};

///////////////////////////////////////////////////////////////////////////
// k3dChannelButton

typedef k3dUserInterfaceElement k3dChannelButtonBase;

/// Provides a UI for editing objects that support k3dIChannel
class k3dChannelButton :
	public k3dChannelButtonBase,
	public k3dEventObserver
{
public:
	k3dChannelButton(sdpIUnknown* CommandNodeParent, SDPCSTRING CommandNodeName);
	~k3dChannelButton();

	/// Called by the framework when instantiating the button from a GTKML (XML) document
	bool Create(sdpGtkIObjectContainer* ObjectContainer, sdpxml::Document& Document, sdpxml::Element& Element);

	/// Attaches the button to the channel it fronts for
	bool Attach(k3dIChannel* Channel);

	const sdpString CustomType() const;

	// k3dICommandNode implementation
	sdpResult Command(SDPCSTRING Name, SDPCSTRING Arguments, sdpIUnknown** Result);

protected:
	/// Called to handle SDPGTK events
	void OnEvent(sdpGtkEvent* Event);
	/// Called when the GTK+ widgets are about to disappear
	void OnDestroy();
	/// Called when the custom-drawn value needs to be updated
	void OnValueDraw();
	/// Called when the user begins editing the channel in place
	void OnStartValueFocusIn();
	/// Called when the user finishes editing the channel in place
	void OnStartValueFocusOut();
	/// Called when the user begins editing the channel in place
	void OnEndValueFocusIn();
	/// Called when the user finishes editing the channel in place
	void OnEndValueFocusOut();
	/// Called to open the edit the channel
	void OnEdit();
	/// Called to edit the channel quickly
	void OnQuickEdit();
	/// Called when the the button needs to be updated
	void Update();

	/// Called to handle K-3D events from the channel we're fronting for
	sdpResult ObserveEvent(k3dIEventSource* const Source, SDPCSTRING Event, SDPCSTRING EventData, SDPCSTRING ObserverData);

	/// Stores the channel we represent
	k3dIChannel* m_Channel;
	/// Allows the developer to choose whether to display the channel start-value or its curve on the button face
	bool m_ShowCurve;

	/// Stores the OpenGL context for the custom-drawn value
	sdpGtkOpenGLDrawingArea m_ValueWidget;
	
	static sdpVector3 m_BackgroundColor;
	static sdpVector3 m_NormalColor;
	
	static const sdpString control_startvalue;
	static const sdpString control_endvalue;
	static const sdpString control_quickedit;
	static const sdpString control_edit;
};

#endif // K3DCHANNELBUTTON_H
