#ifndef K3DCOMMANDNODE_H
#define K3DCOMMANDNODE_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares k3dCommandNode, which provides a default implementation of k3dICommandNode
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3dicommandnode.h>

/// Provides a default implementation of k3dICommandNode
class k3dCommandNode :
	public k3dICommandNode
{
public:
	k3dCommandNode(sdpIUnknown* const Parent, SDPCSTRING Name);
	virtual ~k3dCommandNode();

	// k3dICommandNode overrides ...
	sdpResult Command(SDPCSTRING Name, SDPCSTRING Arguments, sdpIUnknown** Result);

	SDPCSTRING CommandNodeName();
	k3dICommandNode* CommandNodeParent();
	k3dICommandNode* CommandNodeByPath(SDPCSTRING Path);
	
	K3DCOMMANDNODEHANDLE FirstCommandNode();
	K3DCOMMANDNODEHANDLE NextCommandNode(K3DCOMMANDNODEHANDLE CommandNode);
	k3dICommandNode* CommandNode(K3DCOMMANDNODEHANDLE CommandNode);
	sdpResult AddCommandNode(k3dICommandNode* CommandNode);
	sdpResult RemoveCommandNode(k3dICommandNode* CommandNode);

protected:
	void SetCommandNodeName(SDPCSTRING Name);
	void SetParentCommandNode(sdpIUnknown* const Parent);
	k3dICommandNode* GetRootCommandNode();
	
private:
	/// Stores the name of this node
	sdpString m_Name;

	/// Stores the parent of this node
	k3dICommandNode* m_Parent;

	/// A collection of child k3dICommandNode objects
	typedef std::vector<k3dICommandNode*> CommandNodes;
	typedef CommandNodes::iterator CommandNodeIterator;

	/// Contains child nodes
	CommandNodes m_Children;
};

#endif // K3DCOMMANDNODE_H
