// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Implements the k3dControl class, which provides a UI for numeric quantities
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3dcontrol.h>
#include <k3dmisc.h>

#include <k3dieventsource.h>
#include <k3divariable.h>

//////////////////////////////////////////////////////////////////////
// k3dControl

k3dControl::k3dControl(sdpIUnknown* CommandNodeParent, SDPCSTRING CommandNodeName) :
	k3dControlBase(CommandNodeParent, CommandNodeName),
	m_CommandRecorder(0),
	m_StateRecorder(0),
	m_PushEditingAction(0),
	m_ChangeAction(0),
	m_PopEditingAction(0)
{
}

k3dControl::~k3dControl()
{
	delete m_PushEditingAction;
	delete m_ChangeAction;
	delete m_PopEditingAction;
}

bool k3dControl::Attach(k3dICommandRecorder* const CommandRecorder, k3dIStateRecorder* const StateRecorder, SDPCSTRING StateChangeName, k3dIAction* const PushEditingAction, k3dIAction* const ChangeAction, k3dIAction* const PopEditingAction)
{
	 // You only get to attach once!
	g_assert(!m_CommandRecorder);
	g_assert(!m_StateRecorder);
	g_assert(!m_PushEditingAction);
	g_assert(!m_ChangeAction);
	g_assert(!m_PopEditingAction);

	// Store references ...
	m_CommandRecorder = CommandRecorder;
	m_StateRecorder = StateRecorder;
	m_StateChangeName = StateChangeName;
	m_PushEditingAction = PushEditingAction;
	m_ChangeAction = ChangeAction;
	m_PopEditingAction = PopEditingAction;
	
	return true;
}

