#ifndef K3DCONTROL_H
#define K3DCONTROL_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the k3dControl class, which provides features common to all K-3D UI controls
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3daction.h>
#include <k3deventobserver.h>
#include <k3duserinterfaceelement.h>

// Forward class declarations
class k3dICommandRecorder;
class k3dIDocument;
class k3dIStateRecorder;
class k3dIVariable;

///////////////////////////////////////////////////////////////////////////
// k3dControl

typedef k3dUserInterfaceElement k3dControlBase;

/// Provides features common to all K-3D UI controls
class k3dControl :
	public k3dControlBase,
	public k3dEventObserver
{
public:
	k3dControl(sdpIUnknown* CommandNodeParent, SDPCSTRING CommandNodeName);
	virtual ~k3dControl();

protected:
	bool Attach(k3dICommandRecorder* const CommandRecorder, k3dIStateRecorder* const StateRecorder, SDPCSTRING StateChangeName, k3dIAction* const PushEditingAction, k3dIAction* const ChangeAction, k3dIAction* const PopEditingAction);

	/// Stores an optional object that will record commands for the tutorial/macro recording system
	k3dICommandRecorder* m_CommandRecorder;
	/// Stores an optional object that will record state changes for the undo/redo system
	k3dIStateRecorder* m_StateRecorder;
	/// Stores the state change name for undo / redo purposes
	sdpString m_StateChangeName;
	/// Stores an optional action chain that will be activated when the control begins editing its data source
	k3dIAction* m_PushEditingAction;
	/// Stores an optional action chain that will be activated whenever the control alters its data source
	k3dIAction* m_ChangeAction;
	/// Stores an optional action chain that will be activated when the control finishes editing its data source
	k3dIAction* m_PopEditingAction;
};

///////////////////////////////////////////////////////////////////////////
// k3dControlFormat

/// Provides text I/O with formatting for the given variable type
template<class Type>
class k3dControlFormat
{
public:
	virtual ~k3dControlFormat() {}

	virtual bool FormattedOutput(const Type& Value, sdpString& Output) = 0;
	virtual bool ParsedInput(const sdpString& Input, Type& InputValue) = 0;
};

#endif // K3DCONTROL_H

