
// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Implements the k3dDependencies class, which provides an implementation of k3dIDependencies
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3ddependencies.h>

sdpResult k3dDependencies::RegisterObject(k3dIObject* const Object)
{
	// Sanity checks ...
	g_return_val_if_fail(Object, SDP_E_FAIL);

	// Add this object to the list ...
	m_Objects.insert(Object);

	return SDP_S_OK;
}

sdpResult k3dDependencies::RegisterFile(SDPCSTRING FilePath)
{
	// Sanity checks ...
	g_return_val_if_string_fail(FilePath, SDP_E_FAIL);

	// Add this file to the list ...
	m_Files.insert(FilePath);

	return SDP_S_OK;
}

const k3dDependencies::ObjectSet& k3dDependencies::Objects() const
{
	return m_Objects;
}

const sdpStringSet& k3dDependencies::Files() const
{
	return m_Files;
}


