#ifndef K3DDIALOG_H
#define K3DDIALOG_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the k3dObjectDialog class, which provides a standard dialog implementation for K-3D objects
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3deventobserver.h>
#include <k3duserinterfaceelement.h>
#include <k3dwindowgeometrystore.h>

///////////////////////////////////////////////////////////////////////////
// k3dDialog

typedef k3dUserInterfaceElement k3dDialogBase;

class k3dDialog :
	public k3dDialogBase,
	public k3dEventObserver
{
public:
	k3dDialog(sdpIUnknown* ParentCommandNode, SDPCSTRING CommandNodeName, k3dWindowGeometryStore* const WindowGeometryStore);
	virtual ~k3dDialog();

	/// Standard UI destruction (deletes this object)
	virtual void CloseDialog();

protected:
	/// Called to load a GTKML template file (connects to some standard events)
	virtual bool LoadGTKMLTemplate(SDPCSTRING Template);
	/// Called to load a GTKML template file (connects to some standard events)
	virtual bool LoadGTKMLTemplate(SDPCSTRING Subdirectory, SDPCSTRING Template);

	/// Called to make this dialog visible (also handles restoring saved window positions)
	virtual void Show();

	/// Called to cleanly refresh dialog controls (temporarily suspends event-handling)
	virtual void CallUpdateControls();

	/// Does the work of updating controls when their underlying data changes
	virtual void UpdateControls();

	// Event-handling ...
	void OnEvent(sdpGtkEvent* Event);

	// Built-in events ...
	virtual void OnOK();
	virtual void OnCancel();
	virtual void OnClose();
	/// Called when the user hits the window manager "close" button
	virtual void OnDelete(sdpGtkEvent* Event);

private:
	/// Stores an object that can serialize our window configuration
	k3dWindowGeometryStore* const m_WindowGeometryStore;

	static const sdpString control_delete;
};

#endif // K3DDIALOG_H

