#ifndef K3DEDITCONTROL_H
#define K3DEDITCONTROL_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the k3dEditControl class, which provides a standard MVC UI for string values
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3dcontrol.h>
#include <k3dvariable.h>

// Forward class declarations
class k3dICommandRecorder;
class k3dIStateRecorder;
class sdpGtkIObjectContainer;
class sdpxml::Element;

typedef k3dVariable<sdpString> k3dEditControlDataSource;

///////////////////////////////////////////////////////////////////////////
// k3dEditControl

typedef k3dControl k3dEditControlBase;

/// Provides a UI for editing string objects
class k3dEditControl :
	public k3dEditControlBase
{
public:
	k3dEditControl(sdpIUnknown* CommandNodeParent, SDPCSTRING CommandNodeName);
	virtual ~k3dEditControl();

	/// Called by the framework when instantiating the button from a GTKML (XML) document
	bool Create(sdpGtkIObjectContainer* ObjectContainer, sdpxml::Document& Document, sdpxml::Element& Element);
	/// Called to set edit control parameters manually
	bool Create();

	/// Attaches the button to the object it fronts for
	bool Attach(k3dEditControlDataSource* const Source, k3dICommandRecorder* const CommandRecorder, k3dIStateRecorder* const StateRecorder, SDPCSTRING StateChangeName, k3dIAction* const PushEditingAction, k3dIAction* const ChangeAction, k3dIAction* const PopEditingAction);

	const sdpString CustomType() const;

	// k3dIEventObserver implementation
	sdpResult ObserveEvent(k3dIEventSource* const Source, SDPCSTRING Event, SDPCSTRING EventData, SDPCSTRING ObserverData);

	// k3dICommandNode overrides
	sdpResult Command(SDPCSTRING Name, SDPCSTRING Arguments, sdpIUnknown** Result);

protected:
	/// Called to handle SDPGTK events
	void OnEvent(sdpGtkEvent* Event);
	
	/// Called when the GTK+ widgets are about to disappear
	void OnDestroy();
	/// Called when the user begins altering the value
	void OnValueFocusIn();
	/// Called when the user finishes altering the value
	void OnValueFocusOut();

	/// Called when the the button needs to be updated
	void Update();
	
	/// Called to set the contents of the button
	void SetValue(const sdpString Value);

	/// Storeas a reference to the underlying data object
	k3dEditControlDataSource* m_DataSource;

	static const sdpString control_value;
	static const sdpString control_value_focus_in;
	static const sdpString control_value_focus_out;
};

#endif // K3DEDITCONTROL_H
