// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Implements the k3dEventConnector class, which facilitates making connections betwen event clients and event servers
		\author Tim Shead (tshead@k-3d.com)
*/

#include <iostream>

#include <k3deventconnector.h>
#include <k3dieventsource.h>

/////////////////////////////////////////////////////////////////
// k3dEventConnector

k3dEventConnector::k3dEventConnector(k3dIEventObserver* const Observer, SDPCSTRING Event, SDPCSTRING ObserverData, k3dEventConnector* const NextConnector) :
	m_Source(0),
	m_Observer(Observer),
	m_Event(Event),
	m_ObserverData(ObserverData),
	m_NextConnector(NextConnector)
{
	// Sanity checks ...
	g_assert(m_Observer);
	g_assert_string(m_Event);
}

k3dEventConnector::k3dEventConnector(k3dIEventSource* const Source, k3dIEventObserver* const Observer, SDPCSTRING Event, SDPCSTRING ObserverData, k3dEventConnector* const NextConnector) :
	m_Source(Source),
	m_Observer(Observer),
	m_Event(Event),
	m_ObserverData(ObserverData),
	m_NextConnector(NextConnector)
{
	// Sanity checks ...
	g_assert(m_Source);
	g_assert(m_Observer);
	g_assert(m_Event);

	m_Source->AddEventObserver(m_Observer, m_Event, m_ObserverData);	
	
	if(m_NextConnector)
		m_NextConnector->Attach(m_Source);
}

k3dEventConnector::~k3dEventConnector()
{
	if(m_Source)
		{
			m_Source->RemoveEventObserverEvent(m_Observer, m_Event, m_ObserverData);
			
			if(m_NextConnector)
				m_NextConnector->Detach();
		}

	delete m_NextConnector;
}
	
void k3dEventConnector::Attach(k3dIEventSource* const Source)
{
	// Sanity checks ...
	g_assert(Source);
	
	Detach();

	m_Source = Source;
	m_Source->AddEventObserver(m_Observer, m_Event, m_ObserverData);

	if(m_NextConnector)
		m_NextConnector->Attach(Source);
}

bool k3dEventConnector::Attached() const
{
	return m_Source;
}

void k3dEventConnector::Detach()
{
	if(0 == m_Source)
		return;
	
	m_Source->RemoveEventObserverEvent(m_Observer, m_Event, m_ObserverData);
	m_Source = 0;
	
	if(m_NextConnector)
		m_NextConnector->Detach();
}

