#ifndef K3DFILTERSELECTOR_H
#define K3DFILTERSELECTOR_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the k3dFilterSelector class, which provides the user with a standard UI for selecting file import/export filters
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3dfilefilters.h>
#include <sdpgtkobjectcontainer.h>

// Forward class references ...
class k3dIDocument;

namespace k3d
{

////////////////////////////////////////////////////////////////////////////
// FilterSelector

typedef sdpGtkObjectContainer FilterSelectorBase;

/// Implements the application user interface
class FilterSelector :
	public FilterSelectorBase
{
public:
	FilterSelector(k3dIDocument* const Document, const sdpString Title, const sdpString FilePath, const sdp::ITypeInfo& FilterType);
	~FilterSelector();

	/// Returns true iff the user selected a file filter
	bool OK() const;
	/// Returns true iff the user cancelled file filter selection
	bool Cancelled() const;
	/// Returns the filter selected by the user
	template<class Interface>
	Interface* Filter()
	{
		const sdpString choice(m_Choices[FilterChoice()]);
		return choice.size() ? k3d::GetFileFilter<Interface>(m_Document, choice) : k3d::GetAutoFilter<Interface>(m_Document, m_FilePath);
	}

private:
	void OnEvent(sdpGtkEvent* Event);
	void OnOK();
	void OnCancel();
	void OnDelete(sdpGtkEvent* const Event);

	/// Returns the user's combo-box selection
	const sdpString FilterChoice();

	/// Stores a pointer to the owning document
	k3dIDocument* const m_Document;
	/// Stores the filepath to be read-from / written-to
	const sdpString m_FilePath;
	/// Stores a mapping of human-readable-text to format filter names
	typedef std::map<sdpString, sdpString> FilterChoices;
	FilterChoices m_Choices;
	/// Stores whether a file was selected or cancelled
	bool m_OK;
};

} // namespace k3d

#endif // K3DFILTERSELECTOR_H

