#ifndef K3DFRAMERANGE_H
#define K3DFRAMERANGE_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the k3dFrameRange class for working with ranges of numbered files
		\author Timothy M. Shead (tshead@k-3d.com)
*/

#ifndef SDPTYPES_H
#include <sdptypes.h>
#endif // !SDPTYPES_H

///////////////////////////////////////////////////////////////////////////////////////
// k3dFrameRange

/// A utility class for working with ranges of numbered files
class k3dFrameRange
{
public:
	/// Initializes a range of frames based on a single filesystem sample
	k3dFrameRange(const sdpString BaseFileName);
	/// Initializes a range of frames from a user-specified basefilename
	k3dFrameRange(const sdpString BaseFileName, const gulong FirstFrame, const gulong LastFrame);
	/// Initialize a range of frames from scratch
	k3dFrameRange(const sdpString PreDigits, const sdpString PostDigits, const gulong FirstFrame, const gulong LastFrame, const gulong FrameDigits);
	
	/// Generates a filename for the given frame number
	bool Frame(const gulong FrameNumber, sdpString& Result) const;

	/// Returns the largest possible frame number, based on the number of digits
	gulong MaxFrame() const;

	/// Returns the text that precedes frame numbers
	const sdpString& PreDigits() const
	{
		return m_PreDigits;
	}
	
	/// Returns the text that follows frame numbers
	const sdpString& PostDigits() const
	{
		return m_PostDigits;
	}
	
	/// Returns the first frame number
	gulong FirstFrame() const
	{
		return m_FirstFrame;
	}
	
	/// Returns the last frame number
	gulong LastFrame() const
	{
		return m_LastFrame;
	}
	
	/// Returns the number of digits in the frame number
	gulong FrameDigits() const
	{
		return m_FrameDigits;
	}

protected:
	/// Stores the text that precedes frame numbers
	sdpString m_PreDigits;
	/// Stores the text that follows frame numbers
	sdpString m_PostDigits;
	/// Stores the first frame number
	gulong m_FirstFrame;
	/// Stores the last frame number
	gulong m_LastFrame;
	/// Stores the number of digits in the frame number
	gulong m_FrameDigits;
};

#endif // K3DFRAMERANGE_H
