// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Implements the k3dKnot class, which encapsulates a member of a NURBS curve/patch knot vector
		\author Tim Shead (tshead@k-3d.com)
*/

#include <sdpxml.h>

#include <k3dknot.h>
#include <k3dmisc.h>
#include <k3dobject.h>

#include <k3distaterecorder.h>
#include <k3didocument.h>

/////////////////////////////////////////////////////////////////////////
// k3dKnot

k3dKnot::k3dKnot()
{
	m_Parent = 0;
	m_Value = 0.0;
}

k3dKnot::k3dKnot(k3dObject* Parent)
{
	m_Parent = Parent;
	m_Value = 0.0;
}

k3dKnot::~k3dKnot()
{
}

///////////////////////////////////////////////////////
// k3dIKnot overrides

gdouble k3dKnot::Value()
{
	return m_Value;
}

class k3dKnotValueStateChange : public k3dIStateContainer
{
public:
	k3dKnotValueStateChange(k3dKnot* const Knot, const gdouble& Value) :
		m_Knot(Knot),
		m_Value(Value)
	{
	}
	
	void RestoreState()
	{
		m_Knot->m_Value = m_Value;
	}
	
private:
	k3dKnot* const m_Knot;
	const gdouble m_Value;	
};

sdpResult k3dKnot::SetValue(gdouble Value)
{
	if(m_Parent->Document()->StateRecorder()->ChangeSet())
		{
			m_Parent->Document()->StateRecorder()->ChangeSet()->RecordOldState(new k3dKnotValueStateChange(this, m_Value));
			m_Parent->Document()->StateRecorder()->ChangeSet()->RecordNewState(new k3dKnotValueStateChange(this, Value));
		}

	m_Value = Value;
	
	return SDP_S_OK;
}

sdpResult k3dKnot::Save(sdpxml::Element& Element, k3dIDependencies* Dependencies)
{
	// Sanity checks ...
	g_assert(Dependencies);

	// Save our value ...
	Element.Children().push_back(sdpxml::Element("knot", "", sdpxml::Attribute("value", sdpToString(m_Value))));

	return SDP_S_OK;
}

sdpResult k3dKnot::Load(sdpxml::Element& Element)
{
	// Sanity checks ...
	g_return_val_if_fail(Element.Name() == "knot", SDP_E_FAIL);

	// Load our value ...
	sdpxml::ParseAttribute(Element, "value", m_Value);

	return SDP_S_OK;
}


