#ifndef K3DKNOT_H
#define K3DKNOT_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the k3dKnot class, which encapsulates a member of a NURBS curve/patch knot vector
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3diknot.h>

// Forward class references ...
class k3dObject;
class k3dPath;
class k3dNURBSCurve;
class k3dPoint;
class k3dMesh;

class k3dIPath;
class k3dICurve;
class k3dIDependencies;

class sdpxml::Element;

/////////////////////////////////////////////////////////////////////////////
// k3dKnot

class k3dKnot : public k3dIKnot
{
public:
	k3dKnot();
	k3dKnot(k3dObject* Parent);
	
	virtual ~k3dKnot();

	// k3dIKnot overrides
	gdouble Value();
	sdpResult SetValue(gdouble Value);

	// Helper functions ...
	sdpResult Save(sdpxml::Element& Element, k3dIDependencies* Dependencies);
	sdpResult Load(sdpxml::Element& Element);

	/// Stores the parent object
	k3dObject* m_Parent;
	/// Stores the knot value
	gdouble m_Value;
};

#endif // K3DKNOT_H
