#ifndef K3DMODULE_H
#define K3DMODULE_H

// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the k3dModule class, which provides useful boilerplate for implementing K-3D object modules
		\author Timothy M. Shead (tshead@k-3d.com)
*/

#ifndef SDPXML_H
#include <sdpxml.h>
#endif // !SDPXML_H

#include <gmodule.h>

// Forward class declarations
class k3dIApplication;

namespace k3d
{

/// Provides useful boilerplate for implementing K-3D object modules
class Module
{
public:
	Module(const sdpString ModuleName, const sdpString ModuleVersion);
	virtual ~Module();

	virtual sdpResult InitializeModule(sdpIUnknown* Initializer);
	virtual SDPCSTRING ModuleInfo();
	virtual sdpIUnknown* CreateInstance(const sdp::ClassID& ClassID, sdpIUnknown* const Initializer) = 0;
	
protected:
	sdpxml::Element& RegisterObject(const sdp::ClassID& ClassID, const sdpString Name);
	void RegisterInterface(sdpxml::Element& Object, const sdp::ITypeInfo& InterfaceType);
	void RegisterShortDescription(sdpxml::Element& Object, const sdpString Description);
	void RegisterBookmark(sdpxml::Element& Object, const sdpString Bookmark);
	void RegisterIncomplete(sdpxml::Element& Object);
	void RegisterExperimental(sdpxml::Element& Object);

private:
	sdpxml::Document m_ModuleInfo;
	sdpString m_ModuleInfoText;
};

} // namespace k3d

#ifdef K3D_HAVE_STATIC_PLUGINS

/// Provides boilerplate implementations of the required sdpCom module procedures that call into a k3dModule derivative
#define K3DMODULEPROCEDURES(modulename, module) \
extern "C" { \
G_MODULE_EXPORT sdpResult modulename##sdpComInitialize(sdpIUnknown* Initializer) { return module::Instance().InitializeModule(Initializer); } \
G_MODULE_EXPORT SDPCSTRING modulename##sdpComInfo() { return module::Instance().ModuleInfo(); } \
G_MODULE_EXPORT sdpIUnknown* modulename##sdpComCreateInstance(const sdp::ClassID& ClassID, sdpIUnknown* Initializer) { return module::Instance().CreateInstance(ClassID, Initializer); } \
sdpExport const sdpComInitializeModuleProc modulename##Initialize = modulename##sdpComInitialize; \
sdpExport const sdpComModuleInfoProc modulename##Info = modulename##sdpComInfo; \
sdpExport const sdpComCreateInstanceProc modulename##CreateInstance = modulename##sdpComCreateInstance; \
}

#else // K3D_HAVE_STATIC_PLUGINS

/// Provides boilerplate implementations of the required sdpCom module procedures that call into a k3dModule derivative
#define K3DMODULEPROCEDURES(modulename, module) \
extern "C" { \
G_MODULE_EXPORT sdpResult sdpComInitializeModule(sdpIUnknown* Initializer) { return module::Instance().InitializeModule(Initializer); } \
G_MODULE_EXPORT SDPCSTRING sdpComModuleInfo() { return module::Instance().ModuleInfo(); } \
G_MODULE_EXPORT sdpIUnknown* sdpComCreateInstance(const sdp::ClassID& ClassID, sdpIUnknown* Initializer) { return module::Instance().CreateInstance(ClassID, Initializer); } \
}

#endif // !K3D_HAVE_STATIC_PLUGINS

#endif // !K3DMODULE_H

