// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Implements the k3dMouseEventObserver class, which provides a pluggable default implementation of k3dIMouseEventObserver
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3dmouseeventobserver.h>
#include <gdk/gdktypes.h>

k3dMouseEventObserver::k3dMouseEventObserver(SDPCSTRING Usage) :
	m_Usage(Usage)
{
}

k3dMouseEventObserver::~k3dMouseEventObserver()
{
}

///////////////////////////////////////////////////////////////
// k3dIMouseEventObserver implementation

SDPCSTRING k3dMouseEventObserver::Usage()
{
	return m_Usage;
}

sdpResult k3dMouseEventObserver::OnMouseMove(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY)
{
	return SDP_S_FALSE;
}

sdpResult k3dMouseEventObserver::OnLButtonClick(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY)
{
	return SDP_S_FALSE;
}

sdpResult k3dMouseEventObserver::OnLButtonDoubleClick(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY)
{
	return SDP_S_FALSE;
}

sdpResult k3dMouseEventObserver::OnLButtonStartDrag(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY)
{
	return SDP_S_FALSE;
}

sdpResult k3dMouseEventObserver::OnLButtonDrag(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY, gdouble LastX, gdouble LastY, gdouble StartX, gdouble StartY)
{
	return SDP_S_FALSE;
}

sdpResult k3dMouseEventObserver::OnLButtonEndDrag(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY, gdouble LastX, gdouble LastY, gdouble StartX, gdouble StartY)
{
	return SDP_S_FALSE;
}

sdpResult k3dMouseEventObserver::OnMButtonClick(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY)
{
	return SDP_S_FALSE;
}

sdpResult k3dMouseEventObserver::OnMButtonDoubleClick(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY)
{
	return SDP_S_FALSE;
}

sdpResult k3dMouseEventObserver::OnMButtonStartDrag(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY)
{
	return SDP_S_FALSE;
}

sdpResult k3dMouseEventObserver::OnMButtonDrag(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY, gdouble LastX, gdouble LastY, gdouble StartX, gdouble StartY)
{
	return SDP_S_FALSE;
}

sdpResult k3dMouseEventObserver::OnMButtonEndDrag(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY, gdouble LastX, gdouble LastY, gdouble StartX, gdouble StartY)
{
	return SDP_S_FALSE;
}

sdpResult k3dMouseEventObserver::OnRButtonClick(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY)
{
	return SDP_S_FALSE;
}

sdpResult k3dMouseEventObserver::OnRButtonDoubleClick(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY)
{
	return SDP_S_FALSE;
}

sdpResult k3dMouseEventObserver::OnRButtonStartDrag(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY)
{
	return SDP_S_FALSE;
}

sdpResult k3dMouseEventObserver::OnRButtonDrag(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY, gdouble LastX, gdouble LastY, gdouble StartX, gdouble StartY)
{
	return SDP_S_FALSE;
}

sdpResult k3dMouseEventObserver::OnRButtonEndDrag(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY, gdouble LastX, gdouble LastY, gdouble StartX, gdouble StartY)
{
	return SDP_S_FALSE;
}

sdpResult k3dMouseEventObserver::OnLRButtonStartDrag(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY)
{
	return SDP_S_FALSE;
}

sdpResult k3dMouseEventObserver::OnLRButtonDrag(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY, gdouble LastX, gdouble LastY, gdouble StartX, gdouble StartY)
{
	return SDP_S_FALSE;
}

sdpResult k3dMouseEventObserver::OnLRButtonEndDrag(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY, gdouble LastX, gdouble LastY, gdouble StartX, gdouble StartY)
{
	return SDP_S_FALSE;
}

sdpResult k3dMouseEventObserver::OnMouseWheelUp(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY)
{
	return SDP_S_FALSE;
}

sdpResult k3dMouseEventObserver::OnMouseWheelDown(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY)
{
	return SDP_S_FALSE;
}

bool k3dMouseEventObserver::ShiftDown(GdkModifierType Modifiers)
{
	return (Modifiers & GDK_SHIFT_MASK) ? true : false;
}

bool k3dMouseEventObserver::ControlDown(GdkModifierType Modifiers)
{
	return (Modifiers & GDK_CONTROL_MASK) ? true : false;
}

bool k3dMouseEventObserver::Mod1Down(GdkModifierType Modifiers)
{
	return (Modifiers & GDK_MOD1_MASK) ? true : false;
}

bool k3dMouseEventObserver::Mod2Down(GdkModifierType Modifiers)
{
	return (Modifiers & GDK_MOD2_MASK) ? true : false;
}

bool k3dMouseEventObserver::Mod3Down(GdkModifierType Modifiers)
{
	return (Modifiers & GDK_MOD3_MASK) ? true : false;
}

bool k3dMouseEventObserver::Mod4Down(GdkModifierType Modifiers)
{
	return (Modifiers & GDK_MOD4_MASK) ? true : false;
}

bool k3dMouseEventObserver::Mod5Down(GdkModifierType Modifiers)
{
	return (Modifiers & GDK_MOD5_MASK) ? true : false;
}

