#ifndef K3DMOUSEEVENTOBSERVER_H
#define K3DMOUSEEVENTOBSERVER_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the k3dMouseEventObserver class, which provides a pluggable default implementation of k3dIMouseEventObserver
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3dimouseeventobserver.h>

/// Provides an implementation of k3dIMouseEventObserver
class k3dMouseEventObserver :
	public k3dIMouseEventObserver
{
public:
	k3dMouseEventObserver(SDPCSTRING Usage);
	virtual ~k3dMouseEventObserver();

	// k3dIMouseEventObserver implementation ...
	SDPCSTRING Usage();

	sdpResult OnMouseMove(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY);
	sdpResult OnLButtonClick(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY);
	sdpResult OnLButtonDoubleClick(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY);
	sdpResult OnLButtonStartDrag(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY);
	sdpResult OnLButtonDrag(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY, gdouble LastX, gdouble LastY, gdouble StartX, gdouble StartY);
	sdpResult OnLButtonEndDrag(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY, gdouble LastX, gdouble LastY, gdouble StartX, gdouble StartY);
	sdpResult OnMButtonClick(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY);
	sdpResult OnMButtonDoubleClick(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY);
	sdpResult OnMButtonStartDrag(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY);
	sdpResult OnMButtonDrag(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY, gdouble LastX, gdouble LastY, gdouble StartX, gdouble StartY);
	sdpResult OnMButtonEndDrag(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY, gdouble LastX, gdouble LastY, gdouble StartX, gdouble StartY);
	sdpResult OnRButtonClick(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY);
	sdpResult OnRButtonDoubleClick(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY);
	sdpResult OnRButtonStartDrag(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY);
	sdpResult OnRButtonDrag(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY, gdouble LastX, gdouble LastY, gdouble StartX, gdouble StartY);
	sdpResult OnRButtonEndDrag(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY, gdouble LastX, gdouble LastY, gdouble StartX, gdouble StartY);
	sdpResult OnLRButtonStartDrag(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY);
	sdpResult OnLRButtonDrag(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY, gdouble LastX, gdouble LastY, gdouble StartX, gdouble StartY);
	sdpResult OnLRButtonEndDrag(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY, gdouble LastX, gdouble LastY, gdouble StartX, gdouble StartY);
	sdpResult OnMouseWheelUp(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY);
	sdpResult OnMouseWheelDown(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY);

	/// Returns true if the shift key is down
	static bool ShiftDown(GdkModifierType Modifiers);
	/// Returns true if the control key is down
	static bool ControlDown(GdkModifierType Modifiers);
	/// Returns true if the mod1 (usually "alt") key is down
	static bool Mod1Down(GdkModifierType Modifiers);
	/// Returns true if the mod2 key is down
	static bool Mod2Down(GdkModifierType Modifiers);
	/// Returns true if the mod3 key is down
	static bool Mod3Down(GdkModifierType Modifiers);
	/// Returns true if the mod4 key is down
	static bool Mod4Down(GdkModifierType Modifiers);
	/// Returns true if the mod5 key is down
	static bool Mod5Down(GdkModifierType Modifiers);
	
private:
	const sdpString m_Usage;
};

#endif // K3DMOUSEEVENTOBSERVER_H
