#ifndef K3DNURBSCONTROLPOINT_H
#define K3DNURBSCONTROLPOINT_H

// K-3D
// Copyright  1995 - 2002, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the k3dNurbsControlPoint class, which implements a geometric nurbs point
		\author Romain Behar with NURBS students (romainbehar@yahoo.com)
*/

#include <k3deditable.h>
#include <k3dstate.h>

#include <k3dinurbscontrolpoint.h>
#include <k3dilocation.h>

// Forward class declarations
class k3dNurbsMesh;

class k3dNurbsControlPoint :
	public k3dINurbsControlPoint,
	public k3dILocation,
	public k3dEditable,
	public k3dState
{
public:
	k3dNurbsControlPoint(k3dNurbsMesh* const Mesh, const k3dINurbsControlPoint::ID PointID);
	virtual ~k3dNurbsControlPoint();

	// k3dINurbsControlPoint implementation
	k3dIState* State();
	k3dILocation* Location();
	k3dINurbsMesh* Mesh();
	const k3dINurbsControlPoint::ID PointID();

	// k3dILocation implementation
//	k3dIState* State();	Already declared above !
	const sdpVector3& LocalXYZ();
	const sdpVector3& AnimXYZ();
	const sdpVector3& WorldXYZ();
	const sdpVector3& CameraXYZ();
	const sdpVector3& ScreenXYZ();
	const sdpVector3& NormalXYZ();
	const sdpVector3& TextureXYZ();
	const sdpVector3& ImplicitTextureXYZ();
	const gdouble Weight();

	void SetLocalXYZ(const sdpVector3& Coords);
	void SetAnimXYZ(const sdpVector3& Coords);
	void SetWorldXYZ(const sdpVector3& Coords);
	void SetCameraXYZ(const sdpVector3& Coords);
	void SetScreenXYZ(const sdpVector3& Coords);
	void SetNormalXYZ(const sdpVector3& Coords);
	void SetTextureXYZ(const sdpVector3& Coords);
	void SetImplicitTextureXYZ(const sdpVector3& Coords);
	void SetWeight(const gdouble PtWeight);

	// k3dIEditable implementation
	void PushEditing();
	void PopEditing();

	// k3dIState implementation
	sdpResult SetState(const gulong Mask, const bool State);

	// Helper functions
	sdpResult Save(sdpxml::Element& Element, k3dIDependencies* Dependencies);
	sdpResult Load(sdpxml::Element& Element);

	/// Stores the parent nurbs mesh
	k3dNurbsMesh* const m_Mesh;
	/// Stores a unique (within the parent nurbs mesh) ID
	k3dINurbsControlPoint::ID m_PointID;
	/// Set to true iff the position of the point has been changed
	bool m_LocalXYZChanged;
	/// Set to true iff the weight of the point has been changed
	//bool m_WeightChanged;
	/// Set to true iff the point texture has been modified
	bool m_TextureXYZChanged;

	/// Stores current local coordinates
	sdpVector3 m_LocalXYZ;
	/// Stores current local weight
	gdouble m_Weight;
	/// Stores animated local coordinates
	sdpVector3 m_AnimXYZ;
	/// Stores current world coordinates (calculated only on-demand)
	sdpVector3 m_WorldXYZ;
	/// Stores current camera coordinates (calculated only on-demand)
	sdpVector3 m_CameraXYZ;
	/// Stores current screen coordinates (calculated only on-demand)
	sdpVector3 m_ScreenXYZ;
	/// Stores current normal coordinates (currently unused)
	sdpVector3 m_NormalXYZ;
	/// Stores current texture coordinates, for subdivision meshes
	sdpVector3 m_TextureXYZ;
	/// Stores implicit texture coordinates, for subdivision meshes
	sdpVector3 m_ImplicitTextureXYZ;
};

#endif // K3DNURBSCONTROLPOINT_H

