#ifndef K3DNURBSPATHPOINT_H
#define K3DNURBSPATHPOINT_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the k3dNurbsPathPoint class, which encapsulates the many-to-many relationship between k3dINurbsControlPoint and k3dINurbsPath/k3dICurve
		\author Romain Behar with NURBS students (romainbehar@yahoo.com)
*/

#include <k3deditable.h>

#include <k3dinurbspathpoint.h>
#include <k3dilocation.h>

// Forward class references ...
class k3dIDependencies;
class k3dINurbsPath;
class k3dICurve;
class k3dNurbsPath;
class k3dNURBSCurve;
class k3dNurbsControlPoint;
class k3dNurbsMesh;

namespace sdpxml { class Element; }

/////////////////////////////////////////////////////////////////////////////
// k3dNurbsPathPoint

class k3dNurbsPathPoint :
	public k3dINurbsPathPoint,
	public k3dILocation,
	public k3dEditable
{
public:
	k3dNurbsPathPoint();
	k3dNurbsPathPoint(k3dNurbsPath* const Path);
	k3dNurbsPathPoint(k3dNurbsControlPoint* const Point, const k3dINurbsPathPoint::ID PathPointID);
	k3dNurbsPathPoint(k3dNurbsPath* const Path, k3dNurbsControlPoint* const Point, const k3dINurbsPathPoint::ID PathPointID);
	virtual ~k3dNurbsPathPoint();

	k3dNurbsPath* Path() { return m_Path; }

	// k3dINurbsPathPoint implementation
	k3dINurbsControlPoint* Point();
	k3dIState* State();
	k3dILocation* Location();
	const k3dINurbsPathPoint::ID PathPointID();

	// k3dILocation implementation
//	k3dIState* State();	Already declared above ... the compiler will sort it out!
	const sdpVector3& LocalXYZ();
	const sdpVector3& AnimXYZ();
	const sdpVector3& WorldXYZ();
	const sdpVector3& CameraXYZ();
	const sdpVector3& ScreenXYZ();
	const sdpVector3& NormalXYZ();
	const sdpVector3& TextureXYZ();
	const sdpVector3& ImplicitTextureXYZ();
	void SetLocalXYZ(const sdpVector3& Coords);
	void SetAnimXYZ(const sdpVector3& Coords);
	void SetWorldXYZ(const sdpVector3& Coords);
	void SetCameraXYZ(const sdpVector3& Coords);
	void SetScreenXYZ(const sdpVector3& Coords);
	void SetNormalXYZ(const sdpVector3& Coords);
	void SetTextureXYZ(const sdpVector3& Coords);
	void SetImplicitTextureXYZ(const sdpVector3& Coords);

	// k3dIEditable implementation
	void PushEditing();
	void PopEditing();

	// Helper functions ...
	sdpResult Save(sdpxml::Element& Element, k3dIDependencies* Dependencies);
	sdpResult Load(sdpxml::Element& Element);
	sdpResult LoadComplete(k3dNurbsMesh* Mesh);

	k3dNurbsPath* m_Path;
	k3dNurbsControlPoint* m_Point;
	k3dINurbsPathPoint::ID m_PathPointID;
	
	/// Set to true iff the texture coordinates have been modified
	bool m_TextureXYZChanged;
	
	sdpVector3 m_NormalXYZ;
	sdpVector3 m_TextureXYZ;
	sdpVector3 m_ImplicitTextureXYZ;
};

#endif // POLYPOINT_H

