#ifndef K3DOBJECT_H
#define K3DOBJECT_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares k3dObject, a default implementation of the k3dIObject interface, for use as a base class for K-3D objects
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3dcommandnode.h>
#include <k3deventobserver.h>
#include <k3deventsource.h>
#include <k3dstate.h>
#include <k3dvariablecontainer.h>

#include <k3diobject.h>
#include <k3didocument.h>

// Forward class declarations ...
class k3dObject;
class k3dIApplication;
class k3dIDocument;
class k3dIHierarchy;

/// A default implementation of the k3dIObject interface, for use as a base class for K-3D objects
class k3dObject :
	public k3dIObject,
	public sdpIDeletable,
	public k3dCommandNode,
	public k3dEventObserver,
	public k3dEventSource,
	public k3dState,
	public k3dVariableContainer
{
public:
	k3dObject(sdpIUnknown* Initializer);
	virtual ~k3dObject();

	// k3dIObject implementation ...
	sdpResult InitComplete();
	const ID ObjectID();
	void SetObjectID(const ID ObjectID);
	SDPCSTRING Name();
	sdpResult SetName(SDPCSTRING Name);
	k3dIState* State();
	k3dIDocument* Document() { return m_Document; }
	k3dIObjectInfo* ObjectInfo() { return m_ObjectInfo; }
	
	sdpResult EditObject();
	sdpResult ObjectDeleted();
	sdpResult ObjectUnDeleted();

	// k3dIState implementation ...
	sdpResult SetState(const gulong Mask, const bool State);

	/// Returns the object's owning hierarchy
	k3dIHierarchy* Hierarchy() { return m_Hierarchy; }

	/// An event for transmitting human-readable messages to properties dialogs
	static const sdpString Event_UserMessage1;

protected:
	/// Stores the name for this object
	sdpString m_Name;

private:
	/// Stores the ID for this object
	ID m_ObjectID;
	/// Stores the Document for this object
	k3dIDocument* m_Document;
	/// Stores the Object Info for this object
	k3dIObjectInfo* m_ObjectInfo;
	/// Stores the Hierarchy for this object
	k3dIHierarchy* m_Hierarchy;
};

#endif // K3DOBJECT_H


