#ifndef K3DOBJECTDIALOG_H
#define K3DOBJECTDIALOG_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the k3dObjectDialog class, which provides a standard dialog implementation for K-3D objects
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3ddialog.h>
#include <k3dichannel.h>

// Forward class declarations ...
class k3dObject;

///////////////////////////////////////////////////////////////////////////
// k3dObjectDialog

typedef k3dDialog k3dObjectDialogBase;

class k3dObjectDialog :
	public k3dObjectDialogBase
{
public:
	k3dObjectDialog(k3dObject* const Object, const bool GrabMouseFocus, const bool AutoDestroy, const bool ShowObjectName, SDPCSTRING CommandNodeName, k3dWindowGeometryStore* const WindowGeometryStore);
	virtual ~k3dObjectDialog();

	/// Standard UI shutdown
	void CloseDialog();

	/// Returns the owning object
	k3dObject* Object() const
	{
		return m_Object;
	}

	// k3dICommandNode overrides
	sdpResult Command(SDPCSTRING Name, SDPCSTRING Arguments, sdpIUnknown** Result);

	// Events that most dialog-owners need to call ...
	virtual void ObjectRenamed(SDPCSTRING NewName);
	virtual void ObjectDeleted();

	// k3dIEventObserver implementation
	sdpResult ObserveEvent(k3dIEventSource* const Source, SDPCSTRING Event, SDPCSTRING EventData, SDPCSTRING ObserverData);

protected:
	/// Called to load a GTKML template file (calculates the path to the file and connects to some standard events)
	virtual bool LoadGTKMLTemplate(SDPCSTRING Subdirectory, SDPCSTRING Template);

	// Built-in events that we need to handle ...
	virtual void OnOK();
	virtual void OnCancel();
	virtual void OnClose();

	/// Implements grabbing the current mouse focus for our object
	void GrabMouseFocus();

private:
	// Internal user actions
	static const sdpString control_setfocus;
	static const sdpString control_ok;
	static const sdpString control_cancel;
	static const sdpString control_close;

	/// Stores the object that owns us
	k3dObject* const m_Object;
	/// Stores whether the owning object will be destroyed when this dialog is closed (typical for tools)
	bool m_AutoDestroy;
	/// Stores whether this dialog should display the owning object's name in its titlebar
	const bool m_ShowObjectName;
};

#endif // K3DOBJECTDIALOG_H

