// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Implements helper functions for accessing user options
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3doptions.h>
#include <k3diapplication.h>

#ifndef SDPXML_H
#include <sdpxml.h>
#endif // !SDPXML_H

////////////////////////////////////////////////////////////////////////////////////
// Preferred Language

static sdpxml::Element& PreferredLanguageElement(sdpxml::Document& Options)
{
	return sdpxml::SafeXPathSelect(Options, Options, "application/userinterface/variables/variable[@name=\"preferredlanguage\"]", sdpxml::Element("variable", "", sdpxml::Attribute("name", "preferredlanguage"), sdpxml::Attribute("value", "en")));
}

sdpString k3dOptionsGetPreferredLanguage(sdpxml::Document& Options)
{
	return sdpxml::GetAttribute<sdpString>(PreferredLanguageElement(Options), "value", "");
}

void k3dOptionsSetPreferredLanguage(sdpxml::Document& Options, const sdpString Language)
{
	sdpxml::SetAttribute(PreferredLanguageElement(Options), sdpxml::Attribute("value", Language));
}

////////////////////////////////////////////////////////////////////////////////////
// Tip-of-the-Day

static sdpxml::Element& TipOfTheDayElement(sdpxml::Document& Options)
{
	return sdpxml::SafeXPathSelect(Options, Options, "application/userinterface/variables/variable[@name=\"showtipoftheday\"]", sdpxml::Element("variable", "", sdpxml::Attribute("name", "showtipoftheday"), sdpxml::Attribute("value", "true")));
}

bool k3dOptionsGetTipOfTheDay(sdpxml::Document& Options)
{
	return sdpxml::GetAttribute<bool>(TipOfTheDayElement(Options), "value", false);
}

void k3dOptionsSetTipOfTheDay(sdpxml::Document& Options, const bool State)
{
	sdpxml::SetAttribute(TipOfTheDayElement(Options), sdpxml::Attribute("value", sdpToString(State)));
}

////////////////////////////////////////////////////////////////////////////////////
// Show Tutorial Menu

static sdpxml::Element& TutorialMenuElement(sdpxml::Document& Options)
{
	return sdpxml::SafeXPathSelect(Options, Options, "application/userinterface/variables/variable[@name=\"showtutorialmenu\"]", sdpxml::Element("variable", "", sdpxml::Attribute("name", "showtutorialmenu"), sdpxml::Attribute("value", "true")));
}

bool k3dOptionsGetTutorialMenu(sdpxml::Document& Options)
{
	return sdpxml::GetAttribute<bool>(TutorialMenuElement(Options), "value", false);
}

void k3dOptionsSetTutorialMenu(sdpxml::Document& Options, const bool State)
{
	sdpxml::SetAttribute(TutorialMenuElement(Options), sdpxml::Attribute("value", sdpToString(State)));
}

////////////////////////////////////////////////////////////////////////////////////
// Tutorial Speed

static sdpxml::Element& TutorialSpeedElement(sdpxml::Document& Options)
{
	return sdpxml::SafeXPathSelect(Options, Options, "application/userinterface/variables/variable[@name=\"tutorialspeed\"]", sdpxml::Element("variable", "", sdpxml::Attribute("name", "tutorialspeed"), sdpxml::Attribute("value", "1")));
}

gdouble k3dOptionsGetTutorialSpeed(sdpxml::Document& Options)
{
	return sdpxml::GetAttribute<gdouble>(TutorialSpeedElement(Options), "value", 1.0);
}

void k3dOptionsSetTutorialSpeed(sdpxml::Document& Options, const gdouble Speed)
{
	sdpxml::SetAttribute(TutorialSpeedElement(Options), sdpxml::Attribute("value", sdpToString(Speed)));
}

////////////////////////////////////////////////////////////////////////////////////
// Restore Windows

static sdpxml::Element& RestoreWindowsElement(sdpxml::Document& Options)
{
	return sdpxml::SafeXPathSelect(Options, Options, "application/userinterface/variables/variable[@name=\"restorewindows\"]", sdpxml::Element("variable", "", sdpxml::Attribute("name", "restorewindows"), sdpxml::Attribute("value", "true")));
}

bool k3dOptionsGetRestoreWindows(sdpxml::Document& Options)
{
	return sdpxml::GetAttribute(RestoreWindowsElement(Options), "value", false);
}

void k3dOptionsSetRestoreWindows(sdpxml::Document& Options, const bool State)
{
	sdpxml::SetAttribute(RestoreWindowsElement(Options), sdpxml::Attribute("value", sdpToString(State)));
}

////////////////////////////////////////////////////////////////////////////////////
// HTML viewer

static sdpxml::Element& HTMLViewerElement(sdpxml::Document& Options)
{
#ifdef SDPWIN32
	const sdpString htmlviewercommand = "mozilla %p";// What's a sensible default for Windoze?
#else // SDPWIN32
	sdpString htmlviewercommand = sdpGetEnv("BROWSER");
	if(htmlviewercommand.size())
		{
			htmlviewercommand += " %p";
		}
	else
		{
			htmlviewercommand = "mozilla %p";
		}
#endif // !SDPWIN32

	return sdpxml::SafeXPathSelect(Options, Options, "application/helpers/command[@type=\"htmlviewer\"]", sdpxml::Element("command", htmlviewercommand, sdpxml::Attribute("type", "htmlviewer")));
}

sdpString k3dOptionsGetHTMLViewer(sdpxml::Document& Options)
{
	return HTMLViewerElement(Options).Text();
}

void k3dOptionsSetHTMLViewer(sdpxml::Document& Options, const sdpString CommandLine)
{
	HTMLViewerElement(Options).Text() = CommandLine;
}

////////////////////////////////////////////////////////////////////////////////////
// Bitmap viewer

static sdpxml::Element& BitmapViewerElement(sdpxml::Document& Options)
{
#ifdef SDPWIN32
	const sdpString bitmapviewercommand = "";// What's a sensible default for Windoze?
#else // SDPWIN32
	const sdpString bitmapviewercommand = "xview %p";
#endif // !SDPWIN32

	return sdpxml::SafeXPathSelect(Options, Options, "application/helpers/command[@type=\"bitmapviewer\"]", sdpxml::Element("command", bitmapviewercommand, sdpxml::Attribute("type", "bitmapviewer")));
}

sdpString k3dOptionsGetBitmapViewer(sdpxml::Document& Options)
{
	return BitmapViewerElement(Options).Text();
}

void k3dOptionsSetBitmapViewer(sdpxml::Document& Options, const sdpString CommandLine)
{
	BitmapViewerElement(Options).Text() = CommandLine;
}

////////////////////////////////////////////////////////////////////////////////////
// MPEG viewer

static sdpxml::Element& MPEGViewerElement(sdpxml::Document& Options)
{
#ifdef SDPWIN32
	const sdpString mpegviewercommand = "mplayer2.exe %p";// What's a sensible default for Windoze?
#else // SDPWIN32
	const sdpString mpegviewercommand = "plaympeg %p";
#endif // !SDPWIN32

	return sdpxml::SafeXPathSelect(Options, Options, "application/helpers/command[@type=\"mpegviewer\"]", sdpxml::Element("command", mpegviewercommand, sdpxml::Attribute("type", "mpegviewer")));
}

sdpString k3dOptionsGetMPEGViewer(sdpxml::Document& Options)
{
	return MPEGViewerElement(Options).Text();
}

void k3dOptionsSetMPEGViewer(sdpxml::Document& Options, const sdpString CommandLine)
{
	MPEGViewerElement(Options).Text() = CommandLine;
}

////////////////////////////////////////////////////////////////////////////////////
// Template document

static sdpxml::Element& TemplateDocumentElement(sdpxml::Document& Options)
{
	return sdpxml::SafeXPathSelect(Options, Options, "application/userinterface/variables/variable[@name=\"templatedocument\"]", sdpxml::Element("variable", "", sdpxml::Attribute("name", "templatedocument")));
}

sdpString k3dOptionsGetTemplateDocument(sdpxml::Document& Options)
{
	return TemplateDocumentElement(Options).Text();
}

void k3dOptionsSetTemplateDocument(sdpxml::Document& Options, const sdpString Document)
{
	TemplateDocumentElement(Options).Text() = Document;
}

////////////////////////////////////////////////////////////////////////////////////
// Shading Language preprocessor

static sdpxml::Element& SLPreprocessorElement(sdpxml::Document& Options)
{
#ifdef SDPWIN32
	const sdpString slpreprocessorcommand = "cpp %p";
#else // SDPWIN32
	const sdpString slpreprocessorcommand = "gcc -E -P -x c %p";
#endif // !SDPWIN32

	return sdpxml::SafeXPathSelect(Options, Options, "application/helpers/command[@type=\"slpreprocessor\"]", sdpxml::Element("command", slpreprocessorcommand, sdpxml::Attribute("type", "slpreprocessor")));
}

sdpString k3dOptionsGetSLPreprocessor(sdpxml::Document& Options)
{
	return SLPreprocessorElement(Options).Text();
}

void k3dOptionsSetSLPreprocessor(sdpxml::Document& Options, const sdpString CommandLine)
{
	SLPreprocessorElement(Options).Text() = CommandLine;
}

////////////////////////////////////////////////////////////////////////////////////
// Shading Language parser

static sdpxml::Element& SLParserElement(sdpxml::Document& Options)
{
#ifdef SDPWIN32
	const sdpString slparsercommand = "sl2xml";
#else // SDPWIN32
	const sdpString slparsercommand = "sl2xml";
#endif // !SDPWIN32

	return sdpxml::SafeXPathSelect(Options, Options, "application/helpers/command[@type=\"slparser\"]", sdpxml::Element("command", slparsercommand, sdpxml::Attribute("type", "slparser")));
}

sdpString k3dOptionsGetSLParser(sdpxml::Document& Options)
{
	return SLParserElement(Options).Text();
}

void k3dOptionsSetSLParser(sdpxml::Document& Options, const sdpString CommandLine)
{
	SLParserElement(Options).Text() = CommandLine;
}

////////////////////////////////////////////////////////////////////////////////////
// Default render engine

static sdpxml::Element& RenderEnginesElement(sdpxml::Document& Options)
{
	return sdpxml::SafeXPathSelect(Options, Options, "application/renderengines", sdpxml::Element("renderengines", "", sdpxml::Attribute("defaulttype", "ri"), sdpxml::Attribute("defaultengine", "aqsis")));
}

void k3dOptionsGetDefaultRenderEngine(sdpxml::Document& Options, sdpString& Type, sdpString& Engine, sdpString& Name)
{
	sdpxml::Element& element = RenderEnginesElement(Options);
	
	sdpxml::ParseAttribute(element, "defaulttype", Type);
	sdpxml::ParseAttribute(element, "defaultengine", Engine);
	
	k3dOptionsRenderEngines engines;
	k3dOptionsGetRenderEngines(Options, engines);
	for(k3dOptionsRenderEngines::iterator engine = engines.begin(); engine != engines.end(); engine++)
		{
			// Look for the default engine's name ...
			if((Type == engine->Type()) && (Engine == engine->Engine()))
				{
					Name = engine->Name();		
					return;
				}
		}
}

void k3dOptionsSetDefaultRenderEngine(sdpxml::Document& Options, const sdpString Type, const sdpString Engine)
{
	sdpxml::Element& element = RenderEnginesElement(Options);
	sdpxml::SetAttribute(element, sdpxml::Attribute("defaulttype", Type));
	sdpxml::SetAttribute(element, sdpxml::Attribute("defaultengine", Engine));
}

/////////////////////////////////////////////////////////////////////////////////////
// Render farm path

static sdpxml::Element& RenderFarmPathElement(sdpxml::Document& Options)
{
	return sdpxml::SafeXPathSelect(Options, Options, "application/paths/path[@type=\"renderfarm\"]", sdpxml::Element("path", sdpGetTempDirectory(), sdpxml::Attribute("type", "renderfarm")));
}

sdpString k3dOptionsGetRenderFarmPath(sdpxml::Document& Options)
{
	return RenderFarmPathElement(Options).Text();
}

void k3dOptionsSetRenderFarmPath(sdpxml::Document& Options, const sdpString Path)
{
	RenderFarmPathElement(Options).Text() = Path;
}

/////////////////////////////////////////////////////////////////////////////////////
// Render engines

static sdpxml::Element& GetRenderEngine(sdpxml::Document& Options, const sdpString Type, const sdpString Engine)
{
	return sdpxml::SafeXPathSelect(Options, RenderEnginesElement(Options), "renderengine[@type=\"" + Type + "\" and @engine=\"" + Engine + "\"]", sdpxml::Element("renderengine", "", sdpxml::Attribute("type", Type), sdpxml::Attribute("engine", Engine)));
}

static void CreateRenderEngine(sdpxml::Document& Options, const sdpString Type, const sdpString Engine, const sdpString Name, const sdpString Render, const sdpString VisibleRender, const sdpString ShaderBinary, const sdpString CompileShaderCommand)
{
	// Sanity checks ...
	g_assert(Type.size());
	g_assert(Engine.size());
	g_assert(Name.size());

	sdpxml::Element& renderengine = GetRenderEngine(Options, Type, Engine);
	sdpxml::SetAttribute(renderengine, sdpxml::Attribute("name", Name));
	
	if(Render.size())
		sdpxml::SafeXPathSelect(Options, renderengine, "render", sdpxml::Element("render", Render));
		
	if(VisibleRender.size())
		sdpxml::SafeXPathSelect(Options, renderengine, "visiblerender", sdpxml::Element("visiblerender", VisibleRender));
		
	if(ShaderBinary.size() && CompileShaderCommand.size())
		sdpxml::SafeXPathSelect(Options, renderengine, "shaders/shader[@shaderbinary=\"" + ShaderBinary + "\"]", sdpxml::Element("shader", CompileShaderCommand, sdpxml::Attribute("shaderbinary", ShaderBinary)));
}

void k3dOptionsGetRenderEngines(sdpxml::Document& Options, k3dOptionsRenderEngines& Engines)
{
	// Make sure we have some default engines ...
	CreateRenderEngine(Options, "ri", "aqsis", "Aqsis", "aqsis %p", "aqsis %p", "%p.slx", "aqsl %p");
	CreateRenderEngine(Options, "ri", "bmrt", "BMRT", "rendrib %p", "rendrib -d 8 %p", "%p.slc", "slc %p");
	CreateRenderEngine(Options, "ri", "prman", "PRman", "prman -progress %p", "prman -progress %p", "%p.slo", "slcomp %p");
	CreateRenderEngine(Options, "ri", "netprman", "NETPRman", "netrender -progress -f %p", "netrender -progress -f %p", "%p.slo", "slcomp %p");
	CreateRenderEngine(Options, "ri", "3delight", "3Delight", "renderdl %p", "renderdl %p", "%p.sdl", "shaderdl --dont-keep-cpp-file --dont-keep-c++-file %p");
	CreateRenderEngine(Options, "ri", "rdc", "Render Dot C", "renderdc %p", "renderdc %p", "%p.so", "shaderdc %p");
	CreateRenderEngine(Options, "ri", "editrib", "Edit RIB File", "gvim %p", "", "", "");

	sdpxml::ElementPointerCollection engines = sdpxml::XPathSelect(Options, RenderEnginesElement(Options), "renderengine");
	for(sdpxml::ElementPointerCollection::const_iterator engine = engines.begin(); engine != engines.end(); engine++)
		{
			const sdpString type = sdpxml::GetAttribute<sdpString>(**engine, "type", "");
			if(!type.size())
				{
					std::cerr << "k3dOptionsGetRenderEngines(): empty engine type!" << std::endl;
					continue;
				}
			
			const sdpString enginename = sdpxml::GetAttribute<sdpString>(**engine, "engine", "");
			if(!enginename.size())
				{
					std::cerr << "k3dOptionsGetRenderEngines(): empty engine!" << std::endl;
					continue;
				}
			
			const sdpString name = sdpxml::GetAttribute<sdpString>(**engine, "name", "");
			if(!name.size())
				{
					std::cerr << "k3dOptionsGetRenderEngines(): empty engine name" << std::endl;
					continue;
				}
			
			const sdpString render = sdpxml::SafeXPathSelect(Options, **engine, "render", sdpxml::Element("render", "")).Text();
			const sdpString visiblerender = sdpxml::SafeXPathSelect(Options, **engine, "visiblerender", sdpxml::Element("visiblerender", "")).Text();
			
			sdpxml::Element& shader = sdpxml::SafeXPathSelect(Options, **engine, "shaders/shader", sdpxml::Element("shader"));
			const sdpString shaderbinary = sdpxml::GetAttribute<sdpString>(shader, "shaderbinary", "");
			const sdpString compileshadercommand = shader.Text();
			
			Engines.push_back(k3dOptionsRenderEngine(type, enginename, name, render, visiblerender, shaderbinary, compileshadercommand));
		}
}


