#ifndef K3DOPTIONS_H
#define K3DOPTIONS_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares helper functions for accessing user options
		\author Tim Shead (tshead@k-3d.com)
*/

#ifndef SDPTYPES_H
#include <sdptypes.h>
#endif // !SDPTYPES_H

// Forward class declarations
namespace sdpxml { class Document; }

/// Returns the user's preferred language
sdpString k3dOptionsGetPreferredLanguage(sdpxml::Document& Options);
/// Sets whether the Tip-of-the-Day dialog should be displayed at startup
void k3dOptionsSetPreferredLanguage(sdpxml::Document& Options, const sdpString Language);

/// Returns whether the Tip-of-the-Day dialog should be displayed at startup
bool k3dOptionsGetTipOfTheDay(sdpxml::Document& Options);
/// Sets whether the Tip-of-the-Day dialog should be displayed at startup
void k3dOptionsSetTipOfTheDay(sdpxml::Document& Options, const bool State);

/// Returns whether the Tutorials menu should be displayed at startup
bool k3dOptionsGetTutorialMenu(sdpxml::Document& Options);
/// Sets whether the Tutorials menu should be displayed at startup
void k3dOptionsSetTutorialMenu(sdpxml::Document& Options, const bool State);

/// Returns the tutorial speed
gdouble k3dOptionsGetTutorialSpeed(sdpxml::Document& Options);
/// Sets the tutorial speed
void k3dOptionsSetTutorialSpeed(sdpxml::Document& Options, const gdouble Speed);

/// Returns whether a window's size & position should be restored when opened
bool k3dOptionsGetRestoreWindows(sdpxml::Document& Options);
/// Sets whether a window's size & position should be restored when opened
void k3dOptionsSetRestoreWindows(sdpxml::Document& Options, const bool State);

/// Returns the user's preferred HTML viewer command line
sdpString k3dOptionsGetHTMLViewer(sdpxml::Document& Options);
/// Sets the user's preferred HTML viewer command line
void k3dOptionsSetHTMLViewer(sdpxml::Document& Options, const sdpString CommandLine);

/// Returns the user's preferred bitmap viewer command line
sdpString k3dOptionsGetBitmapViewer(sdpxml::Document& Options);
/// Sets the user's preferred bitmap viewer command line
void k3dOptionsSetBitmapViewer(sdpxml::Document& Options, const sdpString CommandLine);

/// Returns the user's preferred MPEG viewer command line
sdpString k3dOptionsGetMPEGViewer(sdpxml::Document& Options);
/// Sets the user's preferred MPEG viewer command line
void k3dOptionsSetMPEGViewer(sdpxml::Document& Options, const sdpString CommandLine);

/// Returns the user's preferred template document
sdpString k3dOptionsGetTemplateDocument(sdpxml::Document& Options);
/// Sets the user's preferred template document
void k3dOptionsSetTemplateDocument(sdpxml::Document& Options, const sdpString Document);

/// Returns the user's preferred shading language pre-processor command line
sdpString k3dOptionsGetSLPreprocessor(sdpxml::Document& Options);
/// Sets the user's preferred shading language pre-processor command line
void k3dOptionsSetSLPreprocessor(sdpxml::Document& Options, const sdpString CommandLine);

/// Returns the user's preferred shading language parser command line
sdpString k3dOptionsGetSLParser(sdpxml::Document& Options);
/// Sets the user's preferred shading language parser command line
void k3dOptionsSetSLParser(sdpxml::Document& Options, const sdpString CommandLine);

/// Returns the user's preferred default render engine
void k3dOptionsGetDefaultRenderEngine(sdpxml::Document& Options, sdpString& Type, sdpString& Engine, sdpString& Name);
/// Sets the user's preferred default render engine
void k3dOptionsSetDefaultRenderEngine(sdpxml::Document& Options, const sdpString Type, const sdpString Engine);

/// Returns the path to the render farm's shared filesystem
sdpString k3dOptionsGetRenderFarmPath(sdpxml::Document& Options);
/// Sets the path to the render farm's shared filesystem
void k3dOptionsSetRenderFarmPath(sdpxml::Document& Options, const sdpString Path);

/// Used to store/retrieve data about a render engine
class k3dOptionsRenderEngine
{
public:
	k3dOptionsRenderEngine(const sdpString Type, const sdpString Engine, const sdpString Name, const sdpString Render, const sdpString VisibleRender, const sdpString ShaderBinary, const sdpString CompileShader) :
		m_Type(Type),
		m_Engine(Engine),
		m_Name(Name),
		m_Render(Render),
		m_VisibleRender(VisibleRender),
		m_ShaderBinary(ShaderBinary),
		m_CompileShader(CompileShader)
	{
		// Sanity checks ...
		g_assert(m_Type.size());
		g_assert(m_Engine.size());
		g_assert(m_Name.size());
	}
	
	const sdpString& Type() { return m_Type; }
	const sdpString& Engine() { return m_Engine; }
	const sdpString& Name() { return m_Name; }
	
	const sdpString& Render() { return m_Render; }
	const sdpString& VisibleRender() { return m_VisibleRender; }
	
	const sdpString& ShaderBinary() { return m_ShaderBinary; }
	const sdpString& CompileShader() { return m_CompileShader; }

protected:
	const sdpString m_Type;
	const sdpString m_Engine;
	const sdpString m_Name;
	
	const sdpString m_Render;
	const sdpString m_VisibleRender;
	
	const sdpString m_ShaderBinary;
	const sdpString m_CompileShader;
};

/// A collection of render engines
typedef std::list<k3dOptionsRenderEngine> k3dOptionsRenderEngines;

/// Returns the user's collection of configured render engines
void k3dOptionsGetRenderEngines(sdpxml::Document& Options, k3dOptionsRenderEngines& Engines);

#endif // !K3DOPTIONS_H

