#ifndef K3DPATHPOINT_H
#define K3DPATHPOINT_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the k3dPathPoint class, which encapsulates the many-to-many relationship between k3dIPoint and k3dIPath/k3dICurve
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3deditable.h>

#include <k3dipathpoint.h>
#include <k3dilocation.h>

// Forward class references ...
class k3dIDependencies;
class k3dIPath;
class k3dICurve;
class k3dPath;
class k3dNURBSCurve;
class k3dPoint;
class k3dMesh;

namespace sdpxml { class Element; }

/////////////////////////////////////////////////////////////////////////////
// k3dPathPoint

class k3dPathPoint :
	public k3dIPathPoint,
	public k3dILocation,
	public k3dEditable
{
public:
	k3dPathPoint();
	k3dPathPoint(k3dPath* const Path);
	k3dPathPoint(k3dPoint* const Point, const k3dIPathPoint::ID PathPointID);
	k3dPathPoint(k3dPath* const Path, k3dPoint* const Point, const k3dIPathPoint::ID PathPointID);
	virtual ~k3dPathPoint();

	k3dPath* Path() { return m_Path; }

	// k3dIPathPoint implementation
	k3dIPoint* Point();
	k3dIState* State();
	k3dILocation* Location();
	const k3dIPathPoint::ID PathPointID();

	// k3dILocation implementation
//	k3dIState* State();	Already declared above ... the compiler will sort it out!
	const sdpVector3& LocalXYZ();
	const sdpVector3& AnimXYZ();
	const sdpVector3& WorldXYZ();
	const sdpVector3& CameraXYZ();
	const sdpVector3& ScreenXYZ();
	const sdpVector3& NormalXYZ();
	const sdpVector3& TextureXYZ();
	const sdpVector3& ImplicitTextureXYZ();
	void SetLocalXYZ(const sdpVector3& Coords);
	void SetAnimXYZ(const sdpVector3& Coords);
	void SetWorldXYZ(const sdpVector3& Coords);
	void SetCameraXYZ(const sdpVector3& Coords);
	void SetScreenXYZ(const sdpVector3& Coords);
	void SetNormalXYZ(const sdpVector3& Coords);
	void SetTextureXYZ(const sdpVector3& Coords);
	void SetImplicitTextureXYZ(const sdpVector3& Coords);

	// k3dIEditable implementation
	void PushEditing();
	void PopEditing();

	// Helper functions ...
	sdpResult Save(sdpxml::Element& Element, k3dIDependencies* Dependencies);
	sdpResult Load(sdpxml::Element& Element);
	sdpResult LoadComplete(k3dMesh* Mesh);

	k3dPath* m_Path;
	k3dPoint* m_Point;
	k3dIPathPoint::ID m_PathPointID;
	
	/// Set to true iff the texture coordinates have been modified
	bool m_TextureXYZChanged;
	
	sdpVector3 m_NormalXYZ;
	sdpVector3 m_TextureXYZ;
	sdpVector3 m_ImplicitTextureXYZ;
};

#endif // POLYPOINT_H

