
// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Implements the k3dPersistObject class, which encapsulates a K-3D object capable of automatically serializing itself
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3dpersistobject.h>
#include <k3dvariable.h>
#include <k3dmisc.h>

#include <sdpxml.h>

//////////////////////////////////////////////////////////////////////////////////
// k3dPersistObject

k3dPersistObject::k3dPersistObject(sdpIUnknown* Initializer) :
	k3dPersistObjectBase(Initializer)
{
	RawSetState(k3dIState::SAVEABLE, true);
}


k3dPersistObject::~k3dPersistObject()
{
}

///////////////////////////////////////////////////////////////////////////////////
// k3dIPersist implementation

sdpResult k3dPersistObject::InitNew()
{
	g_return_val_if_fail(sdpSucceeded(k3dVariableContainer::InitNew()), SDP_E_FAIL);
	return SDP_S_OK;
}

sdpResult k3dPersistObject::Load(sdpxml::Document& Document, sdpxml::Element& Element)
{
	// Load object name ...
	sdpxml::ParseAttribute(Element, "name", m_Name);

	// Load object state ...
	k3dState::Load(Element);

	// Maintain the command-node hierarchy ...
	SetCommandNodeName(Name());
	SetParentCommandNode(dynamic_cast<k3dICommandNode*>(k3dPersistObject::Document()));

	// Load Variables
	g_return_val_if_fail(sdpSucceeded(k3dVariableContainer::Load(Document, Element)), SDP_E_FAIL);

	return SDP_S_OK;
}

sdpResult k3dPersistObject::LoadComplete()
{
	g_return_val_if_fail(sdpSucceeded(k3dVariableContainer::LoadComplete()), SDP_E_FAIL);
	return SDP_S_OK;
}

sdpResult k3dPersistObject::Save(sdpxml::Element& Element, k3dIDependencies* Dependencies)
{
	// Sanity checks ...
	g_return_val_if_fail(Dependencies, SDP_E_FAIL);

	// Save object state ...
	g_return_val_if_fail(sdpSucceeded(k3dState::Save(Element, Dependencies)), SDP_E_FAIL);

	// Save variables
	g_return_val_if_fail(sdpSucceeded(k3dVariableContainer::Save(Element, Dependencies)), SDP_E_FAIL);

	return SDP_S_OK;
}

sdpResult k3dPersistObject::SaveComplete()
{
	g_return_val_if_fail(sdpSucceeded(k3dVariableContainer::SaveComplete()), SDP_E_FAIL);
	return SDP_S_OK;
}


