#ifndef K3DRIENGINE_H
#define K3DRIENGINE_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the k3dRiEngine class, which provides an implementation of k3dIRiEngine
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3diriengine.h>

// Forward class declarations ...
class k3dICamera;

class k3dRiEngine : public k3dIRiEngine
{
public:
	/// Keeps track of declarations
	typedef std::map<sdpString, sdpString> Declarations;
	typedef Declarations::iterator DeclarationIterator;

	k3dRiEngine();
	virtual ~k3dRiEngine();
	
	gulong RiAreaLightSourceV(SDPCSTRING Name, const gulong ParameterCount, SDPCSTRING Tokens[], gpointer Values[], gulong ValueCounts[]);
	sdpResult RiAtmosphereV(SDPCSTRING Name, const gulong ParameterCount, SDPCSTRING Tokens[], gpointer Values[], gulong ValueCounts[]);
	sdpResult RiAttributeBegin();
	sdpResult RiAttributeEnd();
	sdpResult RiAttributeV(SDPCSTRING Name, const gulong ParameterCount, SDPCSTRING Tokens[], gpointer Values[], gulong ValueCounts[]);
	sdpResult RiBasis(gdouble UBasis[16], const gulong UStep, gdouble VBasis[16], const gulong VStep);
	sdpResult RiBound(gdouble Bound[6]);
	sdpResult RiClipping(const gdouble Hither, const gdouble Yon);
	sdpResult RiColor(gdouble Color[3]);
	sdpResult RiColorSamples(const gulong ParameterCount, gdouble nRGB[], gdouble RGBn[]);
	sdpResult RiComment(SDPCSTRING Comment);
	sdpResult RiConcatTransform(gdouble Transform[16]);
	sdpResult RiConeV(const gdouble Height, const gdouble Radius, const gdouble ThetaMax, const gulong ParameterCount, SDPCSTRING Tokens[], gpointer Values[], gulong ValueCounts[]);
	sdpResult RiCoordSysTransform(SDPCSTRING Space);
	sdpResult RiCoordinateSystem(SDPCSTRING Space);
	sdpResult RiCropWindow(const gdouble XMin, const gdouble XMax, const gdouble YMin, const gdouble YMax);
	sdpResult RiCurvesV(SDPCSTRING Degree, const gulong ParameterCountCurves, const gulong ParameterCountVerts[], SDPCSTRING Wrap, const gulong ParameterCount, SDPCSTRING Tokens[], gpointer Values[], gulong ValueCounts[]);
	sdpResult RiCylinderV(const gdouble Radius, const gdouble ZMin, const gdouble ZMax, const gdouble ThetaMax, const gulong ParameterCount, SDPCSTRING Tokens[], gpointer Values[], gulong ValueCounts[]);
	sdpResult RiDeclare(SDPCSTRING Name, SDPCSTRING Type);
	sdpResult RiDeformationV(SDPCSTRING Name, const gulong ParameterCount, SDPCSTRING Tokens[], gpointer Values[], gulong ValueCounts[]);
	sdpResult RiDepthOfField(const gdouble FStop, const gdouble FocalLength, const gdouble FocalDistance);
	sdpResult RiDetail(gdouble Bound[6]);
	sdpResult RiDetailRange(const gdouble MinVis, const gdouble LowTran, const gdouble UpTran, const gdouble MaxVis);
	sdpResult RiDiskV(gdouble Height, gdouble Radius, gdouble ThetaMax, 	const gulong ParameterCount, SDPCSTRING Tokens[], gpointer Values[], gulong ValueCounts[]);
	sdpResult RiDisplacementV(SDPCSTRING Name, const gulong ParameterCount, SDPCSTRING Tokens[], gpointer Values[], gulong ValueCounts[]);
	sdpResult RiDisplayV(SDPCSTRING Name, SDPCSTRING Type, SDPCSTRING Mode, const gulong ParameterCount, SDPCSTRING Tokens[], gpointer Values[], gulong ValueCounts[]);
	sdpResult RiErrorAbort(const gint Code, const gint Severity, SDPCSTRING Message);
	sdpResult RiErrorIgnore(const gint Code, const gint Severity, SDPCSTRING Message);
	sdpResult RiErrorPrint(const gint Code, const gint Severity, SDPCSTRING Message);
	sdpResult RiExposure(const gdouble Gain, const gdouble Gamma);
	sdpResult RiExteriorV(SDPCSTRING Name, const gulong ParameterCount, SDPCSTRING Tokens[], gpointer Values[], gulong ValueCounts[]);
	sdpResult RiFormat(const gulong XResolution, const gulong YResolution, const gdouble AspectRatio);
	sdpResult RiFrameAspectRatio(const gdouble AspectRatio);
	sdpResult RiFrameBegin(const gulong FrameNumber);
	sdpResult RiFrameEnd();
	sdpResult RiGeneralPathV(const gulong LoopCount, const gulong VertexCounts[], const gulong ParameterCount, SDPCSTRING Tokens[], gpointer Values[], gulong ValueCounts[]);
	sdpResult RiGeometricApproximation(SDPCSTRING Type, const gdouble Value);
	sdpResult RiGeometricRepresentation(SDPCSTRING Type);
	sdpResult RiGeometryV(SDPCSTRING Type, const gulong ParameterCount, SDPCSTRING Tokens[], gpointer Values[], gulong ValueCounts[]);
	sdpResult RiHiderV(SDPCSTRING Type, const gulong ParameterCount, SDPCSTRING Tokens[], gpointer Values[], gulong ValueCounts[]);
	sdpResult RiHyperboloidV(gdouble Point1[3], gdouble Point2[3], const gdouble ThetaMax, const gulong ParameterCount, SDPCSTRING Tokens[], gpointer Values[], gulong ValueCounts[]);
	sdpResult RiIdentity();
	sdpResult RiIlluminate(const gulong LightHandle, const bool OnOff);
	sdpResult RiImagerV(SDPCSTRING Name, const gulong ParameterCount, SDPCSTRING Tokens[], gpointer Values[], gulong ValueCounts[]);
	sdpResult RiInteriorV(SDPCSTRING Name, const gulong ParameterCount, SDPCSTRING Tokens[], gpointer Values[], gulong ValueCounts[]);
	gulong RiLightSourceV(SDPCSTRING Name, const gulong ParameterCount, SDPCSTRING Tokens[], gpointer Values[], gulong ValueCounts[]);
	sdpResult RiMatte(const bool OnOff);
	sdpResult RiMotionBeginV(const gulong ParameterCount, gdouble Times[]);
	sdpResult RiMotionEnd();
	sdpResult RiNewline();
	sdpResult RiNuPatchV(const gulong UCount, const gulong UOrder, gdouble UKnot[], const gdouble UMin, const gdouble UMax, const gulong VCount, const gulong VOrder, gdouble VKnot[], const gdouble VMin, const gdouble VMax, const gulong ParameterCount, SDPCSTRING Tokens[], gpointer Values[], gulong ValueCounts[]);
	gulong RiObjectBegin();
	sdpResult RiObjectEnd();
	sdpResult RiObjectInstance(const gulong Object);
	sdpResult RiOpacity(gdouble Color[3]);
	sdpResult RiOptionV(SDPCSTRING Name, const gulong ParameterCount, SDPCSTRING Tokens[], gpointer Values[], gulong ValueCounts[]);
	sdpResult RiOrientation(SDPCSTRING Orientation);
	sdpResult RiParaboloidV(const gdouble RMax, const gdouble ZMin, const gdouble ZMax, const gdouble ThetaMax, const gulong ParameterCount, SDPCSTRING Tokens[], gpointer Values[], gulong ValueCounts[]);
	sdpResult RiPatchMeshV(SDPCSTRING Type, const gulong UCount, SDPCSTRING UWrap, const gulong VCount, SDPCSTRING VWrap, const gulong ParameterCount, SDPCSTRING Tokens[], gpointer Values[], gulong ValueCounts[]);
	sdpResult RiPatchV(SDPCSTRING Type, const gulong ParameterCount, SDPCSTRING Tokens[], gpointer Values[], gulong ValueCounts[]);
	sdpResult RiPerspective(const gdouble FieldOfView);
	sdpResult RiPixelFilter(SDPCSTRING FilterName, const gdouble XWidth, const gdouble YWidth);
	sdpResult RiPixelSamples(const gdouble XSamples, const gdouble YSamples);
	sdpResult RiPixelVariance(const gdouble Variation);
	sdpResult RiPointsGeneralPathsV(const gulong PathCount, gulong LoopCounts[], gulong VertexCounts[], gulong Vertices[], const gulong ParameterCount, SDPCSTRING Tokens[], gpointer Values[], gulong ValueCounts[]);
	sdpResult RiPointsPathsV(const gulong PathCount, gulong VertexCounts[], gulong Vertices[], const gulong ParameterCount, SDPCSTRING Tokens[], gpointer Values[], gulong ValueCounts[]);
	sdpResult RiPathV(const gulong VertexCount, const gulong ParameterCount, SDPCSTRING Tokens[], gpointer Values[], gulong ValueCounts[]);
	sdpResult RiProjectionV(SDPCSTRING Name, const gulong ParameterCount, SDPCSTRING Tokens[], gpointer Values[], gulong ValueCounts[]);
	sdpResult RiQuantize(SDPCSTRING Type, const gint One, const gint QMin, const gint QMax, const gdouble Amplitude);
	sdpResult RiReadArchive(SDPCSTRING Archive);
	sdpResult RiRelativeDetail(const gdouble RelativeDetail);
	sdpResult RiReverseOrientation();
	sdpResult RiRotate(const gdouble angle, const gdouble DX, const gdouble DY, const gdouble DZ);
	sdpResult RiScale(const gdouble DX, const gdouble DY, const gdouble DZ);
	sdpResult RiScreenWindow(const gdouble Left, const gdouble Right, const gdouble Bottom, const gdouble Top);
	sdpResult RiShadingInterpolation(SDPCSTRING Type);
	sdpResult RiShadingRate(const gdouble Size);
	sdpResult RiShutter(const gdouble SMin, const gdouble SMax);
	sdpResult RiSides(const gulong Sides);
	sdpResult RiSkew(const gdouble Angle, const gdouble DX1, const gdouble DY1, const gdouble DZ1, const gdouble DX2, const gdouble DY2, const gdouble DZ2);
	sdpResult RiSolidBegin(SDPCSTRING Type);
	sdpResult RiSolidEnd();
	sdpResult RiSphereV(const gdouble Radius, const gdouble ZMin, const gdouble ZMax, const gdouble ThetaMax, const gulong ParameterCount, SDPCSTRING Tokens[], gpointer Values[], gulong ValueCounts[]);
	sdpResult RiStructure(SDPCSTRING Structure);
	sdpResult RiSubdivisionMeshV(SDPCSTRING Scheme, const gulong PathCount, gulong VertexCounts[], gulong Vertices[], const gulong TagCount, SDPSTRING Tags[], gulong ArgCounts[], gulong IntegerArgs[], gdouble FloatArgs[], const gulong ParameterCount, SDPCSTRING Tokens[], gpointer Values[], gulong ValueCounts[]);
	sdpResult RiSurfaceV(SDPCSTRING Name, const gulong ParameterCount, SDPCSTRING Tokens[], gpointer Values[], gulong ValueCounts[]);
	sdpResult RiTextureCoordinates(const gdouble S1, const gdouble T1, const gdouble S2, const gdouble T2, const gdouble S3, const gdouble T3, const gdouble S4, const gdouble T4);
	sdpResult RiTorusV(const gdouble MajorRadius, const gdouble MinorRadius, const gdouble PhiMin,	const gdouble PhiMax, const gdouble ThetaMax, const gulong ParameterCount, SDPCSTRING Tokens[], gpointer Values[], gulong ValueCounts[]);
	sdpResult RiTransform(gdouble Transform[16]);
	sdpResult RiTransformBegin();
	sdpResult RiTransformEnd();
	sdpResult RiTranslate(const gdouble DX, const gdouble DY, const gdouble DZ);
	sdpResult RiTrimCurve(const gulong LoopCount, gulong CuveCounts[], gulong Order[], gdouble Knot[], gdouble AMin[], gdouble AMax[], const gulong ParameterCount, gdouble U[], gdouble V[], gdouble W[]);
	sdpResult RiWorldBegin();
	sdpResult RiWorldEnd();
		
protected:
	/// Redirects RIB output to the given stream
	sdpResult SetOutputStream(std::ostream* const Stream);

	/// Writes header data to the stream
	sdpResult WriteRIBHeader();

private:
	/// Stores the RIB output stream
	std::ostream* m_Stream;
	/// Stores the current object handle
	gulong m_ObjectHandle;
	/// Stores the current light handle
	gulong m_LightHandle;
	
	/// Stores declarations
	Declarations m_Declarations;
};

#endif // K3DRIENGINE_H
