// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Implements the k3dScriptEditor class, which provides an extensible UI for editing textfiles
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3dscripteditor.h>
#include <k3dmisc.h>
#include <k3dscripting.h>

#include <sdpgtkevents.h>

//////////////////////////////////////////////////////////////////////
// k3dScriptEditor

const sdpString k3dScriptEditor::control_play = "play";

k3dScriptEditor::k3dScriptEditor(sdpIUnknown* ParentCommandNode, SDPCSTRING CommandNodeName) :
	k3dScriptEditorBase(ParentCommandNode, CommandNodeName),
	m_Running(false)
{
}

k3dScriptEditor::~k3dScriptEditor()
{
}

sdpString k3dScriptEditor::GetTitlebarText()
{
	sdpString result = k3dScriptEditorBase::GetTitlebarText();
	
	if(m_Running)
		result += " [ running ]";
	
	return result;
}

void k3dScriptEditor::OnFileSaveAs()
{
	sdpString filepath;
	if(!k3dGetFilePath("script", "Save Script As:", true, m_FilePath, filepath))
		return;

	Save(filepath);
}

void k3dScriptEditor::OnEvent(sdpGtkEvent* Event)
{
	if(Event->Name() == control_play)
		OnPlay();
	else
		k3dScriptEditorBase::OnEvent(Event);
}

void k3dScriptEditor::OnPlay()
{
	m_Running = true;
	Widget(control_play).SetSensitive(false);
	UpdateTitlebar();

	std::strstream script;
	script << TextControl().GetText() << std::ends;
	
	k3d::ExecuteScript(script, GetTitle());
	
	m_Running = false;
	Widget(control_play).SetSensitive(true);
	UpdateTitlebar();
}


