
// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Implements convenience methods for working with scripts 
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3dcom.h>
#include <k3dmisc.h>
#include <k3dscripting.h>

#include <k3diapplication.h>
#include <k3discriptengine.h>

namespace k3d
{

bool ExecuteScript(std::istream& Script, const sdpString ScriptName)
{
	return ExecuteScript(Script, ScriptName, k3d::classid::JavaScriptEngine);
}

bool ExecuteScript(std::istream& Script, const sdpString ScriptName, const sdp::ClassID Engine)
{
	// Sanity checks ...
	g_return_val_if_fail(Script.good(), false);
	g_return_val_if_fail(ScriptName.size(), false);

	// Get the requested scripting engine ...
	k3dIScriptEngine* const engine = k3dApplication()->ScriptEngine(Engine);
	g_return_val_if_fail(engine, false);

	// Extract a script from the stream ...
	std::strstream script;
	Script.get(*script.rdbuf(), '\0');
	script << std::ends;
	
	// Run that bad-boy ...
	const sdpResult result = engine->Execute(script.str(), ScriptName);
	script.freeze(false);

	return SDP_S_OK == result ? true : false;
}

} // namespace k3d

