// K-3D
// Copyright  1995-2002, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Implements utility classes / templates / functions related to working with the current selection
		\author Timothy M. Shead (tshead@k-3d.com)
*/

#include <k3dselection.h>

namespace k3d
{

//////////////////////////////////////////////////////////////////////////////////
/// Dimensions

Dimensions::Dimensions() :
	m_Locations(),
	m_Min(DBL_MAX),
	m_Max(-DBL_MAX)
{
}

void Dimensions::Insert(const sdpVector3 Coords)
{
	m_Locations.push_back(Coords);
	m_Total += Coords;

	// Compare against our min/max coordinates ...
	if(Coords[0] < m_Min[0])
		m_Min[0] = Coords[0];
	if(Coords[0] > m_Max[0])
		m_Max[0] = Coords[0];
	if(Coords[1] < m_Min[1])
		m_Min[1] = Coords[1];
	if(Coords[1] > m_Max[1])
		m_Max[1] = Coords[1];
	if(Coords[2] < m_Min[2])
		m_Min[2] = Coords[2];
	if(Coords[2] > m_Max[2])
		m_Max[2] = Coords[2];
}

gulong Dimensions::LocationCount() const
{
	return m_Locations.size();
}

sdpVector3 Dimensions::Min() const
{
	g_assert(m_Locations.size());
	return m_Min;
}

sdpVector3 Dimensions::Max() const
{
	g_assert(m_Locations.size());
	return m_Max;
}

sdpVector3 Dimensions::Center() const
{
	g_assert(m_Locations.size());
	return (m_Min + m_Max) * 0.5;
}

sdpVector3 Dimensions::Average() const
{
	g_assert(m_Locations.size());
	return m_Total / m_Locations.size();
}

sdpVector3 Dimensions::Extents() const
{
	g_assert(m_Locations.size());

	sdpVector3 tempmin(m_Min);
	sdpVector3 tempmax(m_Max);
	return sdpVectorMax(tempmin.Abs(), tempmax.Abs());
}

sdpVector3 Dimensions::Size() const
{
	g_assert(m_Locations.size());
	return m_Max - m_Min;
}

sdpVector3 Dimensions::Normal() const
{
        // Handle the simplest case ...
	if(m_Locations.size() < 3)
		return sdpVector3(0, 0, 0);

	sdpVector3 result;

	const sdpVector3 segment1 = m_Locations.back() - m_Locations.front();
	for(gulong i = 1; i < m_Locations.size() - 1; i++)
		result += (segment1) ^ (m_Locations[i] - m_Locations[0]);

	return result.Normalize();
}

} // namespace k3d

