#ifndef K3DSHADERARGUMENTSPAGE_H
#define K3DSHADERARGUMENTSPAGE_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the k3dShaderArgumentsPage class, which provides a UI for editing a set of shader arguments
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3dchannelbutton.h>
#include <k3deditcontrol.h>
#include <k3deventobserver.h>
#include <k3dobjectselector.h>
#include <k3dshaderinstance.h>
#include <k3duserinterfaceelement.h>

#include <list>

///////////////////////////////////////////////////////////////////////////
// k3dShaderArgumentsPageButton

/// Base class for a button that represents one shader argument
class k3dShaderArgumentsPageButton
{
public:
	virtual ~k3dShaderArgumentsPageButton()
	{
	}

	/// Creates the appropriate GTK+ widget(s)
	virtual bool Create(GtkWidget* Parent) = 0;
};

/////////////////////////////////////////////////////////////////////////
// k3dShaderArgumentsPageChannelButton

/// A button that fronts for channel values
class k3dShaderArgumentsPageChannelButton :
	public k3dShaderArgumentsPageButton
{
public:
	k3dShaderArgumentsPageChannelButton(k3dShaderInstanceChannelArgument* const Argument, sdpIUnknown* ParentCommandNode);

	/// Creates the appropriate GTK+ widget
	bool Create(GtkWidget* Parent);

protected:
	k3dChannelButton m_Button;
	k3dIChannel* m_Channel;
};

/////////////////////////////////////////////////////////////////////////
// k3dShaderArgumentsPageStringButton

/// A button that fronts for string values
class k3dShaderArgumentsPageStringButton :
	public k3dShaderArgumentsPageButton
{
public:
	k3dShaderArgumentsPageStringButton(k3dShaderInstanceStringArgument* const Argument, sdpIUnknown* ParentCommandNode);
	
	/// Creates the appropriate GTK+ widget
	bool Create(GtkWidget* Parent);
	
protected:
	k3dEditControl m_Button;
	k3dVariable<sdpString>& m_String;
};

/////////////////////////////////////////////////////////////////////////
// k3dShaderArgumentsPageTextureButton

/// A button that fronts for texture object values
class k3dShaderArgumentsPageTextureButton :
	public k3dShaderArgumentsPageButton
{
public:
	k3dShaderArgumentsPageTextureButton(k3dShaderInstanceTextureArgument* const Argument, sdpIUnknown* ParentCommandNode);
	~k3dShaderArgumentsPageTextureButton();

	/// Creates the appropriate GTK+ widget
	bool Create(GtkWidget* Widget);

protected:
	k3dObjectSelector m_Selector;
	k3dObjectInterfaceVariable<k3dITexture>* m_ObjectContainer;
};

///////////////////////////////////////////////////////////////////////////
// k3dShaderArgumentsPage

typedef k3dUserInterfaceElement k3dShaderArgumentsPageBase;

/// Manages an instance of a shader and its arguments
class k3dShaderArgumentsPage :
	public k3dShaderArgumentsPageBase,
	public k3dEventObserver
{
public:
	k3dShaderArgumentsPage(sdpIUnknown* ParentCommandNode, SDPCSTRING CommandNodeName);
	~k3dShaderArgumentsPage();

	// k3dIEventObserver implementation
	sdpResult ObserveEvent(k3dIEventSource* const Source, SDPCSTRING Event, SDPCSTRING EventData, SDPCSTRING ObserverData);

	/// Called by the framework when this object is created from GTKML (XML)
	bool Create(sdpGtkIObjectContainer* ObjectContainer, sdpxml::Document& Document, sdpxml::Element& Element);
	
	/// Attaches this UI to the shader instance it fronts for ...
	bool Attach(k3dIDocument* const Document, k3dShaderInstance* const ShaderInstance);
	
	const sdpString CustomType() const;

protected:
	/// Rebuilds UI controls to reflect the current state of the shader instance
	void UpdateButtons();
	/// Frees all UI controls
	void ClearButtons();

	/// Stores our parent document
	k3dIDocument* m_Document;
	/// Stores the shader instance we're attached to
	k3dShaderInstance* m_ShaderInstance;

	/// A table widget for organizing the argument buttons & labels
	sdpGtkTable m_Table;
	
	/// Declares a list of argument buttons
	typedef std::vector<k3dShaderArgumentsPageButton*> Buttons;
	typedef Buttons::iterator ButtonIterator;
	
	/// Stores buttons for each argument
	Buttons m_Buttons;
};

#endif // K3DSHADERARGUMENTSPAGE_H

