#ifndef K3DSHADERSELECTOR_H
#define K3DSHADERSELECTOR_H

// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the k3dShaderSelector class, which provides a user interface for choosing Pixar RenderMan shaders
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3deventobserver.h>
#include <k3duserinterfaceelement.h>

#include <sdpsl.h>

// Forward class declarations
class k3dShaderInstance;
class k3dIDocument;

///////////////////////////////////////////////////////////////////////////
// k3dShaderSelector

typedef k3dUserInterfaceElement k3dShaderSelectorBase;

/// Provides a user interface for selecting new & existing objects
class k3dShaderSelector :
	public k3dShaderSelectorBase,
	public k3dEventObserver
{
public:
	k3dShaderSelector(sdpIUnknown* ParentCommandNode, SDPCSTRING CommandNodeName);
	~k3dShaderSelector();

	// k3dIEventObserver implementation
	sdpResult ObserveEvent(k3dIEventSource* const Source, SDPCSTRING Event, SDPCSTRING EventData, SDPCSTRING ObserverData);

	// k3dICommandNode overrides
	sdpResult Command(SDPCSTRING Name, SDPCSTRING Arguments, sdpIUnknown** Result);

	/// Called by the framework when being instantiated from a GTKML (XML) document
	bool Create(sdpGtkIObjectContainer* ObjectContainer, sdpxml::Document& Document, sdpxml::Element& Element);
	/// Connects the selector to an existing shader instance
	bool Attach(k3dIDocument* Document, k3dShaderInstance* ShaderInstance, bool AllowNone, sdpsl::IShader::SLType Type);

	const sdpString CustomType() const;

protected:
	/// Custom event-handling ...
	void OnEvent(sdpGtkEvent* Event);

	/// Called to expand/collapse the list of available shaders
	void OnToggleSelection();
	/// Called when the user selects a shader
	void OnSelect(sdpGtkEvent* Event);
	void OnSelect(const gint Row);
	/// Called to do the actual work of selecting a shader
	void SetShader(sdpsl::IShader* Shader);
	/// Called to display the currently selected shader
	void UpdateShader();

	void ShowSelection();
	void HideSelection();	
						
	/// Stores a pointer to the owning document
	k3dIDocument* m_Document;
	/// Stores the shader instance that we'll be fronting for
	k3dShaderInstance* m_ShaderInstance;
	
	static const sdpString control_toggleselection;
	static const sdpString control_currentselection;
	static const sdpString control_arrow;
	static const sdpString control_shaders;
	static const sdpString control_select;
};

#endif // K3DSHADERSELECTOR_H

