// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Implements the k3dShadingNode class, which caches information used to simulate smooth-shading of adjacent paths
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3dshadingnode.h>
#include <k3dpathpoint.h>
#include <k3dpath.h>
#include <k3dmesh.h>

#include <k3dipath.h>
#include <k3distate.h>

////////////////////////////////////////////////////////////////////////
// k3dShadingNode

k3dShadingNode::k3dShadingNode()
{
}

k3dShadingNode::~k3dShadingNode()
{
}

void k3dShadingNode::RemovePaths(gulong State)
{
	// For each pathpoint ...
	PathPointIterator nextpathpoint;
	PathPointIterator end = m_PathPoints.end();
	for(PathPointIterator pathpoint = m_PathPoints.begin(); pathpoint != end; pathpoint = nextpathpoint)
		{
			nextpathpoint = pathpoint;
			nextpathpoint++;

			// If the pathpoint's path matches our criteria, remove it (the pathpoint) from the list ...
			if(SDP_S_OK == (*pathpoint)->Path()->State()->QueryState(State))
				m_PathPoints.erase(pathpoint);
		}
}

sdpResult k3dShadingNode::Save(sdpxml::Element& Element, k3dIDependencies* Dependencies)
{
	// Sanity checks ...
	g_assert(Dependencies);

	sdpxml::Element& node = Element.Append(sdpxml::Element("node"));

	for(PathPointIterator pathpoint = m_PathPoints.begin(); pathpoint != m_PathPoints.end(); pathpoint++)
		node.Append(sdpxml::Element("pathpoint", "", sdpxml::Attribute("id", sdpToString((*pathpoint)->PathPointID()))));

	return SDP_S_OK;
}

/// Functor object for loading each shading-node pathpoint
class k3dShadingNodeLoadShadingNodePathPoint
{
public:
	k3dShadingNodeLoadShadingNodePathPoint(k3dMesh* const Mesh, k3dShadingNode* const Node) :
		m_Mesh(Mesh),
		m_Node(Node)
	{
	}

	void operator()(sdpxml::Element& Element)
	{
		// Get the ID for this pathpoint ...
		const k3dIPathPoint::ID id = sdpxml::GetAttribute<k3dIPathPoint::ID>(Element, "id", 0);
		g_return_if_fail(id);

		// Convert it into a CPathPoint pointer and push it onto our list ...
		k3dPathPoint* const pathpoint = m_Mesh->FindPathPoint(id);
		g_return_if_fail(pathpoint);
		m_Node->m_PathPoints.push_back(pathpoint);
	}

	k3dMesh* const m_Mesh;
	k3dShadingNode* const m_Node;
};

sdpResult k3dShadingNode::Load(sdpxml::Element& Element, k3dMesh* Mesh)
{
	// Sanity checks ...
	g_assert(Mesh);	

	k3dShadingNodeLoadShadingNodePathPoint functor(Mesh, this);
	sdpxml::ForEach(Element, sdpxml::SameName("pathpoint"), functor);

	return SDP_S_OK;
}


