// K-3D
// Copyright  1995-2002, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Implements the k3dSoftObject class, which implements k3dIImplicitObject
		\author Romain Behar (romainbehar@yahoo.com)
*/

#include <k3dboundingbox.h>
#include <k3dmesh.h>
#include <k3dmisc.h>
#include <k3dpoint.h>
#include <k3dpath.h>
#include <k3dpathpoint.h>
#include <k3drishadersetup.h>
#include <k3dshadingnode.h>
#include <k3dstaterecorder.h>

#include <k3diapplication.h>
#include <k3dicamera.h>
#include <k3didocument.h>
#include <k3dieditorengine.h>
#include <k3dieditormaterial.h>
#include <k3dimaterial.h>
#include <k3direnderframe.h>
#include <k3direnderjob.h>
#include <k3dirimaterial.h>
#include <k3distaterecorder.h>

#include <strstream>

#include <k3dsoftobject.h>

//////////////////////////////////////////////////////////////////////////
// k3dSoftObject

k3dSoftObject::k3dSoftObject(sdpIUnknown* Initializer) :
	k3dSoftObjectBase(Initializer)
{
}

k3dSoftObject::~k3dSoftObject()
{
}

////////////////////////////////////////////////////////////////////////////
// k3dIObject implementation

/*
k3dIState* k3dSoftObject::State()
{
	return this;
}
*/

/*
////////////////////////////////////////////////////////////////////////////
// k3dIState implementation

sdpResult k3dSoftObject::SetState(gulong Mask, bool State)
{
	// If we're changing our selection state, make it a deep select of all points/paths/curves ...
	if(Mask & k3dIState::SELECTED)
		{
			NURBSCurveIterator curveend = m_NURBSCurves.end();
			for(NURBSCurveIterator curve = m_NURBSCurves.begin(); curve != curveend; curve++)
				(*curve)->SetState(k3dIState::SELECTED, State);

			PathIterator pathend = m_Paths.end();
			for(PathIterator path = m_Paths.begin(); path != pathend; path++)
				(*path)->SetState(k3dIState::SELECTED, State);

			PointIterator pointend = m_Points.end();
			for(PointIterator point = m_Points.begin(); point != pointend; point++)
				(*point)->SetState(k3dIState::SELECTED, State);
		}

	return k3dSoftObjectBase::SetState(Mask, State);
}
*/

///////////////////////////////////////////////////////////////
// k3dIMesh implementation

k3dIDocument* k3dSoftObject::Document()
{
	return k3dSoftObjectBase::Document();
}



class k3dSoftObjectSetState
{
public:
	k3dSoftObjectSetState(gulong Mask, bool State) { m_Mask = Mask; m_State = State; }

	void operator()(k3dPoint* Point)
	{
		Point->SetState(m_Mask, m_State);
	}

	gulong m_Mask;
	bool m_State;
};

///////////////////////////////////////////////////////////////
// k3dIEditable implementation

void k3dSoftObject::PushEditing()
{
	k3dSoftObjectBase::PushEditing();
}

void k3dSoftObject::PopEditing()
{
	k3dSoftObjectBase::PopEditing();
}

///////////////////////////////////////////////////////////////
// k3dIPersist implementation

sdpResult k3dSoftObject::Save(sdpxml::Element& Element, k3dIDependencies* Dependencies)
{
	// Sanity checks ...
	g_assert(Dependencies);

	// Call the base-class implementation ...
	g_return_val_if_fail(sdpSucceeded(k3dSoftObjectBase::Save(Element, Dependencies)), SDP_E_FAIL);

	return SDP_S_OK;
}

sdpResult k3dSoftObject::Load(sdpxml::Document& Document, sdpxml::Element& Element)
{
	// Call the base-class implementation ...
	g_return_val_if_fail(sdpSucceeded(k3dSoftObjectBase::Load(Document, Element)), SDP_E_FAIL);

	return SDP_S_OK;
}

sdpResult k3dSoftObject::LoadComplete()
{
	// Call the base-class version ...
	g_return_val_if_fail(sdpSucceeded(k3dSoftObjectBase::LoadComplete()), SDP_E_FAIL);

	return SDP_S_OK;
}


