#ifndef K3DSTATE_H
#define K3DSTATE_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the k3dState class, which provides a default implementation of k3dIState
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3distate.h>
#include <k3didependencies.h>

// Forward class declarations
class k3dIStateRecorder;
namespace sdpxml { class Element; }

/// Provides a default implementation of k3dIState
class k3dState :
	public k3dIState
{
public:
	k3dState(const gulong State);

	// k3dIState implementation
	sdpResult QueryState(const gulong State);

	sdpResult Save(sdpxml::Element& Element, k3dIDependencies* const Dependencies);
	sdpResult Load(sdpxml::Element& Element);

	/// Provides an efficient query for a collection of set flags
	bool AdvancedQuery(const gulong Set)
	{
		return (m_StateFlags & Set) == Set;
	}
	
	/// Provides an efficient query for a combination of set and unset flags, mainly for rendering
	bool AdvancedQuery(const gulong Set, const gulong Unset)
	{
		return ((m_StateFlags & Set) == Set) && ((m_StateFlags & Unset) == 0);
	}

protected:
	void RawSetState(const gulong Mask, const bool State, k3dIStateRecorder* const StateRecorder);
	void RawSetState(const gulong Mask, const bool State);

	gulong m_StateFlags;
};

#endif // K3DSTATE_H
