#ifndef K3DTARGETCONTROLLER_H
#define K3DTARGETCONTROLLER_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the k3dTargetController class, which provides a standard MVC UI for manual control of objects that support k3dITransform
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3dspinbutton.h>

// Forward class declarations
class k3dObjectSelector;
class k3dICommandRecorder;
class k3dIDocument;
class k3dIStateRecorder;
class sdpGtkIObjectContainer;
class sdpGtkEvent;
class sdpxml::Element;

///////////////////////////////////////////////////////////////////////////
// k3dTargetController

typedef k3dControl k3dTargetControllerBase;

/// Provides a UI for editing objects that support k3dIChannel
class k3dTargetController :
	public k3dTargetControllerBase
{
public:
	k3dTargetController(sdpIUnknown* CommandNodeParent, SDPCSTRING CommandNodeName);
	virtual ~k3dTargetController();

	/// Called by the framework when instantiating the button from a GTKML (XML) document
	bool Create(sdpGtkIObjectContainer* ObjectContainer, sdpxml::Document& Document, sdpxml::Element& Element);

	/// Attaches the button to the object it fronts for
	bool Attach(k3dIObject* const Object, k3dObjectVariable* const DataSource, k3dICommandRecorder* const CommandRecorder, k3dIStateRecorder* const StateRecorder, SDPCSTRING StateChangeName, k3dIAction* const PushEditingAction, k3dIAction* const ChangeAction, k3dIAction* const PopEditingAction);

	const sdpString CustomType() const;

protected:
	/// Called to handle SDPGTK events
	void OnEvent(sdpGtkEvent* Event);
	
	/// Called when the GTK+ widgets are about to disappear
	void OnDestroy();
	/// Called to reset the object position to the origin
	void OnReset();

	/// Stores a reference to the owning document object
	k3dIDocument* m_Document;
	/// Stores a reference to the underlying data object
	k3dObjectVariable* m_DataSource;
	
	k3dObjectSelector* m_Selector;
	
	static const sdpString control_object;
	static const sdpString control_reset;
};

#endif // K3DTARGETCONTROLLER_H

