#ifndef K3DTEXTEDITOR_H
#define K3DTEXTEDITOR_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the k3dTextEditor class, which provides an extensible UI for editing textfiles
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3ddialog.h>
#include <iostream>

// Forward class references ...
class k3dIApplication;

////////////////////////////////////////////////////////////////////////////
// k3dTextEditor

typedef k3dDialog k3dTextEditorBase;

/// Implements the application user interface
class k3dTextEditor : public k3dTextEditorBase
{
public:
	k3dTextEditor(sdpIUnknown* ParentCommandNode, SDPCSTRING CommandNodeName);
	virtual ~k3dTextEditor();
	
protected:
	virtual bool LoadGTKMLTemplate(SDPCSTRING Template);

	/// Returns the GtkText control for the editor
	virtual sdpGtkText TextControl();
	/// Updates the window titlebar
	virtual void UpdateTitlebar();
	/// Returns the text to display in the window titlebar
	virtual sdpString GetTitlebarText();
	/// Returns the document title
	virtual sdpString GetTitle();
	
	/// Loads the editor contents from a file
	virtual void Load(sdpString Filepath);
	/// Loads the editor contents from a stream
	virtual void Load(std::istream& Stream);
	
	/// Saves the editor contents to a file
	virtual void Save(sdpString Filepath);
	/// Saves the editor contents to a stream
	virtual void Save(std::ostream& Stream);
	/// Conditionally saves changes to the file
	virtual void RescueChanges();

	/// Overrides sdpGtkObjectContainer to handle custom events
	virtual void OnEvent(sdpGtkEvent* Event);
	/// Called by the WM "close" button
	virtual void OnDelete(sdpGtkEvent* Event);
	/// Called to begin a new script
	virtual void OnFileNew();
	/// Called to load a script
	virtual void OnFileOpen();
	/// Called to save the script
	virtual void OnFileSave();
	/// Called to save the script under a new filename
	virtual void OnFileSaveAs() = 0;
	/// Called to revert to a previously-saved version of the file
	virtual void OnFileRevert();
	/// Called to close the editor window
	virtual void OnFileClose();
	/// Called when the text in the editor window changes
	virtual void OnChanged();
	
	/// The current editor filepath
	sdpString m_FilePath;
	/// Contains "true" if there are unsaved changes to the text
	bool m_Changed;
};

#endif // K3DTEXTEDITOR_H
