#ifndef K3DTIMECONTROLLER_H
#define K3DTIMECONTROLLER_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the k3dTimeController facade which provides high-level, frame-oriented manipulation of a document's k3dITime interface
		\author Tim Shead (tshead@k-3d.com)
*/

#ifndef SDPTYPES_H
#include <sdptypes.h>
#endif // !SDPTYPES_H

// Forward class references
class k3dITime;

/// Provides a high-level, frame-oriented facade for a document's k3dITime interface
class k3dTimeController
{
public:
	k3dTimeController(k3dITime* const Time);

	/// Returns the animation start time as a one-based integer frame number
	gulong StartFrame() const;
	/// Returns the current view time as a one-based integer frame number
	gulong ViewFrame() const;
	/// Returns the animation end time as a one-based integer frame number
	gulong EndFrame() const;
	/// Returns the length of the animation in frames
	gulong TotalFrames() const;
	/// Returns the animation duration in seconds
	gdouble Duration() const;
	/// Returns the animation frame rate
	gdouble FrameRate() const;

	/// Sets the animation start time as a one-based integer frame number
	sdpResult SetStartFrame(const gulong Frame);
	/// Sets the current view time as a one-based integer frame number
	sdpResult SetViewFrame(const gulong Frame);
	/// Sets the animation end time as a one-based integer frame number
	sdpResult SetEndFrame(const gulong Frame);
	/// Sets the length of the animation in frames
	sdpResult SetTotalFrames(const gulong Frames);
	/// Sets the animation duration in seconds
	sdpResult SetDuration(const gdouble Seconds);
	/// Sets the animation frame rate
	sdpResult SetFrameRate(const gdouble Rate);
	
protected:
	k3dITime* const m_Time;
	
private:
	gdouble UserToReal(const gulong UserTime) const;
	gulong RealToUser(const gdouble RealTime) const;
	gdouble RealToInternal(const gdouble RealTime) const;
	gdouble InternalToReal(const gdouble InternalTime) const;
	gdouble RealDuration() const;
};

#endif // K3DTIMECONTROLLER_H
