// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Implements the k3dTransform class, which provides a pluggable implementation of k3dITransform
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3dtransform.h>

k3dTransform::k3dTransform()
{
	for(gulong i = 0; i < sizeof(m_Scale) / sizeof(sdpVector3); i++)
		m_Scale[i] = 1.0;
}

////////////////////////////////////////////////////////////////
// k3dITransform overrides

const sdpVector3& k3dTransform::Position(const ReferenceFrame Frame)
{
	return m_Position[Frame];
}

const sdpAngleAxis& k3dTransform::Orientation(const ReferenceFrame Frame)
{
	return m_Orientation[Frame];
}

const sdpVector3& k3dTransform::Scale(const ReferenceFrame Frame)
{
	return m_Scale[Frame];
}

void k3dTransform::SetPosition(const ReferenceFrame Frame, const sdpVector3& Position)
{
	m_Position[Frame] = Position;
}

void k3dTransform::SetOrientation(const ReferenceFrame Frame, const sdpAngleAxis& Orientation)
{
	m_Orientation[Frame] = Orientation;
}

void k3dTransform::SetScale(const ReferenceFrame Frame, const sdpVector3& Scale)
{
	m_Scale[Frame] = Scale;
}

////////////////////////////////////////////////////////////////////////////
// Helper functions ...

sdpResult k3dTransform::Save(sdpxml::Element& Element)
{
	Element.Children().push_back(sdpxml::Element("transformation", "", 
		sdpxml::Attribute("frame", "self"),
		sdpxml::Attribute("position", sdpToString(m_Position[k3dITransform::CURRENT_FRAME])),
		sdpxml::Attribute("orientation", sdpToString(m_Orientation[k3dITransform::CURRENT_FRAME])),
		sdpxml::Attribute("scale", sdpToString(m_Scale[k3dITransform::CURRENT_FRAME]))));
		
	Element.Children().push_back(sdpxml::Element("transformation", "",
		sdpxml::Attribute("frame", "child"),
		sdpxml::Attribute("position", sdpToString(m_Position[k3dITransform::CURRENT_CHILD_FRAME])),
		sdpxml::Attribute("orientation", sdpToString(m_Orientation[k3dITransform::CURRENT_CHILD_FRAME])),
		sdpxml::Attribute("scale", sdpToString(m_Scale[k3dITransform::CURRENT_CHILD_FRAME]))));

	return SDP_S_OK;
}

sdpResult k3dTransform::Load(sdpxml::Element& Element)
{
	for(sdpxml::ElementCollection::iterator element = Element.Children().begin(); element != Element.Children().end(); element++)
		{
			if(element->Name() != "transformation")
				continue;
				
			const sdpString frame = sdpxml::GetAttribute<sdpString>(*element, "frame", "");
			if(frame == "self")
				{
					sdpxml::ParseAttribute(*element, "position", m_Position[k3dITransform::CURRENT_FRAME]);
					sdpxml::ParseAttribute(*element, "orientation", m_Orientation[k3dITransform::CURRENT_FRAME]);
					sdpxml::ParseAttribute(*element, "scale", m_Scale[k3dITransform::CURRENT_FRAME]);
				}
			else if(frame == "child")
				{
					sdpxml::ParseAttribute(*element, "position", m_Position[k3dITransform::CURRENT_CHILD_FRAME]);
					sdpxml::ParseAttribute(*element, "orientation", m_Orientation[k3dITransform::CURRENT_CHILD_FRAME]);
					sdpxml::ParseAttribute(*element, "scale", m_Scale[k3dITransform::CURRENT_CHILD_FRAME]);
				}
			else
				{
					std::cerr << "k3dTransform::Load(): unknown frame [" << frame << "]" << std::endl;
				}
				
			break;
		}

	return SDP_S_OK;
}

