#ifndef K3DVARIABLECONTAINER_H
#define K3DVARIABLECONTAINER_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the k3dVariableContainer class, which encapsulates a set of k3dVariable objects that can be serialized
		\author Tim Shead (tshead@k-3d.com)
*/

#ifndef SDPTYPES_H
#include <sdptypes.h>
#endif // !SDPTYPES_H

// Forward class declarations
class k3dIDependencies;
class k3dIVariable;

namespace sdpxml
{
class Document;
class Element;
} // namespace sdpxml

/// Encapsulates a K-3D object capable of automatically serializing itself
class k3dVariableContainer
{
public:
	virtual ~k3dVariableContainer();

	// Persistence methods
	virtual sdpResult InitNew();
	virtual sdpResult Load(sdpxml::Document& Document, sdpxml::Element& Element);
	virtual sdpResult LoadComplete();
	virtual sdpResult Save(sdpxml::Element& Element, k3dIDependencies* Dependencies);
	virtual sdpResult SaveComplete();
	
protected:
	/// Inserts a variable object into the collection, so it can automatically be serialized / deserialized
	void AddVariable(k3dIVariable* Variable);

	/// A collection of variables	
	typedef std::list<k3dIVariable*> VariableList;
	typedef VariableList::iterator VariableIterator;
	VariableList m_Variables;
};

#endif // K3DVARIABLECONTAINER_H
