// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Implements helper functions for working with view volumes
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3dviewvolume.h>

#include <k3diriengine.h>
#include <k3diviewvolume.h>

#include <sdpgl.h>

void k3dOpenGLViewVolumeSetup(k3dIViewVolume* const ViewVolume)
{
	// Sanity checks ...
	g_assert(ViewVolume);

	if(ViewVolume->Orthographic())
		glOrtho(ViewVolume->Left(), ViewVolume->Right(), ViewVolume->Bottom(), ViewVolume->Top(), ViewVolume->NearPlane(), ViewVolume->FarPlane());
	else
		glFrustum(ViewVolume->NearPlane() * ViewVolume->Left(), ViewVolume->NearPlane() * ViewVolume->Right(), ViewVolume->NearPlane() * ViewVolume->Bottom(), ViewVolume->NearPlane() * ViewVolume->Top(), ViewVolume->NearPlane(), ViewVolume->FarPlane());
}

void k3dRiViewVolumeSetup(k3dIViewVolume* const ViewVolume, k3dIRiEngine* const Engine)
{
	// Sanity checks ...
	g_assert(ViewVolume);
	g_assert(Engine);

	if(ViewVolume->Orthographic())
		{
			Engine->RiProjectionV("orthographic", 0, 0, 0, 0);
			Engine->RiScreenWindow(ViewVolume->Left(), ViewVolume->Right(), ViewVolume->Bottom(), ViewVolume->Top());
			Engine->RiClipping(ViewVolume->NearPlane(), ViewVolume->FarPlane());
		}
	else
		{
			Engine->RiProjectionV("perspective", 0, 0, 0, 0);
			Engine->RiScreenWindow(ViewVolume->Left(), ViewVolume->Right(), ViewVolume->Bottom(), ViewVolume->Top());
			Engine->RiClipping(ViewVolume->NearPlane(), ViewVolume->FarPlane());
		}
}


