#ifndef K3DWINDOWGEOMETRYSTORE_H
#define K3DWINDOWGEOMETRYSTORE_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the k3dWindowGeometryStore and derived classes, which encapsulate persisting window geometry (position & size) to various storage types
		\author Timothy M. Shead (tshead@k-3d.com)
*/

#ifndef SDPTYPES_H
#include <sdptypes.h>
#endif // !SDPTYPES_H

// Forward class declarations
class k3dIApplication;

//////////////////////////////////////////////////////////////////////////////////
// k3dWindowGometryStore

/// Abstract interface for objects that can store/retrieve window geometry
class k3dWindowGeometryStore
{
public:
	k3dWindowGeometryStore(k3dWindowGeometryStore* const NextStore);
	virtual ~k3dWindowGeometryStore();

	virtual void SetWindowGeometry(SDPCSTRING WindowName, const gint Left, const gint Top, const guint Width, const guint Height) = 0;
	virtual bool GetWindowGeometry(SDPCSTRING WindowName, gint& Left, gint& Top, guint& Width, guint& Height) = 0;
	
protected:
	k3dWindowGeometryStore* const m_NextStore;
};

//////////////////////////////////////////////////////////////////////////////////
// k3dOptionsWindowGeometryStore

/// Stores window geometry in the user preferences
class k3dOptionsWindowGeometryStore :
	public k3dWindowGeometryStore
{
public:
	k3dOptionsWindowGeometryStore(k3dIApplication* const Application, k3dWindowGeometryStore* const NextStore);

	void SetWindowGeometry(SDPCSTRING WindowName, const gint Left, const gint Top, const guint Width, const guint Height);
	bool GetWindowGeometry(SDPCSTRING WindowName, gint& Left, gint& Top, guint& Width, guint& Height);
	
private:
	k3dIApplication* const m_Application;
};

//////////////////////////////////////////////////////////////////////////////////
// k3dPersistWindowGeometryStore

/// Stores window geometry in a K-3D document using standard mechanisms
class k3dPersistWindowGeometryStore :
	public k3dWindowGeometryStore
{
public:
	k3dPersistWindowGeometryStore(k3dWindowGeometryStore* const NextStore);

	void SetWindowGeometry(SDPCSTRING WindowName, const gint Left, const gint Top, const guint Width, const guint Height);
	bool GetWindowGeometry(SDPCSTRING WindowName, gint& Left, gint& Top, guint& Width, guint& Height);
	
private:
	gint m_Left;
	gint m_Top;
	guint m_Width;
	guint m_Height;
};

#endif // K3DWINDOWGEOMETRYSTORE_H
