// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Implements procedures required to export K-3D objects from the bitmaplib module
		\author Timothy M. Shead (tshead@k-3d.com)
*/
		
#include "bitmap.h"
#include "netpbmreader.h"
#include "netpbmwriter.h"

#include <k3dcom.h>
#include <k3dmodule.h>

/// Implements procedures required to export K-3D objects from the bitmap module
class BitmapModule :
	public k3d::Module
{
public:
	sdpIUnknown* CreateInstance(const sdp::ClassID& ClassID, sdpIUnknown* Initializer)
	{
		if(ClassID == k3d::classid::Bitmap)
			return sdpCreateClassInstance<CBitmap>(Initializer);
		if(ClassID == k3d::classid::BitmapReadFormatNetPBM)
			return sdpCreateClassInstance<CNetPBMReader>(Initializer);
		if(ClassID == k3d::classid::BitmapWriteFormatNetPBM)
			return sdpCreateClassInstance<CNetPBMWriter>(Initializer);

		return 0;
	}
	
private:
	friend class sdpSingleton<BitmapModule>;

	BitmapModule() :
		k3d::Module("bitmaplib", VERSION)
	{
		// Bitmap
		sdpxml::Element& bitmap = RegisterObject(k3d::classid::Bitmap, "Bitmap");
		RegisterBookmark(bitmap, "Objects/Bitmap");
		RegisterInterface(bitmap, sdp::TypeInfo<k3dIBitmap>());
		RegisterInterface(bitmap, sdp::TypeInfo<k3dITexture>());
		
		// NetPBMReader
		sdpxml::Element& netpbmreader = RegisterObject(k3d::classid::BitmapReadFormatNetPBM, "NetPBMReader");
		RegisterInterface(netpbmreader, sdp::TypeInfo<k3dIBitmapReadFormat>());
		RegisterShortDescription(netpbmreader, "NetPBM Formats ( many )");
		
		// NetPBMWriter
		sdpxml::Element& netpbmwriter = RegisterObject(k3d::classid::BitmapWriteFormatNetPBM, "NetPBMWriter");
		RegisterInterface(netpbmwriter, sdp::TypeInfo<k3dIBitmapWriteFormat>() );
		RegisterShortDescription(netpbmwriter, "NetPBM Formats ( many )");
	}
};

K3DMODULEPROCEDURES(BitmapModule, sdpSingleton<BitmapModule>);


