#ifndef NETPBMREADER_H
#define NETPBMREADER_H

// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the CBitmapFormatNetPBM K-3D object, which reads and writes a large number of image file formats, using the NetPBM package
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3dobject.h>
#include <k3difileformat.h>
#include <k3dibitmapreadformat.h>

#ifndef SDPXML_H
#include <sdpxml.h>
#endif // !SDPXML_H

// Forward class declarations ...
class k3dPixel;

typedef k3dObject CNetPBMReaderBase;

/// File filter for reading bitmap images, using the NetPBM package
class CNetPBMReader :
	public CNetPBMReaderBase,
	public k3dIFileFormat,
	public k3dIBitmapReadFormat
{
public:
	CNetPBMReader(sdpIUnknown* Initializer);
	virtual ~CNetPBMReader();

	// k3dIFileFormat implementation
	bool QueryCanHandle(SDPCSTRING FilePath);

	// k3dIBitmapReadFormat implementation
	sdpResult PreRead(k3dIBitmap* const Bitmap, SDPCSTRING FilePath, gulong* const Width, gulong* const Height);
	sdpResult ReadOptions(k3dIBitmap* const Bitmap, SDPCSTRING FilePath);
	sdpResult ReadFile(k3dIBitmap* const Bitmap, SDPCSTRING FilePath);

private:
	/// Loads the netpbm.conf XML file, which describes how to identify and convert filetypes supported by NetPBM
	bool LoadFormats();

	/// Enumerates the three most common NetPBM file formats
	typedef enum
	{
		PBM,
		PGM,
		PPM
	} ImageType;

	/// Returns a filter description from the netpbm.conf XML file, based on the filename
	sdpxml::ElementPointer GetReader(SDPCSTRING FilePath);
	
	/// Converts a file to a temp file in the native NetPBM file format
	bool ConvertToPNM(SDPCSTRING FilePath, sdpString& PNMFilePath);
	
	bool ReadPNMHeader(std::istream& Stream, gulong& Width, gulong& Height, ImageType& Type);
	bool ReadPNM(std::istream& Stream, k3dIBitmap* Bitmap);
	bool ReadPBM(std::istream& Stream, k3dPixel* Destination, gulong Width, gulong Height);
	bool ReadPGM(std::istream& Stream, k3dPixel* Destination, gulong Width, gulong Height);
	bool ReadPPM(std::istream& Stream, k3dPixel* Destination, gulong Width, gulong Height);

	sdpxml::Document m_Formats;
	sdpString m_PNMFile;
};

#endif // NETPBMREADER
