#ifndef NETPBMWRITER_H
#define NETPBMWRITER_H

// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the CNetPBMWriter K-3D object, which writes a large number of bitmap image file formats, using the NetPBM package
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3dobject.h>
#include <k3difileformat.h>
#include <k3dibitmapwriteformat.h>

#ifndef SDPXML_H
#include <sdpxml.h>
#endif // !SDPXML_H

// Forward class declarations ...
class k3dPixel;

typedef k3dObject CNetPBMWriterBase;

/// File filter for writing bitmap images, using the NetPBM package
class CNetPBMWriter :
	public CNetPBMWriterBase,
	public k3dIFileFormat,
	public k3dIBitmapWriteFormat
{
public:
	CNetPBMWriter(sdpIUnknown* Initializer);
	virtual ~CNetPBMWriter();

	// k3dIFileFormat implementation
	bool QueryCanHandle(SDPCSTRING FilePath);

	// k3dIBitmapWriteFormat implementation
	sdpResult PreWrite(k3dIBitmap* Bitmap, SDPCSTRING FilePath);
	sdpResult WriteOptions(k3dIBitmap* Bitmap, SDPCSTRING FilePath);
	sdpResult WriteFile(k3dIBitmap* Bitmap, SDPCSTRING FilePath);

private:
	/// Loads the netpbm.conf XML file, which describes how to identify and convert filetypes supported by NetPBM
	bool LoadFormats();

	/// Returns a filter description from the netpbm.conf XML file, corresponding to the requested file format
	sdpxml::ElementPointer GetWriter(SDPCSTRING FilePath);

	/// Saves a K-3D bitmap in PPM format
	bool SaveAsPPM(k3dIBitmap* Bitmap, SDPCSTRING FilePath);
	/// Saves image data in PPM format
	bool SaveAsPPM(k3dPixel* Data, gulong Width, gulong Height, std::ostream& Stream);

	sdpxml::Document m_Formats;
};

#endif // NETPBMWRITER_H
