#ifndef AXES_H
#define AXES_H

// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the CAxes K-3D object, which provides a configurable set of axes to help in visualizing the 3D workspace
		\author Tim Shead (tshead@k-3d.com)
*/

// Forward class declarations ...
class CAxes;

#include <k3dobjectdialog.h>
#include <k3dtransformobject.h>
#include <k3dvariable.h>

#include <k3disnaptarget.h>

/////////////////////////////////////////////////////////////////////////////
// CAxesDialog

typedef k3dObjectDialog CAxesDialogBase;

class CAxesDialog : public CAxesDialogBase
{
public:
	CAxesDialog(CAxes* const Object);

protected:
	CAxes* const m_Object;

	void OnEvent(sdpGtkEvent* Event);

	static const sdpString dialogname;
	static const sdpString control_axes;
	static const sdpString control_xyplane;
	static const sdpString control_yzplane;
	static const sdpString control_xzplane;
	static const sdpString control_gridsize;
	static const sdpString control_gridcount;
	static const sdpString control_position;
	static const sdpString control_orientation;
	static const sdpString control_scale;
	static const sdpString control_target;
};


/////////////////////////////////////////////////////
// CAxes

typedef k3dTransformObject CAxesBase;

class CAxes :	public CAxesBase
{
friend class CAxesDialog;
public:
	CAxes(sdpIUnknown* Initializer);
	~CAxes();

	// k3dIObject implementation
	sdpResult EditObject();

	// k3dIEditorRender implementation
	sdpResult EditorRender(k3dIEditorEngine* const Engine);

	// k3dISnapTarget implementation
	sdpResult Snap(sdpVector3& Coordinates, sdpString& Description);

protected:
	k3dVariable<bool> m_Axes;
	k3dVariable<bool> m_XYPlane;
	k3dVariable<bool> m_YZPlane;
	k3dVariable<bool> m_XZPlane;
	k3dVariable<gdouble> m_GridSize;
	k3dVariable<gdouble> m_GridCount;

	static double m_XAxisColor[4];
	static double m_YAxisColor[4];
	static double m_ZAxisColor[4];
	static double m_GridColor[4];
};


#endif // AXES_H
