#ifndef DISK_H
#define DISK_H

// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the CDisk K-3D object, which renders a RenderMan sphere primitive
		\author Tim Shead (tshead@k-3d.com)
*/

// Forward class declarations ...
class CDisk;

#include <k3dobjectdialog.h>
#include <k3dtransformobject.h>
#include <k3dvariable.h>

#include <k3dimaterialcollection.h>
#include <k3disnaptarget.h>

/////////////////////////////////////////////////////////////////////////////
// CDiskDialog

typedef k3dObjectDialog CDiskDialogBase;

class CDiskDialog :
	public CDiskDialogBase
{
public:
	CDiskDialog(CDisk* const Object);

protected:
	CDisk* const m_Object;

	void OnEvent(sdpGtkEvent* Event);

	static const sdpString dialogname;
	static const sdpString control_radius;
	static const sdpString control_height;
	static const sdpString control_sweepangle;
	static const sdpString control_position;
	static const sdpString control_orientation;
	static const sdpString control_scale;
	static const sdpString control_target;
};


/////////////////////////////////////////////////////
// CDisk

typedef k3dTransformObject CDiskBase;

class CDisk :
	public CDiskBase,
	public k3dIMaterialCollection
{
friend class CDiskDialog;
public:
	CDisk(sdpIUnknown* Initializer);
	~CDisk();

	// k3dIObject implementation
	sdpResult EditObject();

	// k3dIEditorRender implementation
	sdpResult EditorRender(k3dIEditorEngine* const Engine);
	
	// k3dIRiRender implementation
	sdpResult RiRender(k3dIRenderFrame* const Frame, k3dIRiEngine* const Engine);

	// k3dIMaterialCollection implementation
	k3dIMaterial* Material();
	sdpResult SetMaterial(k3dIMaterial* const Material);

	// k3dIPersist implementation
	sdpResult InitComplete();

protected:
	/// Stores our assigned material
	k3dObjectInterfaceVariable<k3dIMaterial> m_Material;

	k3dScalarChannel m_Radius;
	k3dScalarChannel m_Height;
	k3dScalarChannel m_SweepAngle;
};


#endif // DISK_H
