// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Implements the CMesh class, which encapsulates a collection of points, paths, material, NURBS curves, and shading information
		\author Timothy M. Shead (tshead@k-3d.com)
		\author Romain Behar (romainbehar@yahoo.com)
*/

#include "mesh.h"

#include <k3dchooser.h>
#include <k3doptions.h>
#include <k3dorientationcontroller.h>
#include <k3dpositioncontroller.h>
#include <k3dscalecontroller.h>
#include <k3dtargetcontroller.h>
#include <k3dmisc.h>

#include <k3diapplication.h>

#include <sdpgtkevent.h>

//////////////////////////////////////////////////////////////////////////
// CMesh

CMesh::CMesh(sdpIUnknown* Initializer) :
	CMeshBase(Initializer)
{
}

///////////////////////////////////////////////////////////////
// k3dIObject implementation

sdpResult CMesh::EditObject()
{
	new Properties(this);
	return SDP_S_OK;
}

/////////////////////////////////////////////////////////////////////////////
// CMesh::Properties

CMesh::Properties::Properties(CMesh* const Object) :
	PropertiesBase(Object, true, false, true, "properties", new k3dOptionsWindowGeometryStore(k3dApplication(), 0)),
	m_Object(Object)
{
	// Load the dialog template ...
	g_return_if_fail(LoadGTKMLTemplate("corelib", "mesh.gtkml"));

	// Setup choosers ...
	if(Getk3dChooser("rendermode"))
		Getk3dChooser("rendermode")->Attach(new k3dChooserVariableDataSource<k3dMesh::RenderMode>(m_Object->m_RenderMode), k3dApplication()->CommandRecorder(), m_Object->Document()->StateRecorder(), "Render Mode", 0, new k3dActionRedrawAll(m_Object->Document(), false, 0), 0);
		
	// Setup transformation controls ...
	if(Getk3dPositionController("position"))
		Getk3dPositionController("position")->Attach(Object->Document(), m_Object, k3dApplication()->CommandRecorder(), m_Object->Document()->StateRecorder(), "Position", 0, new k3dActionRedrawAll(m_Object->Document(), false, 0), 0);

	if(Getk3dOrientationController("orientation"))
		Getk3dOrientationController("orientation")->Attach(Object->Document(), m_Object, k3dApplication()->CommandRecorder(), m_Object->Document()->StateRecorder(), "Orientation", 0, new k3dActionRedrawAll(m_Object->Document(), false, 0), 0);

	if(Getk3dScaleController("scale"))
		Getk3dScaleController("scale")->Attach(Object->Document(), m_Object, k3dApplication()->CommandRecorder(), m_Object->Document()->StateRecorder(), "Scale", 0, new k3dActionRedrawAll(m_Object->Document(), false, 0), 0);

	if(Getk3dTargetController("target"))
		Getk3dTargetController("target")->Attach(m_Object, &m_Object->m_Target, k3dApplication()->CommandRecorder(), m_Object->Document()->StateRecorder(), "Target", 0, new k3dActionRedrawAll(m_Object->Document(), false, 0), 0);

	Show();
}


