// K-3D
// Copyright  1995-2002, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Implements procedures required to export K-3D objects from the core module
		\author Timothy M. Shead (tshead@k-3d.com)
*/

#include "axes.h"
#include "camera.h"
#include "channel.h"
#include "cone.h"
#include "cylinder.h"
#include "disk.h"
#include "light.h"
#include "material.h"
#include "mesh.h"
#include "paraboloid.h"
#include "sphere.h"
#include "torus.h"
//#include "group.h"
//#include "olbtarget.h"

#include <k3dcom.h>
#include <k3dmodule.h>

/// Implements procedures required to export K-3D objects from the core module
class CoreModule :
	public k3d::Module
{
public:
	sdpIUnknown* CreateInstance(const sdp::ClassID& ClassID, sdpIUnknown* Initializer)
	{
		if(ClassID == k3d::classid::Axes)
			return sdpCreateClassInstance<CAxes>(Initializer);
		if(ClassID == k3d::classid::Camera)
			return sdpCreateClassInstance<CCamera>(Initializer);
		if(ClassID == k3d::classid::Channel)
			return sdpCreateClassInstance<CChannel>(Initializer);
		if(ClassID == k3d::classid::Cone)
			return sdpCreateClassInstance<CCone>(Initializer);
		if(ClassID == k3d::classid::Cylinder)
			return sdpCreateClassInstance<CCylinder>(Initializer);
		if(ClassID == k3d::classid::Disk)
			return sdpCreateClassInstance<CDisk>(Initializer);
		if(ClassID == k3d::classid::Light)
			return sdpCreateClassInstance<CLight>(Initializer);
		if(ClassID == k3d::classid::Material)
			return sdpCreateClassInstance<CMaterial>(Initializer);
		if(ClassID == k3d::classid::Mesh)
			return sdpCreateClassInstance<CMesh>(Initializer);
		if(ClassID == k3d::classid::Paraboloid)
			return sdpCreateClassInstance<CParaboloid>(Initializer);
		if(ClassID == k3d::classid::Sphere)
			return sdpCreateClassInstance<CSphere>(Initializer);
		if(ClassID == k3d::classid::Torus)
			return sdpCreateClassInstance<CTorus>(Initializer);

		return 0;
	}

private:
	friend class sdpSingleton<CoreModule>;

	CoreModule() :
		k3d::Module("corelib", VERSION)
	{
		// Axes ...
		sdpxml::Element& axes = RegisterObject(k3d::classid::Axes, "Axes");
		RegisterBookmark(axes, "Objects/Axes");

		// Camera ...
		sdpxml::Element& camera = RegisterObject(k3d::classid::Camera, "Camera");
		RegisterBookmark(camera, "Objects/Camera");
		RegisterInterface(camera, sdp::TypeInfo<k3dICamera>());

		// Channel ...
		sdpxml::Element& channel = RegisterObject(k3d::classid::Channel, "Channel");
		RegisterInterface(channel, sdp::TypeInfo<k3dIChannel>());

		// Cone ...
		sdpxml::Element& cone = RegisterObject(k3d::classid::Cone, "Cone");
		RegisterBookmark(cone, "Objects/Cone");
		RegisterIncomplete(cone);

		// Cylinder ...
		sdpxml::Element& cylinder = RegisterObject(k3d::classid::Cylinder, "Cylinder");
		RegisterBookmark(cylinder, "Objects/Cylinder");
		RegisterIncomplete(cylinder);

		// Disk ...
		sdpxml::Element& disk = RegisterObject(k3d::classid::Disk, "Disk");
		RegisterBookmark(disk, "Objects/Disk");

		// Light ...
		sdpxml::Element& light = RegisterObject(k3d::classid::Light, "Light");
		RegisterBookmark(light, "Objects/Light");
		RegisterInterface(light, sdp::TypeInfo<k3dILight>());

		// Material ...
		sdpxml::Element& material = RegisterObject(k3d::classid::Material, "Material");
		RegisterBookmark(material, "Objects/Material");
		RegisterInterface(material, sdp::TypeInfo<k3dIMaterial>());

		// Mesh ...
		sdpxml::Element& mesh = RegisterObject(k3d::classid::Mesh, "Mesh");
		RegisterBookmark(mesh, "Objects/Mesh");
		RegisterInterface(mesh, sdp::TypeInfo<k3dIMesh>());

		// Paraboloid ...
		sdpxml::Element& paraboloid = RegisterObject(k3d::classid::Paraboloid, "Paraboloid");
		RegisterBookmark(paraboloid, "Objects/Paraboloid");
		RegisterIncomplete(paraboloid);

		// Sphere ...
		sdpxml::Element& sphere = RegisterObject(k3d::classid::Sphere, "Sphere");
		RegisterBookmark(sphere, "Objects/Sphere");
		RegisterIncomplete(sphere);

		// Torus ...
		sdpxml::Element& torus = RegisterObject(k3d::classid::Torus, "Torus");
		RegisterBookmark(torus, "Objects/Torus");
		RegisterIncomplete(torus);
	}
};

K3DMODULEPROCEDURES(CoreModule, sdpSingleton<CoreModule>);

