// K-3D
// Copyright  1995-2002, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Implements the CRiEngine class, which can render the 3D workspace to a RIB file
		\author Tim Shead (tshead@k-3d.com)
*/

//#include "riengine.h"
#include "camera.h"

#include <k3dframerange.h>
#include <k3dmisc.h>
#include <k3dobjectcollection.h>
#include <k3drishadersetup.h>
#include <k3dtimecontroller.h>
#include <k3dviewvolume.h>

#include <k3diapplication.h>
#include <k3dichannel.h>
#include <k3dihierarchy.h>
#include <k3direnderfarm.h>
#include <k3direnderframe.h>
#include <k3direnderjob.h>
#include <k3dirilight.h>
#include <k3dirimaterial.h>
#include <k3diritexture.h>
#include <k3ditexture.h>

#include <sdpgtkfileselector.h>

#include <iomanip>

/*
///////////////////////////////////////////////////////////////
// JobFunctor

class JobFunctor :
	public k3dIObjectVisitor
{
public:
	JobFunctor(k3dIRenderJob* const Job, k3dIRiEngine* const Engine) :
		m_Job(Job),
		m_Engine(Engine)
	{
		// Sanity checks ...
		g_assert(m_Job);
		g_assert(m_Engine);
	}

protected:
	k3dIRenderJob* const m_Job;
	k3dIRiEngine* const m_Engine;
};

/// Functor for calling k3dIRiRender::RiStartJob()
class RenderStartJob :
	public JobFunctor
{
public:
	RenderStartJob(k3dIRenderJob* const Job, k3dIRiEngine* const Engine) :
		JobFunctor(Job, Engine)
	{
	}

	bool VisitObject(k3dIObject* const Object)
	{
		k3dIRiRender* const render = dynamic_cast<k3dIRiRender*>(Object);
		if(render)
			render->RiStartJob(m_Job, m_Engine);

		return true;
	}
};

/// Functor for calling k3dIRiLight::RiLightStartJob()
class LightStartJob :
	public JobFunctor
{
public:
	LightStartJob(k3dIRenderJob* const Job, k3dIRiEngine* const Engine) :
		JobFunctor(Job, Engine)
	{
	}

	bool VisitObject(k3dIObject* const Object)
	{
		k3dIRiLight* const light = dynamic_cast<k3dIRiLight*>(Object);
		if(light)
			light->RiLightStartJob(m_Job, m_Engine);

		return true;
	}
};

/// Functor for calling k3dIRiTexture::RiTextureStartJob()
class TextureStartJob :
	public JobFunctor
{
public:
	TextureStartJob(k3dIRenderJob* const Job, k3dIRiEngine* const Engine) :
		JobFunctor(Job, Engine)
	{
	}

	bool VisitObject(k3dIObject* const Object)
	{
		k3dIRiTexture* const texture = dynamic_cast<k3dIRiTexture*>(Object);
		if(texture)
			texture->RiTextureStartJob(m_Job, m_Engine);

		return true;
	}
};

/// Functor for calling k3dIRiMaterial::RiMaterialStartJob()
class MaterialStartJob :
	public JobFunctor
{
public:
	MaterialStartJob(k3dIRenderJob* const Job, k3dIRiEngine* const Engine) :
		JobFunctor(Job, Engine)
	{
	}

	bool VisitObject(k3dIObject* const Object)
	{
		k3dIRiMaterial* const material = dynamic_cast<k3dIRiMaterial*>(Object);
		if(material)
			material->RiMaterialStartJob(m_Job, m_Engine);

		return true;
	}
};

/// Functor for calling k3dIRiRender::RiFinishJob()
class RenderFinishJob :
	public JobFunctor
{
public:
	RenderFinishJob(k3dIRenderJob* const Job, k3dIRiEngine* const Engine) :
		JobFunctor(Job, Engine)
	{
	}

	bool VisitObject(k3dIObject* const Object)
	{
		k3dIRiRender* const render = dynamic_cast<k3dIRiRender*>(Object);
		if(render)
			render->RiFinishJob(m_Job, m_Engine);

		return true;
	}
};

/// Functor for calling k3dIRiLight::RiLightFinishJob()
class LightFinishJob :
	public JobFunctor
{
public:
	LightFinishJob(k3dIRenderJob* const Job, k3dIRiEngine* const Engine) :
		JobFunctor(Job, Engine)
	{
	}

	bool VisitObject(k3dIObject* const Object)
	{
		k3dIRiLight* const light = dynamic_cast<k3dIRiLight*>(Object);
		if(light)
			light->RiLightFinishJob(m_Job, m_Engine);

		return true;
	}
};

/// Functor for calling k3dIRiTexture::RiTextureFinishJob()
class TextureFinishJob :
	public JobFunctor
{
public:
	TextureFinishJob(k3dIRenderJob* const Job, k3dIRiEngine* const Engine) :
		JobFunctor(Job, Engine)
	{
	}

	bool VisitObject(k3dIObject* const Object)
	{
		k3dIRiTexture* const texture = dynamic_cast<k3dIRiTexture*>(Object);
		if(texture)
			texture->RiTextureFinishJob(m_Job, m_Engine);

		return true;
	}
};

/// Functor for calling k3dIRiMaterial::RiMaterialFinishJob()
class MaterialFinishJob :
	public JobFunctor
{
public:
	MaterialFinishJob(k3dIRenderJob* const Job, k3dIRiEngine* const Engine) :
		JobFunctor(Job, Engine)
	{
	}

	bool VisitObject(k3dIObject* const Object)
	{
		k3dIRiMaterial* const material = dynamic_cast<k3dIRiMaterial*>(Object);
		if(material)
			material->RiMaterialFinishJob(m_Job, m_Engine);

		return true;
	}
};
*/

///////////////////////////////////////////////////////////////
// FrameFunctor

class FrameFunctor
{
public:
	FrameFunctor(k3dIRenderFrame* const Frame, k3dIRiEngine* const Engine) :
		m_Frame(Frame),
		m_Engine(Engine)
	{
		// Sanity checks ...
		g_assert(m_Frame);
		g_assert(m_Engine);
	}

protected:
	k3dIRenderFrame* const m_Frame;
	k3dIRiEngine* const m_Engine;
};

/// Functor for calling k3dIRiRender::RiPreRender()
class RenderPreRender :
	public FrameFunctor
{
public:
	RenderPreRender(k3dIRenderFrame* const Frame, k3dIRiEngine* const Engine) :
		FrameFunctor(Frame, Engine)
	{
	}

	void operator()(k3dIRiRender* const Render)
	{
		Render->RiSetSolid(false);
		Render->RiPreRender(m_Frame, m_Engine);
	}
};

/// Functor for calling k3dIRiLight::RiLightPreRender()
class LightPreRender :
	public FrameFunctor
{
public:
	LightPreRender(k3dIRenderFrame* const Frame, k3dIRiEngine* const Engine) :
		FrameFunctor(Frame, Engine)
	{
	}

	void operator()(k3dIRiLight* const Light)
	{
		Light->RiLightPreRender(m_Frame, m_Engine);
	}
};

/// Functor for calling k3dIRiTexture::RiTexturePreRender()
class TexturePreRender :
	public FrameFunctor
{
public:
	TexturePreRender(k3dIRenderFrame* const Frame, k3dIRiEngine* const Engine) :
		FrameFunctor(Frame, Engine)
	{
	}

	void operator()(k3dIRiTexture* const Texture)
	{
		Texture->RiTexturePreRender(m_Frame, m_Engine);
	}
};

/// Functor for calling k3dIRiMaterial::RiMaterialPreRender()
class MaterialPreRender :
	public FrameFunctor
{
public:
	MaterialPreRender(k3dIRenderFrame* const Frame, k3dIRiEngine* const Engine) :
		FrameFunctor(Frame, Engine)
	{
	}

	void operator()(k3dIRiMaterial* const Material)
	{
		Material->RiMaterialPreRender(m_Frame, m_Engine);
	}
};

/// Functor for calling k3dIRiLight::RiLightSetup()
class LightSetup :
	public FrameFunctor
{
public:
	LightSetup(k3dIRenderFrame* const Frame, k3dIRiEngine* const Engine) :
		FrameFunctor(Frame, Engine)
	{
	}

	void operator()(k3dIRiLight* const Light)
	{
		m_Engine->RiComment("Setup light [" + sdpString(dynamic_cast<k3dIObject*>(Light)->Name()) + "]");
		Light->RiLightSetup(m_Frame, m_Engine);
		m_Engine->RiNewline();
	}
};

/// Functor for calling k3dIRiRender::RiPostRender()
class RenderPostRender :
	public FrameFunctor
{
public:
	RenderPostRender(k3dIRenderFrame* const Frame, k3dIRiEngine* const Engine) :
		FrameFunctor(Frame, Engine)
	{
	}

	void operator()(k3dIRiRender* const Render)
	{
		Render->RiPostRender(m_Frame, m_Engine);
	}
};

/// Functor for calling k3dIRiLight::RiLightPostRender()
class LightPostRender :
	public FrameFunctor
{
public:
	LightPostRender(k3dIRenderFrame* const Frame, k3dIRiEngine* const Engine) :
		FrameFunctor(Frame, Engine)
	{
	}

	void operator()(k3dIRiLight* const Light)
	{
		Light->RiLightPostRender(m_Frame, m_Engine);
	}
};

/// Functor for calling k3dIRiTexture::RiTexturePostRender()
class TexturePostRender :
	public FrameFunctor
{
public:
	TexturePostRender(k3dIRenderFrame* const Frame, k3dIRiEngine* const Engine) :
		FrameFunctor(Frame, Engine)
	{
	}

	void operator()(k3dIRiTexture* const Texture)
	{
		Texture->RiTexturePostRender(m_Frame, m_Engine);
	}
};

/// Functor for calling k3dIRiMaterial::RiMaterialPostRender()
class MaterialPostRender :
	public FrameFunctor
{
public:
	MaterialPostRender(k3dIRenderFrame* const Frame, k3dIRiEngine* const Engine) :
		FrameFunctor(Frame, Engine)
	{
	}

	void operator()(k3dIRiMaterial* const Material)
	{
		Material->RiMaterialPostRender(m_Frame, m_Engine);
	}
};

/// A collection of shaders
typedef std::set<sdpsl::IShader*> ShaderSet;

/// Functor for retrieving the set of shaders currently in use
class UsedShaders :
	public ShaderSet,
	public k3dIObjectVisitor,
	public sdpsl::IShaderVisitor
{
public:
	virtual ~UsedShaders()
	{
	}

	// k3dIObjectVisitor implementation
	void VisitObject(k3dIObject* const Object)
	{
		sdpsl::IShaderCollection* const shaders = dynamic_cast<sdpsl::IShaderCollection*>(Object);
		if(!shaders)
			return;

		shaders->VisitShaders(this);
	}

	// sdpsl::IShaderVisitor implementation
	void VisitShader(sdpsl::IShader* const Shader)
	{
		insert(Shader);
	}
};

///////////////////////////////////////////////////////////////
// CRiEngine

CRiEngine::CRiEngine()
{
	m_Camera = 0;
}

CRiEngine::~CRiEngine()
{
}

bool CRiEngine::Render(k3dIRenderFrame* Frame, SDPCSTRING OutputImagePath, const bool VisibleRender)
{
	// Sanity checks ...
	g_return_val_if_fail(Frame, false);
	g_return_val_if_string_fail(OutputImagePath, false);

	// Make sure all shaders for this document are up-to-date ...
	UsedShaders shaders;
	m_Camera->Document()->Objects()->VisitObjects(&shaders);
	for(UsedShaders::iterator shader = shaders.begin(); shader != shaders.end(); shader++)
		if(!k3dUpdateShader(*shader, "ri", m_Camera->m_RiRenderEngine.Value()))
			g_return_val_if_fail(0, false);

	// Calculate the current time
	gdouble frametime = m_Camera->m_Time->ViewTime();
	gdouble totalframes = m_Camera->m_Time->Duration() * m_Camera->m_Time->FrameRate();
	gdouble actualtime = frametime;
	m_Camera->m_Time->SetRenderTime(actualtime);
	m_Camera->m_Time->SetRenderDeltaTime(1.0 / totalframes);

	// Animate the document
	m_Camera->Document()->Hierarchy()->Animate(actualtime, 1.0 / totalframes);
	k3dCalculateNormals(m_Camera->Document()->Objects());

	// Start our RIB file ...
	sdpString ribfilepath;
	gulong ribbufferlength = 1024;
	sdpResult result = Frame->AddInputFile("world.rib", ribfilepath.buffer(ribbufferlength), &ribbufferlength);
	if(SDP_S_BUFFER_TOO_SMALL == result)
		result = Frame->AddInputFile("world.rib", ribfilepath.buffer(ribbufferlength), &ribbufferlength);
	g_return_val_if_fail(SDP_S_OK == result, false);

	// Setup the frame for RI rendering with the user's preferred engine ...
	Frame->AddRenderOperation("ri", m_Camera->m_RiRenderEngine.Value(), sdpFile(ribfilepath), VisibleRender);

	// Open the RIB file stream ...
	std::ofstream ribfile(ribfilepath);
	g_return_val_if_fail(ribfile.good(), false);

	// Start the RIB file ...
	SetOutputStream(&ribfile);

	WriteRIBHeader();
	RiNewline();

	RiComment("Created with K-3D Version " + sdpToString(VERSION));
	RiNewline();

	// Set path options ...
	RiComment("Setup file search paths");

	k3dRiParameterList searchpaths;
	searchpaths.AddTokenValuePair(new k3dRiTokenValuePair("shader", new k3dRiStringArray("string", sdpMakePosixFilePath(k3dApplication()->ShadersPath()) + ":&")));
	searchpaths.DeclareParameters(this);
	searchpaths.SetOptions("searchpath", this);
	RiNewline();

	// Set limits options
	RiComment("Setup limits options");

	k3dRiIntegerArray* bucketsize = new k3dRiIntegerArray("integer[2]");
	bucketsize->Append(static_cast<gint>(m_Camera->m_RiXBucketSize.Value()));
	bucketsize->Append(static_cast<gint>(m_Camera->m_RiYBucketSize.Value()));

	k3dRiParameterList limits;
	limits.AddTokenValuePair(new k3dRiTokenValuePair("gridsize", new k3dRiIntegerArray("integer", static_cast<gint>(m_Camera->m_RiGridSize.Value()))));
	limits.AddTokenValuePair(new k3dRiTokenValuePair("bucketsize", bucketsize));
	limits.AddTokenValuePair(new k3dRiTokenValuePair("eyesplits", new k3dRiIntegerArray("integer", static_cast<gint>(m_Camera->m_RiEyeSplits.Value()))));
	limits.AddTokenValuePair(new k3dRiTokenValuePair("texturememory", new k3dRiIntegerArray("integer", static_cast<gint>(m_Camera->m_RiTextureMemory.Value()))));
	limits.DeclareParameters(this);
	limits.SetOptions("limits", this);
	RiNewline();

	// Set texture options...
	RiComment("Setup texture options");

	const gint enablelerpvalue = m_Camera->m_RiEnableLerp.Value() ? 1 : 0;
	const gint enablegaussianvalue = m_Camera->m_RiEnableGaussian.Value() ? 1 : 0;

	k3dRiParameterList texture;
	texture.AddTokenValuePair(new k3dRiTokenValuePair("enable lerp", new k3dRiIntegerArray("integer", static_cast<gint>(enablelerpvalue))));
	texture.AddTokenValuePair(new k3dRiTokenValuePair("enable gaussian", new k3dRiIntegerArray("integer", static_cast<gint>(enablegaussianvalue))));
	texture.DeclareParameters(this);
	texture.SetOptions("texture", this);
	RiNewline();

	// Setup custom attributes for use by lights ...
	RiComment("Setup light options");
	RiDeclare("shadows", "string");
	RiDeclare("casts_shadows", "string");
	RiNewline();

	// Set material options ...
	RiComment("Setup material options");

	RiDeclare("visibility", "integer");
	RiDeclare("emissioncolor", "color");
	RiDeclare("truedisplacement", "integer");
	RiDeclare("sphere", "float");
	RiDeclare("coordinatesystem", "string");
	RiNewline();

	// Set the display type ...
	RiComment("Setup Display");

	if(VisibleRender)
		RiDisplayV(sdpFile(OutputImagePath), k3dIRiEngine::RI_FRAMEBUFFER, k3dIRiEngine::RI_RGB, 0, 0, 0, 0);
	else
		RiDisplayV(sdpFile(OutputImagePath), k3dIRiEngine::RI_FILE, k3dIRiEngine::RI_RGB, 0, 0, 0, 0);

	RiFormat(static_cast<gint>(m_Camera->m_OutputPixelWidth.Value()), static_cast<gint>(m_Camera->m_OutputPixelHeight.Value()), m_Camera->m_OutputPixelAspectRatio.Value());

	// Set pixel sampling rates ...
	RiPixelSamples(m_Camera->m_RiXSamples.Value(), m_Camera->m_RiYSamples.Value());

	// Set pixel filtering ...
	RiPixelFilter(m_Camera->m_RiPixelFilterName.Value(), m_Camera->m_RiPixelFilterWidth.Value(), m_Camera->m_RiPixelFilterHeight.Value());

	// Set global shading rate ...
	RiShadingRate(m_Camera->m_RiShadingRate.Value());

	// Set global shading interpolation ...
	RiShadingInterpolation(m_Camera->m_RiShadingInterpolation.Value());

	// We use LH orientation, the way Our Lord Who Art In Heaven wants us to ...
	RiOrientation(RI_LH);

	// Double-sided surfaces ...
	RiSides(m_Camera->m_RiTwoSided.Value() ? 2 : 1);

	// Set gain & gamma ...
	RiExposure(m_Camera->m_Exposure->Value1(m_Camera->m_CurrentTime, m_Camera->m_DeltaTime),
						m_Camera->m_Gamma->Value1(m_Camera->m_CurrentTime, m_Camera->m_DeltaTime));

	// Set depth-of-field options ...
	if(m_Camera->m_RiDepthOfField.Value())
		RiDepthOfField(m_Camera->m_Fstop->Value1(m_Camera->m_CurrentTime, m_Camera->m_DeltaTime),
									m_Camera->m_Focallength->Value1(m_Camera->m_CurrentTime, m_Camera->m_DeltaTime),
									m_Camera->m_Focalplane->Value1(m_Camera->m_CurrentTime, m_Camera->m_DeltaTime));

	RiNewline();

	// Setup up viewing transformations
	RiComment("Setup viewing transformations");
	k3dRiViewVolumeSetup(m_Camera->ViewVolume(), this);

	sdpQuaternion q(m_Camera->m_CurrentOrientation);
	sdpEulerAngles a(q, sdpEulerAngles::ZXYstatic);
	RiRotate(-sdpDegrees(a[0]), 0.0f, 0.0f, 1.0f);
	RiRotate(-sdpDegrees(a[1]), 1.0f, 0.0f, 0.0f);
	RiRotate(-sdpDegrees(a[2]), 0.0f, 1.0f, 0.0f);
	RiTranslate(-m_Camera->m_CurrentPosition[0], -m_Camera->m_CurrentPosition[1], -m_Camera->m_CurrentPosition[2]);

	RiNewline();

	// If radiosity is enabled, set it up ...
	if(m_Camera->m_RiRadiosity.Value())
		{
			RiComment("Setup radiosity options");

			// Setup light attributes ...
			k3dRiParameterList radiosity(new k3dRiTokenValuePair("steps", new k3dRiIntegerArray("integer", static_cast<gint>(m_Camera->m_RiRadiositySteps.Value()))));
			radiosity.DeclareParameters(this);
			radiosity.SetOptions("radiosity", this);

		/*
			tokens[0] = "steps";
			intarg = m_Camera->m_RiRadiositySteps();
			pointers[0] = (RtPointer)&intarg;
			RiOptionV("radiosity", 1, tokens, pointers);

			intarg = m_Camera->m_RiRadiosityPatchSamples();
			RiOption("radiosity", "minpatchsamples", (RtInt*)&intarg, k3dIRiEngine::RI_NULL);

			RiDeclare("patchsize", "float");
			RiDeclare("elemsize", "float");
			RiDeclare("minsize", "float");

			floatarg = (RtFloat)m_Camera->m_RiRadiosityPatchSize();
			floatarg2 = (RtFloat)m_Camera->m_RiRadiosityElementSize();
			floatarg3 = (RtFloat)m_Camera->m_RiRadiosityMinimumSize();
			RiAttribute("radiosity", "patchsize", (RtFloat*)&floatarg, "elemsize", (RtFloat*)&floatarg2, "minsize", (RtFloat*)&floatarg3, k3dIRiEngine::RI_NULL);
		*/

			RiNewline();
		}

	// Begin the world ...
	RiComment("Start World");
	RiWorldBegin();

	// Setup our identity transformation matrix
	RiTransformBegin();
	RiIdentity();
	RiNewline();

	// Pre-render objects ...
	k3d::for_each<k3dIRiTexture>(m_Camera->Document()->Objects(), TexturePreRender(Frame, this));
	k3d::for_each<k3dIRiMaterial>(m_Camera->Document()->Objects(), MaterialPreRender(Frame, this));
	k3d::for_each<k3dIRiLight>(m_Camera->Document()->Objects(), LightPreRender(Frame, this));
	k3d::for_each<k3dIRiRender>(m_Camera->Document()->Objects(), RenderPreRender(Frame, this));

	// If atmospheric rendering is enabled, set it up ...
	if(m_Camera->m_RiAtmosphere.Value())
		{
			RiComment("Setup atmosphere options");

			k3dRiVolumeShader volumeshader;
			m_Camera->m_RiAtmosphereShader.InsertInto(Frame, this, m_Camera->m_CurrentTime, m_Camera->m_DeltaTime, volumeshader);
			volumeshader.SetupAtmosphere(this);

			RiNewline();
		}

	// Setup lights ...
	k3d::for_each<k3dIRiLight>(m_Camera->Document()->Objects(), LightSetup(Frame, this));
	RiNewline();

	// Render objects ...
	k3dIHierarchy* const hierarchy = m_Camera->Hierarchy();
	K3DHIERARCHYNODEHANDLE handle;
	for(k3dIHierarchy::NodeType type = hierarchy->FirstHierarchyNode(&handle); k3dIHierarchy::COMPLETE != type; type = hierarchy->NextHierarchyNode(&handle))
		{
			k3dIObject* object = hierarchy->HierarchyObject(handle);

			k3dIRiRender* render = dynamic_cast<k3dIRiRender*>(object);
			if(render)
				{
					switch(type)
						{
							case k3dIHierarchy::PUSH:
								RiComment("Render [" + sdpString(object->Name()) + "]");
								render->RiPushRender(Frame, this);
								break;

							case k3dIHierarchy::NODE:
								render->RiRender(Frame, this);
								render->RiPushChildRender(Frame, this);
								break;

							case k3dIHierarchy::POP:
								render->RiPopChildRender(Frame, this);
								render->RiPopRender(Frame, this);
								break;

							default:
								g_return_val_if_fail(0, false);
						}
				}
		}

	// PostRender objects ...
	k3d::for_each<k3dIRiRender>(m_Camera->Document()->Objects(), RenderPostRender(Frame, this));
	k3d::for_each<k3dIRiLight>(m_Camera->Document()->Objects(), LightPostRender(Frame, this));
	k3d::for_each<k3dIRiMaterial>(m_Camera->Document()->Objects(), MaterialPostRender(Frame, this));
	k3d::for_each<k3dIRiTexture>(m_Camera->Document()->Objects(), TexturePostRender(Frame, this));

	// Finished transforming ...
	RiComment("Finish World");
	RiTransformEnd();

	// Finish the world ...
	RiWorldEnd();

	// Finish the RIB file ...
	ribfile.close();

	return true;
}

/////////////////////////////////////////////////////////////////
// k3dIRenderEngine overrides ...

sdpResult CRiEngine::SetCamera(k3dICamera* Camera)
{
	// Sanity checks ...
	g_return_val_if_fail(Camera, SDP_E_FAIL);

	m_Camera = (CCamera*)Camera;

	return SDP_S_OK;
}

sdpResult CRiEngine::Redraw()
{
	// Sanity checks ...
	g_return_val_if_fail(m_Camera, SDP_E_FAIL);

	// Not gonna do it!
	return SDP_S_FALSE;
}

sdpResult CRiEngine::RenderPreview()
{
	k3dTimeController time(m_Camera->Document()->Time());

	// Start a new render job ...
	k3dIRenderJob* const job = k3dApplication()->RenderFarm()->CreateJob("k3d-preview", k3dIRenderFarm::IMMEDIATE_PRIORITY);
	g_return_val_if_fail(job, SDP_E_FAIL);

	// Add a single render frame to the job ...
	std::strstream buffer;
	buffer << "frame" << std::setw(6) << std::setfill('0') << time.ViewFrame() << std::ends;
	k3dIRenderFrame* const frame = job->CreateFrame(buffer.str());
	buffer.freeze(false);
	g_return_val_if_fail(frame, SDP_E_FAIL);

	// Create an output image path ...
	gulong outputimagebuffer = 1024;
	sdpString outputimagepath;
	if(SDP_S_BUFFER_TOO_SMALL == frame->AddOutputFile("outputimage", outputimagepath.buffer(outputimagebuffer), &outputimagebuffer))
		frame->AddOutputFile("outputimage", outputimagepath.buffer(outputimagebuffer), &outputimagebuffer);

	// Render it (visible rendering) ...
	g_return_val_if_fail(Render(frame, outputimagepath, true), SDP_E_FAIL);

	// Start the job running ...
	return k3dApplication()->RenderFarm()->StartJob(job);
}

sdpResult CRiEngine::RenderFrame()
{
	sdpString filepath;
	if(!k3dGetFilePath("renderframe", "Render Frame:", true, 0, filepath))
		return SDP_S_FALSE;

	k3dTimeController time(m_Camera->Document()->Time());

	// Start a new render job ...
	k3dIRenderJob* const job = k3dApplication()->RenderFarm()->CreateJob("k3d-render-frame", k3dIRenderFarm::HIGHEST_PRIORITY);
	g_return_val_if_fail(job, SDP_E_FAIL);

	// Add a single render frame to the job ...
	std::strstream buffer;
	buffer << "frame" << std::setw(6) << std::setfill('0') << time.ViewFrame() << std::ends;
	k3dIRenderFrame* const frame = job->CreateFrame(buffer.str());
	buffer.freeze(false);
	g_return_val_if_fail(frame, SDP_E_FAIL);

	// Create an output image path ...
	gulong outputimagebuffer = 1024;
	sdpString outputimagepath;
	if(SDP_S_BUFFER_TOO_SMALL == frame->AddOutputFile("outputimage", outputimagepath.buffer(outputimagebuffer), &outputimagebuffer))
		frame->AddOutputFile("outputimage", outputimagepath.buffer(outputimagebuffer), &outputimagebuffer);

	// Copy the output image to its requested destination ...
	frame->AddCopyOperation(outputimagepath, filepath);

	// View the output image when it's done ...
	frame->AddViewOperation(filepath);

	// Render it (hidden rendering) ...
	g_return_val_if_fail(Render(frame, outputimagepath, false), SDP_E_FAIL);

	// Start the job running ...
	return k3dApplication()->RenderFarm()->StartJob(job);
}

sdpResult CRiEngine::RenderAnimation()
{
	sdpString filepath;
	if(!k3dGetFilePath("renderanimation", "Choose Animation Base Filepath:", false, 0, filepath))
		return SDP_S_FALSE;

	// Setup a k3dFrameRange object for generating frame filepaths ...
	k3dTimeController time(m_Camera->Document()->Time());
	k3dFrameRange framerange(filepath, time.StartFrame(), time.EndFrame());
	if(framerange.MaxFrame() < time.EndFrame())
		{
			SDPCSTRING message =	"The Base Filepath doesn't contain enough digits to render the entire animation.\n"
														"Try a filepath of the form [ myanim0000.tif ] ... the placement of digits is flexible,\n"
														"and any prefix / postfix / file extension is optional, but the path must contain\n"
														"enough consecutive digits to enumerate all of the frames in the animation.";


			k3dApplication()->Error(message, "Render Animation:");
			return SDP_S_FALSE;
		}

	// See if the user wants to view frames as they're completed ...
	sdpStringArray buttons;
	buttons.push_back("Yes");
	buttons.push_back("No");
	buttons.push_back("Cancel");

	const gulong result = k3dApplication()->Query("Do you want to see rendered frames as they're completed?", "Render Animation:", 1, buttons);
	if(0 == result || 3 == result)
		return SDP_S_FALSE;

	const bool viewcompleted = (1 == result);

	// Start a new render job ...
	k3dIRenderJob* const job = k3dApplication()->RenderFarm()->CreateJob("k3d-render-animation", k3dIRenderFarm::MODERATE_PRIORITY);
	g_return_val_if_fail(job, SDP_E_FAIL);

	// For each frame to be rendered ...
	for(gulong animationframe = time.StartFrame(); animationframe < time.EndFrame(); animationframe++)
		{
			// Set the frame time ...
			time.SetViewFrame(animationframe);

			// Redraw everything ...
			k3dRedrawAll(m_Camera->Document(), true);

			// Add a render frame to the job ...
			std::strstream buffer;
			buffer << std::setw(6) << std::setfill('0') << time.ViewFrame() << std::ends;
			k3dIRenderFrame* const frame = job->CreateFrame(buffer.str());
			buffer.freeze(false);
			g_return_val_if_fail(frame, SDP_E_FAIL);

			// Create an output image path ...
			gulong outputimagebuffer = 1024;
			sdpString outputimagepath;
			if(SDP_S_BUFFER_TOO_SMALL == frame->AddOutputFile("outputimage", outputimagepath.buffer(outputimagebuffer), &outputimagebuffer))
				frame->AddOutputFile("outputimage", outputimagepath.buffer(outputimagebuffer), &outputimagebuffer);

			// Copy the output image to its requested destination ...
			sdpString destination;
			framerange.Frame(animationframe, destination);
			frame->AddCopyOperation(outputimagepath, destination);

			// View the output image when it's done ...
			if(viewcompleted)
				frame->AddViewOperation(destination);

			// Render it (hidden rendering) ...
			g_return_val_if_fail(Render(frame, outputimagepath, false), SDP_E_FAIL);
		}

	// Start the job running ...
	return k3dApplication()->RenderFarm()->StartJob(job);
}

///////////////////////////////////////////////////////////////
// k3dIRiEngine overrides ...

k3dICamera* CRiEngine::Camera()
{
	return m_Camera;
}








