#ifndef TORUS_H
#define TORUS_H

// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the CTorus K-3D object, which renders a RenderMan sphere primitive
		\author Tim Shead (tshead@k-3d.com)
*/

// Forward class declarations ...
class CTorus;

#include <k3dobjectdialog.h>
#include <k3dtransformobject.h>
#include <k3dvariable.h>

#include <k3dimaterialcollection.h>
#include <k3disnaptarget.h>

/////////////////////////////////////////////////////////////////////////////
// CTorusDialog

typedef k3dObjectDialog CTorusDialogBase;

class CTorusDialog :
	public CTorusDialogBase
{
public:
	CTorusDialog(CTorus* const Object);

protected:
	CTorus* const m_Object;

	void OnEvent(sdpGtkEvent* Event);

	static const sdpString dialogname;
	static const sdpString control_majorradius;
	static const sdpString control_minorradius;
	static const sdpString control_phimin;
	static const sdpString control_phimax;
	static const sdpString control_sweepangle;
	static const sdpString control_position;
	static const sdpString control_orientation;
	static const sdpString control_scale;
	static const sdpString control_target;
};


/////////////////////////////////////////////////////
// CTorus

typedef k3dTransformObject CTorusBase;

class CTorus :
	public CTorusBase,
	public k3dIMaterialCollection
{
friend class CTorusDialog;
public:
	CTorus(sdpIUnknown* Initializer);
	~CTorus();

	// k3dIObject implementation
	sdpResult EditObject();

	// k3dIEditorRender implementation
	sdpResult EditorRender(k3dIEditorEngine* const Engine);
	
	// k3dIRiRender implementation
	sdpResult RiRender(k3dIRenderFrame* const Frame, k3dIRiEngine* const Engine);

	// k3dIMaterialCollection implementation
	k3dIMaterial* Material();
	sdpResult SetMaterial(k3dIMaterial* const Material);

	// k3dIPersist implementation
	sdpResult InitComplete();

protected:
	/// Stores our assigned material
	k3dObjectInterfaceVariable<k3dIMaterial> m_Material;

	k3dScalarChannel m_MajorRadius;
	k3dScalarChannel m_MinorRadius;
	k3dScalarChannel m_PhiMin;
	k3dScalarChannel m_PhiMax;
	k3dScalarChannel m_SweepAngle;
};


#endif // TORUS_H
