// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Implements CBooleanOps, a K-3D implementing boolean operations for meshes
		\author Romain Behar (romainbehar@yahoo.com)
*/

#include "booleanops.h"

#include <k3dchooser.h>
#include <k3dmisc.h>
#include <k3doptions.h>

#include <k3diapplication.h>
#include <k3distaterecorder.h>
#include <k3dicamera.h>
#include <k3dilocation.h>
#include <k3dimaterialcollection.h>
#include <k3dipathpointcollection.h>

#include <sdpgtkevents.h>

#include <algorithm>
#include <iterator>
#include <iostream>
#include <numeric>
#include <utility>

#include "csg.h"

CBooleanOps::PathProxyA::PathProxyA(CBooleanOps& Object, k3d::GeometryEngine& Engine, k3d::EngineMeshProxy& Mesh) :
	PathProxyBase(Engine, Mesh),
	m_Object(Object)
{
}

CBooleanOps::PathProxyA::PathProxyA(CBooleanOps& Object, k3d::GeometryEngine& Engine, k3dIPath* const Path) :
	PathProxyBase(Engine, Path),
	m_Object(Object)
{
}

CBooleanOps::PathProxyB::PathProxyB(CBooleanOps& Object, k3d::GeometryEngine& Engine, k3d::EngineMeshProxy& Mesh) :
	PathProxyBase(Engine, Mesh),
	m_Object(Object)
{
}

CBooleanOps::PathProxyB::PathProxyB(CBooleanOps& Object, k3d::GeometryEngine& Engine, k3dIPath* const Path) :
	PathProxyBase(Engine, Path),
	m_Object(Object)
{
}


void CBooleanOps::PathProxyA::BuildSolidA()
{
	SolidA = new SOLID();
	SolidA->id = 0;
	SolidA->filename = "fileA";

	for(PointCollection::iterator point = m_Points.begin(); point != m_Points.end(); ++point)
		{
			k3dIPoint* const newpoint = (*point)->Actual();
			std::cout << newpoint->Location()->LocalXYZ() << std::endl;
		}


	SolidA->DataCloseCallback();
}

/*
void CBooleanOps::PathProxy::BuildGeometry()
{
	// Skip anything with < 3 sides ...
	if(m_Points.size() >= 3)
		{
			// Create a new point for each existing point ...
			for(PointCollection::iterator point = m_Points.begin(); point != m_Points.end(); ++point)
				m_NewPoints.push_back(&Engine().Point(Mesh()));

			// Create a new face path ...
			if(m_Object.m_CreateFaces.Value())
				{
					m_FacePaths.push_back(&Engine().Path(Mesh()));
					for(PointCollection::iterator newpoint = m_NewPoints.begin(); newpoint != m_NewPoints.end(); ++newpoint)
						m_FacePaths.back()->Actual()->Points()->AddPoint((*newpoint)->Actual());
				}

			// Create new edge paths ...
			if(m_Object.m_CreateEdges.Value())
				{
					for(gulong i = 0; i < m_Points.size(); i++)
						{
							m_EdgePaths.push_back(&Engine().Path(Mesh()));
					
							m_EdgePaths.back()->Actual()->Points()->AddPoint(m_Points[(i+0) % m_Points.size()]->Actual());
							m_EdgePaths.back()->Actual()->Points()->AddPoint(m_Points[(i+1) % m_Points.size()]->Actual());
							m_EdgePaths.back()->Actual()->Points()->AddPoint(m_NewPoints[(i+1) % m_NewPoints.size()]->Actual());
							m_EdgePaths.back()->Actual()->Points()->AddPoint(m_NewPoints[(i+0) % m_NewPoints.size()]->Actual());
						}
				}

/
			// Mark the original for deletion ...
			if(m_Object.m_DeleteOriginals.Value())
				MarkForDeletion();
/
		}

	PathProxyBase::BuildGeometry();
}

void CBooleanOps::PathProxy::UpdateGeometry()
{
	// Calculate some generic data about the path ...
	k3d::Dimensions dimensions;
	k3d::copy(Actual()->Points(), k3d::Dimensions::Inserter(dimensions));
	
	// Get the path center and normal vector ...
	const sdpVector3 center = dimensions.Center();
	const sdpVector3 normal = dimensions.Normal();
	
	// Calculate an offset perpendicular to the path face ...
	const sdpVector3 normaloffset = normal * m_Object.m_Offset.Value();

	// Set new point positions ...
	if(CBooleanOps::SCALE == m_Object.m_InsetType.Value())
		{
			PointCollection::iterator newpoint = m_NewPoints.begin();
			for(PointCollection::iterator point = m_Points.begin(); point != m_Points.end(); ++point)
				(*newpoint++)->Actual()->Location()->SetLocalXYZ(sdpLerp(center, (*point)->Actual()->Location()->LocalXYZ(), m_Object.m_Scale.Value()) + normaloffset);
		}
	else
		{
			for(gulong i = 0; i < m_Points.size(); i++)
				{
				
					// Get the point we're going to offset from ...
					const sdpVector3 point2 = m_Points[i]->Actual()->Location()->LocalXYZ();

					// Look for the first point preceeding it by a non-zero distance ...
					sdpVector3 point1;
					for(gulong p1 = (i + m_Points.size()-1) % m_Points.size(); p1 != i; p1 = (p1 + m_Points.size()-1) % m_Points.size())
						{
							point1 = m_Points[p1]->Actual()->Location()->LocalXYZ();
							
							if(point1 != point2)
								break;
						}
					
					// Look for the first point following it by a non-zero distance ...
					sdpVector3 point3;
					for(gulong p3 = (i + 1) % m_Points.size(); p3 != i; p3 = (p3 + 1) % m_Points.size())
						{
							point3 = m_Points[p3]->Actual()->Location()->LocalXYZ();
							
							if(point3 != point2)
								break;
						}
				
					const sdpVector3 edge1 = point2 - point1;
					const sdpVector3 edge2 = point3 - point2;

					sdpVector3 edge3 = edge1 ^ normal; // Cross product
					sdpVector3 edge4 = edge2 ^ normal; // Cross product

					edge3.Normalize();
					edge4.Normalize();

					sdpVector3 inset = edge3 + edge4;
					inset.Normalize();
					inset *= m_Object.m_Inset.Value();

					// Calculate angle adjustment
					const gdouble theta = acos((edge1 * edge2) / (edge1.Length() * edge2.Length()));
					inset *= 1.0 / cos(theta * 0.5);

					m_NewPoints[i]->Actual()->Location()->SetLocalXYZ(m_Points[i]->Actual()->Location()->LocalXYZ() + normaloffset + inset);
				}
		}
	

	PathProxyBase::UpdateGeometry();
}

void CBooleanOps::PathProxy::Cancel()
{
	m_FacePaths.clear();
	m_EdgePaths.clear();
	m_NewPoints.clear();
	
	PathProxyBase::Cancel();
}
*/



///////////////////////////////////////////////////////////////
// CBooleanOps

CBooleanOps::CBooleanOps(sdpIUnknown* Initializer) :
	CBooleanOpsBase(Initializer),
	m_Operation("operation", INTERSECTION, Document()->StateRecorder(), 0),
	m_EngineA(
		Document(),
		new k3d::EngineMeshProxyFactory<k3d::EngineMeshProxy>(),
		new k3d::EnginePathProxyFactory2<CBooleanOps, CBooleanOps::PathProxyA>(*this),
		new k3d::EnginePointProxyFactory<k3d::EnginePointProxy>(),
		new k3d::EngineEdgeProxyFactory<k3d::EngineEdgeProxy>()
		),
	m_EngineB(
		Document(),
		new k3d::EngineMeshProxyFactory<k3d::EngineMeshProxy>(),
		new k3d::EnginePathProxyFactory2<CBooleanOps, CBooleanOps::PathProxyB>(*this),
		new k3d::EnginePointProxyFactory<k3d::EnginePointProxy>(),
		new k3d::EngineEdgeProxyFactory<k3d::EngineEdgeProxy>()
		)
{
}

///////////////////////////////////////////////////////////////
// k3dIObject implementation

sdpResult CBooleanOps::EditObject()
{
	new Properties(this);

	BuildGeometry();
	k3dRedrawAll(Document(), false);

	return SDP_S_OK;
}

void CBooleanOps::BuildGeometry()
{
	// Get rid of any old geometry
	CancelGeometry();

	// Create new meshes, keeping track of them for undo purposes ...
	m_NewGeometry.StartChangeSet(Document());
	//CreateNewellMeshes(Document(), m_PrimitiveType.Value(), m_NewGeometry);
	m_NewGeometry.FinishChangeSet(Document());

	// Add geometry ...
	m_NewGeometry.PushEditing();
	//CreateNewellGeometry(Document(), m_PrimitiveType.Value(), static_cast<gulong>(m_Subdivisions.Value()), m_NewGeometry);
	m_NewGeometry.PopEditing();

	UpdateGeometry();
	m_NewGeometry.Manage();

	Notify(k3dObject::Event_UserMessage1, m_NewGeometry.Description());
}

void CBooleanOps::UpdateGeometry()
{
	std::for_each(m_NewGeometry.Points().begin(), m_NewGeometry.Points().end(), std::mem_fun(&k3d::NewPointProxy::PushEditing));
	//std::for_each(m_NewGeometry.Points().begin(), m_NewGeometry.Points().end(), NewellPoints(m_Scale.Value()));
	std::for_each(m_NewGeometry.Points().begin(), m_NewGeometry.Points().end(), std::mem_fun(&k3d::NewPointProxy::PopEditing));
}

void CBooleanOps::CancelGeometry()
{
	m_NewGeometry.Cancel();
}

void CBooleanOps::FinishGeometry()
{
	m_NewGeometry.Finish(Document(), "Create Boolean Oped Object");
}

void CBooleanOps::SetObjectA()
{
	//m_ObjectAPaths.clear();
	//m_ObjectAPoints.clear();

	//k3dGetPaths(Document(), k3dIState::VISIBLESELECTED, m_ObjectAPaths);
	//k3dGetPoints(Document(), k3dIState::VISIBLESELECTED, m_ObjectAPoints);

	m_EngineA.Cancel();
	k3d::GetSelectedPaths(Document()->Objects(), std::back_inserter(m_SelectionA));
	std::copy(m_SelectionA.begin(), m_SelectionA.end(), k3d::GeometryEngine::Inserter(m_EngineA));
}

void CBooleanOps::SetObjectB()
{
	//m_ObjectBPaths.clear();
	//m_ObjectBPoints.clear();

	//k3dGetPaths(Document(), k3dIState::VISIBLESELECTED, m_ObjectBPaths);
	//k3dGetPoints(Document(), k3dIState::VISIBLESELECTED, m_ObjectBPoints);

	m_EngineB.Cancel();
	k3d::GetSelectedPaths(Document()->Objects(), std::back_inserter(m_SelectionB));
	std::copy(m_SelectionB.begin(), m_SelectionB.end(), k3d::GeometryEngine::Inserter(m_EngineB));
}

/*
typedef std::pair<gulong, gulong> CSGlibEdgeBase;
class CSGlibEdge :
	public CSGlibEdgeBase
{
public:
	CSGlibEdge(const gulong Index1, const gulong Index2) :
		CSGlibEdgeBase(Index1 < Index2 ? Index1 : Index2, Index1 < Index2 ? Index2 : Index1)
	{
	}
};

/// Converts a set of K-3D paths and points into CSGLib-compatible data structures
class CSGlibObject
{
public:
	CSGlibObject(const k3dPathArray& Paths, const k3dPointArray& Points) :
		m_Object(new Object())
	{
		// Convert K-3D points to CSGLib nodes, and create a 0-based mapping ...
		std::vector<Node*> nodes;
		std::map<k3dIPoint*, gulong> indexlookup;

		for(k3dPointArray::const_iterator point = Points.begin(); point != Points.end(); point++)
			{
				sdpVector3 coords;
				(*point)->Location()->WorldXYZ(coords);
				nodes.push_back(new Node(Point(coords[0], coords[1], coords[2])));
				indexlookup[*point] = nodes.size()-1;
			}

		// Keep track of edges as we go ...
		std::map<CSGlibEdge, CSGCnxn*> edges;
		// Closed-surface test : each edge should belong to 2 and only 2 faces
		std::map<CSGlibEdge, gulong> edgetest;


		// Edges, connections, faces and object
		for(gulong i = 0; i < Paths.size(); i++)
			{
				// Skip degenerate cases ...
				k3dIPathPointCollection* const pathpoints = Paths[i]->Points();
				if(pathpoints->PointCount() < 3)
					continue;

				// Create a face structure for this path ...
				Face* const face = new Face();

				// Save points to create face's plane
				std::vector<Point> facepoints;

				// For each pathpoint in the path ...
				K3DPATHPOINTHANDLE pathpointhandle = pathpoints->FirstPathPoint();
				K3DPATHPOINTHANDLE pathpointnexthandle = pathpoints->NextPathPoint(pathpointhandle);
				while(pathpointhandle)
					{
						// Create an edge ...
						k3dIPoint* const point1 = pathpoints->PathPoint(pathpointhandle)->Point();
						if(!pathpointnexthandle)
							pathpointnexthandle = pathpoints->FirstPathPoint();
						k3dIPoint* const point2 = pathpoints->PathPoint(pathpointnexthandle)->Point();
						const gulong index1 = indexlookup[point1];
						const gulong index2 = indexlookup[point2];
						const CSGlibEdge edge(index1, index2);

						// Create a CSGlib edge ...
						CSGCnxn* cnxn = 0;
						if(edges.find(edge) != edges.end())
							{
								cnxn = edges[edge];
								edgetest[edge]++;
							}
						else
							{
								cnxn = newCSGCnxn(nodes[index1], nodes[index2]);
								edges[edge] = cnxn;
								edgetest[edge] = 1;
							}

						cnxn->addFace(face);
						face->addCnxn(cnxn);
						Node* node1 = nodes[index1];
						face->addNode(node1);
						facepoints.push_back(node1->data);
						if(pathpointnexthandle == pathpoints->FirstPathPoint())
							break;

						pathpointhandle = pathpointnexthandle;
						if(pathpointnexthandle)
							pathpointnexthandle = pathpoints->NextPathPoint(pathpointhandle);
					}

				// Face plane (a path is coplanar)
				face->pl = Plane(facepoints[0], facepoints[1], facepoints[2]);

				m_Object->addFace(face);
			}

//		std::map<CSGlibEdge, gulong>::iterator closedsurface = std::find_if(edgetest.begin(), edgetest.end(), std::bind2nd(not_equal_to<gulong>(), 2));
		std::map<CSGlibEdge, gulong>::iterator test = edgetest.begin();
		for(; test != edgetest.end(); test++)
			if((*test).second != 2)
				break;

		if(test != edgetest.end())
			std::cerr << std::endl << " NON CLOSED SURFACE ";
	}

	CSGlibObject()
	{
		clean(m_Object);
	}

	operator Object*()
	{
		return m_Object;
	}

private:
	Object* const m_Object;
};

class nextcnxn
{
public:
	nextcnxn(Cnxn* const CurrentCnxn, Node* const CurrentNode) :
		m_CurrentCnxn(CurrentCnxn),
		m_CurrentNode(CurrentNode)
	{
	}

	bool operator()(Cnxn* const Cnxn)
	{
		return (m_CurrentCnxn != Cnxn) && ((m_CurrentNode == Cnxn->iCnxn) || (m_CurrentNode == Cnxn->oCnxn));
	}

private:
	Cnxn* const m_CurrentCnxn;
	Node* const m_CurrentNode;
};

/// Converts a CSGLib object to a K-3D mesh
bool CSGlib2K3D(k3dIDocument* const Document, const Object* const CSGObject, k3dMeshArray& NewMeshes, k3dPathArray& NewPaths, k3dPointArray& NewPoints)
{
//std::cout << "Edges: " << CSGObject->cnxns.size() << std::endl;
//std::cout << "Nodes: " << CSGObject->nodes.size() << std::endl;
//std::cout << "Faces: " << CSGObject->faces.size() << std::endl;

	// Create a new mesh ...
	k3dIObject* const object = Document->CreateObject(sdpMesh);
	g_return_val_if_fail(object, false);

	k3dIMesh* const mesh = dynamic_cast<k3dIMesh*>(object);
	g_return_val_if_fail(mesh, false);

	NewMeshes.push_back(mesh);

	// Create vertices ...
	std::map<Node*, k3dIPoint*> k3dpointmap;
	for(std::set<Node*>::iterator node = CSGObject->nodes.begin(); node != CSGObject->nodes.end(); node++)
		{
			k3dIPoint* const newpoint = mesh->CreatePoint();
			g_return_val_if_fail(newpoint, false);
			NewPoints.push_back(newpoint);

			Point nodepoint = (*node)->data;
			sdpVector3 coords(nodepoint[0], nodepoint[1], nodepoint[2]);
			newpoint->Location()->SetLocalXYZ(coords);

			k3dpointmap[*node] = newpoint;
		}

	// Create paths ...
	for(std::slist<Face*>::const_iterator face = CSGObject->faces.begin(); face != CSGObject->faces.end(); face++)
		{
//std::cout << "  face with " << (*face)->cnxns.size() << " edges" << std::endl;

			// Skip degenerate cases ...
			std::set<Cnxn*>& cnxns((*face)->cnxns);
			if(cnxns.size() < 3)
				continue;

			k3dIPath* const newpath = mesh->CreatePath();
			g_return_val_if_fail(newpath, false);
			NewPaths.push_back(newpath);

			// Extract edge data (unfortunately, it isn't in any particular order) ...
			std::set<Cnxn*>::iterator cnxn = cnxns.begin();
			Node* currentnode = (*cnxn)->oCnxn;


// Ugly tweak


			// Find face orientation

			sdpVector3 point1, point2, point3;
			(k3dpointmap[currentnode])->Location()->LocalXYZ(point2);
			Node* firstnode = (*cnxn)->iCnxn;
			(k3dpointmap[firstnode])->Location()->LocalXYZ(point1);

			Node* nextnode = (*cnxn)->oCnxn;
			cnxn = std::find_if(cnxns.begin(), cnxns.end(), nextcnxn(*cnxn, nextnode));
			if((*cnxn)->iCnxn == nextnode)
				nextnode = (*cnxn)->oCnxn;
			else
				nextnode = (*cnxn)->iCnxn;

			(k3dpointmap[nextnode])->Location()->LocalXYZ(point3);
			sdpVector3 fnorm = (point2-point1)^(point3-point1);
			Point planepoint = (*face)->pl.n;
			sdpVector3 fplanenorm = sdpVector3(planepoint[0], planepoint[1], planepoint[2]);
			bool invertface = false;
			if(fnorm*fplanenorm < 0)
				invertface = true;

			cnxn = cnxns.begin();

//

			newpath->Points()->AddPoint(k3dpointmap[currentnode]);
			while(1)
				{
					cnxn = std::find_if(cnxns.begin(), cnxns.end(), nextcnxn(*cnxn, currentnode));
					if(cnxn == cnxns.begin() || cnxn == cnxns.end())
						break;

					if((*cnxn)->iCnxn == currentnode)
						currentnode = (*cnxn)->oCnxn;
					else
						currentnode = (*cnxn)->iCnxn;

					newpath->Points()->AddPoint(k3dpointmap[currentnode]);
				}

			if(invertface)
				newpath->FlipOrientation();
		}

	return true;
}

void CBooleanOps::UpdateGeometry()
{
	// Remove old geometry as necessary ...
	if(CreateGeometry)
		DeleteAll();

	// If we don't enough geometry, we're done ...
	if(0 == m_APaths.size() || 0 == m_APoints.size() || 0 == m_BPaths.size() || 0 == m_BPoints.size())
		return;

	// Create new geometry
	if(CreateGeometry)
		{
			if(SaveUndoAction)
				k3dStartStateChangeSet(Document());

			k3dMeshArray newmeshes;
			k3dPathArray newpaths;
			k3dPointArray newpoints;

			// Deal with world coordinates
			Document()->SnapshotWorldCoordinates();

			// Convert our two objects to something CSGlib can use ...
std::cerr << std::endl << " Init object A";
			CSGlibObject objecta(m_APaths, m_APoints);
std::cerr << std::endl << " Init object B";
			CSGlibObject objectb(m_BPaths, m_BPoints);

std::cerr << std::endl << " Intersect objects";
			// Calculate the intersection of the two objects ...
			IntersectionResult* const result = intersect(objecta, objectb);

std::cerr << std::endl << " Done";
			// Choose which parts of the intersection we want ...
			Object* newobject = 0;
			switch(m_Operation.Value())
				{
					case CBooleanOps::UNION:
						newobject = result->aUnionB();
						break;
					case CBooleanOps::INTERSECTION:
						newobject = result->aAndB();
						break;
					case CBooleanOps::DIFFERENCE:
						newobject = result->aNotB();
						break;
					default:
						g_return_if_fail(0);
				}
			g_assert(newobject);

			// Convert back to K-3D data structures ...
			CSGlib2K3D(Document(), newobject, newmeshes, newpaths, newpoints);
			// test CSGlib2K3D return value?

			if(SaveUndoAction)
				k3dFinishStateChangeSet(Document(), "BooleanOps");

			gulong i;
			for(i = 0; i < newmeshes.size(); i++)
				ManageMesh(newmeshes[i], "BooleanOps");

			for(i = 0; i < newpaths.size(); i++)
				BumpPathCount();
//				AddPath(newpaths[i]);

			for(i = 0; i < newpoints.size(); i++)
				BumpPointCount();
//				AddPoint(newpoints[i]);

			// Set materials (we can't use k3dPrimTool::SetDefaultMaterials() because the paths aren't managed)
			k3dIMaterial* const defaultmaterial = k3dDefaultMaterial(Document());
			for(i = 0; i < newpaths.size(); i++)
				newpaths[i]->Materials()->SetMaterial(defaultmaterial);
		}

	Notify(k3dIObject::Event_UpdateUI, 0);
}
*/


////////////////////////////////////////////////////////////////////////////
// CBooleanOps::Properties

const sdpString CBooleanOps::Properties::dialogname = "properties";
const sdpString CBooleanOps::Properties::control_setobjecta = "setobjecta";
const sdpString CBooleanOps::Properties::control_setobjectb = "setobjectb";

CBooleanOps::Properties::Properties(CBooleanOps* const Object) :
	PropertiesBase(Object, false, true, false, "properties", new k3dOptionsWindowGeometryStore(k3dApplication(), 0)),
	m_Object(Object)
{
	// Load the dialog template ...
	g_return_if_fail(LoadGTKMLTemplate("experimentallib", "booleanops.gtkml"));

	AddCommand(control_setobjecta, control_setobjecta, CONTROL_BUTTON);
	AddCommand(control_setobjectb, control_setobjectb, CONTROL_BUTTON);

	// Setup chooser...
	if(Getk3dChooser("operation"))
		Getk3dChooser("operation")->Attach(new k3dChooserVariableDataSource<CBooleanOps::Operation>(m_Object->m_Operation), k3dApplication()->CommandRecorder(), 0, "Operation", 0, new k3dActionMethodCall<CBooleanOps>(m_Object, &CBooleanOps::BuildGeometry, new k3dActionRedrawAll(m_Object->Document(), false, 0)), 0);

	Show();
}

void CBooleanOps::Properties::OnOK()
{
//	m_Object->FinishGeometry();
	k3dRedrawAll(m_Object->Document(), false);

	PropertiesBase::OnOK();
}

void CBooleanOps::Properties::OnCancel()
{
//	m_Object->CancelGeometry();
	k3dRedrawAll(m_Object->Document(), false);

	PropertiesBase::OnCancel();
}

void CBooleanOps::Properties::OnEvent(sdpGtkEvent* Event)
{
	// Sanity checks ...
	g_assert(Event);

	if(Event->Name() == control_setobjecta)
		OnSetObjectA();
	else if(Event->Name() == control_setobjectb)
		OnSetObjectB();
	else
		PropertiesBase::OnEvent(Event);
}

void CBooleanOps::Properties::OnSetObjectA()
{
	k3dRecordCommand(this, k3dICommandNode::USERINTERFACE, control_setobjecta, 0);

	m_Object->SetObjectA();
	k3dDeselectAll(m_Object->Document());

	m_Object->BuildGeometry();
	k3dRedrawAll(m_Object->Document(), false);
}

void CBooleanOps::Properties::OnSetObjectB()
{
	k3dRecordCommand(this, k3dICommandNode::USERINTERFACE, control_setobjectb, 0);

	m_Object->SetObjectB();
	k3dDeselectAll(m_Object->Document());

	m_Object->BuildGeometry();
	k3dRedrawAll(m_Object->Document(), false);
}


