#ifndef BOOLEANOPS_H
#define BOOLEANOPS_H

// K-3D
// Copyright  1995-2002, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares CBooleanOps, a K-3D tool implementing boolean operations for meshes
		\author Romain Behar (romainbehar@yahoo.com)
*/

#include <k3dbuildgeometry.h>
#include <k3dobjectdialog.h>
#include <k3dtool.h>
#include "csg.h"

/////////////////////////////////////////////////////////////////////////////
// CBooleanOps

typedef k3dTool CBooleanOpsBase;

class CBooleanOps :
	public CBooleanOpsBase
{
public:
	CBooleanOps(sdpIUnknown* Initializer);

	// k3dIObject implementation
	sdpResult EditObject();

	void BuildGeometry();
	void UpdateGeometry();
	void CancelGeometry();
	void FinishGeometry();

	// Tells the tool to store any visible-selected geometry as object A
	void SetObjectA();
	// Tells the tool to store any visible-selected geometry as object B
	void SetObjectB();

private:
	typedef k3dObjectDialog PropertiesBase;

	/// Provides the user interface for the CBooleanOps tool
	class Properties :
		public PropertiesBase
	{
	public:
		Properties(CBooleanOps* const Object);

	private:
		/// Stores a pointer to the owning BooleanOps object
		CBooleanOps* const m_Object;

		void OnEvent(sdpGtkEvent* Event);

		// Stores any visible-selected geometry as object A
		void OnSetObjectA();
		// Stores any visible-selected geometry as object B
		void OnSetObjectB();

		void OnOK();
		void OnCancel();

		static const sdpString dialogname;
		static const sdpString control_setobjecta;
		static const sdpString control_setobjectb;
	};
	friend class Properties;

	///////////////////////////////////////////////////////////////////////////////////
	// PathProxies

	typedef k3d::EnginePathProxy PathProxyBase;
	class PathProxyA :
		public PathProxyBase
	{
	public:
		PathProxyA(CBooleanOps& Object, k3d::GeometryEngine& Engine, k3d::EngineMeshProxy& Mesh);
		PathProxyA(CBooleanOps& Object, k3d::GeometryEngine& Engine, k3dIPath* const Path);
		void BuildSolidA();
		SOLID* SolidA;

	private:
		CBooleanOps& m_Object;
		PathCollection m_FacePaths;
		PathCollection m_EdgePaths;
		PointCollection m_NewPoints;
	};
	friend class PathProxyA;

	class PathProxyB :
		public PathProxyBase
	{
	public:
		PathProxyB(CBooleanOps& Object, k3d::GeometryEngine& Engine, k3d::EngineMeshProxy& Mesh);
		PathProxyB(CBooleanOps& Object, k3d::GeometryEngine& Engine, k3dIPath* const Path);
		void BuildSolidB();
		SOLID* SolidB;

	private:
		CBooleanOps& m_Object;
		PathCollection m_FacePaths;
		PathCollection m_EdgePaths;
		PointCollection m_NewPoints;
	};
	friend class PathProxyB;

	/// Enumerates CSG operations
	typedef enum
	{
		UNION,
		INTERSECTION,
		AMINB,
		BMINA
	} Operation;

	friend std::ostream& operator<<(std::ostream& Stream, const Operation& Value)
	{
		switch(Value)
			{
				case UNION:
					Stream << "union";
					break;
				case INTERSECTION:
					Stream << "intersection";
					break;
				case AMINB:
					Stream << "differenceAB";
					break;
				case BMINA:
					Stream << "differenceBA";
					break;
			}

		return Stream;
	}

	friend std::istream& operator>>(std::istream& Stream, Operation& Value)
	{
		sdpString text;
		Stream >> text;

		if(text == "union")
			Value = UNION;
		else if(text == "intersection")
			Value = INTERSECTION;
		else if(text == "differenceAB")
			Value = AMINB;
		else if(text == "differenceBA")
			Value = BMINA;
		else
			std::cerr << g_pretty_function() << ": unknown enumeration [" << text << "]"<< std::endl;

		return Stream;
	}

	/// Controls the type of CSG operation to be performed
	k3dVariable<Operation> m_Operation;

	/// Stores newly-created geometry
	k3d::NewGeometryCollection m_NewGeometry;

	k3d::SelectedPathArray m_SelectionA;
	k3d::GeometryEngine m_EngineA;

	k3d::SelectedPathArray m_SelectionB;
	k3d::GeometryEngine m_EngineB;
};


#endif // BOOLEANOPS_H

