#ifndef MAGNETISER_H
#define MAGNETISER_H

// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the CMagnetiser K-3D object, which pulls geometry towards a point, animatedly
		\author Andy Gill (buzz@ucky.com)
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3deditorrender.h>
#include <k3dpersistobject.h>
#include <k3dobjectdialog.h>

// Forward class references
class CMagnetiser;

/////////////////////////////////////////////////////////////////////////////
// CMagnetDialog

typedef k3dObjectDialog CMagnetiserDialogBase;

class CMagnetiserDialog : public CMagnetiserDialogBase
{
public:
	CMagnetiserDialog(CMagnetiser* const Object);
	
protected:
	CMagnetiser* const m_Object;

	void OnLinear();
	void OnRadial();

	static const sdpString dialogname;
	static const sdpString control_magnetfactor;
	static const sdpString control_offsetx;
	static const sdpString control_offsety;
	static const sdpString control_offsetz;
	static const sdpString control_radius;
	static const sdpString control_profile;
};

/////////////////////////////////////////////////////////////////////////////
// CMagnet

typedef k3dPersistObject CMagnetiserBase;

class CMagnetiser :
	public CMagnetiserBase,
	public k3dEditorRender
{
friend class CMagnetiserDialog;
public:
	CMagnetiser(sdpIUnknown* Initializer);

	// k3dIObject overrides
	sdpResult EditObject();

	// k3dAnimate overrides
	sdpResult PopAnimate();
	
	// k3dPersist overrides
	sdpResult InitComplete();

	// k3dIEventObserver implementation
	sdpResult ObserveEvent(k3dIEventSource* const Source, SDPCSTRING Event, SDPCSTRING EventData, SDPCSTRING ObserverData);

	// k3dIEditorRender overrides
	sdpResult EditorPostRender(k3dIEditorEngine* const Engine);

protected:
	k3dScalarChannel m_MagnetFactor;
	k3dScalarChannel m_OffsetX;
	k3dScalarChannel m_OffsetY;
	k3dScalarChannel m_OffsetZ;
	k3dScalarChannel m_Radius;
	
	k3dScalarChannel m_Profile;
};

#endif // MAGNETISER_H

