// K-3D
// Copyright  1995-2002, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Implements the functions necessary to export K-3D objects from the experimentallib module
		\author Timothy M. Shead (tshead@k-3d.com)
*/

#include "booleanops.h"
#include "magnetiser.h"

#include <k3dcom.h>
#include <k3dmodule.h>

class ExperimentalModule :
	public k3d::Module
{
public:
	sdpIUnknown* CreateInstance(const sdp::ClassID& ClassID, sdpIUnknown* Initializer)
	{
		if(ClassID == k3d::classid::BooleanOps)
			return sdpCreateClassInstance<CBooleanOps>(Initializer);
		if(ClassID == k3d::classid::Magnetiser)
			return sdpCreateClassInstance<CMagnetiser>(Initializer);

		return 0;
	}

private:
	friend class sdpSingleton<ExperimentalModule>;

	ExperimentalModule() :
		k3d::Module("experimentallib", VERSION)
	{
		// Booleans ...
		sdpxml::Element& booleans = RegisterObject(k3d::classid::BooleanOps, "Booleans");
		RegisterBookmark(booleans, "Tools/Booleans");
		RegisterExperimental(booleans);

		// Magnetiser Object ...
		sdpxml::Element& magnetiser = RegisterObject(k3d::classid::Magnetiser, "Magnetiser");
		RegisterBookmark(magnetiser, "Objects/Magnetiser");
		RegisterExperimental(magnetiser);
	}
};

K3DMODULEPROCEDURES(ExperimentalModule, sdpSingleton<ExperimentalModule>);


