#ifndef MAXREADER_H
#define MAXREADER_H

// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the CMaxReader K-3D object, which reads .3ds files from 3D Studio Max
		\author Ben Campbell (bcampbell@ntlworld.com)
*/

// STATUS (27Aug2001)
//
// Basically loads points and faces but not much else.
//
// TODO:
// - Reconstruct object hierarchy
// - sort out normal issues
// - add UVs to mesh
// - material handling
// - craps out on big models
// - etc etc

#include <k3dobject.h>
#include <k3difileformat.h>
#include <k3digeometryreadformat.h>

// Forward class declarations
class k3dIMesh;

///////////////////////////////////////////////////////////////////////////////
// CMaxReader

typedef k3dObject CMaxReaderBase;

class CMaxReader :
	public CMaxReaderBase,
	public k3dIFileFormat,
	public k3dIGeometryReadFormat
{
public:
	CMaxReader(sdpIUnknown* Initializer);

	// k3dIFileFormat implementation ...
	bool QueryCanHandle(SDPCSTRING FilePath);

	// k3dIGeometryReadFormat implementation ...
	sdpResult PreRead(SDPCSTRING FilePath);
	sdpResult ReadOptions(SDPCSTRING FilePath);
	sdpResult ReadFile(SDPCSTRING FilePath);

private:
	gushort GetShort();
	gulong GetLong();
	float GetFloat();
	sdpString GetString();

	sdpResult GetChunkInfo();
	void SkipChunk();

	sdpResult LoadMesh();
	sdpResult Handle_OBJMESH(int chunkbytes);
	sdpResult Handle_OBJBLOCK(int chunkbytes);
	sdpResult Handle_TRIMESH(int chunkbytes, k3dIMesh* mesh);

	gushort m_ChunkId;
	gulong m_ChunkLength;

	guchar *m_Buf;
	gulong m_Bufsize;
	guchar *m_Ptr;

	//std::map<sdpString, k3dIMaterial*> materials;
	//k3dIMaterial* currentmaterial = 0;
};

#endif // MAXREADER_H
