// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Implements the functions necessary to export K-3D objects from the geometrylib module
		\author Timothy M. Shead (tshead@k-3d.com)
*/

#include "objreader.h"
#include "objwriter.h"
#include "gtsreader.h"
#include "gtswriter.h"
#include "maxreader.h"
#include "xwriter.h"

#include <k3dcom.h>
#include <k3dmodule.h>

class GeometryModule :
	public k3d::Module
{
public:
	sdpIUnknown* CreateInstance(const sdp::ClassID& ClassID, sdpIUnknown* Initializer)
	{
		if(ClassID == k3d::classid::GeometryReadFormatOBJ)
			return sdpCreateClassInstance<COBJReader>(Initializer);
		if(ClassID == k3d::classid::GeometryWriteFormatOBJ)
			return sdpCreateClassInstance<COBJWriter>(Initializer);
		if(ClassID == k3d::classid::GeometryReadFormatGTS)
			return sdpCreateClassInstance<CGTSReader>(Initializer);
		if(ClassID == k3d::classid::GeometryWriteFormatGTS)
			return sdpCreateClassInstance<CGTSWriter>(Initializer);
		if(ClassID == k3d::classid::GeometryReadFormat3DS)
			return sdpCreateClassInstance<CMaxReader>(Initializer);
		if(ClassID == k3d::classid::GeometryWriteFormatX)
			return sdpCreateClassInstance<CXWriter>(Initializer);
		
		return 0;
	}

private:
	friend class sdpSingleton<GeometryModule>;

	GeometryModule() :
		k3d::Module("geometry", VERSION)
	{
		// Wavefront (.obj) geometry import ...
		sdpxml::Element& objreader = RegisterObject(k3d::classid::GeometryReadFormatOBJ, "OBJReader");
		RegisterInterface(objreader, sdp::TypeInfo<k3dIGeometryReadFormat>());
		RegisterShortDescription(objreader, "Wavefront ( .obj )");
	
		// Wavefront (.obj) geometry export ...
		sdpxml::Element& objwriter = RegisterObject(k3d::classid::GeometryWriteFormatOBJ, "OBJWriter");
		RegisterInterface(objwriter, sdp::TypeInfo<k3dIGeometryWriteFormat>());
		RegisterShortDescription(objwriter, "Wavefront ( .obj )");

		// GNU Triangulated Surface (.gts) geometry import ...
		sdpxml::Element& gtsreader = RegisterObject(k3d::classid::GeometryReadFormatGTS, "GTSReader");
		RegisterInterface(gtsreader, sdp::TypeInfo<k3dIGeometryReadFormat>());
		RegisterShortDescription(gtsreader, "GNU Triangulated Surface ( .gts )");

		// GNU Triangulated Surface (.gts) geometry export ...
		sdpxml::Element& gtswriter = RegisterObject(k3d::classid::GeometryWriteFormatGTS, "GTSWriter");
		RegisterInterface(gtswriter, sdp::TypeInfo<k3dIGeometryWriteFormat>());
		RegisterShortDescription(gtswriter, "GNU Triangulated Surface ( .gts )");

		// 3d studio max (.3ds) geometry import ...
		sdpxml::Element& maxreader = RegisterObject(k3d::classid::GeometryReadFormat3DS, "3DSReader");
		RegisterInterface(maxreader, sdp::TypeInfo<k3dIGeometryReadFormat>());
		RegisterShortDescription(maxreader, "3D Studio Max ( .3ds )");

		// DirectX (.x) geometry export ...
		sdpxml::Element& xwriter = RegisterObject(k3d::classid::GeometryWriteFormatX, "XWriter");
		RegisterInterface(xwriter, sdp::TypeInfo<k3dIGeometryWriteFormat>());
		RegisterShortDescription(xwriter, "DirectX ( .x )");
	}
};

K3DMODULEPROCEDURES(GeometryModule, sdpSingleton<GeometryModule>);


