#ifndef JAVASCRIPTENGINE_H
#define JAVASCRIPTENGINE_H

// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the CJavaScriptEngine and related classes, which allow K-3D scripts to be written in the JavaScript language
		\author Tim Shead (tshead@k-3d.com)
		\author Dave Crane (ulunum@bigfoot.com)
*/

#include <k3discriptengine.h>
#include <jsapi.h>

/////////////////////////////////////////////////////
// CJavaScriptEngine

class CJavaScriptEngine :
	public k3dIScriptEngine,
	public sdpIDeletable
{
public:
	CJavaScriptEngine(sdpIUnknown* Initializer);
	virtual ~CJavaScriptEngine();

	// k3dIScriptEngine overrides ...
	SDPCSTRING Language();
	sdpResult Execute(SDPCSTRING Script, SDPCSTRING ScriptName);
	sdpResult ConvertCommand(k3dICommandNode* CommandNode, SDPCSTRING Name, SDPCSTRING Arguments, SDPSTRING Buffer, gulong& BufferLength);
	
private:
	JSRuntime* m_Runtime;

	/// Provides a callback that is called by the JavaScript engine periodically to allow script interruptions
	static JSBool RawBranchCallback(JSContext* Context, JSScript* Script);
	/// Provides a callback that is called by the JavaScript engine periodically to allow script interruptions
	JSBool BranchCallback(JSContext* Context, JSScript* Script);

	/// Provides an error-report callback that can be called by the JavaScript engine during error conditions
	static void RawErrorReporter(JSContext* Context, const char *Message, JSErrorReport *Report);
	/// Provides an error-report callback that can be called by the JavaScript engine during error conditions
	static void ErrorReporter(const char *Message, JSErrorReport *Report);
};


#endif // JAVASCRIPTENGINE_H

