#ifndef JAVASCRIPTGTKML_H
#define JAVASCRIPTGTKML_H

// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the CJavaScriptGTKMLContainer class, which provides JavaScript support for creating user interfaces with GTKML
		\author Tim Shead (tshead@k-3d.com)
*/

#include <jsapi.h>
#include <sdpgtkobjectcontainer.h>

///////////////////////////////////////////////////////////////////////////
// CJavaScriptGTKMLContainer

typedef sdpGtkObjectContainer CJavaScriptGTKMLContainerBase;

class CJavaScriptGTKMLContainer :
	public CJavaScriptGTKMLContainerBase
{
public:
	static void InitializeClass(JSContext* Context, JSObject* Object);
	static CJavaScriptGTKMLContainer* Container(JSContext* Context, JSObject* Object);

protected:
/*
	// Standard JS object data ...
	typedef enum
	{
		PROPERTY_MESH,
		PROPERTY_POINTS,
	} PropertyID;
*/

	static JSClass Class;
	static JSPropertySpec Properties[];
	static JSFunctionSpec Methods[];

	// Object constructor ...
	static JSBool Constructor(JSContext* Context, JSObject* Object, uintN argc, jsval* argv, jsval* Result);
	// Object destructor ...
	static void Destructor(JSContext* Context, JSObject* Object);

	// Object methods ...
	static JSBool JSDoModal(JSContext* Context, JSObject* Object, uintN argc, jsval* argv, jsval* Result);	
	static JSBool JSField(JSContext* Context, JSObject* Object, uintN argc, jsval* argv, jsval* Result);	
	
	/// Overrides the base DoModal() implementation to reset the result code
	void DoModal();
	
	// Event handling ...
	void OnEvent(sdpGtkEvent* Event);
	/// Called when the root widget is about to be deleted through the WM
	void OnDelete(sdpGtkEvent* const Event);
	/// Called when the user hits the "OK" button
	void OnOK();
	/// Called when the user hits the "Cancel" button
	void OnCancel();
	/// Called when the root widget is about to be destroyed
	void OnDestroy();
	
	/// Stores a result code returned by DoModal()
	gint m_ResultCode;
};

#endif // JAVASCRIPTGTKML_H

