#ifndef JAVASCRIPTOBJECTS_H
#define JAVASCRIPTOBJECTS_H

// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares classes for implementing a scripting object model in JavaScript
		\author Tim Shead (tshead@k-3d.com)
		\author Dave Crane (ulunum@bigfoot.com)
*/

#include "javascriptskeleton.h"

// Forward class declarations ...
class k3dIApplication;
class k3dICommandNode;
class k3dIDocument;
class k3dIRenderJob;
class k3dIRenderFrame;
class k3dICamera;
class k3dIEditable;
class k3dIObject;
class k3dIObjectCollection;
class k3dIMesh;
class k3dIPoint;
class k3dIPath;
class k3dIPathPointCollection;
class k3dILocation;
class k3dIPointCollection;

///////////////////////////////////////////////////////////////////////////
// CJavaScriptEditable

class CJavaScriptEditable :
	public JavaScriptSkeleton<k3dIEditable, CJavaScriptEditable>
{
public:
	// Methods
	static JSBool PushEditing(JSContext* Context, JSObject* Object, uintN argc, jsval* argv, jsval* Result);
	static JSBool PopEditing(JSContext* Context, JSObject* Object, uintN argc, jsval* argv, jsval* Result);
};

///////////////////////////////////////////////////////////////////////////
// CJavaScriptLocation

class CJavaScriptLocation :
	public JavaScriptSkeleton<k3dILocation, CJavaScriptLocation>
{
public:
	// Properties
	static JSBool GetLocal(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);	
	static JSBool SetLocal(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);	
	static JSBool GetAnimated(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);	
	static JSBool SetAnimated(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);	
	static JSBool GetWorld(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);	
	static JSBool SetWorld(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);	
	static JSBool GetCamera(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);	
	static JSBool SetCamera(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);	
	static JSBool GetScreen(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);	
	static JSBool SetScreen(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);	
	static JSBool GetNormal(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);	
	static JSBool SetNormal(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);	
	static JSBool GetTexture(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);	
	static JSBool SetTexture(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);	
	static JSBool GetImplicitTexture(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);	
	static JSBool SetImplicitTexture(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);	
};

///////////////////////////////////////////////////////////////////////////
// CJavaScriptPoint

class CJavaScriptPoint :
	public JavaScriptSkeleton<k3dIPoint, CJavaScriptPoint>
{
public:
	// Properties
	static JSBool GetMesh(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);	
	static JSBool GetLocation(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);
	static JSBool GetEditable(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);

	// Methods
	static JSBool PushEditing(JSContext* Context, JSObject* Object, uintN argc, jsval* argv, jsval* Result);
	static JSBool PopEditing(JSContext* Context, JSObject* Object, uintN argc, jsval* argv, jsval* Result);
};

///////////////////////////////////////////////////////////////////////////
// CJavaScriptPathPointCollection

class CJavaScriptPathPointCollection :
	public JavaScriptSkeleton<k3dIPathPointCollection, CJavaScriptPathPointCollection>
{
public:
	// Properties
	static JSBool AddPoint(JSContext* Context, JSObject* Object, uintN argc, jsval* argv, jsval* Result);
	static JSBool RemovePoint(JSContext* Context, JSObject* Object, uintN argc, jsval* argv, jsval* Result);
};

///////////////////////////////////////////////////////////////////////////
// CJavaScriptPath

class CJavaScriptPath :
	public JavaScriptSkeleton<k3dIPath, CJavaScriptPath>
{
public:
	// Properties
	static JSBool GetMesh(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);	
	static JSBool GetPoints(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);
	static JSBool GetEditable(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);

	// Methods
	static JSBool PushEditing(JSContext* Context, JSObject* Object, uintN argc, jsval* argv, jsval* Result);
	static JSBool PopEditing(JSContext* Context, JSObject* Object, uintN argc, jsval* argv, jsval* Result);
};

///////////////////////////////////////////////////////////////////////////
// CJavaScriptMesh

class CJavaScriptMesh :
	public JavaScriptSkeleton<k3dIMesh, CJavaScriptMesh>
{
public:
	// Properties
	static JSBool GetPoints(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);	
	static JSBool GetPaths(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);	

	// Methods
	static JSBool CreatePoint(JSContext* Context, JSObject* Object, uintN argc, jsval* argv, jsval* Result);
	static JSBool CreatePath(JSContext* Context, JSObject* Object, uintN argc, jsval* argv, jsval* Result);
};

///////////////////////////////////////////////////////////////////////////
// CJavaScriptObject

class CJavaScriptObject :
	public JavaScriptSkeleton<k3dIObject, CJavaScriptObject>
{
public:
	// Properties
	static JSBool GetDocument(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);	
	static JSBool GetName(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);	
	static JSBool SetName(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);	
	static JSBool GetPosition(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);	
	static JSBool SetPosition(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);	
	static JSBool GetOrientation(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);	
	static JSBool SetOrientation(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);	
	static JSBool GetScale(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);	
	static JSBool SetScale(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);	
	static JSBool GetMesh(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);
	static JSBool GetEditable(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);
	
	// Methods
	static JSBool PushEditing(JSContext* Context, JSObject* Object, uintN argc, jsval* argv, jsval* Result);
	static JSBool PopEditing(JSContext* Context, JSObject* Object, uintN argc, jsval* argv, jsval* Result);
	static JSBool EditObject(JSContext* Context, JSObject* Object, uintN argc, jsval* argv, jsval* Result);
};

///////////////////////////////////////////////////////////////////////////
// CJavaScriptCamera

class CJavaScriptCamera :
	public JavaScriptSkeleton<k3dICamera, CJavaScriptCamera>
{
public:
	// Properties
	static JSBool GetDocument(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);
	static JSBool GetEditMode(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);
	static JSBool SetEditMode(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);

	// Methods
	static JSBool Redraw(JSContext* Context, JSObject* Object, uintN argc, jsval* argv, jsval* Result);
};

///////////////////////////////////////////////////////////////////////////
// CJavaScriptObjectCollection

class CJavaScriptObjectCollection :
	public JavaScriptSkeleton<k3dIObjectCollection, CJavaScriptObjectCollection>
{
public:
	// Properties
	static JSBool GetEditable(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);

	// Methods
	static JSBool PushEditing(JSContext* Context, JSObject* Object, uintN argc, jsval* argv, jsval* Result);
	static JSBool PopEditing(JSContext* Context, JSObject* Object, uintN argc, jsval* argv, jsval* Result);
	static JSBool CreateObject(JSContext* Context, JSObject* Object, uintN argc, jsval* argv, jsval* Result); // "CreateObject" to avoid a clash with the skeleton
	static JSBool Get(JSContext* Context, JSObject* Object, uintN argc, jsval* argv, jsval* Result);
	static JSBool Object(JSContext* Context, JSObject* Object, uintN argc, jsval* argv, jsval* Result);
	static JSBool Delete(JSContext* Context, JSObject* Object, uintN argc, jsval* argv, jsval* Result);
};

///////////////////////////////////////////////////////////////////////////
// CJavaScriptDocument

class CJavaScriptDocument :
	public JavaScriptSkeleton<k3dIDocument, CJavaScriptDocument>
{
public:
	// Properties
	static JSBool GetApplication(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);
	static JSBool GetObjects(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);

	// Methods
	static JSBool Import(JSContext* Context, JSObject* Object, uintN argc, jsval* argv, jsval* Result);
	static JSBool Export(JSContext* Context, JSObject* Object, uintN argc, jsval* argv, jsval* Result);
	static JSBool Save(JSContext* Context, JSObject* Object, uintN argc, jsval* argv, jsval* Result);
	static JSBool StartStateChangeSet(JSContext* Context, JSObject* Object, uintN argc, jsval* argv, jsval* Result);
	static JSBool FinishStateChangeSet(JSContext* Context, JSObject* Object, uintN argc, jsval* argv, jsval* Result);
	static JSBool RedrawAll(JSContext* Context, JSObject* Object, uintN argc, jsval* argv, jsval* Result);
};

/*
///////////////////////////////////////////////////////////////////////////
// CJavaScriptRenderFrame

class CJavaScriptRenderFrame :
	public JavaScriptSkeleton<k3dIRenderFrame, CJavaScriptRenderFrame>
{
public:
	// Properties
	static JSBool GetJob(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);
	static JSBool GetName(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);
	static JSBool GetPath(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);
	static JSBool GetPriority(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);
	static JSBool SetPriority(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);
	static JSBool GetViewDuringRender(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);
	static JSBool SetViewDuringRender(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);
	static JSBool GetViewAfterRender(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);
	static JSBool SetViewAfterRender(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);
	static JSBool GetEngine(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);
	static JSBool SetEngine(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);
};

///////////////////////////////////////////////////////////////////////////
// CJavaScriptRenderJob

class CJavaScriptRenderJob :
	public JavaScriptSkeleton<k3dIRenderJob, CJavaScriptRenderJob>
{
public:
	// Properties
	static JSBool GetApplication(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);
	static JSBool GetName(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);
	static JSBool GetPath(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);

	// Methods
	static JSBool CreateFrame(JSContext* Context, JSObject* Object, uintN argc, jsval* argv, jsval* Result);
};
*/

///////////////////////////////////////////////////////////////////////////
// CJavaScriptCommandNode

class CJavaScriptCommandNode :
	public JavaScriptSkeleton<k3dICommandNode, CJavaScriptCommandNode>
{
public:
	// Methods
	static JSBool Command(JSContext* Context, JSObject* Object, uintN argc, jsval* argv, jsval* Result);
};

//////////////////////////////////////////////////////
// CJavaScriptApplication

class CJavaScriptApplication
{
public:
  	static JSObject* Create(JSContext* const Context, JSObject* const GlobalObject);
    	
private:
	static JSClass Class;
	static JSPropertySpec Properties[];
	static JSFunctionSpec Methods[]; 
			
	// Properties
	static JSBool GetActiveDocument(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);
	static JSBool GetBasePath(JSContext* Context, JSObject* Object, jsval ID, jsval* Value);

	// Methods
	static JSBool Close(JSContext* Context, JSObject* Object, uintN argc, jsval *argv, jsval* Result);
	
	static JSBool NewDocument(JSContext* Context, JSObject* Object, uintN argc, jsval *argv, jsval* Result);
	static JSBool OpenDocument(JSContext* Context, JSObject* Object, uintN argc, jsval *argv, jsval* Result);
	static JSBool CloseDocument(JSContext* Context, JSObject* Object, uintN argc, jsval *argv, jsval* Result);
	
	static JSBool BrowserNavigate(JSContext* Context, JSObject* Object, uintN argc, jsval *argv, jsval *Result);
	static JSBool Message(JSContext* Context, JSObject* Object, uintN argc, jsval *argv, jsval *Result);
	static JSBool Query(JSContext* Context, JSObject* Object, uintN argc, jsval *argv, jsval *Result);
	static JSBool Error(JSContext* Context, JSObject* Object, uintN argc, jsval *argv, jsval *Result);
	static JSBool DebugOutput(JSContext* Context, JSObject* Object, uintN argc, jsval *argv, jsval *Result);

	static JSBool CommandNode(JSContext* Context, JSObject* Object, uintN argc, jsval *argv, jsval *Result);
};

#endif // JAVASCRIPTOBJECTS_H

