#ifndef JAVASCRIPTSKELETON_H
#define JAVASCRIPTSKELETON_H

#ifndef SDPTYPES_H
#include <sdptypes.h>
#endif // !SDPTYPES_H

#include <jsapi.h>

template<class InterfaceType, class BaseType>
class JavaScriptSkeleton
{
public:
	typedef JavaScriptSkeleton<InterfaceType, BaseType> Skeleton;
	typedef BaseType Owner;

	static JSObject* Create(InterfaceType* const Interface, JSContext* const Context, JSObject* const Parent)
	{
		// Sanity checks ...
		g_assert(Interface);
		g_assert(Context);
		g_assert(Parent);

		JSObject* const result = JS_NewObject(Context, &Class, 0, Parent);
		JS_SetPrivate(Context, result, Interface);
		JS_DefineProperties(Context, result, Properties);
		JS_DefineFunctions(Context, result, Methods);
	
		return result;
	}

	static InterfaceType* Interface(JSContext* const Context, JSObject* const Object)
	{
		if(JS_GetClass(Object) == &Class)
			return reinterpret_cast<InterfaceType*>(JS_GetPrivate(Context, Object));
		else
			return 0;
	}

	template<class NewInterfaceType, class NewActualType>
	static JSBool DynamicCast(JSContext* const Context, JSObject* const Object, const jsval ID, jsval* const Value)
	{
		NewInterfaceType* const i = dynamic_cast<NewInterfaceType*>(Interface(Context, Object));
		g_return_val_if_fail(i, JS_FALSE);
	
		*Value = OBJECT_TO_JSVAL(NewActualType::Create(i, Context, Object));
		return JS_TRUE;
	}
	
private:
	static JSClass Class;
	static JSPropertySpec Properties[];
	static JSFunctionSpec Methods[];
};

#endif // !JAVASCRIPTSKELETON_H
