#ifndef BOOLEANOPERATOR_H
#define BOOLEANOPERATOR_H


// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the CBooleanOperator K-3D object, which allows geometry to be rendered with boolean effects
		\author Tim Shead (tshead@k-3d.com)
*/

// Forward class declarations ...
class CBooleanOperator;

#include <k3dobjectdialog.h>
#include <k3dpersistobject.h>
#include <k3drirender.h>
#include <k3dvariable.h>

/////////////////////////////////////////////////////////////////////////////
// CBooleanOperatorDialog

typedef k3dObjectDialog CBooleanOperatorDialogBase;

class CBooleanOperatorDialog :
	public CBooleanOperatorDialogBase
{
public:
	CBooleanOperatorDialog(CBooleanOperator* const Object);

protected:
	CBooleanOperator* const m_Object;

	static const sdpString dialogname;
	static const sdpString control_booleantype;
};

/////////////////////////////////////////////////////
// CBooleanOperator
typedef k3dPersistObject CBooleanOperatorBase;

/// Allows geometry to be rendered with boolean effects
class CBooleanOperator :
	public CBooleanOperatorBase,
	public k3dRiRender
{
friend class CBooleanOperatorDialog;
public:
	CBooleanOperator(sdpIUnknown* Initializer);
	~CBooleanOperator();

	typedef enum
	{
		BOOLEAN_INTERSECTION,
		BOOLEAN_UNION,
		BOOLEAN_DIFFERENCE
	} BooleanType;

	friend std::ostream& operator<<(std::ostream& Stream, const BooleanType& Value)
	{
		switch(Value)
			{
				case BOOLEAN_UNION:
					Stream << "union";
					break;
				case BOOLEAN_INTERSECTION:
					Stream << "intersection";
					break;
				case BOOLEAN_DIFFERENCE:
					Stream << "difference";
					break;
			}

		return Stream;
	}

	friend std::istream& operator>>(std::istream& Stream, BooleanType& Value)
	{
		sdpString text;
		Stream >> text;
		
		if(text == "union")
			Value = BOOLEAN_UNION;
		else if(text == "intersection")
			Value = BOOLEAN_INTERSECTION;
		else if(text == "difference")
			Value = BOOLEAN_DIFFERENCE;
		else
			std::cerr << g_pretty_function() << ": unknown enumeration [" << text << "]"<< std::endl;
		
		return Stream;
	}

	k3dVariable<BooleanType> m_Type;

	// k3dIObject overrides
	sdpResult EditObject();

	// k3dIRiRender overrides
	sdpResult RiPushRender(k3dIRenderFrame* const Frame, k3dIRiEngine* const Engine);
	sdpResult RiPopRender(k3dIRenderFrame* const Frame, k3dIRiEngine* const Engine);
};

#endif // BOOLEANOPERATOR_H
