#ifndef IMAGELAYER_H
#define IMAGELAYER_H

// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the CImageLayer K-3D object, which renders 2D graphics either in front of or behind all 3D geometry.
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3deditorrender.h>
#include <k3dobjectdialog.h>
#include <k3dpersistobject.h>
#include <k3drirender.h>
#include <k3dshaderinstance.h>
#include <k3dvariable.h>


// Forward class declarations ...
class CImageLayer;

/////////////////////////////////////////////////////////////////////////////
// CImageLayerDialog

typedef k3dObjectDialog CImageLayerDialogBase;

class CImageLayerDialog :
	public CImageLayerDialogBase
{
public:
	CImageLayerDialog(CImageLayer* const Object);

protected:
	CImageLayer* const m_Object;

	// Event handling ...
	void OnEvent(sdpGtkEvent* Event);

	static const sdpString dialogname;
	static const sdpString control_editor;
	static const sdpString control_layer;
	static const sdpString control_color;
	static const sdpString control_strength;
	static const sdpString control_shaderselector;
	static const sdpString control_shadercontainer;
};

/////////////////////////////////////////////////////////////////////////////
// CImageLayer
typedef k3dPersistObject CImageLayerBase;

/// Renders 2D images either in-front-of or behind 3D geometry
class CImageLayer :
	public CImageLayerBase,
	public k3dEditorRender,
	public k3dRiRender,
	public sdpsl::IShaderCollection
{
friend class CImageLayerDialog;
public:
	CImageLayer(sdpIUnknown* Initializer);
	~CImageLayer();

	// k3dIObject implementation
	sdpResult EditObject();

	// sdpsl::IShaderCollection implementation
	void VisitShaders(sdpsl::IShaderVisitor* const Visitor);

	// k3dIEditorRender implementation
	sdpResult EditorPreRender(k3dIEditorEngine* const Engine);
	sdpResult EditorPostRender(k3dIEditorEngine* const Engine);

	// k3dIRiRender implementation
	sdpResult RiPreRender(k3dIRenderFrame* const Frame, k3dIRiEngine* const Engine);
	sdpResult RiRender(k3dIRenderFrame* const Frame, k3dIRiEngine* const Engine);

	// k3dIPersist implementation
	sdpResult InitComplete();

protected:
	// Helper functions
	sdpResult DrawEditor(k3dIEditorEngine* const Engine);

	typedef enum
	{
		FOREGROUND = 1,
		BACKGROUND = 2,
	} Layer;

	friend std::ostream& operator<<(std::ostream& Stream, const Layer& Value)
	{
		switch(Value)
			{
				case FOREGROUND:
					Stream << "foreground";
					break;
				case BACKGROUND:
					Stream << "background";
					break;
			}
			
		return Stream;
	}
	
	friend std::istream& operator>>(std::istream& Stream, Layer& Value)
	{
		sdpString text;
		Stream >> text;
		
		if(text == "foreground")
			Value = FOREGROUND;
		else if(text == "background")
			Value = BACKGROUND;
		else
			std::cerr << g_pretty_function() << ": unknown enumeration [" << text << "]" << std::endl;
		
		return Stream;
	}


	k3dVariable<bool> m_Editor;
	k3dVariable<Layer> m_Layer;

	k3dColorChannel m_Color;
	k3dScalarChannel m_Strength;
	k3dShaderInstance m_Shader;

	// This is so not thread safe!!
	static double m_ForegroundCount;
	static double m_BackgroundCount;
};

#endif // IMAGELAYER_H
