// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Implements the functions necessary to export K-3D objects from the sfxlib module
		\author Timothy M. Shead (tshead@k-3d.com)
*/

#include "booleanoperator.h"
#include "fractalterrain.h"
#include "imagelayer.h"
#include "particleburst.h"
#include "polygonizer.h"
#include "softobject.h"

#include <k3dcom.h>
#include <k3dmodule.h>

class SFXModule :
	public k3d::Module
{
public:
	sdpIUnknown* CreateInstance(const sdp::ClassID& ClassID, sdpIUnknown* Initializer)
	{
		if(ClassID == k3d::classid::BooleanOperator)
			return sdpCreateClassInstance<CBooleanOperator>(Initializer);
		if(ClassID == k3d::classid::FractalTerrain)
			return sdpCreateClassInstance<CFractalTerrain>(Initializer);
		if(ClassID == k3d::classid::ImageLayer)
			return sdpCreateClassInstance<CImageLayer>(Initializer);
		if(ClassID == k3d::classid::ParticleBurst)
			return sdpCreateClassInstance<CParticleBurst>(Initializer);
		if(ClassID == k3d::classid::Polygonizer)
			return sdpCreateClassInstance<CPolygonizer>(Initializer);
		if(ClassID == k3d::classid::ImplicitObject)
			return sdpCreateClassInstance<CSoftObject>(Initializer);

		return 0;
	}

private:
	friend class sdpSingleton<SFXModule>;
	SFXModule() :
		k3d::Module("sfx", VERSION)
	{
		// Boolean operator ...
		sdpxml::Element& boolean = RegisterObject(k3d::classid::BooleanOperator, "Boolean");
		RegisterBookmark(boolean, "Objects/Boolean");
	
		// Fractal Terrain ...
		sdpxml::Element& fractalterrain = RegisterObject(k3d::classid::FractalTerrain, "FractalTerrain");
		RegisterBookmark(fractalterrain, "Objects/Fractal Terrain");
	
		// Image Layer ...
		sdpxml::Element& imagelayer = RegisterObject(k3d::classid::ImageLayer, "ImageLayer");
		RegisterBookmark(imagelayer, "Objects/Image Layer");

		// Particle Burst ...
		sdpxml::Element& particleburst = RegisterObject(k3d::classid::ParticleBurst, "ParticleBurst");
		RegisterBookmark(particleburst, "Objects/Particle Burst");

		// Polygonizer Object ...
		sdpxml::Element& polygonizer = RegisterObject(k3d::classid::Polygonizer, "Polygonizer");
		RegisterBookmark(polygonizer, "Objects/Polygonizer");
		RegisterIncomplete(polygonizer);

		// ImplicitObject ...
		sdpxml::Element& softobject = RegisterObject(k3d::classid::ImplicitObject, "SoftObject");
		RegisterBookmark(softobject, "Objects/Soft Object");
		RegisterIncomplete(softobject);
	}
};

K3DMODULEPROCEDURES(SFXModule, sdpSingleton<SFXModule>);


