#ifndef BEND_H
#define BEND_H

// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the CBend K-3D tool, which applies a "Bend" transformation to geometry
		\author Andy Gill (buzz@ucky.com)
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3daxis.h>
#include <k3dmouseeventobserver.h>
#include <k3dobjectdialog.h>
#include <k3dselection.h>
#include <k3dtool.h>

// Forward class references
class CBend;

/////////////////////////////////////////////////////////////////////////////
// CBendDialog

typedef k3dObjectDialog CBendDialogBase;

class CBendDialog :
	public CBendDialogBase
{
public:
	CBendDialog(CBend* const Object);
	
private:
	CBend* const m_Object;

	static const sdpString dialogname;
	static const sdpString control_bendangle;
	static const sdpString control_along;
	static const sdpString control_around;
	static const sdpString control_tightness;
	static const sdpString control_jointposition;
};

/////////////////////////////////////////////////////////////////////////////
// CBend

typedef k3dTool CBendBase;

class CBend :
	public CBendBase,
	public k3dMouseEventObserver
{
friend class CBendDialog;
public:
	CBend(sdpIUnknown* Initializer);

	// k3dIObject implementation
	sdpResult InitComplete();
	sdpResult EditObject();

	// k3dIMouseEventObserver implementation
	sdpResult OnLButtonStartDrag(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY);
	sdpResult OnLButtonDrag(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY, gdouble LastX, gdouble LastY, gdouble StartX, gdouble StartY);
	sdpResult OnLButtonEndDrag(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY, gdouble LastX, gdouble LastY, gdouble StartX, gdouble StartY);

	void PushEditing();
	void UpdateGeometry();
	void CancelEditing();
	void PopEditing();

private:
	k3dVariable<gdouble> m_BendAngle;
	k3dVariable<gdouble> m_Tightness;
	k3dVariable<gdouble> m_JointPosition;
	k3dVariable<k3d::Axis> m_Along;
	k3dVariable<k3d::Axis> m_Around;
	
	k3d::SelectedLocationArray m_CurrentSelection;
	k3d::Dimensions m_CurrentSelectionDimensions;
};

#endif // BEND_H

