#ifndef BEVEL_H
#define BEVEL_H

// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the CBevel K-3D tool, which creates bevelled geometry
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3dbuildgeometry.h>
#include <k3dmouseeventobserver.h>
#include <k3dobjectdialog.h>
#include <k3dtool.h>

/////////////////////////////////////////////////////////////////////////////
// CBevel

typedef k3dTool CBevelBase;

class CBevel :
	public CBevelBase,
	public k3dMouseEventObserver
{
public:
	CBevel(sdpIUnknown* Initializer);

	// k3dIObject implementation
	sdpResult InitComplete();
	sdpResult EditObject();

	// k3dIMouseEventObserver implementation
	sdpResult OnLButtonStartDrag(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY);
	sdpResult OnLButtonDrag(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY, gdouble LastX, gdouble LastY, gdouble StartX, gdouble StartY);
	sdpResult OnLButtonEndDrag(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY, gdouble LastX, gdouble LastY, gdouble StartX, gdouble StartY);

	void BuildGeometry();
	void UpdateGeometry();
	void CancelGeometry();
	void FinishGeometry();

private:
	typedef k3dObjectDialog PropertiesBase;
	class Properties :
		public PropertiesBase
	{
	public:
		Properties(CBevel* const Object);

	private:
		CBevel* const m_Object;

		void OnOK();
		void OnCancel();
	};
	friend class Properties;

	///////////////////////////////////////////////////////////////////////////////////
	// PathProxy
	
	typedef k3d::EnginePathProxy PathProxyBase;
	class PathProxy :
		public PathProxyBase
	{
	public:
		PathProxy(CBevel& Object, k3d::GeometryEngine& Engine, k3d::EngineMeshProxy& Mesh);
		PathProxy(CBevel& Object, k3d::GeometryEngine& Engine, k3dIPath* const Path);

		void BuildGeometry();
		void UpdateGeometry();
		void Cancel();

	private:
		CBevel& m_Object;
		PathCollection m_FacePaths;
		PathCollection m_EdgePaths;
		PointCollection m_NewPoints;
	};
	friend class PathProxy;
	
	typedef enum
	{
		INSET,
		SCALE,
	} InsetType;
	
	friend std::ostream& operator<<(std::ostream& Stream, const InsetType& Value)
	{
		switch(Value)
			{
				case INSET:
					Stream << "inset";
					break;
				case SCALE:
					Stream << "scale";
					break;
			}
	
		return Stream;
	}
	
	friend std::istream& operator>>(std::istream& Stream, InsetType& Value)
	{
		sdpString text;
		Stream >> text;
		
		if(text == "inset")
			Value = INSET;
		else if(text == "scale")
			Value = SCALE;
		else
			std::cerr << g_pretty_function() << ": unknown enumeration [" << text << "]" << std::endl;
		
		return Stream;
	}

	k3dVariable<gdouble> m_Offset;
	k3dVariable<gdouble> m_Inset;
	k3dVariable<gdouble> m_Scale;
	
	k3dVariable<bool> m_DeleteOriginals;
	k3dVariable<bool> m_CreateFaces;
	k3dVariable<bool> m_CreateEdges;

	k3dVariable<InsetType> m_InsetType;
	
	k3d::SelectedPathArray m_Selection;
	k3d::GeometryEngine m_Engine;
};

#endif // BEVEL_H
