#ifndef BULGE_H
#define BULGE_H

// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the CBulge K-3D tool, which applies a "Bulge" transformation to geometry
		\author Andy Gill (buzz@ucky.com)
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3daxis.h>
#include <k3dmouseeventobserver.h>
#include <k3dobjectdialog.h>
#include <k3dselection.h>
#include <k3dtool.h>

// Forward class references
class CBulge;

/////////////////////////////////////////////////////////////////////////////
// CBulgeDialog

typedef k3dObjectDialog CBulgeDialogBase;

class CBulgeDialog :
	public CBulgeDialogBase
{
public:
	CBulgeDialog(CBulge* const Object);
	
private:
	CBulge* const m_Object;

	static const sdpString dialogname;
	static const sdpString control_displacex;
	static const sdpString control_displacey;
	static const sdpString control_displacez;
	static const sdpString control_along;
	static const sdpString control_bulgefactor;
	static const sdpString control_profile;
	static const sdpString control_type;
};

/////////////////////////////////////////////////////////////////////////////
// CBulge

typedef k3dTool CBulgeBase;

class CBulge :
	public CBulgeBase,
	public k3dMouseEventObserver
{
friend class CBulgeDialog;
public:
	CBulge(sdpIUnknown* Initializer);

	// k3dIObject implementation
	sdpResult InitComplete();
	sdpResult EditObject();

	// k3dIMouseEventObserver implementation
	sdpResult OnLButtonStartDrag(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY);
	sdpResult OnLButtonDrag(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY, gdouble LastX, gdouble LastY, gdouble StartX, gdouble StartY);
	sdpResult OnLButtonEndDrag(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY, gdouble LastX, gdouble LastY, gdouble StartX, gdouble StartY);

	// k3dIEventObserver implementation
	sdpResult ObserveEvent(k3dIEventSource* const Source, SDPCSTRING Event, SDPCSTRING EventData, SDPCSTRING ObserverData);

	void PushEditing();
	void UpdateGeometry();
	void CancelEditing();
	void PopEditing();

	typedef enum
	{
		LINEAR,
		RADIAL,
	} Type;

	friend std::ostream& operator<<(std::ostream& Stream, const Type& Value)
	{
		switch(Value)
			{
				case LINEAR:
					Stream << "linear";
					break;
				case RADIAL:
					Stream << "radial";
					break;
			}
			
		return Stream;
	}
	
	friend std::istream& operator>>(std::istream& Stream, Type& Value)
	{
		sdpString text;
		Stream >> text;
		
		if(text == "linear")
			Value = LINEAR;
		else if(text == "radial")
			Value = RADIAL;
		else
			std::cerr << g_pretty_function() << ": unknown enumeration [" << text << "]" << std::endl;
		
		return Stream;
	}

private:
	k3dVariable<gdouble> m_BulgeFactor;
	k3dVariable<bool> m_DisplaceX;
	k3dVariable<bool> m_DisplaceY;
	k3dVariable<bool> m_DisplaceZ;
	k3dVariable<k3d::Axis> m_Along;
	k3dVariable<Type> m_Type;

	k3dScalarChannel m_Profile;
	
	k3d::SelectedLocationArray m_CurrentSelection;
	k3d::Dimensions m_CurrentSelectionDimensions;
};

#endif // BULGE_H

