#ifndef EXTRUDE_H
#define EXTRUDE_H

// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares CExtrude, a K-3D tool for creating ducted solids from existing geometry
		\author Tim Shead (tshead@k-3d.com)
		\author Romain Behar (romainbehar@yahoo.com)
*/

#include <k3dbuildgeometry.h>
#include <k3dmouseeventobserver.h>
#include <k3dobjectdialog.h>
#include <k3dtool.h>

// Forward class declarations
class sdpGtkEventNotebookSwitchPage;

/////////////////////////////////////////////////////////////////////////////
// CExtrude

typedef k3dTool CExtrudeBase;

class CExtrude :
	public CExtrudeBase,
	public k3dMouseEventObserver
{
public:
	CExtrude(sdpIUnknown* Initializer);

	// k3dIObject implementation
	sdpResult InitComplete();
	sdpResult EditObject();

	// k3dIMouseEventObserver implementation
	sdpResult OnLButtonStartDrag(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY);
	sdpResult OnLButtonDrag(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY, gdouble LastX, gdouble LastY, gdouble StartX, gdouble StartY);
	sdpResult OnLButtonEndDrag(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY, gdouble LastX, gdouble LastY, gdouble StartX, gdouble StartY);

	void BuildGeometry();
	void UpdateGeometry();
	void CancelGeometry();
	void FinishGeometry();

private:
	typedef k3dObjectDialog PropertiesBase;
	class Properties :
		public PropertiesBase
	{
	public:
		Properties(CExtrude* const Object);

	private:
		/// Stores a pointer to the owning Extrude object
		CExtrude* const m_Object;

		void OnEvent(sdpGtkEvent* Event);
		void OnPageSwitch(sdpGtkEventNotebookSwitchPage* const Event);

		void OnOK();
		void OnCancel();
	};
	friend class Properties;

	///////////////////////////////////////////////////////////////////////////////////
	// PathProxy
	
	typedef k3d::EnginePathProxy PathProxyBase;
	class PathProxy :
		public PathProxyBase
	{
	public:
		PathProxy(CExtrude& Object, k3d::GeometryEngine& Engine, k3d::EngineMeshProxy& Mesh);
		PathProxy(CExtrude& Object, k3d::GeometryEngine& Engine, k3dIPath* const Path);
		
		void BuildGeometry();
		void Cancel();
		
	private:
		void TouchEdges();
		
		CExtrude& m_Object;
		PathCollection m_LayerPaths;
	};
	friend class PathProxy;

	///////////////////////////////////////////////////////////////////////////////////
	// PointProxy
	
	typedef k3d::EnginePointProxy PointProxyBase;
	class PointProxy :
		public PointProxyBase
	{
	public:
		PointProxy(CExtrude& Object, k3d::GeometryEngine& Engine, k3d::EngineMeshProxy& Mesh);
		PointProxy(CExtrude& Object, k3d::GeometryEngine& Engine, k3dIPoint* const Point);

		void UpdateGeometry();
		void Cancel();

		PointProxy& LayerPoint(const gulong Index);

	private:
		CExtrude& m_Object;
		PointCollection m_LayerPoints;
	};
	friend class PointProxy;
	
	////////////////////////////////////////////////////////////////////////////////////
	// EdgeProxy
	
	typedef k3d::EngineEdgeProxy EdgeProxyBase;
	class EdgeProxy :
		public EdgeProxyBase
	{
	public:
		EdgeProxy(CExtrude& Object, k3d::GeometryEngine& Engine, k3d::EnginePointProxy& Point1, k3d::EnginePointProxy& Point2, const bool Original);

		void BuildGeometry();
		void Cancel();

		void AddPath();

	private:
		CExtrude& m_Object;
		gulong m_PathCount;
		PathCollection m_LayerPaths;
	};
	friend class EdgeProxy;
	
	k3dVariable<gdouble> m_Segments;

	k3dVariable<bool> m_ConnectExterior;
	k3dVariable<bool> m_ConnectInterior;
	k3dVariable<bool> m_CreateLayers;
	k3dVariable<bool> m_CreateEndCap;

	k3dVariable<gdouble> m_Sensitivity;
	k3dVariable<gdouble> m_OffsetX;
	k3dVariable<gdouble> m_OffsetY;
	k3dVariable<gdouble> m_OffsetZ;
	k3dVariable<gdouble> m_ScaleX;
	k3dVariable<gdouble> m_ScaleY;
	k3dVariable<gdouble> m_ScaleZ;
	k3dVariable<gdouble> m_RotationX;
	k3dVariable<gdouble> m_RotationY;
	k3dVariable<gdouble> m_RotationZ;

	/// Enumerates "modes" for interactive control
	typedef enum
	{
		EXTRUDE,
		OFFSETS,
		SCALING,
		ROTATION
	} InteractiveModeType;

	InteractiveModeType m_InteractiveMode;
	
	k3d::SelectedPathArray m_Selection;
	k3d::GeometryEngine m_Engine;
};


#endif // EXTRUDE_H

