#ifndef FRACTALS_H
#define FRACTALS_H

// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the CFractal class which encapsulates a fractal construction
		\author Andy Gill (buzz@ucky.com)
*/

#ifndef SDPTYPES_H
#include <sdptypes.h>
#endif // !SDPTYPES_H

class CFractal
{
public:
	CFractal();
	
	virtual gdouble Value( const sdpVector3 Point ) = 0;
	
	virtual void Smoothness( gdouble smoothness );
 	void Octaves( gdouble octaves );

	virtual void Seed( gulong seed );
		
protected:
	void CalculateExponents();
	void CalculateBounds();

	gdouble m_H;
	gdouble m_Lacunarity;
	gdouble m_Octaves;

	gdouble* m_Exponents;
	
	gdouble m_Min; // Used to normalise the values between [0 1]
	gdouble m_Max;

	gdouble m_Seed;
};


// Fractional Brownian motion
class CFBm : public CFractal
{
public:
	CFBm();
	
	gdouble Value( const sdpVector3 Point );
};

// Perlin's turbulence funtion
class CTurbulence : public CFractal
{
public:
	CTurbulence();
	
	gdouble Value( const sdpVector3 Point );
};

// Ken Musgrave's Ridged multifractal function
class CRidged : public CFractal
{
public:
	CRidged();
	
	gdouble Value( const sdpVector3 Point );

	void Offset( gdouble offset ){ m_Offset = offset; }
	void Gain( gdouble gain ){ m_Gain = gain; }
	
protected:
	gdouble m_Offset;
	gdouble m_Gain;
};


// not really fractal but ...
class CNoise : public CFractal
{
public:
	gdouble Value( const sdpVector3 Point );
};	

class CRand : public CFractal
{
public:
	gdouble Value( const sdpVector3 Point );
	
	void Seed( gulong seed );

protected:
	gulong m_Seed;
};


#endif //FRACTALS_H

