#ifndef FREEHAND_H
#define FREEHAND_H

// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares CFreehand, a K-3D tool for creating freehand geometry
		\author Timothy M. Shead (tshead@k-3d.com)
		\author Romain Behar (romainbehar@yahoo.com)
*/

#include <k3dbuildgeometry.h>
#include <k3deditorrender.h>
#include <k3dmouseeventobserver.h>
#include <k3dobjectdialog.h>
#include <k3dtool.h>

#include <sdpgl.h>

/////////////////////////////////////////////////////////////////////////////
// CFreehand

typedef k3dTool CFreehandBase;

class CFreehand :
	public CFreehandBase,
	public k3dMouseEventObserver,
	public k3dEditorRender
{
public:
	CFreehand(sdpIUnknown* Initializer);

	//k3dIObject implementation
	sdpResult EditObject();

	// k3dIMouseEventObserver implementation
	sdpResult OnMouseMove(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY);
	sdpResult OnLButtonClick(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY);
	sdpResult OnLButtonDrag(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY, gdouble LastX, gdouble LastY, gdouble StartX, gdouble StartY);
	sdpResult OnLButtonEndDrag(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY, gdouble LastX, gdouble LastY, gdouble StartX, gdouble StartY);

	// k3dIEditorRender implementation
	sdpResult EditorPostRender(k3dIEditorEngine* const Engine);

	void BuildGeometry();
	void UpdateGeometry();
	void CancelGeometry();
	void FinishGeometry();
	
private:
	typedef k3dObjectDialog PropertiesBase;
	class Properties :
		public PropertiesBase
	{
	public:
		Properties(CFreehand* const Object);

	private:
		void OnOK();
		void OnCancel();

		CFreehand* const m_Object;
	};
	friend class Properties;

	/// Stores the position of the mouse in three dimensions
	sdpVector3 m_MouseCursor;

	/// Stores the 3D path points
	sdpVector3Array m_PathPoints;

	k3d::NewGeometryCollection m_NewGeometry;
	
	/// Converts 2D mouse coordinates into 3D coordinates
	sdpVector3 ScreenToWorld(k3dICamera* const Camera, const GdkModifierType Modifiers, const gdouble CurrentX, const gdouble CurrentY);
};

#endif // FREEHAND_H

