#ifndef KNOT_H
#define KNOT_H

// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares CKnot, a K-3D tool which generates knot paths
		\author Romain Behar (romainbehar@yahoo.com)
*/

#include <k3dbuildgeometry.h>
#include <k3dobjectdialog.h>
#include <k3dtool.h>

/////////////////////////////////////////////////////////////////////////////
// CKnots

typedef k3dTool CKnotsBase;

class CKnots :
	public CKnotsBase
{
public:
	CKnots(sdpIUnknown* Initializer);

	// k3dIObject implementation
	sdpResult EditObject();

	void BuildGeometry();
	void UpdateGeometry();
	void CancelGeometry();
	void FinishGeometry();

private:
	friend class Properties;
	typedef k3dObjectDialog PropertiesBase;
	/// Provides a GUI for the CKnots tool
	class Properties :
		public PropertiesBase
	{
	public:
		Properties(CKnots* const Object);

	private:
		void OnOK();
		void OnCancel();

		CKnots* const m_Object;
	};
	friend class Properties;

	k3dVariable<gdouble> m_EdgeCount;
	k3dVariable<gdouble> m_MeridianWraps;
	k3dVariable<gdouble> m_LongitudinalWraps;
	k3dVariable<gdouble> m_Scale;
	
	k3d::NewGeometryCollection m_NewGeometry;
};



#endif // KNOT_H
