#ifndef LISSACURVE_H
#define LISSACURVE_H

// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the CLissaCurve K-3D tool, which generates geometry composed of Lissajous curves
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3dbuildgeometry.h>
#include <k3dobjectdialog.h>
#include <k3dtool.h>

/////////////////////////////////////////////////////////////////////////////
// CLissaCurve

typedef k3dTool CLissaCurveBase;

class CLissaCurve :
	public CLissaCurveBase
{
public:
	CLissaCurve(sdpIUnknown* Initializer);

	// k3dIObject implementation
	sdpResult EditObject();

	void BuildGeometry();
	void UpdateGeometry();
	void CancelGeometry();
	void FinishGeometry();

private:

	typedef k3dObjectDialog PropertiesBase;

	/// Provides a GUI for the CLissaCurve tool
	class Properties :
		public PropertiesBase
	{
	public:
		Properties(CLissaCurve* const Object);

	private:
		void OnOK();
		void OnCancel();

		CLissaCurve* const m_Object;
	};
	friend class Properties;

	k3dVariable<gdouble> m_EdgeCount;

	k3dVariable<gdouble> m_MAmp;
	k3dVariable<gdouble> m_MFreq;
	k3dVariable<gdouble> m_MPhase;
//	bool		m_CustomM;
	k3dVariable<gdouble> m_XAmp;
	k3dVariable<gdouble> m_XFreq;
	k3dVariable<gdouble> m_XPhase;
//	bool		m_CustomX;
	k3dVariable<gdouble> m_YAmp;
	k3dVariable<gdouble> m_YFreq;
	k3dVariable<gdouble> m_YPhase;
//	bool		m_CustomY;
	k3dVariable<gdouble> m_ZAmp;
	k3dVariable<gdouble> m_ZFreq;
	k3dVariable<gdouble> m_ZPhase;
//	bool		m_CustomZ;

	k3d::NewGeometryCollection m_NewGeometry;
};

#endif // LISSACURVE_H

