#ifndef MAGNET_H
#define MAGNET_H

// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the CMagnet K-3D tool, which pulls geometry towards a point
		\author Andy Gill (buzz@ucky.com)
		\author Tim Shead (tshead@k-3d.com)
*/

//#include <k3deditorrender.h>
#include <k3dtransformobject.h>
#include <k3dobjectdialog.h>

// Forward class references
class CMagnet;

/////////////////////////////////////////////////////////////////////////////
// CMagnetDialog

typedef k3dObjectDialog CMagnetDialogBase;

class CMagnetDialog : public CMagnetDialogBase
{
public:
	CMagnetDialog(CMagnet* const Object);
	
protected:
	CMagnet* const m_Object;

	void OnOK();
	void OnCancel();

	static const sdpString dialogname;
	static const sdpString control_magnetfactor;
	static const sdpString control_radius;
	static const sdpString control_profile;
	static const sdpString control_position;
};

/////////////////////////////////////////////////////////////////////////////
// CMagnet

typedef k3dTransformObject CMagnetBase;

class CMagnet :
	public CMagnetBase
{
friend class CMagnetDialog;
public:
	CMagnet(sdpIUnknown* Initializer);

	// k3dTool overrides ...
	void Update(bool SaveUndoAction);

	// k3dIObject overrides
	sdpResult EditObject();

	// k3dIEventObserver implementation
	sdpResult ObserveEvent(k3dIEventSource* const Source, SDPCSTRING Event, SDPCSTRING EventData, SDPCSTRING ObserverData);

	// k3dIEditorRender overrides
	sdpResult EditorRender(k3dIEditorEngine* const Engine);

	// k3dITransform overrides
	sdpResult SetPosition(const ReferenceFrame Frame, gdouble Offsets[3]);

/*
	// k3dIMouseEventObserver implementation
	sdpResult OnLButtonEndDrag(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY, gdouble LastX, gdouble LastY, gdouble StartX, gdouble StartY);
*/

protected:
	k3dVariable<gdouble> m_MagnetFactor;
	k3dVariable<gdouble> m_Radius;
	k3dScalarChannel m_Profile;
	
	bool m_InteractiveUpdate;
};

#endif // MAGNET_H

