#ifndef MIRROR_H
#define MIRROR_H

// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the CMirror K-3D tool, which reflects geometry about an axis
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3daxis.h>
#include <k3dobjectdialog.h>
#include <k3dselection.h>
#include <k3dtool.h>

#include <k3dilocation.h>

// Forward class declarations
class CMirror;

/////////////////////////////////////////////////////////////////////////////
// CMirrorDialog

typedef k3dObjectDialog CMirrorDialogBase;

class CMirrorDialog :
	public CMirrorDialogBase
{
public:
	CMirrorDialog(CMirror* const Object);

protected:
	void OnEvent(sdpGtkEvent* Event);
	
	void OnMirror();
	void OnOK();
	void OnCancel();

	CMirror* const m_Object;

	static const sdpString dialogname;
	static const sdpString control_axis;
	static const sdpString control_mirror;
};

/////////////////////////////////////////////////////////////////////////////
// CMirror

typedef k3dTool CMirrorBase;

class CMirror :
	public CMirrorBase
{
friend class CMirrorDialog;
public:
	CMirror(sdpIUnknown* Initializer);

	// k3dIObject implementation
	sdpResult InitComplete();
	sdpResult EditObject();

	void PushEditing();
	void UpdateGeometry();
	void CancelEditing();
	void PopEditing();
			
private:
	k3dVariable<k3d::Axis> m_Axis;
	k3d::SelectedLocationArray m_CurrentSelection;
};

#endif // MIRROR_H
