#ifndef NEWELL_H
#define NEWELL_H

// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares CNewell, a K-3D tool for creating Newell primitives
		\author Romain Behar (romainbehar@yahoo.com)
		\author Timothy M. Shead (tshead@k-3d.com)
*/

#include <k3dbuildgeometry.h>
#include <k3dobjectdialog.h>
#include <k3dtool.h>

/////////////////////////////////////////////////////////////////////////////
// CNewell

typedef k3dTool CNewellBase;

class CNewell :
	public CNewellBase
{
public:
	CNewell(sdpIUnknown* Initializer);

	//k3dIObject implementation
	sdpResult EditObject();

	void BuildGeometry();
	void UpdateGeometry();
	void CancelGeometry();
	void FinishGeometry();

	/// Enumerates supported primitive types
	typedef enum
	{
		TEAPOT,
		TEACUP,
		TEASPOON,
		TEAPOTINTOPIECES
	} PrimitiveType;

	friend std::ostream& operator<<(std::ostream& Stream, const PrimitiveType& Value)
	{
		switch(Value)
			{
				case TEAPOT:
					Stream << "teapot";
					break;
				case TEACUP:
					Stream << "teacup";
					break;
				case TEASPOON:
					Stream << "teaspoon";
					break;
				case TEAPOTINTOPIECES:
					Stream << "teapotintopieces";
					break;
			}
		return Stream;
	}

	friend std::istream& operator>>(std::istream& Stream, PrimitiveType& Value)
	{
		sdpString text;
		Stream >> text;

		if(text == "teapot")
			Value = TEAPOT;
		else if(text == "teacup")
			Value = TEACUP;
		else if(text == "teaspoon")
			Value = TEASPOON;
		else if(text == "teapotintopieces")
			Value = TEAPOTINTOPIECES;
		else
			std::cerr << g_pretty_function() << ": unknown enumeration [" << text << "]" << std::endl;

		return Stream;
	}

private:
	typedef k3dObjectDialog PropertiesBase;

	/// Provides the user interface for the CNewell tool
	class Properties :
		public PropertiesBase
	{
	public:
		Properties(CNewell* const Object);

	private:
		void OnOK();
		void OnCancel();
	
		CNewell* const m_Object;
	};
	friend class Properties;

	/// Controls the type of primitive to be created
	k3dVariable<PrimitiveType> m_PrimitiveType;
	/// Controls the number of times each NURBS patch will be subdivided to create a polygonal approximation	
	k3dVariable<gdouble> m_Subdivisions;
	/// Generic scale control
	k3dVariable<gdouble> m_Scale;
	
	/// Stores newly-created geometry for easier management
	k3d::NewGeometryCollection m_NewGeometry;
};


#endif // NEWELL_H

