#ifndef PATHEXTRUDE_H
#define PATHEXTRUDE_H

// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares CPathExtrude, a K-3D tool for creating ducted solids from existing geometry
		\author Tim Shead (tshead@k-3d.com)
		\author Romain Behar (romainbehar@yahoo.com)
*/

/*
#include <k3dductedsolid.h>
*/

#include <k3dmouseeventobserver.h>
#include <k3dtool.h>
#include <k3dobjectdialog.h>

#include <sdpgl.h>

/////////////////////////////////////////////////////////////////////////////
// CPathExtrude

typedef k3dTool CPathExtrudeBase;

class CPathExtrude :
	public CPathExtrudeBase,
	public k3dMouseEventObserver
{
public:
	CPathExtrude(sdpIUnknown* Initializer);

/*
	void Update(bool SaveUndoAction, bool CreateGeometry);
*/

	// Tells the tool to store any visible-selected geometry as the shape to be extruded
	void SetShape();
	// Tells the tool to store any visible-selected geometry as the path to extrude along
	void SetPath();

	//k3dIObject implementation
	sdpResult EditObject();

	// k3dIMouseEventObserver implementation
	sdpResult OnLButtonDrag(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY, gdouble LastX, gdouble LastY, gdouble StartX, gdouble StartY);

	/// Enumerates interactive "modes" for the tool
	typedef enum
	{
		EXTRUDE,
		OFFSETS,
		SCALING,
		ROTATION
	} InteractiveModeType;

	InteractiveModeType InteractiveMode() { return m_InteractiveMode; }
	void SetInteractiveMode(InteractiveModeType Mode) { m_InteractiveMode = Mode; }

private:
	typedef k3dObjectDialog PropertiesBase;
	class Properties :
		public PropertiesBase
	{
	public:
		Properties(CPathExtrude* const Object);

	protected:
		/// Stores a pointer to the owning PathExtrude object
		CPathExtrude* const m_Object;

		void OnEvent(sdpGtkEvent* Event);
		void OnPageSwitch(sdpGtkEvent* Event);

		void OnSetShape();
		void OnSetPath();
			
		static const sdpString dialogname;
		static const sdpString control_segments;
		static const sdpString control_connectexterior;
		static const sdpString control_connectinterior;
		static const sdpString control_createlayers;
		static const sdpString control_createendcaps;
		static const sdpString control_offsetx;
		static const sdpString control_offsety;
		static const sdpString control_offsetz;
		static const sdpString control_scalex;
		static const sdpString control_scaley;
		static const sdpString control_scalez;
		static const sdpString control_rotationx;
		static const sdpString control_rotationy;
		static const sdpString control_rotationz;
		static const sdpString control_pageswitch;
		static const sdpString control_tabs;
		static const sdpString control_setshape;
		static const sdpString control_setpath;
	};

/*
	k3dPathArray m_PathPaths;
	k3dPointArray m_PathPoints;
	
	k3dPathArray m_ShapePaths;
	k3dPointArray m_ShapePoints;
*/

	k3dVariable<gdouble> m_Segments;

	k3dVariable<bool> m_ConnectExterior;
	k3dVariable<bool> m_ConnectInterior;
	k3dVariable<bool> m_CreateLayers;
	k3dVariable<bool> m_CreateEndCaps;

	k3dVariable<gdouble> m_Sensitivity;
	k3dVariable<gdouble> m_OffsetX;
	k3dVariable<gdouble> m_OffsetY;
	k3dVariable<gdouble> m_OffsetZ;
	k3dVariable<gdouble> m_ScaleX;
	k3dVariable<gdouble> m_ScaleY;
	k3dVariable<gdouble> m_ScaleZ;
	k3dVariable<gdouble> m_RotationX;
	k3dVariable<gdouble> m_RotationY;
	k3dVariable<gdouble> m_RotationZ;

/*
	k3dDuctedSolidPointMap m_NewPoints;
*/

	InteractiveModeType m_InteractiveMode;
};


#endif // PATHEXTRUDE_H

