#ifndef SLANT_H
#define SLANT_H

// K-3D
// Copyright  1995-2002, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the CSlant class, which applies a "slant" transformation to an object's geometry
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3dmouseeventobserver.h>
#include <k3dobjectdialog.h>
#include <k3dselection.h>
#include <k3dtool.h>

#include <k3dicamera.h>

// Forward class references
class CSlant;

/////////////////////////////////////////////////////////////////////////////
// CSlantDialog

typedef k3dObjectDialog CSlantDialogBase;

class CSlantDialog :
	public CSlantDialogBase
{
public:
	CSlantDialog(CSlant* const Object);

private:
	CSlant* const m_Object;

	static const sdpString dialogname;
	static const sdpString control_x;
	static const sdpString control_y;
	static const sdpString control_z;
	static const sdpString control_sensitivity;
};

/////////////////////////////////////////////////////////////////////////////
// CSlant

typedef k3dTool CSlantBase;

class CSlant :
	public CSlantBase,
	public k3dMouseEventObserver
{
friend class CSlantDialog;
public:
	CSlant(sdpIUnknown* Initializer);

	// k3dIObject implementation
	sdpResult InitComplete();
	sdpResult EditObject();

	// k3dIMouseEventObserver implementation
	sdpResult OnLButtonStartDrag(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY);
	sdpResult OnLButtonDrag(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY, gdouble LastX, gdouble LastY, gdouble StartX, gdouble StartY);
	sdpResult OnLButtonEndDrag(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY, gdouble LastX, gdouble LastY, gdouble StartX, gdouble StartY);

	void PushEditing();
	void UpdateGeometry();
	void CancelEditing();
	void PopEditing();

private:
	k3dICamera::Axis m_CameraActiveAxis;
	k3dVariable<gdouble> m_X;
	k3dVariable<gdouble> m_Y;
	k3dVariable<gdouble> m_Z;
	k3dVariable<gdouble> m_Sensitivity;
	bool m_Mirror;

	k3d::SelectedLocationArray m_CurrentSelection;
	k3d::Dimensions m_CurrentSelectionDimensions;
};

#endif // SLANT_H
