#ifndef SNAP_H
#define SNAP_H

// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the CSnap K-3D tool, which "snaps" a point to another point's location
		\author Tim Shead (tshead@k-3d.com)
		\author Romain Behar (romainbehar@yahoo.com)
*/

#include <k3deditorrender.h>
#include <k3dmouseeventobserver.h>
#include <k3dobjectdialog.h>
#include <k3dtool.h>

// Forward class references
class CSnap;
class k3dIPoint;

/////////////////////////////////////////////////////////////////////////////
// CSnapDialog

typedef k3dObjectDialog CSnapDialogBase;

/// Provides the user interface for the Snap tool
class CSnapDialog :
	public CSnapDialogBase
{
public:
	CSnapDialog(CSnap* const Object);

private:
	/// Stores a reference to the owning Snap object
	CSnap* const m_Object;
	
	static const sdpString dialogname;
};


/////////////////////////////////////////////////////////////////////////////
// CSnap

typedef k3dTool CSnapBase;

class CSnap :
	public CSnapBase,
	public k3dMouseEventObserver,
	public k3dEditorRender
{
public:
	CSnap(sdpIUnknown* Initializer);

	// k3dIObject implementation
	sdpResult EditObject();

	// k3dIMouseEventObserver implementation
	sdpResult OnLButtonClick(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY);

	// k3dIEditorRender implementation
	sdpResult EditorPostRender(k3dIEditorEngine* const Engine);
		
	void PushEditing();
	void UpdateGeometry();
	void PopEditing();
	
private:
	k3dIPoint* m_Point1;
	k3dIPoint* m_Point2;
};


#endif // SNAP_H
