#ifndef SPHEREIZE_H
#define SPHEREIZE_H

// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the CSphereize K-3D tool, which "sphereizes" objects by making them conform to the surface of a sphere
		\author Timothy M. Shead (tshead@k-3d.com)
		\author Romain Behar (romainbehar@yahoo.com)
*/

#include <k3dmouseeventobserver.h>
#include <k3dobjectdialog.h>
#include <k3dselection.h>
#include <k3dtool.h>

// Forward class references
class CSphereize;

/////////////////////////////////////////////////////////////////////////////
// CSphereizeDialog

typedef k3dObjectDialog CSphereizeDialogBase;

class CSphereizeDialog :
	public CSphereizeDialogBase
{
public:
	CSphereizeDialog(CSphereize* const Object);
	
protected:
	CSphereize* const m_Object;

	static const sdpString dialogname;
	static const sdpString control_spherefactor;
};

/////////////////////////////////////////////////////////////////////////////
// CSphereize

typedef k3dTool CSphereizeBase;

class CSphereize :
	public CSphereizeBase,
	public k3dMouseEventObserver
{
friend class CSphereizeDialog;
public:
	CSphereize(sdpIUnknown* Initializer);

	//k3dIObject implementation
	sdpResult InitComplete();
	sdpResult EditObject();

	//k3dIMouseEventObserver implementation
	sdpResult OnLButtonStartDrag(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY);
	sdpResult OnLButtonDrag(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY, gdouble LastX, gdouble LastY, gdouble StartX, gdouble StartY);
	sdpResult OnLButtonEndDrag(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY, gdouble LastX, gdouble LastY, gdouble StartX, gdouble StartY);

	void PushEditing();
	void UpdateGeometry();
	void CancelEditing();
	void PopEditing();

private:
	k3dVariable<gdouble> m_SphereizeFactor;
	
	k3d::SelectedLocationArray m_CurrentSelection;
	k3d::Dimensions m_CurrentSelectionDimensions;
};


#endif // SPHEREIZE_H

