#ifndef TAPER_H
#define TAPER_H

// K-3D
// Copyright  1995-2002, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the CTaper K-3D tool, which applies a "taper" transformation to geometry
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3dmouseeventobserver.h>
#include <k3dobjectdialog.h>
#include <k3dselection.h>
#include <k3dtool.h>

#include <k3dicamera.h>

// Forward class references
class CTaper;

/////////////////////////////////////////////////////////////////////////////
// CTaperDialog

typedef k3dObjectDialog CTaperDialogBase;

class CTaperDialog :
	public CTaperDialogBase
{
public:
	CTaperDialog(CTaper* const Object);

private:
	CTaper* const m_Object;

	static const sdpString dialogname;
	static const sdpString control_displacex;
	static const sdpString control_displacey;
	static const sdpString control_displacez;
	static const sdpString control_taperfactor;
};

/////////////////////////////////////////////////////////////////////////////
// CTaper

typedef k3dTool CTaperBase;

class CTaper :
	public CTaperBase,
	public k3dMouseEventObserver
{
friend class CTaperDialog;
public:
	CTaper(sdpIUnknown* Initializer);

	// k3dIObject overrides
	sdpResult InitComplete();
	sdpResult EditObject();

	// k3dIMouseEventObserver implementation
	sdpResult OnLButtonStartDrag(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY);
	sdpResult OnLButtonDrag(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY, gdouble LastX, gdouble LastY, gdouble StartX, gdouble StartY);
	sdpResult OnLButtonEndDrag(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY, gdouble LastX, gdouble LastY, gdouble StartX, gdouble StartY);

	void PushEditing();
	void UpdateGeometry();
	void CancelEditing();
	void PopEditing();

private:
	k3dICamera::Axis m_CameraActiveAxis;
	k3dVariable<gdouble> m_TaperFactor;
	k3dVariable<bool> m_DisplaceX;
	k3dVariable<bool> m_DisplaceY;
	k3dVariable<bool> m_DisplaceZ;

	k3d::SelectedLocationArray m_CurrentSelection;
	k3d::Dimensions m_CurrentSelectionDimensions;
};

#endif // TAPER_H
