#ifndef TRANSLATE_H
#define TRANSLATE_H

// K-3D
// Copyright  1995-2002, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the CTranslate K-3D tool, which alters the position of objects, or their geometry
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3dmouseeventobserver.h>
#include <k3dobjectdialog.h>
#include <k3dselection.h>
#include <k3dtool.h>

/////////////////////////////////////////////////////////////////////////////
// CTranslate

typedef k3dTool CTranslateBase;

class CTranslate :
	public CTranslateBase,
	public k3dMouseEventObserver
{
public:
	CTranslate(sdpIUnknown* Initializer);

	// k3dIObject implementation
	sdpResult InitComplete();
	sdpResult EditObject();

	void PushEditing();
	void UpdateGeometry();
	void CancelEditing();
	void PopEditing();

	// Controls update
	void UpdateControl();

	// k3dIMouseEventObserver implementation
	sdpResult OnLButtonStartDrag(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY);
	sdpResult OnLButtonDrag(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY, gdouble LastX, gdouble LastY, gdouble StartX, gdouble StartY);
	sdpResult OnLButtonEndDrag(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY, gdouble LastX, gdouble LastY, gdouble StartX, gdouble StartY);

private:
	typedef k3dObjectDialog PropertiesBase;
	class Properties :
		public PropertiesBase
	{
	public:
		Properties(CTranslate* const Object);

	private:
		CTranslate* const m_Object;

		static const sdpString dialogname;
		static const sdpString control_x;
		static const sdpString control_y;
		static const sdpString control_z;
	};
	friend class Properties;

	k3dVariable<gdouble> m_RawX;
	k3dVariable<gdouble> m_RawY;
	k3dVariable<gdouble> m_RawZ;
	k3dVariable<gdouble> m_Sensitivity;

	sdpVector3 m_Translate;

	k3d::SelectedLocationArray m_CurrentSelection;
};

#endif // TRANSLATE_H
