#ifndef TRIANGULATE_H
#define TRIANGULATE_H

// K-3D
// Copyright  1995-2002, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the CTriangulate K-3D tool, which triangulates polygons
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3dbuildgeometry.h>
#include <k3dobjectdialog.h>
#include <k3dtool.h>

/////////////////////////////////////////////////////////////////////////////
// CTriangulate

typedef k3dTool CTriangulateBase;

class CTriangulate :
	public CTriangulateBase
{
public:
	CTriangulate(sdpIUnknown* Initializer);

	// k3dIObject implementation
	sdpResult InitComplete();
	sdpResult EditObject();

	void BuildGeometry();
	void CancelGeometry();
	void FinishGeometry();

private:
	typedef k3dObjectDialog PropertiesBase;
	class Properties :
		public PropertiesBase
	{
	public:
		Properties(CTriangulate* const Object);

	private:
		CTriangulate* const m_Object;

		void OnOK();
		void OnCancel();
	};
	friend class Properties;

	///////////////////////////////////////////////////////////////////////////////////
	// PathProxy

	typedef k3d::EnginePathProxy PathProxyBase;
	class PathProxy :
		public PathProxyBase
	{
	public:
		PathProxy(CTriangulate& Object, k3d::GeometryEngine& Engine, k3d::EngineMeshProxy& Mesh);
		PathProxy(CTriangulate& Object, k3d::GeometryEngine& Engine, k3dIPath* const Path);

		void BuildGeometry();
		void UpdateGeometry();
		void Cancel();

	private:
		CTriangulate& m_Object;
		PathCollection m_NewPaths;
	};
	friend class PathProxy;

	k3d::SelectedPathArray m_Selection;
	k3d::GeometryEngine m_Engine;
};

#endif // TRIANGULATE_H
