#ifndef WAVE_H
#define WAVE_H

// K-3D
// Copyright  1995-2002, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Defines the CWave K-3D tool, which applies a "wave" transformation to geometry
		\author Timothy M. Shead (tshead@k-3d.com)
*/

#include <k3dmouseeventobserver.h>
#include <k3dobjectdialog.h>
#include <k3dselection.h>
#include <k3dtool.h>

#include <k3dicamera.h>

// Forward class references
class CWave;

/////////////////////////////////////////////////////////////////////////////
// CWaveDialog

typedef k3dObjectDialog CWaveDialogBase;

class CWaveDialog :
	public CWaveDialogBase
{
public:
	CWaveDialog(CWave* const Object);

private:
	CWave* const m_Object;

	static const sdpString dialogname;
	static const sdpString control_displacex;
	static const sdpString control_displacey;
	static const sdpString control_displacez;
	static const sdpString control_amplitude;
	static const sdpString control_wavelength;
	static const sdpString control_phase;
};

/////////////////////////////////////////////////////////////////////////////
// CWave

typedef k3dTool CWaveBase;

class CWave :
	public CWaveBase,
	public k3dMouseEventObserver
{
friend class CWaveDialog;
public:
	CWave(sdpIUnknown* Initializer);

	// k3dIObject implementation
	sdpResult InitComplete();
	sdpResult EditObject();

	// k3dIMouseEventObserver implementation
	sdpResult OnLButtonStartDrag(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY);
	sdpResult OnLButtonDrag(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY, gdouble LastX, gdouble LastY, gdouble StartX, gdouble StartY);
	sdpResult OnLButtonEndDrag(k3dICamera* Camera, GdkModifierType Modifiers, gdouble CurrentX, gdouble CurrentY, gdouble LastX, gdouble LastY, gdouble StartX, gdouble StartY);

	void PushEditing();
	void UpdateGeometry();
	void CancelEditing();
	void PopEditing();

protected:
	k3dICamera::Axis m_CameraActiveAxis;
	k3dVariable<bool> m_DisplaceX;
	k3dVariable<bool> m_DisplaceY;
	k3dVariable<bool> m_DisplaceZ;
	k3dVariable<gdouble> m_Amplitude;
	k3dVariable<gdouble> m_Wavelength;
	k3dVariable<gdouble> m_Phase;
	k3dVariable<bool> m_FiniteRange;
	k3dVariable<gdouble> m_Range;

	k3d::SelectedLocationArray m_CurrentSelection;
};

#endif // WAVE_H
