#ifndef BENDER_H
#define BENDER_H

// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the CBender K-3D tool, which applies an animated "Bend" transformation to geometry
		\author Andy Gill (buzz@ucky.com)
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3daxis.h>
#include <k3dpersistobject.h>
#include <k3dobjectdialog.h>
#include <k3dvariable.h>

// Forward class references
class CBender;

/////////////////////////////////////////////////////////////////////////////
// CBenderDialog

typedef k3dObjectDialog CBenderDialogBase;

class CBenderDialog :
	public CBenderDialogBase
{
public:
	CBenderDialog(CBender* const Object);
	
protected:
	CBender* const m_Object;

	static const sdpString dialogname;
	static const sdpString control_bendangle;
	static const sdpString control_along;
	static const sdpString control_around;
	static const sdpString control_tightness;
	static const sdpString control_jointposition;
};

/////////////////////////////////////////////////////////////////////////////
// CBender

typedef k3dPersistObject CBenderBase;

class CBender : public CBenderBase
{
friend class CBenderDialog;
public:
	CBender(sdpIUnknown* Initializer);

	// k3dIObject overrides
	sdpResult EditObject();

	// k3dAnimate overrides
	sdpResult PopAnimate();
	
	// k3dPersist overrides
	sdpResult InitComplete();
	
protected:
	k3dScalarChannel m_BendAngle;
	k3dScalarChannel m_Tightness;
	k3dScalarChannel m_JointPosition;
	k3dVariable<k3d::Axis> m_Along;
	k3dVariable<k3d::Axis> m_Around;
};

#endif // BENDER_H

