#ifndef BULGER_H
#define BULGER_H

// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the CBulger K-3D object, which applies an animated "Bulge" transformation to geometry
		\author Andy Gill (buzz@ucky.com)
		\author Tim Shead (tshead@k-3d.com)
*/

#include <k3daxis.h>
#include <k3dpersistobject.h>
#include <k3dobjectdialog.h>
#include <k3dvariable.h>

// Forward class references
class CBulger;

/////////////////////////////////////////////////////////////////////////////
// CBulgerDialog

typedef k3dObjectDialog CBulgerDialogBase;

class CBulgerDialog :
	public CBulgerDialogBase
{
public:
	CBulgerDialog(CBulger* const Object);
	
protected:
	CBulger* const m_Object;

	static const sdpString dialogname;
	static const sdpString control_displacex;
	static const sdpString control_displacey;
	static const sdpString control_displacez;
	static const sdpString control_along;
	static const sdpString control_bulgefactor;
	static const sdpString control_profile;
	static const sdpString control_displacetype;
};

/////////////////////////////////////////////////////////////////////////////
// CBulger

typedef k3dPersistObject CBulgerBase;

class CBulger : public CBulgerBase
{
friend class CBulgerDialog;
public:
	CBulger(sdpIUnknown* Initializer);

	// k3dIObject overrides
	sdpResult EditObject();

	// k3dAnimate overrides
	sdpResult PopAnimate();
	
	// k3dPersist overrides
	sdpResult InitComplete();
	
	typedef enum
	{
		LINEAR,
		RADIAL,
	} DisplaceType;

	friend std::ostream& operator<<(std::ostream& Stream, const DisplaceType& Value)
	{
		switch(Value)
			{
				case LINEAR:
					Stream << "linear";
					break;
				case RADIAL:
					Stream << "radial";
					break;
			}
			
		return Stream;
	}
			
	friend std::istream& operator>>(std::istream& Stream, DisplaceType& Value)
	{
		sdpString text;
		Stream >> text;

		if(text == "linear")
			Value = LINEAR;
		else if(text == "radial")
			Value = RADIAL;
		else
			std::cerr << g_pretty_function() << ": unknown enumeration [" << text << "]" << std::endl;
		
		return Stream;
	}

protected:
	k3dScalarChannel m_BulgeFactor;
	k3dVariable<bool> m_DisplaceX;
	k3dVariable<bool> m_DisplaceY;
	k3dVariable<bool> m_DisplaceZ;
	k3dVariable<k3d::Axis> m_Along;
	k3dScalarChannel m_Profile;
	k3dVariable<DisplaceType> m_DisplaceType;
};

#endif // BULGER_H

