// K-3D
// Copyright  1995-2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Implements the functions necessary to export K-3D objects from the transformerlib module
		\author Timothy M. Shead (tshead@k-3d.com)
*/

#include "bender.h"
#include "bulger.h"
#include "displacer.h"
#include "slanter.h"
#include "sphereizer.h"
#include "taperer.h"
#include "transformer.h"
#include "twister.h"
#include "waveizer.h"

#include <k3dcom.h>
#include <k3dmodule.h>

class TransformerModule :
	public k3d::Module
{
public:
	sdpIUnknown* CreateInstance(const sdp::ClassID& ClassID, sdpIUnknown* Initializer)
	{
        if(ClassID == k3d::classid::Bender)
            return sdpCreateClassInstance<CBender>(Initializer);
		if(ClassID == k3d::classid::Bulger)
			return sdpCreateClassInstance<CBulger>(Initializer);
		if(ClassID == k3d::classid::Displacer)
			return sdpCreateClassInstance<CDisplacer>(Initializer);
		if(ClassID == k3d::classid::Slanter)
			return sdpCreateClassInstance<CSlanter>(Initializer);
		if(ClassID == k3d::classid::Sphereizer)
			return sdpCreateClassInstance<CSphereizer>(Initializer);
		if(ClassID == k3d::classid::Taperer)
			return sdpCreateClassInstance<CTaperer>(Initializer);
		if(ClassID == k3d::classid::Transformer)
			return sdpCreateClassInstance<CTransformer>(Initializer);
		if(ClassID == k3d::classid::Twister)
			return sdpCreateClassInstance<CTwister>(Initializer);
		if(ClassID == k3d::classid::Waveizer)
			return sdpCreateClassInstance<CWaveizer>(Initializer);

		return 0;
	}
	
private:
	friend class sdpSingleton<TransformerModule>;
	
	TransformerModule() :
		k3d::Module("transformers", VERSION)
	{
        // Bender Object ...
        sdpxml::Element& bender = RegisterObject(k3d::classid::Bender, "Bender");
        RegisterBookmark(bender, "Objects/Bender");

		// Bulger Object ...
		sdpxml::Element& bulger = RegisterObject(k3d::classid::Bulger, "Bulger");
		RegisterBookmark(bulger, "Objects/Bulger");

		// Displacer Object ...
		sdpxml::Element& displacer = RegisterObject(k3d::classid::Displacer, "Displacer");
		RegisterBookmark(displacer, "Objects/Dispalcer");

		// Slanter object ...
		sdpxml::Element& slanter = RegisterObject( k3d::classid::Slanter, "Slanter");
		RegisterBookmark(slanter, "Objects/Slanter");
	
		// Sphereizer object ...
		sdpxml::Element& sphereizer = RegisterObject( k3d::classid::Sphereizer, "Sphereizer");
		RegisterBookmark(sphereizer, "Objects/Sphereizer");
	
		// Taperer object ...
		sdpxml::Element& taperer = RegisterObject( k3d::classid::Taperer, "Taperer");
		RegisterBookmark(taperer, "Objects/Taperer");
	
		// Transformer object ...
		sdpxml::Element& transformer = RegisterObject( k3d::classid::Transformer, "Transformer");
		RegisterBookmark(transformer, "Objects/Transformer");
	
		// Twister object ...
		sdpxml::Element& twister = RegisterObject( k3d::classid::Twister, "Twister");
		RegisterBookmark(twister, "Objects/Twister");
	
		// Waveizer object ...
		sdpxml::Element& waveizer = RegisterObject( k3d::classid::Waveizer, "Waveizer");
		RegisterBookmark(waveizer, "Objects/Waveizer");
	}
};

K3DMODULEPROCEDURES(TransformerModule, sdpSingleton<TransformerModule>);

