#ifndef SLANTER_H
#define SLANTER_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Defines the CSlanter K-3D object, which applies an animated "slant" transformation to an object's geometry
		\author Timothy M. Shead (tshead@k-3d.com)
*/

#include <k3daxis.h>
#include <k3dpersistobject.h>
#include <k3dobjectdialog.h>
#include <k3dvariable.h>

// Forward class references ...
class CSlanter;

/////////////////////////////////////////////////////////////////////////////////////
// CSlanterDialog

typedef k3dObjectDialog CSlanterDialogBase;

/// Encapsulates the Slanter UI
class CSlanterDialog :
	public CSlanterDialogBase
{
public:
	CSlanterDialog(CSlanter* const Object);

protected:
	/// Stores a pointer to the owning object
	CSlanter* const m_Object;

	static const sdpString dialogname;
	static const sdpString control_displace;
	static const sdpString control_along;
	static const sdpString control_mirror;
	static const sdpString control_slantfactor;
	static const sdpString control_slantfalloff;
};

////////////////////////////////////////////////////////////////////////////////////
// CSlanter

typedef k3dPersistObject CSlanterBase;

/// Encapsulates the Slanter object
class CSlanter : public CSlanterBase
{
friend class CSlanterDialog;
friend class CSlanterSlantObject;
public:
	CSlanter(sdpIUnknown* Initializer);
	~CSlanter();

	// k3dIObject overrides
	sdpResult EditObject();

	// k3dIAnimate overrides
	sdpResult PopAnimate();

	// k3dIPersist overrides
	sdpResult InitComplete();

protected:
	k3dVariable<k3d::Axis> m_Displace;
	k3dVariable<k3d::Axis> m_Along;	
	k3dVariable<bool> m_Mirror;
	k3dScalarChannel m_SlantFactor;
	k3dScalarChannel m_SlantFalloff;
};

#endif // SLANTER_H
