#ifndef TAPERER_H
#define TAPERER_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Defines the CTaperer K-3D object, which applies an animated "taper" transformation to geometry
		\author Timothy M. Shead (tshead@k-3d.com)
*/

#include <k3daxis.h>
#include <k3dpersistobject.h>
#include <k3dobjectdialog.h>
#include <k3dvariable.h>

// Forward class references ...
class CTaperer;

/////////////////////////////////////////////////////////////////////////////////////
// CTapererDialog

typedef k3dObjectDialog CTapererDialogBase;

/// Encapsulates the Taperer UI
class CTapererDialog :
	public CTapererDialogBase
{
public:
	CTapererDialog(CTaperer* const Object);

protected:
	/// Stores a pointer to the owning object
	CTaperer* const m_Object;

	static const sdpString dialogname;
	static const sdpString control_displacex;
	static const sdpString control_displacey;
	static const sdpString control_displacez;
	static const sdpString control_along;
	static const sdpString control_taperfactor;
	static const sdpString control_taperfalloff;
};

////////////////////////////////////////////////////////////////////////////////////
// CTaperer

typedef k3dPersistObject CTapererBase;

/// Encapsulates the Taperer object
class CTaperer : public CTapererBase
{
friend class CTapererDialog;
friend class CTapererTaperObject;
public:
	CTaperer(sdpIUnknown* Initializer);
	~CTaperer();

	// k3dIObject overrides
	sdpResult EditObject();

	// k3dIAnimate overrides
	sdpResult PopAnimate();

	// k3dIPersist overrides
	sdpResult InitComplete();

protected:
	k3dVariable<bool> m_DisplaceX;
	k3dVariable<bool> m_DisplaceY;
	k3dVariable<bool> m_DisplaceZ;
	k3dVariable<k3d::Axis> m_Along;
	k3dScalarChannel m_TaperFactor;
	k3dScalarChannel m_TaperFalloff;
};

#endif // TAPERER_H
