#ifndef TRANSFORMER_H
#define TRANSFORMER_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Declares the CTransformer class, which is used to animate the position, orientation, and scale of objects
		\author Timothy M. Shead (tshead@k-3d.com)
		\author Romain Behar (romainbehar@yahoo.com)
*/

#include <k3dmesh.h>

#include <k3dobjectdialog.h>

// Forward class declarations ...
class CTransformer;
class sdpGtkEvent;

/////////////////////////////////////////////////////////////////////////////
// CTransformerDialog

typedef k3dObjectDialog CTransformerDialogBase;

class CTransformerDialog :
	public CTransformerDialogBase
{
public:
	CTransformerDialog(CTransformer* const Object);

protected:
	CTransformer* const m_Object;
 
	static const sdpString dialogname;
	static const sdpString control_translatelast;
	static const sdpString control_translatetype;
	static const sdpString control_translation;
	static const sdpString control_xrotation;
	static const sdpString control_yrotation;
	static const sdpString control_zrotation;
	static const sdpString control_xscale;
	static const sdpString control_yscale;
	static const sdpString control_zscale;
	static const sdpString control_xoffset;
	static const sdpString control_yoffset;
	static const sdpString control_zoffset;
	static const sdpString control_position;
	static const sdpString control_orientation;
	static const sdpString control_scale;
	static const sdpString control_target;
};

//////////////////////////////////////////////////////////////////////
// CTransformer

typedef k3dMesh CTransformerBase;

/// Used to animate position, orientation, and scale
class CTransformer :
	public CTransformerBase
{
friend class CTransformerDialog;
public:
	CTransformer(sdpIUnknown* Initializer);
	~CTransformer();

	// k3dIObject overrides
	sdpResult EditObject();

	// k3dIAnimate overrides
	sdpResult PopAnimate();

	// k3dIPersist overrides
	sdpResult InitComplete();

	k3dVariable<gdouble> m_GlobalCycles;
	k3dVariable<gdouble> m_GlobalPhase;

	k3dVariable<bool> m_EnableTranslation;
	k3dVariable<bool> m_AbsolutePath;
	k3dVariable<bool> m_TranslateLast;

	typedef enum
	{
		POINT,
		LINEAR,
		NURBS,
	} TranslateType;

	friend std::ostream& operator<<(std::ostream& Stream, const TranslateType& Value)
	{
		switch(Value)
			{
				case POINT:
					Stream << "point";
					break;
				case LINEAR:
					Stream << "linear";
					break;
				case NURBS:
					Stream << "nurbs";
					break;
			}
			
		return Stream;
	}
	
	friend std::istream& operator>>(std::istream& Stream, TranslateType& Value)
	{
		sdpString text;
		Stream >> text;
		
		if(text == "point")
			Value = POINT;
		else if(text == "linear")
			Value = LINEAR;
		else if(text == "nurbs")
			Value = NURBS;
		else
			std::cerr << g_pretty_function() << ": unknown enumeration [" << text << "]" << std::endl;
		
		return Stream;
	}

	k3dVariable<TranslateType> m_TranslateType;

	k3dScalarChannel m_XRotation;
	k3dScalarChannel m_YRotation;
	k3dScalarChannel m_ZRotation;
	k3dScalarChannel m_XScale;
	k3dScalarChannel m_YScale;
	k3dScalarChannel m_ZScale;
	k3dScalarChannel m_XOffset;
	k3dScalarChannel m_YOffset;
	k3dScalarChannel m_ZOffset;
	k3dScalarChannel m_Translation;
	
	sdpVector3 GetPointPosition(gdouble PathPercent);
	sdpVector3 GetLinearPosition(gdouble PathPercent);
	sdpVector3 GetNURBSPosition(gdouble PathPercent);
	
	sdpVector3 GetPosition(gdouble Time, gdouble Delta);
	sdpVector3 GetOrientation(gdouble Time, gdouble Delta);
	sdpVector3 GetScale(gdouble Time, gdouble Delta);
};

#endif // !TRANSFORMER_H
