#ifndef TWISTER_H
#define TWISTER_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Defines the CTwister K-3D object, which applies an animated "twist" transformation to geometry
		\author Timothy M. Shead (tshead@k-3d.com)
*/

#include <k3dpersistobject.h>
#include <k3dobjectdialog.h>
#include <k3dvariable.h>

// Forward class references ...
class CTwister;

/////////////////////////////////////////////////////////////////////////////////////
// CTwisterDialog

typedef k3dObjectDialog CTwisterDialogBase;

/// Encapsulates the Twister UI
class CTwisterDialog :
	public CTwisterDialogBase
{
public:
	CTwisterDialog(CTwister* const Object);
	
	
protected:
	/// Stores a pointer to the owning object
	CTwister* const m_Object;

	/// Handles our custom events
	void OnEvent(sdpGtkEvent* Event);

	static const sdpString dialogname;
	static const sdpString control_x;
	static const sdpString control_y;
	static const sdpString control_z;
};

////////////////////////////////////////////////////////////////////////////////////
// CTwister

typedef k3dPersistObject CTwisterBase;

/// Encapsulates the Twister object
class CTwister : public CTwisterBase
{
friend class CTwisterDialog;
public:
	CTwister(sdpIUnknown* Initializer);
	~CTwister();

	// k3dIObject overrides
	sdpResult EditObject();

	// k3dIAnimate overrides
	sdpResult PopAnimate();

	// k3dIPersist overrides
	sdpResult InitComplete();

protected:
	k3dScalarChannel m_X;
	k3dScalarChannel m_Y;
	k3dScalarChannel m_Z;
	
/*
	k3dVariablebool						m_XFalloff;
	k3dVariablebool						m_YFalloff;
	k3dVariablebool						m_ZFalloff;
	KObjectContainer<&sdpChannel, &IID_k3dIChannel, k3dIChannel> m_Xfalloff;
	KObjectContainer<&sdpChannel, &IID_k3dIChannel, k3dIChannel> m_Yfalloff;
	KObjectContainer<&sdpChannel, &IID_k3dIChannel, k3dIChannel> m_Zfalloff;

	double m_StartPoint[3];
*/	
};

#endif // TWISTER_H
