#ifndef WAVEIZER_H
#define WAVEIZER_H

// K-3D
// Copyright  1995 - 2000, Timothy M. Shead
//
// Contact: tshead@k-3d.com
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/** \file
		\brief Defines the CWaveizer K-3D object, which applies an animated "wave" transformation to geometry
		\author Timothy M. Shead (tshead@k-3d.com)
*/

#include <k3daxis.h>
#include <k3dpersistobject.h>
#include <k3dobjectdialog.h>
#include <k3dvariable.h>

// Forward class references ...
class CWaveizer;

/////////////////////////////////////////////////////////////////////////////////////
// CWaveizerDialog

typedef k3dObjectDialog CWaveizerDialogBase;

/// Encapsulates the Waveizer UI
class CWaveizerDialog :
	public CWaveizerDialogBase
{
public:
	CWaveizerDialog(CWaveizer* const Object);

protected:
	/// Stores a pointer to the owning object
	CWaveizer* const m_Object;

	static const sdpString dialogname;
	static const sdpString control_displacex;
	static const sdpString control_displacey;
	static const sdpString control_displacez;
	static const sdpString control_along;
	static const sdpString control_amplitude;
	static const sdpString control_wavelength;
	static const sdpString control_phase;
};

////////////////////////////////////////////////////////////////////////////////////
// CWaveizer

typedef k3dPersistObject CWaveizerBase;

/// Encapsulates the Waveizer object
class CWaveizer : public CWaveizerBase
{
friend class CWaveizerDialog;
friend class CWaveizerWaveizeObject;
public:
	CWaveizer(sdpIUnknown* Initializer);
	~CWaveizer();

	// k3dIObject overrides
	sdpResult EditObject();

	// k3dIAnimate overrides
	sdpResult PopAnimate();

	// k3dIPersist overrides
	sdpResult InitComplete();

protected:
	k3dVariable<bool> m_DisplaceX;
	k3dVariable<bool> m_DisplaceY;
	k3dVariable<bool> m_DisplaceZ;

	k3dVariable<k3d::Axis> m_Along;

//	k3dVariable<bool> m_FiniteRange;

	k3dScalarChannel m_Amplitude;
	k3dScalarChannel m_Wavelength;
	k3dScalarChannel m_Phase;
//	k3dScalarChannel m_Range;
//	k3dScalarChannel m_Falloff;
};

#endif // WAVEIZER_H
