#!/usr/bin/perl

# Stupid C++ beautifier (1.0, romainbehar@yahoo.com)

# beautifies indentation only, assumes code and C-comments are properly written:
#	- no syntax error
#	- no more than one /* and */ per line
# has no option >:->
# doesnt care about indenting statement 'for(1) foo();'


use strict;
use strict 'subs';


# Globals
use vars qw(
	$tab
	$ccomment
);


$tab = "\t";		# Tab string
$ccomment = 0;		# Boolean == 'inside C-comment'


if(! defined $ARGV[0])
{
	print "cb.pl file1 file2 etc. (including wildcards, replaces original files)\n";
}
else
{
	# beautify each file given as parameter
	# Perl automagically processes wildcards
	foreach (@ARGV)
	{
		&BeautifyFile($_)
			if(-f $_);
	}
}


sub	BeautifyFile
{
	my $fname = shift;
	my $source = LoadFile($fname);
	return if($source eq '');

	# Get rid of whitespaces before and after braces
	$source =~ s/\s*\{\s*/\{/mg;
	$source =~ s/\s*\}\s*/\}/mg;
	# One brace per line, alone!
	$source =~ s/([^\{])\{/$1\n\{/g;
	$source =~ s/\{\{/\{\n\{\n/g;
	$source =~ s/\{([^\{\}])/\{\n$1/g;
	$source =~ s/([^\}])\}/$1\n\}/g;
	$source =~ s/\}\}/\}\n\}\n/g;
	$source =~ s/\}\s*([^\};])/\}\n\n$1/g;
	$source =~ s/\}\s+\}/\}\n\}/g;	# only semi-colon is glued to closing brace
	$source =~ s/\s+;/;/g;	# no space before semi-colon
	$source =~ s/\}\s*(\w+)\s*;/\}\n$1;/g;	# structure definition
	# Glue empty braces together
	$source =~ s/\s*\{\s*\}/ \{\}/g;
	# Semi-colon following closing brace
	$source =~ s/\}\s+;/\};/g;

	# overwrite original file
	open OUTFILE, ">$fname";

	my $indent = 0;		# Indentation level
	my $classdef = 0;	# Level (of nested classes)
	my @saveindent = ();
	# process each line
	for(split /\n/, $source)
	{
		# indent current line, from current indentation level and class definition
		my ($line, $newindent, $newclassdef) =
			IndentLine($_, $indent, $classdef);

		if($newclassdef > $classdef)
		{
			# stack indentation level when entering a new class definition
			push @saveindent, $indent;
			$newindent = 0;
		}
		elsif($newclassdef > 0)
		{
			# end of classdefinition : restore indentation level
			if($newindent == 0)
			{
				$newindent = pop @saveindent;
				$newclassdef--;
			}
		}

		$indent = $newindent;
		$classdef = $newclassdef;

		print OUTFILE $line."\n";
	}

	close OUTFILE;
}


sub	IndentLine
{
	my $indent;
	my $classdef;

	($_, $indent, $classdef) = @_;

	# mark C-like comments
	$ccomment = 1 if(/\/\*/);
	$ccomment = 0 if(/\*\//);

	# don't indent inside comments
	if(!$ccomment)
		{ s/^\s*(.*?)\s*$/$1/; }	# strip leading and trailing spaces
	else	{ s/^(.*?)\s*$/$1/; }		# strip only trailing spaces

	# Compute indentation (easy with one brace per line :-)
	# (we dont care about braces inside strings)
	$indent-- if(/\}/ and !/\{\}/);


	if($_ ne '' and !$ccomment)
	{	# don't tab empty lines nor C-comments

		# we're inside a nested class, indent it
		print OUTFILE $tab x ($classdef -1) if($classdef > 1);

		my $level = $indent * 2 - 2;

		if(/(public|private|protected)\s*:/ or /friend\s+/)
		{
			# inheritance attributes are at the same level as 'class Foo'
			print OUTFILE $tab x ($level);
		}
		elsif((/\{/ or /\}/) and !/\{\}/)
		{
			# indent functions defined inside classes as classes
			my $offset = 0;
			$offset = -1 if($classdef > 0 and $indent > 0);

			print OUTFILE $tab x ($level + 2 + $offset)
				if($indent > 0);
		}
		else
		{
			# indent functions defined inside classes as classes
			my $offset = 0;
			$offset = -1 if($classdef > 0 and $indent > 1);

			print OUTFILE $tab x ($level + 1 + $offset)
				if($indent > 0);
		}
	}

	# Compute indentation
	$indent++ if(/\{/ and !/\{\}/);

	$classdef++ if(/^class\s(.*)[^;]\s*$/);	# new class

	return ($_, $indent, $classdef);
}


sub	LoadFile
{
	my $file = '';

	undef $/;
	open(INFILE, shift) && do
	{
		$file = <INFILE>;
		close INFILE;
	};
	$/ = "\n";	# Default

	return $file;
}


